@@ -2,7 +2,7 @@
  * Programmer(s): Daniel R. Reynolds @ SMU
  *---------------------------------------------------------------
  * SUNDIALS Copyright Start
- * Copyright (c) 2002-2019, Lawrence Livermore National Security
+ * Copyright (c) 2002-2022, Lawrence Livermore National Security
  * and Southern Methodist University.
  * All rights reserved.
  *
@@ -53,7 +53,7 @@
 #include <sunmatrix/sunmatrix_band.h> /* access to band SUNMatrix */
 #include <sunlinsol/sunlinsol_band.h> /* access to band SUNLinearSolver */
 #include <sundials/sundials_types.h>  /* def. of type 'realtype' */
-#include <sundials/sundials_math.h>   /* def. of SUNRsqrt, etc. */
+
 #ifdef _OPENMP
 #include <omp.h>                      /* OpenMP functions */
 #endif
@@ -133,6 +133,11 @@ int main(int argc, char *argv[])
   int iout, num_threads;
   long int nst, nst_a, nfe, nfi, nsetups, nje, nfeLS, nni, ncfn, netf;
 
+  /* Create the SUNDIALS context object for this simulation */
+  SUNContext ctx;
+  flag = SUNContext_Create(NULL, &ctx);
+  if (check_flag(&flag, "SUNContext_Create", 1)) return 1;
+
   /* allocate udata structure */
   udata = (UserData) malloc(sizeof(*udata));
   if (check_flag((void *) udata, "malloc", 2)) return 1;
@@ -143,7 +148,7 @@ int main(int argc, char *argv[])
   num_threads = omp_get_max_threads();   /* overwrite with OMP_NUM_THREADS environment variable */
 #endif
   if (argc > 1)                          /* overwrite with command line value, if supplied */
-    num_threads = strtol(argv[1], NULL, 0);
+    num_threads = (int) strtol(argv[1], NULL, 0);
 
   /* store the inputs in the UserData structure */
   udata->N  = N;
@@ -169,19 +174,26 @@ int main(int argc, char *argv[])
   printf("    reltol = %.1"ESYM",  abstol = %.1"ESYM"\n\n", reltol, abstol);
 
   /* Initialize vector data structures */
-  y = N_VNew_OpenMP(NEQ, num_threads);      /* Create vector for solution */
+
+  /* Create vector for solution */
+  y = N_VNew_OpenMP(NEQ, num_threads, ctx);
   if (check_flag((void *)y, "N_VNew_OpenMP", 0)) return 1;
-  udata->dx = RCONST(1.0)/(N-1);            /* set spatial mesh spacing */
-  data = N_VGetArrayPointer(y);             /* Access data array for new NVector y */
-  if (check_flag((void *)data, "N_VGetArrayPointer", 0)) return 1;
-  umask = N_VNew_OpenMP(NEQ, num_threads);  /* Create vector masks */
-  if (check_flag((void *)umask, "N_VNew_OpenMP", 0)) return 1;
-  vmask = N_VNew_OpenMP(NEQ, num_threads);
-  if (check_flag((void *)vmask, "N_VNew_OpenMP", 0)) return 1;
-  wmask = N_VNew_OpenMP(NEQ, num_threads);
-  if (check_flag((void *)wmask, "N_VNew_OpenMP", 0)) return 1;
+
+  /* Create vector masks */
+  umask = N_VClone(y);
+  if (check_flag((void *)umask, "N_VClone", 0)) return 1;
+
+  vmask = N_VClone(y);
+  if (check_flag((void *)vmask, "N_VClone", 0)) return 1;
+
+  wmask = N_VClone(y);
+  if (check_flag((void *)wmask, "N_VClone", 0)) return 1;
 
   /* Set initial conditions into y */
+  udata->dx = RCONST(1.0)/(N-1);     /* set spatial mesh spacing */
+  data = N_VGetArrayPointer(y);      /* Access data array for new NVector y */
+  if (check_flag((void *)data, "N_VGetArrayPointer", 0)) return 1;
+
   pi = RCONST(4.0)*atan(RCONST(1.0));
   for (i=0; i<N; i++) {
     data[IDX(i,0)] =  a  + RCONST(0.1)*sin(pi*i*udata->dx);  /* u */
@@ -206,16 +218,16 @@ int main(int argc, char *argv[])
   for (i=0; i<N; i++)  data[IDX(i,2)] = RCONST(1.0);
 
   /* Initialize matrix and linear solver data structures */
-  A = SUNBandMatrix(NEQ, 4, 4);
+  A = SUNBandMatrix(NEQ, 4, 4, ctx);
   if (check_flag((void *)A, "SUNBandMatrix", 0)) return 1;
-  LS = SUNLinSol_Band(y, A);
+  LS = SUNLinSol_Band(y, A, ctx);
   if (check_flag((void *)LS, "SUNLinSol_Band", 0)) return 1;
 
   /* Call ARKStepCreate to initialize the ARK timestepper module and
      specify the right-hand side function in y'=f(t,y), the inital time
      T0, and the initial dependent variable vector y.  Note: since this
      problem is fully implicit, we set f_E to NULL and f_I to f. */
-  arkode_mem = ARKStepCreate(NULL, f, T0, y);
+  arkode_mem = ARKStepCreate(NULL, f, T0, y, ctx);
   if (check_flag((void *)arkode_mem, "ARKStepCreate", 0)) return 1;
 
   /* Set routines */
@@ -262,11 +274,11 @@ int main(int argc, char *argv[])
     flag = ARKStepEvolve(arkode_mem, tout, y, &t, ARK_NORMAL);    /* call integrator */
     if (check_flag(&flag, "ARKStepEvolve", 1)) break;
     u = N_VWL2Norm(y,umask);                               /* access/print solution statistics */
-    u = SUNRsqrt(u*u/N);
+    u = sqrt(u*u/N);
     v = N_VWL2Norm(y,vmask);
-    v = SUNRsqrt(v*v/N);
+    v = sqrt(v*v/N);
     w = N_VWL2Norm(y,wmask);
-    w = SUNRsqrt(w*w/N);
+    w = sqrt(w*w/N);
     printf("  %10.6"FSYM"  %10.6"FSYM"  %10.6"FSYM"  %10.6"FSYM"\n", t, u, v, w);
     if (flag >= 0) {                                       /* successful solve: update output time */
       tout += dTout;
@@ -324,10 +336,12 @@ int main(int argc, char *argv[])
   ARKStepFree(&arkode_mem);     /* Free integrator memory */
   SUNLinSolFree(LS);            /* Free linear solver */
   SUNMatDestroy(A);             /* Free matrix */
-  N_VDestroy_OpenMP(y);         /* Free vectors */
-  N_VDestroy_OpenMP(umask);
-  N_VDestroy_OpenMP(vmask);
-  N_VDestroy_OpenMP(wmask);
+  N_VDestroy(y);                /* Free vectors */
+  N_VDestroy(umask);
+  N_VDestroy(vmask);
+  N_VDestroy(wmask);
+  SUNContext_Free(&ctx);        /* Free context */
+
   return 0;
 }
 
@@ -349,7 +363,7 @@ static int f(realtype t, N_Vector y, N_Vector ydot, void *user_data)
   realtype dx = udata->dx;
   realtype *Ydata=NULL, *dYdata=NULL;
   realtype uconst, vconst, wconst, u, ul, ur, v, vl, vr, w, wl, wr;
-  sunindextype i;
+  sunindextype i = 0;
 
   /* clear out ydot (to be careful) */
   N_VConst(0.0, ydot);
@@ -424,7 +438,7 @@ static int LaplaceMatrix(realtype c, SUNMatrix Jac, UserData udata)
 {
   sunindextype N = udata->N;            /* set shortcuts */
   realtype dx = udata->dx;
-  sunindextype i;
+  sunindextype i  = 0;
   realtype uconst = c*udata->du/dx/dx;
   realtype vconst = c*udata->dv/dx/dx;
   realtype wconst = c*udata->dw/dx/dx;
@@ -456,7 +470,7 @@ static int ReactionJac(realtype c, N_Vector y, SUNMatrix Jac, UserData udata)
 {
   sunindextype N = udata->N;                   /* set shortcuts */
   realtype ep = udata->ep;
-  sunindextype i;
+  sunindextype i = 0;
   realtype u, v, w;
   realtype *Ydata = N_VGetArrayPointer(y);     /* access solution array */
   if (check_flag((void *)Ydata, "N_VGetArrayPointer", 0)) return 1;
