@@ -5,7 +5,7 @@
  * Daniel R. Reynolds and parallelized with OpenMP 4.5.
  *---------------------------------------------------------------
  * SUNDIALS Copyright Start
- * Copyright (c) 2002-2021, Lawrence Livermore National Security
+ * Copyright (c) 2002-2022, Lawrence Livermore National Security
  * and Southern Methodist University.
  * All rights reserved.
  *
@@ -102,6 +102,11 @@ int main() {
   int iout;
   long int nst, nst_a, nfe, nfi, nsetups, nli, nJv, nlcf, nni, ncfn, netf;
 
+  /* Create the SUNDIALS context object for this simulation */
+  SUNContext ctx;
+  flag = SUNContext_Create(NULL, &ctx);
+  if (check_flag(&flag, "SUNContext_Create", 1)) return 1;
+
   /* allocate and fill udata structure */
   udata = (UserData) malloc(sizeof(*udata));
   udata->N = N;
@@ -114,7 +119,7 @@ int main() {
   printf("  diffusion coefficient:  k = %"GSYM"\n", udata->k);
 
   /* Initialize data structures */
-  y = N_VNew_OpenMPDEV(N);          /* Create OpenMPDEV vector for solution */
+  y = N_VNew_OpenMPDEV(N, ctx);          /* Create OpenMPDEV vector for solution */
   if (check_flag((void *) y, "N_VNew_Serial", 0)) return 1;
   N_VConst(0.0, y);                 /* Set initial conditions */
 
@@ -122,7 +127,7 @@ int main() {
      right-hand side function in y'=f(t,y), the inital time T0, and
      the initial dependent variable vector y.  Note: since this
      problem is fully implicit, we set f_E to NULL and f_I to f. */
-  arkode_mem = ARKStepCreate(NULL, f, T0, y);
+  arkode_mem = ARKStepCreate(NULL, f, T0, y, ctx);
   if (check_flag((void *) arkode_mem, "ARKStepCreate", 0)) return 1;
 
   /* Set routines */
@@ -136,7 +141,7 @@ int main() {
   if (check_flag(&flag, "ARKStepSStolerances", 1)) return 1;
 
   /* Initialize PCG solver -- no preconditioning, with up to N iterations  */
-  LS = SUNLinSol_PCG(y, 0, N);
+  LS = SUNLinSol_PCG(y, 0, N, ctx);
   if (check_flag((void *)LS, "SUNLinSol_PCG", 0)) return 1;
 
   /* Linear solver interface -- set user-supplied J*v routine (no 'jtsetup' required) */
@@ -228,8 +233,10 @@ int main() {
   /* Clean up and return with successful completion */
   N_VDestroy(y);               /* Free vectors */
   free(udata);                 /* Free user data */
-  ARKStepFree(&arkode_mem);     /* Free integrator memory */
+  ARKStepFree(&arkode_mem);    /* Free integrator memory */
   SUNLinSolFree(LS);           /* Free linear solver */
+  SUNContext_Free(&ctx);       /* Free context */
+
   return 0;
 }
 
