@@ -1,23 +1,19 @@
 /*-----------------------------------------------------------------
  * Programmer(s): Daniel R. Reynolds @ SMU
  *---------------------------------------------------------------
- * LLNS/SMU Copyright Start
- * Copyright (c) 2015, Southern Methodist University and 
- * Lawrence Livermore National Security
- *
- * This work was performed under the auspices of the U.S. Department 
- * of Energy by Southern Methodist University and Lawrence Livermore 
- * National Laboratory under Contract DE-AC52-07NA27344.
- * Produced at Southern Methodist University and the Lawrence 
- * Livermore National Laboratory.
- *
+ * SUNDIALS Copyright Start
+ * Copyright (c) 2002-2022, Lawrence Livermore National Security
+ * and Southern Methodist University.
  * All rights reserved.
- * For details, see the LICENSE file.
- * LLNS/SMU Copyright End
+ *
+ * See the top-level LICENSE and NOTICE files for details.
+ *
+ * SPDX-License-Identifier: BSD-3-Clause
+ * SUNDIALS Copyright End
  *---------------------------------------------------------------
  * Demonstration program for ARKODE - Krylov linear solver.
  * ODE system from ns-species interaction PDE in 2 dimensions.
- * 
+ *
  * This program solves a stiff ODE system that arises from a system
  * of partial differential equations. The PDE system is a food web
  * population model, with predator-prey interaction and diffusion on
@@ -30,14 +26,14 @@
  *
  *    i               i      i
  *  dc /dt  =  d(i)*(c    + c   )  +  f (x,y,c)  (i=1,...,ns)
- *                    xx     yy        i   
+ *                    xx     yy        i
  *
  * where
  *
  *                 i          ns         j
  *  f (x,y,c)  =  c *(b(i) + sum a(i,j)*c )
- *   i                       j=1                                         
- *                                                                       
+ *   i                       j=1
+ *
  * The number of species is ns = 2*np, with the first np being prey
  * and the last np being predators. The coefficients a(i,j), b(i),
  * d(i) are:
@@ -58,24 +54,24 @@
  *
  * The resulting ODE system is stiff.
  *
- * The ODE system is solved using Newton iteration and the SUNSPGMR
- * linear solver (scaled preconditioned GMRES).
+ * The ODE system is solved using Newton iteration and the
+ * SUNLinSol_SPGMR linear solver (scaled preconditioned GMRES).
  *
  * The preconditioner matrix used is the product of two matrices:
- * (1) A matrix, only defined implicitly, based on a fixed number
- * of Gauss-Seidel iterations using the diffusion terms only.
- * (2) A block-diagonal matrix based on the partial derivatives
- * of the interaction terms f only, using block-grouping (computing
- * only a subset of the ns by ns blocks).
+ * (1) A matrix, only defined implicitly, based on a fixed number of
+ * Gauss-Seidel iterations using the diffusion terms only. (2) A
+ * block-diagonal matrix based on the partial derivatives of the
+ * interaction terms f only, using block-grouping (computing only a
+ * subset of the ns by ns blocks).
  *
- * Four different runs are made for this problem.
- * The product preconditoner is applied on the left and on the
- * right. In each case, both the modified and classical Gram-Schmidt
- * options are tested.
- * In the series of runs, ARKodeInit, SUNSPGMR and 
- * ARKSpilsSetLinearSolver are called only for the first run, whereas 
- * ARKodeReInit, SUNSPGMRSetPrecType, and SUNSPGMRSetGSType are called
- * for each of the remaining three runs.
+ * Four different runs are made for this problem. The product
+ * preconditoner is applied on the left and on the right. In each
+ * case, both the modified and classical Gram-Schmidt options are
+ * tested. In the series of runs, ARKStepCreate, SUNLinSol_SPGMR and
+ * ARKStepSetLinearSolver are called only for the first run, whereas
+ * ARKStepReInit, SUNLinSol_SPGMRSetPrecType, and
+ * SUNLinSol_SPGMRSetGSType are called for each of the remaining
+ * three runs.
  *
  * A problem description, performance statistics at selected output
  * times, and final statistics are written to standard output.
@@ -84,8 +80,10 @@
  * but there should be no such messages.
  *
  * Note: This program requires the dense linear solver functions
- * newDenseMat, newIndexArray, denseAddIdentity, denseGETRF, denseGETRS, 
- * destroyMat and destroyArray.
+ * SUNDlsMat_newDenseMat, SUNDlsMat_newIndexArray,
+ * SUNDlsMat_denseAddIdentity, SUNDlsMat_denseGETRF,
+ * SUNDlsMat_denseGETRS, SUNDlsMat_destroyMat and
+ * SUNDlsMat_destroyArray.
  *
  * Note: This program assumes the sequential implementation for the
  * type N_Vector and uses the N_VGetArrayPointer function to gain
@@ -101,13 +99,31 @@
 #include <stdlib.h>
 #include <math.h>
 
-#include <arkode/arkode.h>              /* main integrator header file                 */
+#include <arkode/arkode_arkstep.h>      /* prototypes for ARKStep fcts., consts        */
 #include <sunlinsol/sunlinsol_spgmr.h>  /* access to SPGMR SUNLinearSolver             */
-#include <arkode/arkode_spils.h>        /* access to ARKSpils interface                */
 #include <nvector/nvector_serial.h>     /* serial N_Vector types, fct. and macros      */
 #include <sundials/sundials_dense.h>    /* use generic DENSE solver in preconditioning */
 #include <sundials/sundials_types.h>    /* definition of realtype                      */
-#include <sundials/sundials_math.h>     /* contains the macros ABS and SUNSQR          */
+
+/* helpful macros */
+
+#ifndef MAX
+#define MAX(A, B) ((A) > (B) ? (A) : (B))
+#endif
+
+#ifndef SQR
+#define SQR(A) ((A)*(A))
+#endif
+
+#ifndef SQRT
+#if defined(SUNDIALS_DOUBLE_PRECISION)
+#define SQRT(x) (sqrt((x)))
+#elif defined(SUNDIALS_SINGLE_PRECISION)
+#define SQRT(x) (sqrtf((x)))
+#elif defined(SUNDIALS_EXTENDED_PRECISION)
+#define SQRT(x) (sqrtl((x)))
+#endif
+#endif
 
 /* Constants */
 
@@ -143,14 +159,14 @@
 #define NGRP  (NGX*NGY)
 #define ITMAX 5
 
-/* CVodeInit Constants */
+/* Init Constants */
 
 #define NEQ  (NS*MX*MY)
 #define T0   ZERO
 #define RTOL RCONST(1.0e-5)
 #define ATOL RCONST(1.0e-5)
 
-/* CVSpgmr Constants */
+/* Spgmr Constants */
 
 #define MAXL 0     /* => use default = MIN(NEQ, 5)            */
 #define DELT ZERO  /* => use default = 0.05                   */
@@ -185,7 +201,7 @@ typedef struct {
 
 /* Private Helper Functions */
 
-static WebData AllocUserData(void);
+static WebData AllocUserData(SUNContext ctx);
 static void InitUserData(WebData wdata);
 static void SetGroups(int m, int ng, int jg[], int jig[], int jr[]);
 static void CInit(N_Vector c, WebData wdata);
@@ -196,9 +212,9 @@ static void PrintOutput(void *arkode_mem, realtype t);
 static void PrintFinalStats(void *arkode_mem);
 static void FreeUserData(WebData wdata);
 static void WebRates(realtype x, realtype y, realtype t, realtype c[],
-		     realtype rate[], WebData wdata);
+                     realtype rate[], WebData wdata);
 static void fblock (realtype t, realtype cdata[], int jx, int jy,
-		    realtype cdotdata[], WebData wdata);
+                    realtype cdotdata[], WebData wdata);
 static void GSIter(realtype gamma, N_Vector z, N_Vector x, WebData wdata);
 
 /* Small Vector Kernels */
@@ -213,7 +229,7 @@ static void v_zero(realtype u[], int n);
 
 static int f(realtype t, N_Vector y, N_Vector ydot, void *user_data);
 
-static int Precond(realtype tn, N_Vector c, N_Vector fc, booleantype jok, 
+static int Precond(realtype tn, N_Vector c, N_Vector fc, booleantype jok,
                    booleantype *jcurPtr, realtype gamma, void *user_data);
 
 static int PSolve(realtype tn, N_Vector c, N_Vector fc, N_Vector r, N_Vector z,
@@ -225,26 +241,31 @@ static int check_flag(void *flagvalue, const char *funcname, int opt);
 
 /* Implementation */
 
-int main()
+int main(int argc, char* argv[])
 {
   realtype abstol=ATOL, reltol=RTOL, t, tout;
-  N_Vector c;
-  WebData wdata;
-  SUNLinearSolver LS;
-  void *arkode_mem;
+  N_Vector c = NULL;
+  WebData wdata = NULL;
+  SUNLinearSolver LS = NULL;
+  void *arkode_mem = NULL;
   booleantype firstrun;
   int jpre, gstype, flag;
   int ns, mxns, iout;
+  int nrmfactor = 0;   /* LS norm conversion factor flag */
+  realtype nrmfac;     /* LS norm conversion factor      */
 
-  c = NULL;
-  wdata = NULL;
-  LS = NULL;
-  arkode_mem = NULL;
+  /* Create the SUNDIALS context object for this simulation */
+  SUNContext ctx;
+  flag = SUNContext_Create(NULL, &ctx);
+  if (check_flag(&flag, "SUNContext_Create", 1)) return 1;
+
+  /* Retrieve the command-line options */
+  if (argc > 1) nrmfactor = atoi(argv[1]);
 
   /* Initializations */
-  c = N_VNew_Serial(NEQ);
+  c = N_VNew_Serial(NEQ, ctx);
   if(check_flag((void *)c, "N_VNew_Serial", 0)) return(1);
-  wdata = AllocUserData();
+  wdata = AllocUserData(ctx);
   if(check_flag((void *)wdata, "AllocUserData", 2)) return(1);
   InitUserData(wdata);
   ns = wdata->ns;
@@ -254,105 +275,124 @@ int main()
   PrintIntro();
 
   /* Loop over jpre and gstype (four cases) */
-  for (jpre = PREC_LEFT; jpre <= PREC_RIGHT; jpre++) {
-    for (gstype = MODIFIED_GS; gstype <= CLASSICAL_GS; gstype++) {
-      
+  for (jpre = SUN_PREC_LEFT; jpre <= SUN_PREC_RIGHT; jpre++) {
+    for (gstype = SUN_MODIFIED_GS; gstype <= SUN_CLASSICAL_GS; gstype++) {
+
       /* Initialize c and print heading */
       CInit(c, wdata);
       PrintHeader(jpre, gstype);
 
-      /* Call ARKodeInit or ARKodeReInit, then ARKSpgmr to set up problem */
-      
-      firstrun = (jpre == PREC_LEFT) && (gstype == MODIFIED_GS);
+      /* Call ARKStepCreate or ARKStepReInit, then SPGMR to set up problem */
+
+      firstrun = (jpre == SUN_PREC_LEFT) && (gstype == SUN_MODIFIED_GS);
       if (firstrun) {
-        arkode_mem = ARKodeCreate();
-        if(check_flag((void *)arkode_mem, "ARKodeCreate", 0)) return(1);
+        arkode_mem = ARKStepCreate(NULL, f, T0, c, ctx);
+        if(check_flag((void *)arkode_mem, "ARKStepCreate", 0)) return(1);
 
         wdata->arkode_mem = arkode_mem;
 
-        flag = ARKodeSetUserData(arkode_mem, wdata);
-        if(check_flag(&flag, "ARKodeSetUserData", 1)) return(1);
+        flag = ARKStepSetUserData(arkode_mem, wdata);
+        if(check_flag(&flag, "ARKStepSetUserData", 1)) return(1);
 
-        flag = ARKodeInit(arkode_mem, NULL, f, T0, c);
-        if(check_flag(&flag, "ARKodeInit", 1)) return(1);
+        flag = ARKStepSStolerances(arkode_mem, reltol, abstol);
+        if(check_flag(&flag, "ARKStepSStolerances", 1)) return(1);
 
-        flag = ARKodeSStolerances(arkode_mem, reltol, abstol);
-        if(check_flag(&flag, "ARKodeSStolerances", 1)) return(1);
+        flag = ARKStepSetMaxNumSteps(arkode_mem, 1000);
+        if(check_flag(&flag, "ARKStepSetMaxNumSteps", 1)) return(1);
 
-        flag = ARKodeSetMaxNumSteps(arkode_mem, 1000);
-        if(check_flag(&flag, "ARKodeSetMaxNumSteps", 1)) return(1);
+        flag = ARKStepSetNonlinConvCoef(arkode_mem, 1.e-3);
+        if(check_flag(&flag, "ARKStepSetNonlinConvCoef", 1)) return(1);
 
-        flag = ARKodeSetNonlinConvCoef(arkode_mem, 1.e-3);
-        if(check_flag(&flag, "ARKodeSetNonlinConvCoef", 1)) return(1);
+        LS = SUNLinSol_SPGMR(c, jpre, MAXL, ctx);
+        if(check_flag((void *)LS, "SUNLinSol_SPGMR", 0)) return(1);
 
-        LS = SUNSPGMR(c, jpre, MAXL);
-        if(check_flag((void *)LS, "SUNSPGMR", 0)) return(1);
+        flag = ARKStepSetLinearSolver(arkode_mem, LS, NULL);
+        if(check_flag(&flag, "ARKStepSetLinearSolver", 1)) return 1;
 
-        flag = ARKSpilsSetLinearSolver(arkode_mem, LS);
-        if(check_flag(&flag, "ARKSpilsSetLinearSolver", 1)) return 1;
+        flag = SUNLinSol_SPGMRSetGSType(LS, gstype);
+        if(check_flag(&flag, "SUNLinSol_SPGMRSetGSType", 1)) return(1);
 
-        flag = SUNSPGMRSetGSType(LS, gstype);
-        if(check_flag(&flag, "SUNSPGMRSetGSType", 1)) return(1);
+        flag = ARKStepSetEpsLin(arkode_mem, DELT);
+        if(check_flag(&flag, "ARKStepSetEpsLin", 1)) return(1);
 
-        flag = ARKSpilsSetEpsLin(arkode_mem, DELT);
-        if(check_flag(&flag, "ARKSpilsSetEpsLin", 1)) return(1);
+        flag = ARKStepSetPreconditioner(arkode_mem, Precond, PSolve);
+        if(check_flag(&flag, "ARKStepSetPreconditioner", 1)) return(1);
 
-        flag = ARKSpilsSetPreconditioner(arkode_mem, Precond, PSolve);
-        if(check_flag(&flag, "ARKSpilsSetPreconditioner", 1)) return(1);
+        /* Set the linear solver tolerance conversion factor */
+        switch(nrmfactor) {
+
+        case(1):
+          /* use the square root of the vector length */
+          nrmfac = SQRT((realtype)NEQ);
+          break;
+        case(2):
+          /* compute with dot product */
+          nrmfac = -ONE;
+          break;
+        default:
+          /* use the default */
+          nrmfac = ZERO;
+          break;
+        }
+
+        flag = ARKStepSetLSNormFactor(arkode_mem, nrmfac);
+        if (check_flag(&flag, "ARKStepSetLSNormFactor", 1)) return(1);
 
       } else {
 
-        flag = ARKodeReInit(arkode_mem, NULL, f, T0, c);
-        if(check_flag(&flag, "ARKodeReInit", 1)) return(1);
+        flag = ARKStepReInit(arkode_mem, NULL, f, T0, c);
+        if(check_flag(&flag, "ARKStepReInit", 1)) return(1);
 
-        flag = SUNSPGMRSetPrecType(LS, jpre);
-        check_flag(&flag, "SUNSPGMRSetPrecType", 1);
-        flag = SUNSPGMRSetGSType(LS, gstype);
-        if(check_flag(&flag, "SUNSPGMRSetGSType", 1)) return(1);
+        flag = SUNLinSol_SPGMRSetPrecType(LS, jpre);
+        if(check_flag(&flag, "SUNLinSol_SPGMRSetPrecType", 1)) return(1);
+
+        flag = SUNLinSol_SPGMRSetGSType(LS, gstype);
+        if(check_flag(&flag, "SUNLinSol_SPGMRSetGSType", 1)) return(1);
 
       }
-      
+
       /* Print initial values */
       if (firstrun) PrintAllSpecies(c, ns, mxns, T0);
-      
-      /* Loop over output points, call ARKode, print sample solution values. */
+
+      /* Loop over output points, call ARKStepEvolve, print sample solution values. */
       tout = T1;
       for (iout = 1; iout <= NOUT; iout++) {
-        flag = ARKode(arkode_mem, tout, c, &t, ARK_NORMAL);
+        flag = ARKStepEvolve(arkode_mem, tout, c, &t, ARK_NORMAL);
         PrintOutput(arkode_mem, t);
         if (firstrun && (iout % 3 == 0)) PrintAllSpecies(c, ns, mxns, t);
-        if(check_flag(&flag, "ARKode", 1)) break;
-        if (tout > RCONST(0.9)) tout += DTOUT; else tout *= TOUT_MULT; 
+        if(check_flag(&flag, "ARKStepEvolve", 1)) break;
+        if (tout > RCONST(0.9)) tout += DTOUT; else tout *= TOUT_MULT;
       }
-      
+
       /* Print final statistics, and loop for next case */
       PrintFinalStats(arkode_mem);
-      
+
     }
   }
 
   /* Free all memory */
-  ARKodeFree(&arkode_mem);
+  ARKStepFree(&arkode_mem);
   N_VDestroy(c);
   SUNLinSolFree(LS);
   FreeUserData(wdata);
+  SUNContext_Free(&ctx);
 
   return(0);
 }
 
-static WebData AllocUserData(void)
+static WebData AllocUserData(SUNContext ctx)
 {
   int i, ngrp = NGRP;
   sunindextype ns = NS;
   WebData wdata;
-  
+
   wdata = (WebData) malloc(sizeof *wdata);
   for(i=0; i < ngrp; i++) {
-    (wdata->P)[i] = newDenseMat(ns, ns);
-    (wdata->pivot)[i] = newIndexArray(ns);
+    (wdata->P)[i] = SUNDlsMat_newDenseMat(ns, ns);
+    (wdata->pivot)[i] = SUNDlsMat_newIndexArray(ns);
   }
-  wdata->rewt = N_VNew_Serial(NEQ);
-  wdata->tmp = N_VNew_Serial(NEQ);
+  wdata->rewt = N_VNew_Serial(NEQ, ctx);
+  wdata->tmp = N_VNew_Serial(NEQ, ctx);
   return(wdata);
 }
 
@@ -361,14 +401,14 @@ static void InitUserData(WebData wdata)
   int i, j, ns;
   realtype *bcoef, *diff, *cox, *coy, dx, dy;
   realtype (*acoef)[NS];
-  
+
   acoef = wdata->acoef;
   bcoef = wdata->bcoef;
   diff = wdata->diff;
   cox = wdata->cox;
   coy = wdata->coy;
   ns = wdata->ns = NS;
-  
+
   for (j = 0; j < NS; j++) { for (i = 0; i < NS; i++) acoef[i][j] = 0.; }
   for (j = 0; j < NP; j++) {
     for (i = 0; i < NP; i++) {
@@ -389,8 +429,8 @@ static void InitUserData(WebData wdata)
   dx = wdata->dx = DX;
   dy = wdata->dy = DY;
   for (i = 0; i < ns; i++) {
-    cox[i] = diff[i]/SUNSQR(dx);
-    coy[i] = diff[i]/SUNSQR(dy);
+    cox[i] = diff[i]/SQR(dx);
+    coy[i] = diff[i]/SQR(dy);
   }
 
   /* Set remaining method parameters */
@@ -399,7 +439,7 @@ static void InitUserData(WebData wdata)
   wdata->mq = MQ;
   wdata->mx = MX;
   wdata->my = MY;
-  wdata->srur = SUNRsqrt(UNIT_ROUNDOFF);
+  wdata->srur = sqrt(UNIT_ROUNDOFF);
   wdata->mxmp = MXMP;
   wdata->ngrp = NGRP;
   wdata->ngx = NGX;
@@ -426,7 +466,7 @@ static void SetGroups(int m, int ng, int jg[], int jig[], int jr[])
   mper = m/ng; /* does integer division */
   for (ig=0; ig < ng; ig++) jg[ig] = ig*mper;
   jg[ng] = m;
-  
+
   ngm1 = ng - 1;
   len1 = ngm1*mper;
   for (j = 0; j < len1; j++) jig[j] = j/mper;
@@ -441,22 +481,22 @@ static void CInit(N_Vector c, WebData wdata)
 {
   int jx, jy, ns, mxns, ioff, iyoff, i, ici;
   realtype argx, argy, x, y, dx, dy, x_factor, y_factor, *cdata;
-  
+
   cdata = N_VGetArrayPointer(c);
   ns = wdata->ns;
   mxns = wdata->mxns;
   dx = wdata->dx;
   dy = wdata->dy;
 
-  x_factor = RCONST(4.0)/SUNSQR(AX);
-  y_factor = RCONST(4.0)/SUNSQR(AY);
+  x_factor = RCONST(4.0)/SQR(AX);
+  y_factor = RCONST(4.0)/SQR(AY);
   for (jy = 0; jy < MY; jy++) {
     y = jy*dy;
-    argy = SUNSQR(y_factor*y*(AY-y));
+    argy = SQR(y_factor*y*(AY-y));
     iyoff = mxns*jy;
     for (jx = 0; jx < MX; jx++) {
       x = jx*dx;
-      argx = SUNSQR(x_factor*x*(AX-x));
+      argx = SQR(x_factor*x*(AX-x));
       ioff = iyoff + ns*jx;
       for (i = 1; i <= ns; i++) {
         ici = ioff + i-1;
@@ -468,7 +508,7 @@ static void CInit(N_Vector c, WebData wdata)
 
 static void PrintIntro(void)
 {
-  printf("\n\nDemonstration program for ARKODE - ARKSPGMR linear solver\n\n");
+  printf("\n\nDemonstration program for ARKODE - SPGMR linear solver\n\n");
   printf("Food web problem with ns species, ns = %d\n", NS);
   printf("Predator-prey interaction and diffusion on a 2-D square\n\n");
 #if defined(SUNDIALS_EXTENDED_PRECISION)
@@ -518,22 +558,22 @@ static void PrintIntro(void)
 
 static void PrintHeader(int jpre, int gstype)
 {
-  if(jpre == PREC_LEFT)
-    printf("\n\nPreconditioner type is           jpre = %s\n", "PREC_LEFT");
+  if(jpre == SUN_PREC_LEFT)
+    printf("\n\nPreconditioner type is           jpre = %s\n", "SUN_PREC_LEFT");
   else
-    printf("\n\nPreconditioner type is           jpre = %s\n", "PREC_RIGHT");
+    printf("\n\nPreconditioner type is           jpre = %s\n", "SUN_PREC_RIGHT");
 
-  if(gstype == MODIFIED_GS)
-    printf("\nGram-Schmidt method type is    gstype = %s\n\n\n", "MODIFIED_GS");
+  if(gstype == SUN_MODIFIED_GS)
+    printf("\nGram-Schmidt method type is    gstype = %s\n\n\n", "SUN_MODIFIED_GS");
   else
-    printf("\nGram-Schmidt method type is    gstype = %s\n\n\n", "CLASSICAL_GS");
+    printf("\nGram-Schmidt method type is    gstype = %s\n\n\n", "SUN_CLASSICAL_GS");
 }
 
 static void PrintAllSpecies(N_Vector c, int ns, int mxns, realtype t)
 {
   int i, jx ,jy;
   realtype *cdata;
-  
+
   cdata = N_VGetArrayPointer(c);
 #if defined(SUNDIALS_EXTENDED_PRECISION)
   printf("c values at t = %Lg:\n\n", t);
@@ -566,14 +606,14 @@ static void PrintOutput(void *arkode_mem, realtype t)
   int flag;
   realtype hu;
 
-  flag = ARKodeGetNumSteps(arkode_mem, &nst);
-  check_flag(&flag, "ARKodeGetNumSteps", 1);
-  flag = ARKodeGetNumRhsEvals(arkode_mem, &nfe, &nfi);
-  check_flag(&flag, "ARKodeGetNumRhsEvals", 1);
-  flag = ARKodeGetNumNonlinSolvIters(arkode_mem, &nni);
-  check_flag(&flag, "ARKodeGetNumNonlinSolvIters", 1);
-  flag = ARKodeGetLastStep(arkode_mem, &hu);
-  check_flag(&flag, "ARKodeGetLastStep", 1);
+  flag = ARKStepGetNumSteps(arkode_mem, &nst);
+  check_flag(&flag, "ARKStepGetNumSteps", 1);
+  flag = ARKStepGetNumRhsEvals(arkode_mem, &nfe, &nfi);
+  check_flag(&flag, "ARKStepGetNumRhsEvals", 1);
+  flag = ARKStepGetNumNonlinSolvIters(arkode_mem, &nni);
+  check_flag(&flag, "ARKStepGetNumNonlinSolvIters", 1);
+  flag = ARKStepGetLastStep(arkode_mem, &hu);
+  check_flag(&flag, "ARKStepGetLastStep", 1);
 
 #if defined(SUNDIALS_EXTENDED_PRECISION)
   printf("t = %10.2Le  nst = %ld  nfe = %ld  nfi = %ld  nni = %ld", t, nst, nfe, nfi, nni);
@@ -595,40 +635,40 @@ static void PrintFinalStats(void *arkode_mem)
   long int nli, npe, nps, ncfl, nfeLS;
   int flag;
   realtype avdim;
-  
-  flag = ARKodeGetWorkSpace(arkode_mem, &lenrw, &leniw);
-  check_flag(&flag, "ARKodeGetWorkSpace", 1);
-  flag = ARKodeGetNumSteps(arkode_mem, &nst);
-  check_flag(&flag, "ARKodeGetNumSteps", 1);
-  flag = ARKodeGetNumRhsEvals(arkode_mem, &nfe, &nfi);
-  check_flag(&flag, "ARKodeGetNumRhsEvals", 1);
-  flag = ARKodeGetNumLinSolvSetups(arkode_mem, &nsetups);
-  check_flag(&flag, "ARKodeGetNumLinSolvSetups", 1);
-  flag = ARKodeGetNumErrTestFails(arkode_mem, &netf);
-  check_flag(&flag, "ARKodeGetNumErrTestFails", 1);
-  flag = ARKodeGetNumNonlinSolvIters(arkode_mem, &nni);
-  check_flag(&flag, "ARKodeGetNumNonlinSolvIters", 1);
-  flag = ARKodeGetNumNonlinSolvConvFails(arkode_mem, &ncfn);
-  check_flag(&flag, "ARKodeGetNumNonlinSolvConvFails", 1);
 
-  flag = ARKSpilsGetWorkSpace(arkode_mem, &lenrwLS, &leniwLS);
-  check_flag(&flag, "ARKSpilsGetWorkSpace", 1);
-  flag = ARKSpilsGetNumLinIters(arkode_mem, &nli);
-  check_flag(&flag, "ARKSpilsGetNumLinIters", 1);
-  flag = ARKSpilsGetNumPrecEvals(arkode_mem, &npe);
-  check_flag(&flag, "ARKSpilsGetNumPrecEvals", 1);
-  flag = ARKSpilsGetNumPrecSolves(arkode_mem, &nps);
-  check_flag(&flag, "ARKSpilsGetNumPrecSolves", 1);
-  flag = ARKSpilsGetNumConvFails(arkode_mem, &ncfl);
-  check_flag(&flag, "ARKSpilsGetNumConvFails", 1);
-  flag = ARKSpilsGetNumRhsEvals(arkode_mem, &nfeLS);
-  check_flag(&flag, "ARKSpilsGetNumRhsEvals", 1);
+  flag = ARKStepGetWorkSpace(arkode_mem, &lenrw, &leniw);
+  check_flag(&flag, "ARKStepGetWorkSpace", 1);
+  flag = ARKStepGetNumSteps(arkode_mem, &nst);
+  check_flag(&flag, "ARKStepGetNumSteps", 1);
+  flag = ARKStepGetNumRhsEvals(arkode_mem, &nfe, &nfi);
+  check_flag(&flag, "ARKStepGetNumRhsEvals", 1);
+  flag = ARKStepGetNumLinSolvSetups(arkode_mem, &nsetups);
+  check_flag(&flag, "ARKStepGetNumLinSolvSetups", 1);
+  flag = ARKStepGetNumErrTestFails(arkode_mem, &netf);
+  check_flag(&flag, "ARKStepGetNumErrTestFails", 1);
+  flag = ARKStepGetNumNonlinSolvIters(arkode_mem, &nni);
+  check_flag(&flag, "ARKStepGetNumNonlinSolvIters", 1);
+  flag = ARKStepGetNumNonlinSolvConvFails(arkode_mem, &ncfn);
+  check_flag(&flag, "ARKStepGetNumNonlinSolvConvFails", 1);
+
+  flag = ARKStepGetLinWorkSpace(arkode_mem, &lenrwLS, &leniwLS);
+  check_flag(&flag, "ARKStepGetLinWorkSpace", 1);
+  flag = ARKStepGetNumLinIters(arkode_mem, &nli);
+  check_flag(&flag, "ARKStepGetNumLinIters", 1);
+  flag = ARKStepGetNumPrecEvals(arkode_mem, &npe);
+  check_flag(&flag, "ARKStepGetNumPrecEvals", 1);
+  flag = ARKStepGetNumPrecSolves(arkode_mem, &nps);
+  check_flag(&flag, "ARKStepGetNumPrecSolves", 1);
+  flag = ARKStepGetNumLinConvFails(arkode_mem, &ncfl);
+  check_flag(&flag, "ARKStepGetNumLinConvFails", 1);
+  flag = ARKStepGetNumLinRhsEvals(arkode_mem, &nfeLS);
+  check_flag(&flag, "ARKStepGetNumLinRhsEvals", 1);
 
   printf("\n\n Final statistics for this run:\n\n");
-  printf(" ARKode real workspace length          = %4ld \n", lenrw);
-  printf(" ARKode integer workspace length       = %4ld \n", leniw);
-  printf(" SUNSPGMR real workspace length        = %4ld \n", lenrwLS);
-  printf(" SUNSPGMR integer workspace length     = %4ld \n", leniwLS);
+  printf(" ARKStep real workspace length         = %4ld \n", lenrw);
+  printf(" ARKStep integer workspace length      = %4ld \n", leniw);
+  printf(" ARKLS real workspace length           = %4ld \n", lenrwLS);
+  printf(" ARKLS integer workspace length        = %4ld \n", leniwLS);
   printf(" Number of steps                       = %4ld \n", nst);
   printf(" Number of f-s (explicit)              = %4ld \n", nfe);
   printf(" Number of f-s (implicit)              = %4ld \n", nfi);
@@ -660,11 +700,11 @@ static void FreeUserData(WebData wdata)
 
   ngrp = wdata->ngrp;
   for(i=0; i < ngrp; i++) {
-    destroyMat((wdata->P)[i]);
-    destroyArray((wdata->pivot)[i]);
+    SUNDlsMat_destroyMat((wdata->P)[i]);
+    SUNDlsMat_destroyArray((wdata->pivot)[i]);
   }
-  N_VDestroy_Serial(wdata->rewt);
-  N_VDestroy_Serial(wdata->tmp);
+  N_VDestroy(wdata->rewt);
+  N_VDestroy(wdata->tmp);
   free(wdata);
 }
 
@@ -679,11 +719,11 @@ static int f(realtype t, N_Vector c, N_Vector cdot,void *user_data)
   realtype dcxli, dcxui, dcyli, dcyui, x, y, *cox, *coy, *fsave, dx, dy;
   realtype *cdata, *cdotdata;
   WebData wdata;
-  
+
   wdata = (WebData) user_data;
   cdata = N_VGetArrayPointer(c);
   cdotdata = N_VGetArrayPointer(cdot);
-  
+
   mxns = wdata->mxns;
   ns = wdata->ns;
   fsave = wdata->fsave;
@@ -692,7 +732,7 @@ static int f(realtype t, N_Vector c, N_Vector cdot,void *user_data)
   mxns = wdata->mxns;
   dx = wdata->dx;
   dy = wdata->dy;
-  
+
   for (jy = 0; jy < MY; jy++) {
     y = jy*dy;
     iyoff = mxns*jy;
@@ -725,7 +765,7 @@ static int f(realtype t, N_Vector c, N_Vector cdot,void *user_data)
 
 /*
   This routine computes the interaction rates for the species
-  c_1, ... ,c_ns (stored in c[0],...,c[ns-1]), at one spatial point 
+  c_1, ... ,c_ns (stored in c[0],...,c[ns-1]), at one spatial point
   and at time t.
 */
 static void WebRates(realtype x, realtype y, realtype t, realtype c[],
@@ -734,27 +774,27 @@ static void WebRates(realtype x, realtype y, realtype t, realtype c[],
   int i, j, ns;
   realtype fac, *bcoef;
   realtype (*acoef)[NS];
-  
+
   ns = wdata->ns;
   acoef = wdata->acoef;
   bcoef = wdata->bcoef;
-  
+
   for (i = 0; i < ns; i++)
     rate[i] = ZERO;
-  
-  for (j = 0; j < ns; j++) 
-    for (i = 0; i < ns; i++) 
+
+  for (j = 0; j < ns; j++)
+    for (i = 0; i < ns; i++)
       rate[i] += c[j] * acoef[i][j];
-  
+
   fac = ONE + ALPH*x*y;
-  for (i = 0; i < ns; i++) 
+  for (i = 0; i < ns; i++)
     rate[i] = c[i]*(bcoef[i]*fac + rate[i]);
 }
 
 /*
  This routine generates the block-diagonal part of the Jacobian
  corresponding to the interaction rates, multiplies by -gamma, adds
- the identity matrix, and calls denseGETRF to do the LU decomposition of
+ the identity matrix, and calls SUNDlsMat_denseGETRF to do the LU decomposition of
  each diagonal block. The computation of the diagonal blocks uses
  the preset block and grouping information. One block per group is
  computed. The Jacobian elements are generated by difference
@@ -763,28 +803,27 @@ static void WebRates(realtype x, realtype y, realtype t, realtype c[],
  This routine can be regarded as a prototype for the general case
  of a block-diagonal preconditioner. The blocks are of size mp, and
  there are ngrp=ngx*ngy blocks computed in the block-grouping scheme.
-*/ 
+*/
 static int Precond(realtype t, N_Vector c, N_Vector fc, booleantype jok,
                    booleantype *jcurPtr, realtype gamma, void *user_data)
 {
   realtype ***P;
-  int ier;
+  sunindextype ier;
   sunindextype **pivot;
   int i, if0, if00, ig, igx, igy, j, jj, jx, jy;
-  int *jxr, *jyr, ngrp, ngx, ngy, mxmp, flag;
-  sunindextype mp;
+  int *jxr, *jyr, ngrp, ngx, ngy, mxmp, mp, flag;
   realtype uround, fac, r, r0, save, srur;
   realtype *f1, *fsave, *cdata, *rewtdata;
   WebData wdata;
   void *arkode_mem;
   N_Vector rewt;
-  
+
   wdata = (WebData) user_data;
   arkode_mem = wdata->arkode_mem;
   cdata = N_VGetArrayPointer(c);
   rewt = wdata->rewt;
-  flag = ARKodeGetErrWeights(arkode_mem, rewt);
-  if(check_flag(&flag, "ARKodeGetErrWeights", 1)) return(1);
+  flag = ARKStepGetErrWeights(arkode_mem, rewt);
+  if(check_flag(&flag, "ARKStepGetErrWeights", 1)) return(1);
   rewtdata = N_VGetArrayPointer(rewt);
 
   uround = UNIT_ROUNDOFF;
@@ -800,30 +839,30 @@ static int Precond(realtype t, N_Vector c, N_Vector fc, booleantype jok,
   ngy = wdata->ngy;
   mxmp = wdata->mxmp;
   fsave = wdata->fsave;
-  
+
   /* Make mp calls to fblock to approximate each diagonal block of Jacobian.
-     Here, fsave contains the base value of the rate vector and 
+     Here, fsave contains the base value of the rate vector and
      r0 is a minimum increment factor for the difference quotient. */
-  
+
   f1 = N_VGetArrayPointer(wdata->tmp);
-  
+
   fac = N_VWrmsNorm (fc, rewt);
-  r0 = RCONST(1000.0)*SUNRabs(gamma)*uround*NEQ*fac;
+  r0 = RCONST(1000.0)*fabs(gamma)*uround*NEQ*fac;
   if (r0 == ZERO) r0 = ONE;
-  
+
   for (igy = 0; igy < ngy; igy++) {
     jy = jyr[igy];
     if00 = jy*mxmp;
-    for (igx = 0; igx < ngx; igx++) { 
+    for (igx = 0; igx < ngx; igx++) {
       jx = jxr[igx];
       if0 = if00 + jx*mp;
-      ig = igx + igy*ngx; 
+      ig = igx + igy*ngx;
       /* Generate ig-th diagonal block */
       for (j = 0; j < mp; j++) {
         /* Generate the jth column as a difference quotient */
-        jj = if0 + j; 
+        jj = if0 + j;
         save = cdata[jj];
-        r = SUNMAX(srur*SUNRabs(save),r0/rewtdata[jj]);
+        r = MAX(srur*fabs(save),r0/rewtdata[jj]);
         cdata[jj] += r;
         fac = -gamma/r;
         fblock (t, cdata, jx, jy, f1, wdata);
@@ -834,15 +873,15 @@ static int Precond(realtype t, N_Vector c, N_Vector fc, booleantype jok,
       }
     }
   }
-  
+
   /* Add identity matrix and do LU decompositions on blocks. */
-  
+
   for (ig = 0; ig < ngrp; ig++) {
-    denseAddIdentity(P[ig], mp);
-    ier = denseGETRF(P[ig], mp, mp, pivot[ig]);
+    SUNDlsMat_denseAddIdentity(P[ig], mp);
+    ier = SUNDlsMat_denseGETRF(P[ig], mp, mp, pivot[ig]);
     if (ier != 0) return(1);
   }
-  
+
   *jcurPtr = SUNTRUE;
   return(0);
 }
@@ -857,7 +896,7 @@ static void fblock(realtype t, realtype cdata[], int jx, int jy,
 {
   int iblok, ic;
   realtype x, y;
-  
+
   iblok = jx + jy*(wdata->mx);
   y = jy*(wdata->dy);
   x = jx*(wdata->dx);
@@ -880,20 +919,19 @@ static int PSolve(realtype tn, N_Vector c, N_Vector fc, N_Vector r, N_Vector z,
 {
   realtype   ***P;
   sunindextype **pivot;
-  int jx, jy, igx, igy, iv, ig, *jigx, *jigy, mx, my, ngx;
-  sunindextype mp;
+  int jx, jy, igx, igy, iv, ig, *jigx, *jigy, mx, my, ngx, mp;
   WebData wdata;
-  
+
   wdata = (WebData) user_data;
-  
+
   N_VScale(ONE, r, z);
-  
+
   /* call GSIter for Gauss-Seidel iterations */
-  
+
   GSIter(gamma, z, wdata->tmp, wdata);
-  
+
   /* Do backsolves for inverse of block-diagonal preconditioner factor */
-  
+
   P = wdata->P;
   pivot = wdata->pivot;
   mx = wdata->mx;
@@ -902,18 +940,18 @@ static int PSolve(realtype tn, N_Vector c, N_Vector fc, N_Vector r, N_Vector z,
   mp = wdata->mp;
   jigx = wdata->jigx;
   jigy = wdata->jigy;
-  
+
   iv = 0;
   for (jy = 0; jy < my; jy++) {
     igy = jigy[jy];
     for (jx = 0; jx < mx; jx++) {
       igx = jigx[jx];
       ig = igx + igy*ngx;
-      denseGETRS(P[ig], mp, pivot[ig], &(N_VGetArrayPointer(z)[iv]));
+      SUNDlsMat_denseGETRS(P[ig], mp, pivot[ig], &(N_VGetArrayPointer(z)[iv]));
       iv += mp;
     }
   }
-  
+
   return(0);
 }
 
@@ -932,7 +970,7 @@ static void GSIter(realtype gamma, N_Vector z, N_Vector x, WebData wdata)
   int ns, mxns, i, iyoff, ic, iter;
   realtype beta[NS], beta2[NS], cof1[NS], gam[NS], gam2[NS];
   realtype temp, *cox, *coy, *xd, *zd;
-  
+
   xd = N_VGetArrayPointer(x);
   zd = N_VGetArrayPointer(z);
   ns = wdata->ns;
@@ -941,10 +979,10 @@ static void GSIter(realtype gamma, N_Vector z, N_Vector x, WebData wdata)
   mxns = wdata->mxns;
   cox = wdata->cox;
   coy = wdata->coy;
-  
+
   /* Write matrix as P = D - L - U.
      Load local arrays beta, beta2, gam, gam2, and cof1. */
-  
+
   for (i = 0; i < ns; i++) {
     temp = ONE/(ONE + RCONST(2.0)*gamma*(cox[i] + coy[i]));
     beta[i] = gamma*cox[i]*temp;
@@ -953,10 +991,10 @@ static void GSIter(realtype gamma, N_Vector z, N_Vector x, WebData wdata)
     gam2[i] = RCONST(2.0)*gam[i];
     cof1[i] = temp;
   }
-  
+
   /* Begin iteration loop.
      Load vector x with (D-inverse)*z for first iteration. */
-  
+
   for (jy = 0; jy < my; jy++) {
     iyoff = mxns*jy;
     for (jx = 0; jx < mx; jx++) {
@@ -965,13 +1003,13 @@ static void GSIter(realtype gamma, N_Vector z, N_Vector x, WebData wdata)
     }
   }
   N_VConst(ZERO, z);
-  
+
   /* Looping point for iterations. */
-  
+
   for (iter=1; iter <= ITMAX; iter++) {
-    
+
     /* Calculate (D-inverse)*U*x if not the first iteration. */
-    
+
     if (iter > 1) {
       for (jy=0; jy < my; jy++) {
         iyoff = mxns*jy;
@@ -980,47 +1018,47 @@ static void GSIter(realtype gamma, N_Vector z, N_Vector x, WebData wdata)
           x_loc = (jx == 0) ? 0 : ((jx == mx-1) ? 2 : 1);
           y_loc = (jy == 0) ? 0 : ((jy == my-1) ? 2 : 1);
           switch (3*y_loc+x_loc) {
-          case 0 : 
+          case 0 :
             /* jx == 0, jy == 0 */
             /* x[ic+i] = beta2[i]x[ic+ns+i] + gam2[i]x[ic+mxns+i] */
             v_sum_prods(xd+ic, beta2, xd+ic+ns, gam2, xd+ic+mxns, ns);
             break;
-          case 1 : 
+          case 1 :
             /* 1 <= jx <= mx-2, jy == 0 */
             /* x[ic+i] = beta[i]x[ic+ns+i] + gam2[i]x[ic+mxns+i] */
             v_sum_prods(xd+ic, beta, xd+ic+ns, gam2, xd+ic+mxns, ns);
             break;
-          case 2 : 
+          case 2 :
             /* jx == mx-1, jy == 0 */
             /* x[ic+i] = gam2[i]x[ic+mxns+i] */
             v_prod(xd+ic, gam2, xd+ic+mxns, ns);
             break;
-          case 3 : 
+          case 3 :
             /* jx == 0, 1 <= jy <= my-2 */
             /* x[ic+i] = beta2[i]x[ic+ns+i] + gam[i]x[ic+mxns+i] */
             v_sum_prods(xd+ic, beta2, xd+ic+ns, gam, xd+ic+mxns, ns);
             break;
-          case 4 : 
+          case 4 :
             /* 1 <= jx <= mx-2, 1 <= jy <= my-2 */
             /* x[ic+i] = beta[i]x[ic+ns+i] + gam[i]x[ic+mxns+i] */
             v_sum_prods(xd+ic, beta, xd+ic+ns, gam, xd+ic+mxns, ns);
             break;
-          case 5 : 
+          case 5 :
             /* jx == mx-1, 1 <= jy <= my-2 */
             /* x[ic+i] = gam[i]x[ic+mxns+i] */
             v_prod(xd+ic, gam, xd+ic+mxns, ns);
             break;
-          case 6 : 
+          case 6 :
             /* jx == 0, jy == my-1 */
             /* x[ic+i] = beta2[i]x[ic+ns+i] */
             v_prod(xd+ic, beta2, xd+ic+ns, ns);
             break;
-          case 7 : 
+          case 7 :
             /* 1 <= jx <= mx-2, jy == my-1 */
             /* x[ic+i] = beta[i]x[ic+ns+i] */
             v_prod(xd+ic, beta, xd+ic+ns, ns);
             break;
-          case 8 : 
+          case 8 :
             /* jx == mx-1, jy == my-1 */
             /* x[ic+i] = 0.0 */
             v_zero(xd+ic, ns);
@@ -1029,9 +1067,9 @@ static void GSIter(realtype gamma, N_Vector z, N_Vector x, WebData wdata)
         }
       }
     }  /* end if (iter > 1) */
-    
+
     /* Overwrite x with [(I - (D-inverse)*L)-inverse]*x. */
-    
+
     for (jy=0; jy < my; jy++) {
       iyoff = mxns*jy;
       for (jx=0; jx < mx; jx++) { /* order of loops matters */
@@ -1039,48 +1077,48 @@ static void GSIter(realtype gamma, N_Vector z, N_Vector x, WebData wdata)
         x_loc = (jx == 0) ? 0 : ((jx == mx-1) ? 2 : 1);
         y_loc = (jy == 0) ? 0 : ((jy == my-1) ? 2 : 1);
         switch (3*y_loc+x_loc) {
-        case 0 : 
+        case 0 :
           /* jx == 0, jy == 0 */
           break;
-        case 1 : 
+        case 1 :
           /* 1 <= jx <= mx-2, jy == 0 */
           /* x[ic+i] += beta[i]x[ic-ns+i] */
           v_inc_by_prod(xd+ic, beta, xd+ic-ns, ns);
           break;
-        case 2 : 
+        case 2 :
           /* jx == mx-1, jy == 0 */
           /* x[ic+i] += beta2[i]x[ic-ns+i] */
           v_inc_by_prod(xd+ic, beta2, xd+ic-ns, ns);
           break;
-        case 3 : 
+        case 3 :
           /* jx == 0, 1 <= jy <= my-2 */
           /* x[ic+i] += gam[i]x[ic-mxns+i] */
           v_inc_by_prod(xd+ic, gam, xd+ic-mxns, ns);
           break;
-        case 4 : 
+        case 4 :
           /* 1 <= jx <= mx-2, 1 <= jy <= my-2 */
           /* x[ic+i] += beta[i]x[ic-ns+i] + gam[i]x[ic-mxns+i] */
           v_inc_by_prod(xd+ic, beta, xd+ic-ns, ns);
           v_inc_by_prod(xd+ic, gam, xd+ic-mxns, ns);
           break;
-        case 5 : 
+        case 5 :
           /* jx == mx-1, 1 <= jy <= my-2 */
           /* x[ic+i] += beta2[i]x[ic-ns+i] + gam[i]x[ic-mxns+i] */
           v_inc_by_prod(xd+ic, beta2, xd+ic-ns, ns);
           v_inc_by_prod(xd+ic, gam, xd+ic-mxns, ns);
           break;
-        case 6 : 
+        case 6 :
           /* jx == 0, jy == my-1 */
           /* x[ic+i] += gam2[i]x[ic-mxns+i] */
           v_inc_by_prod(xd+ic, gam2, xd+ic-mxns, ns);
           break;
-        case 7 : 
+        case 7 :
           /* 1 <= jx <= mx-2, jy == my-1 */
           /* x[ic+i] += beta[i]x[ic-ns+i] + gam2[i]x[ic-mxns+i] */
           v_inc_by_prod(xd+ic, beta, xd+ic-ns, ns);
           v_inc_by_prod(xd+ic, gam2, xd+ic-mxns, ns);
           break;
-        case 8 : 
+        case 8 :
           /* jx == mx-1, jy == my-1 */
           /* x[ic+i] += beta2[i]x[ic-ns+i] + gam2[i]x[ic-mxns+i] */
           v_inc_by_prod(xd+ic, beta2, xd+ic-ns, ns);
@@ -1089,36 +1127,36 @@ static void GSIter(realtype gamma, N_Vector z, N_Vector x, WebData wdata)
         }
       }
     }
-    
+
     /* Add increment x to z : z <- z+x */
-    
+
     N_VLinearSum(ONE, z, ONE, x, z);
-    
+
   }
 }
 
 static void v_inc_by_prod(realtype u[], realtype v[], realtype w[], int n)
 {
-  int i;  
+  int i;
   for (i=0; i < n; i++) u[i] += v[i]*w[i];
 }
 
 static void v_sum_prods(realtype u[], realtype p[], realtype q[],
                         realtype v[], realtype w[], int n)
 {
-  int i;  
+  int i;
   for (i=0; i < n; i++) u[i] = p[i]*q[i] + v[i]*w[i];
 }
 
 static void v_prod(realtype u[], realtype v[], realtype w[], int n)
-{ 
+{
   int i;
   for (i=0; i < n; i++) u[i] = v[i]*w[i];
 }
 
 static void v_zero(realtype u[], int n)
 {
-  int i;  
+  int i;
   for (i=0; i < n; i++) u[i] = ZERO;
 }
 
