@@ -1,29 +1,25 @@
 /*-----------------------------------------------------------------
  * Programmer(s): Daniel R. Reynolds @ SMU
  *---------------------------------------------------------------
- * LLNS/SMU Copyright Start
- * Copyright (c) 2015, Southern Methodist University and 
- * Lawrence Livermore National Security
- *
- * This work was performed under the auspices of the U.S. Department 
- * of Energy by Southern Methodist University and Lawrence Livermore 
- * National Laboratory under Contract DE-AC52-07NA27344.
- * Produced at Southern Methodist University and the Lawrence 
- * Livermore National Laboratory.
- *
+ * SUNDIALS Copyright Start
+ * Copyright (c) 2002-2022, Lawrence Livermore National Security
+ * and Southern Methodist University.
  * All rights reserved.
- * For details, see the LICENSE file.
- * LLNS/SMU Copyright End
+ *
+ * See the top-level LICENSE and NOTICE files for details.
+ *
+ * SPDX-License-Identifier: BSD-3-Clause
+ * SUNDIALS Copyright End
  *---------------------------------------------------------------
  * Example problem:
- * 
- * The following is a simple example problem with analytical 
+ *
+ * The following is a simple example problem with analytical
  * solution,
  *     dy/dt = (t+1)*exp(-y)
- * for t in the interval [0.0, 10.0], with initial condition: y=0. 
- * This has analytical solution 
+ * for t in the interval [0.0, 10.0], with initial condition: y=0.
+ * This has analytical solution
  *      y(t) = log(0.5*t^2 + t + 1)
- * 
+ *
  * This program solves the problem with the ERK method.
  * Output is printed every 1.0 units of time (10 total).
  * Run statistics (optional outputs) are printed at the end.
@@ -32,7 +28,7 @@
 /* Header files */
 #include <stdio.h>
 #include <math.h>
-#include <arkode/arkode.h>            /* prototypes for ARKode fcts., consts. */
+#include <arkode/arkode_erkstep.h>    /* prototypes for ERKStep fcts., consts */
 #include <nvector/nvector_serial.h>   /* serial N_Vector types, fcts., macros */
 #include <sundials/sundials_types.h>  /* def. of type 'realtype' */
 #include <sundials/sundials_math.h>   /* def. of SUNRsqrt, etc. */
@@ -68,9 +64,13 @@ int main()
   int flag;                      /* reusable error-checking flag */
   N_Vector y = NULL;             /* empty vector for storing solution */
   void *arkode_mem = NULL;       /* empty ARKode memory structure */
-  FILE *UFID;
+  FILE *UFID, *FID;
   realtype t, tout;
-  long int nst, nst_a, nfe, nfi, netf;
+
+  /* Create the SUNDIALS context object for this simulation */
+  SUNContext ctx;
+  flag = SUNContext_Create(NULL, &ctx);
+  if (check_flag(&flag, "SUNContext_Create", 1)) return 1;
 
   /* Initial problem output */
   printf("\nAnalytical ODE test problem:\n");
@@ -78,31 +78,28 @@ int main()
   printf("   abstol = %.1"ESYM"\n\n",abstol);
 
   /* Initialize data structures */
-  y = N_VNew_Serial(NEQ);          /* Create serial vector for solution */
+  y = N_VNew_Serial(NEQ, ctx);          /* Create serial vector for solution */
   if (check_flag((void *)y, "N_VNew_Serial", 0)) return 1;
   NV_Ith_S(y,0) = 0.0;             /* Specify initial condition */
-  arkode_mem = ARKodeCreate();     /* Create the solver memory */
-  if (check_flag((void *)arkode_mem, "ARKodeCreate", 0)) return 1;
 
-  /* Call ARKodeInit to initialize the integrator memory and specify the
-     right-hand side function in y'=f(t,y), the inital time T0, and
-     the initial dependent variable vector y.  Note: since this
-     problem is fully explicit, we set f_U to NULL and f_E to f. */
-  flag = ARKodeInit(arkode_mem, f, NULL, T0, y);
-  if (check_flag(&flag, "ARKodeInit", 1)) return 1;
+  /* Call ERKStepCreate to initialize the ERK timestepper module and
+     specify the right-hand side function in y'=f(t,y), the inital time
+     T0, and the initial dependent variable vector y. */
+  arkode_mem = ERKStepCreate(f, T0, y, ctx);
+  if (check_flag((void *)arkode_mem, "ERKStepCreate", 0)) return 1;
 
   /* Specify tolerances */
-  flag = ARKodeSStolerances(arkode_mem, reltol, abstol);
-  if (check_flag(&flag, "ARKodeSStolerances", 1)) return 1;
+  flag = ERKStepSStolerances(arkode_mem, reltol, abstol);
+  if (check_flag(&flag, "ERKStepSStolerances", 1)) return 1;
 
   /* Open output stream for results, output comment line */
   UFID = fopen("solution.txt","w");
   fprintf(UFID,"# t u\n");
 
   /* output initial condition to disk */
-  fprintf(UFID," %.16"ESYM" %.16"ESYM"\n", T0, NV_Ith_S(y,0));  
+  fprintf(UFID," %.16"ESYM" %.16"ESYM"\n", T0, NV_Ith_S(y,0));
 
-  /* Main time-stepping loop: calls ARKode to perform the integration, then
+  /* Main time-stepping loop: calls ERKStepEvolve to perform the integration, then
      prints results.  Stops when the final time has been reached */
   t = T0;
   tout = T0+dTout;
@@ -110,10 +107,10 @@ int main()
   printf("   ---------------------\n");
   while (Tf - t > 1.0e-15) {
 
-    flag = ARKode(arkode_mem, tout, y, &t, ARK_NORMAL);       /* call integrator */
-    if (check_flag(&flag, "ARKode", 1)) break;
+    flag = ERKStepEvolve(arkode_mem, tout, y, &t, ARK_NORMAL);       /* call integrator */
+    if (check_flag(&flag, "ERKStepEvolve", 1)) break;
     printf("  %10.6"FSYM"  %10.6"FSYM"\n", t, NV_Ith_S(y,0));           /* access/print solution */
-    fprintf(UFID," %.16"ESYM" %.16"ESYM"\n", t, NV_Ith_S(y,0));  
+    fprintf(UFID," %.16"ESYM" %.16"ESYM"\n", t, NV_Ith_S(y,0));
     if (flag >= 0) {                                          /* successful solve: update time */
       tout += dTout;
       tout = (tout > Tf) ? Tf : tout;
@@ -125,24 +122,20 @@ int main()
   printf("   ---------------------\n");
   fclose(UFID);
 
-  /* Print some final statistics */
-  flag = ARKodeGetNumSteps(arkode_mem, &nst);
-  check_flag(&flag, "ARKodeGetNumSteps", 1);
-  flag = ARKodeGetNumStepAttempts(arkode_mem, &nst_a);
-  check_flag(&flag, "ARKodeGetNumStepAttempts", 1);
-  flag = ARKodeGetNumRhsEvals(arkode_mem, &nfe, &nfi);
-  check_flag(&flag, "ARKodeGetNumRhsEvals", 1);
-  flag = ARKodeGetNumErrTestFails(arkode_mem, &netf);
-  check_flag(&flag, "ARKodeGetNumErrTestFails", 1);
+  /* Print final statistics */
+  printf("\nFinal Statistics:\n");
+  flag = ERKStepPrintAllStats(arkode_mem, stdout, SUN_OUTPUTFORMAT_TABLE);
 
-  printf("\nFinal Solver Statistics:\n");
-  printf("   Internal solver steps = %li (attempted = %li)\n", nst, nst_a);
-  printf("   Total RHS evals:  Fe = %li,  Fi = %li\n", nfe, nfi);
-  printf("   Total number of error test failures = %li\n\n", netf);
+  /* Print final statistics to a file in CSV format */
+  FID = fopen("ark_analytic_nonlin_stats.csv", "w");
+  flag = ERKStepPrintAllStats(arkode_mem, FID, SUN_OUTPUTFORMAT_CSV);
+  fclose(FID);
 
   /* Clean up and return with successful completion */
   N_VDestroy(y);               /* Free y vector */
-  ARKodeFree(&arkode_mem);     /* Free integrator memory */
+  ERKStepFree(&arkode_mem);    /* Free integrator memory */
+  SUNContext_Free(&ctx);       /* Free context */
+
   return 0;
 }
 
@@ -167,7 +160,7 @@ static int f(realtype t, N_Vector y, N_Vector ydot, void *user_data)
     opt == 1 means SUNDIALS function returns a flag so check if
              flag >= 0
     opt == 2 means function allocates memory so check if returned
-             NULL pointer  
+             NULL pointer
 */
 static int check_flag(void *flagvalue, const char *funcname, int opt)
 {
@@ -176,7 +169,7 @@ static int check_flag(void *flagvalue, const char *funcname, int opt)
   /* Check if SUNDIALS function returned NULL pointer - no memory allocated */
   if (opt == 0 && flagvalue == NULL) {
     fprintf(stderr, "\nSUNDIALS_ERROR: %s() failed - returned NULL pointer\n\n",
-	    funcname);
+            funcname);
     return 1; }
 
   /* Check if flag < 0 */
@@ -184,13 +177,13 @@ static int check_flag(void *flagvalue, const char *funcname, int opt)
     errflag = (int *) flagvalue;
     if (*errflag < 0) {
       fprintf(stderr, "\nSUNDIALS_ERROR: %s() failed with flag = %d\n\n",
-	      funcname, *errflag);
+              funcname, *errflag);
       return 1; }}
 
   /* Check if function returned NULL pointer - no memory allocated */
   else if (opt == 2 && flagvalue == NULL) {
     fprintf(stderr, "\nMEMORY_ERROR: %s() failed - returned NULL pointer\n\n",
-	    funcname);
+            funcname);
     return 1; }
 
   return 0;
