@@ -25,6 +25,16 @@
 #include <sunlinsol/sunlinsol_dense.h> /* access to dense SUNLinearSolver          */
 #include <sundials/sundials_types.h>   /* defs. of realtype, sunindextype          */
 
+#if defined(SUNDIALS_EXTENDED_PRECISION)
+#define GSYM "Lg"
+#define ESYM "Le"
+#define FSYM "Lf"
+#else
+#define GSYM "g"
+#define ESYM "e"
+#define FSYM "f"
+#endif
+
 /* Problem Constants */
 #define NEQ  1 /* number of equations */
 
@@ -36,16 +46,17 @@
 static int f(realtype t, N_Vector y, N_Vector ydot, void *f_data);
 
 /* Function for checking return values */
-static int check_flag(void *flagvalue, const char *funcname, int opt);
+static int check_retval(void *flagvalue, const char *funcname, int opt);
 
 int main()
 {
   void *cvode_mem;
   SUNMatrix A;
   SUNLinearSolver LS;
+  SUNContext sunctx;
 
   N_Vector y;
-  int flag, ret;
+  int flag, retval;
   realtype reltol, abstol, t0, t1, t2, t;
   long int nst1, nst2, nst;
 
@@ -56,8 +67,12 @@ int main()
   t1 = RCONST(1.0);
   t2 = RCONST(2.0);
 
+  /* Create the SUNDIALS context */
+  retval = SUNContext_Create(NULL, &sunctx);
+  if(check_retval(&retval, "SUNContext_Create", 1)) return(1);
+
   /* Allocate the vector of initial conditions */
-  y = N_VNew_Serial(NEQ);
+  y = N_VNew_Serial(NEQ, sunctx);
 
   /* Set initial condition */
   NV_Ith_S(y,0) = RCONST(1.0);
@@ -70,35 +85,35 @@ int main()
 
   /* Call CVodeCreate to create CVODE memory block and specify the
    * Backward Differentiaion Formula */
-  cvode_mem = CVodeCreate(CV_BDF);
-  if (check_flag((void *)cvode_mem, "CVodeCreate", 0)) return(1);
+  cvode_mem = CVodeCreate(CV_BDF, sunctx);
+  if (check_retval((void *)cvode_mem, "CVodeCreate", 0)) return(1);
 
   /* Call CVodeInit to initialize integrator memory and specify the
    * user's right hand side function y'=f(t,y), the initial time T0
    * and the initial condiition vector y. */
-  ret = CVodeInit(cvode_mem, f, t0, y);
-  if (check_flag((void *)&ret, "CVodeInit", 1)) return(1);
+  retval = CVodeInit(cvode_mem, f, t0, y);
+  if (check_retval((void *)&retval, "CVodeInit", 1)) return(1);
 
   /* Call CVodeSStolerances to specify integration tolereances,
    * specifically the scalar relative and absolute tolerance. */
-  ret = CVodeSStolerances(cvode_mem, reltol, abstol);
-  if (check_flag((void *)&ret, "CVodeSStolerances", 1)) return(1);
+  retval = CVodeSStolerances(cvode_mem, reltol, abstol);
+  if (check_retval((void *)&retval, "CVodeSStolerances", 1)) return(1);
 
   /* Provide RHS flag as user data which can be access in user provided routines */
-  ret = CVodeSetUserData(cvode_mem, &flag);
-  if (check_flag((void *)&ret, "CVodeSetUserData", 1)) return(1);
+  retval = CVodeSetUserData(cvode_mem, &flag);
+  if (check_retval((void *)&retval, "CVodeSetUserData", 1)) return(1);
 
   /* Create dense SUNMatrix for use in linear solver */
-  A = SUNDenseMatrix(NEQ, NEQ);
-  if (check_flag((void *)A, "SUNDenseMatrix", 0)) return(1);
+  A = SUNDenseMatrix(NEQ, NEQ, sunctx);
+  if (check_retval((void *)A, "SUNDenseMatrix", 0)) return(1);
 
   /* Create dense linear solver for use by CVode */
-  LS = SUNLinSol_Dense(y, A);
-  if (check_flag((void *)LS, "SUNLinSol_Dense", 0)) return(1);
+  LS = SUNLinSol_Dense(y, A, sunctx);
+  if (check_retval((void *)LS, "SUNLinSol_Dense", 0)) return(1);
 
   /* Attach the linear solver and matrix to CVode by calling CVodeSetLinearSolver */
-  ret = CVodeSetLinearSolver(cvode_mem, LS, A);
-  if (check_flag((void *)&ret, "CVodeSetLinearSolver", 1)) return(1);
+  retval = CVodeSetLinearSolver(cvode_mem, LS, A);
+  if (check_retval((void *)&retval, "CVodeSetLinearSolver", 1)) return(1);
 
   /*
    * ---------------------------------------------------------------
@@ -110,55 +125,55 @@ int main()
    */
 
   /* ---- Integrate to the discontinuity */
- 
+
   printf("\nDiscontinuity in solution\n\n");
- 
+
   /* set TSTOP (max time solution proceeds to) - this is not required */
-  ret = CVodeSetStopTime(cvode_mem, t1);
-  if (check_flag((void *)&ret, "CVodeSetStopTime", 1)) return(1);
+  retval = CVodeSetStopTime(cvode_mem, t1);
+  if (check_retval((void *)&retval, "CVodeSetStopTime", 1)) return(1);
 
   flag = RHS1; /* use -y for RHS */
   t = t0; /* set the integrator start time */
 
-  printf("%12.8e  %12.8e\n",t,NV_Ith_S(y,0));
+  printf("%12.8"ESYM"  %12.8"ESYM"\n",t,NV_Ith_S(y,0));
   while (t<t1) {
     /* advance solver just one internal step */
-    ret = CVode(cvode_mem, t1, y, &t, CV_ONE_STEP);
-    if (check_flag((void *)&ret, "CVode", 1)) return(1);
-    printf("%12.8e  %12.8e\n",t,NV_Ith_S(y,0));
+    retval = CVode(cvode_mem, t1, y, &t, CV_ONE_STEP);
+    if (check_retval((void *)&retval, "CVode", 1)) return(1);
+    printf("%12.8"ESYM"  %12.8"ESYM"\n",t,NV_Ith_S(y,0));
   }
   /* Get the number of steps the solver took to get to the discont. */
-  ret = CVodeGetNumSteps(cvode_mem, &nst1);
-  if (check_flag((void *)&ret, "CvodeGetNumSteps", 1)) return(1);
+  retval = CVodeGetNumSteps(cvode_mem, &nst1);
+  if (check_retval((void *)&retval, "CvodeGetNumSteps", 1)) return(1);
 
   /* ---- Integrate from the discontinuity */
 
   /* Include discontinuity */
   NV_Ith_S(y,0) = RCONST(1.0);
- 
+
   /* Reinitialize the solver */
-  ret = CVodeReInit(cvode_mem, t1, y);
-  if (check_flag((void *)&ret, "CVodeReInit", 1)) return(1);
+  retval = CVodeReInit(cvode_mem, t1, y);
+  if (check_retval((void *)&retval, "CVodeReInit", 1)) return(1);
 
   /* set TSTOP (max time solution proceeds to) - this is not required */
-  ret = CVodeSetStopTime(cvode_mem, t2);
-  if (check_flag((void *)&ret, "CVodeSetStopTime", 1)) return(1);
+  retval = CVodeSetStopTime(cvode_mem, t2);
+  if (check_retval((void *)&retval, "CVodeSetStopTime", 1)) return(1);
 
   flag = RHS1; /* use -y for RHS */
   t = t1; /* set the integrator start time */
 
-  printf("%12.8e  %12.8e\n",t,NV_Ith_S(y,0));
+  printf("%12.8"ESYM"  %12.8"ESYM"\n",t,NV_Ith_S(y,0));
 
   while (t<t2) {
     /* advance solver just one internal step */
-    ret = CVode(cvode_mem, t2, y, &t, CV_ONE_STEP);
-    if (check_flag((void *)&ret, "CVode", 1)) return(1);
-    printf("%12.8e  %12.8e\n",t,NV_Ith_S(y,0));
+    retval = CVode(cvode_mem, t2, y, &t, CV_ONE_STEP);
+    if (check_retval((void *)&retval, "CVode", 1)) return(1);
+    printf("%12.8"ESYM"  %12.8"ESYM"\n",t,NV_Ith_S(y,0));
   }
 
   /* Get the number of steps the solver took after the discont. */
-  ret = CVodeGetNumSteps(cvode_mem, &nst2);
-  if (check_flag((void *)&ret, "CvodeGetNumSteps", 1)) return(1);
+  retval = CVodeGetNumSteps(cvode_mem, &nst2);
+  if (check_retval((void *)&retval, "CvodeGetNumSteps", 1)) return(1);
 
   /* Print statistics */
   nst = nst1 + nst2;
@@ -180,29 +195,29 @@ int main()
   /* Reinitialize the solver. CVodeReInit does not reallocate memory
    * so it can only be used when the new problem size is the same as
    * the problem size when CVodeCreate was called. */
-  ret = CVodeReInit(cvode_mem, t0, y);
-  if (check_flag((void *)&ret, "CVodeReInit", 1)) return(1);
+  retval = CVodeReInit(cvode_mem, t0, y);
+  if (check_retval((void *)&retval, "CVodeReInit", 1)) return(1);
 
   /* ---- Integrate to the discontinuity */
 
   /* Set TSTOP (max time solution proceeds to) to location of discont. */
-  ret = CVodeSetStopTime(cvode_mem, t1);
-  if (check_flag((void *)&ret, "CVodeSetStopTime", 1)) return(1);
+  retval = CVodeSetStopTime(cvode_mem, t1);
+  if (check_retval((void *)&retval, "CVodeSetStopTime", 1)) return(1);
 
   flag = RHS1; /* use -y for RHS */
   t = t0; /* set the integrator start time */
 
-  printf("%12.8e  %12.8e\n",t,NV_Ith_S(y,0));
+  printf("%12.8"ESYM"  %12.8"ESYM"\n",t,NV_Ith_S(y,0));
   while (t<t1) {
     /* advance solver just one internal step */
-    ret = CVode(cvode_mem, t1, y, &t, CV_ONE_STEP);
-    if (check_flag((void *)&ret, "CVode", 1)) return(1);
-    printf("%12.8e  %12.8e\n",t,NV_Ith_S(y,0));
+    retval = CVode(cvode_mem, t1, y, &t, CV_ONE_STEP);
+    if (check_retval((void *)&retval, "CVode", 1)) return(1);
+    printf("%12.8"ESYM"  %12.8"ESYM"\n",t,NV_Ith_S(y,0));
   }
 
   /* Get the number of steps the solver took to get to the discont. */
-  ret = CVodeGetNumSteps(cvode_mem, &nst1);
-  if (check_flag((void *)&ret, "CvodeGetNumSteps", 1)) return(1);
+  retval = CVodeGetNumSteps(cvode_mem, &nst1);
+  if (check_retval((void *)&retval, "CvodeGetNumSteps", 1)) return(1);
 
   /* If TSTOP was not set, we'd need to find y(t1): */
   /* CVodeGetDky(cvode_mem, t1, 0, y); */
@@ -210,27 +225,27 @@ int main()
   /* ---- Integrate from the discontinuity */
 
   /* Reinitialize solver */
-  ret = CVodeReInit(cvode_mem, t1, y);
+  retval = CVodeReInit(cvode_mem, t1, y);
 
   /* set TSTOP (max time solution proceeds to) - this is not required */
-  ret = CVodeSetStopTime(cvode_mem, t2);
-  if (check_flag((void *)&ret, "CVodeSetStopTime", 1)) return(1);
+  retval = CVodeSetStopTime(cvode_mem, t2);
+  if (check_retval((void *)&retval, "CVodeSetStopTime", 1)) return(1);
 
   flag = RHS2; /* use -5y for RHS */
   t = t1; /* set the integrator start time */
 
-  printf("%12.8e  %12.8e\n",t,NV_Ith_S(y,0));
+  printf("%12.8"ESYM"  %12.8"ESYM"\n",t,NV_Ith_S(y,0));
 
   while (t<t2) {
     /* advance solver just one internal step */
-    ret = CVode(cvode_mem, t2, y, &t, CV_ONE_STEP);
-    if (check_flag((void *)&ret, "CVode", 1)) return(1);
-    printf("%12.8e  %12.8e\n",t,NV_Ith_S(y,0));
+    retval = CVode(cvode_mem, t2, y, &t, CV_ONE_STEP);
+    if (check_retval((void *)&retval, "CVode", 1)) return(1);
+    printf("%12.8"ESYM"  %12.8"ESYM"\n",t,NV_Ith_S(y,0));
   }
 
   /* Get the number of steps the solver took after the discont. */
-  ret = CVodeGetNumSteps(cvode_mem, &nst2);
-  if (check_flag((void *)&ret, "CvodeGetNumSteps", 1)) return(1);
+  retval = CVodeGetNumSteps(cvode_mem, &nst2);
+  if (check_retval((void *)&retval, "CvodeGetNumSteps", 1)) return(1);
 
   /* Print statistics */
   nst = nst1 + nst2;
@@ -253,51 +268,51 @@ int main()
   /* Reinitialize the solver. CVodeReInit does not reallocate memory
    * so it can only be used when the new problem size is the same as
    * the problem size when CVodeCreate was called. */
-  ret = CVodeReInit(cvode_mem, t0, y);
-  if (check_flag((void *)&ret, "CVodeReInit", 1)) return(1);
+  retval = CVodeReInit(cvode_mem, t0, y);
+  if (check_retval((void *)&retval, "CVodeReInit", 1)) return(1);
 
   /* ---- Integrate to the discontinuity */
 
   /* Set TSTOP (max time solution proceeds to) to location of discont. */
-  ret = CVodeSetStopTime(cvode_mem, t1);
-  if (check_flag((void *)&ret, "CVodeSetStopTime", 1)) return(1);
+  retval = CVodeSetStopTime(cvode_mem, t1);
+  if (check_retval((void *)&retval, "CVodeSetStopTime", 1)) return(1);
 
   flag = RHS1; /* use -y for RHS */
   t = t0; /* set the integrator start time */
 
-  printf("%12.8e  %12.8e\n",t,NV_Ith_S(y,0));
+  printf("%12.8"ESYM"  %12.8"ESYM"\n",t,NV_Ith_S(y,0));
   while (t<t1) {
     /* advance solver just one internal step */
-    ret = CVode(cvode_mem, t1, y, &t, CV_ONE_STEP);
-    if (check_flag((void *)&ret, "CVode", 1)) return(1);
-    printf("%12.8e  %12.8e\n",t,NV_Ith_S(y,0));
+    retval = CVode(cvode_mem, t1, y, &t, CV_ONE_STEP);
+    if (check_retval((void *)&retval, "CVode", 1)) return(1);
+    printf("%12.8"ESYM"  %12.8"ESYM"\n",t,NV_Ith_S(y,0));
   }
 
   /* Get the number of steps the solver took to get to the discont. */
-  ret = CVodeGetNumSteps(cvode_mem, &nst1);
-  if (check_flag((void *)&ret, "CvodeGetNumSteps", 1)) return(1);
+  retval = CVodeGetNumSteps(cvode_mem, &nst1);
+  if (check_retval((void *)&retval, "CvodeGetNumSteps", 1)) return(1);
 
   /* ---- Integrate from the discontinuity */
 
   /* set TSTOP (max time solution proceeds to) - this is not required */
-  ret = CVodeSetStopTime(cvode_mem, t2);
-  if (check_flag((void *)&ret, "CVodeSetStopTime", 1)) return(1);
+  retval = CVodeSetStopTime(cvode_mem, t2);
+  if (check_retval((void *)&retval, "CVodeSetStopTime", 1)) return(1);
 
   flag = RHS2; /* use -5y for RHS */
   t = t1; /* set the integrator start time */
 
-  printf("%12.8e  %12.8e\n",t,NV_Ith_S(y,0));
+  printf("%12.8"ESYM"  %12.8"ESYM"\n",t,NV_Ith_S(y,0));
 
   while (t<t2) {
     /* advance solver just one internal step */
-    ret = CVode(cvode_mem, t2, y, &t, CV_ONE_STEP);
-    if (check_flag((void *)&ret, "CVode", 1)) return(1);
-    printf("%12.8e  %12.8e\n",t,NV_Ith_S(y,0));
+    retval = CVode(cvode_mem, t2, y, &t, CV_ONE_STEP);
+    if (check_retval((void *)&retval, "CVode", 1)) return(1);
+    printf("%12.8"ESYM"  %12.8"ESYM"\n",t,NV_Ith_S(y,0));
   }
 
   /* Get the number of steps the solver took after the discont. */
-  ret = CVodeGetNumSteps(cvode_mem, &nst);
-  if (check_flag((void *)&ret, "CvodeGetNumSteps", 1)) return(1);
+  retval = CVodeGetNumSteps(cvode_mem, &nst);
+  if (check_retval((void *)&retval, "CvodeGetNumSteps", 1)) return(1);
 
   /* Print statistics */
   nst2 = nst - nst1;
@@ -308,6 +323,7 @@ int main()
   SUNMatDestroy(A);
   SUNLinSolFree(LS);
   CVodeFree(&cvode_mem);
+  SUNContext_Free(&sunctx);
 
   return(0);
 }
@@ -347,7 +363,7 @@ static int f(realtype t, N_Vector y, N_Vector ydot, void *f_data)
  *            NULL pointer
  */
 
-static int check_flag(void *flagvalue, const char *funcname, int opt)
+static int check_retval(void *flagvalue, const char *funcname, int opt)
 {
   int *errflag;
 
