@@ -3,7 +3,7 @@
  *                Radu Serban @ LLNL
  * --------------------------------------------------------------------
  * SUNDIALS Copyright Start
- * Copyright (c) 2002-2019, Lawrence Livermore National Security
+ * Copyright (c) 2002-2022, Lawrence Livermore National Security
  * and Southern Methodist University.
  * All rights reserved.
  *
@@ -14,7 +14,7 @@
  * --------------------------------------------------------------------
  * Demonstration program for CVODES - Krylov linear solver.
  * ODE system from ns-species interaction PDE in 2 dimensions.
- * 
+ *
  * This program solves a stiff ODE system that arises from a system
  * of partial differential equations. The PDE system is a food web
  * population model, with predator-prey interaction and diffusion on
@@ -27,14 +27,14 @@
  *
  *    i               i      i
  *  dc /dt  =  d(i)*(c    + c   )  +  f (x,y,c)  (i=1,...,ns)
- *                    xx     yy        i   
+ *                    xx     yy        i
  *
  * where
  *
  *                 i          ns         j
  *  f (x,y,c)  =  c *(b(i) + sum a(i,j)*c )
- *   i                       j=1                                         
- *                                                                       
+ *   i                       j=1
+ *
  * The number of species is ns = 2*np, with the first np being prey
  * and the last np being predators. The coefficients a(i,j), b(i),
  * d(i) are:
@@ -69,8 +69,8 @@
  * The product preconditoner is applied on the left and on the
  * right. In each case, both the modified and classical Gram-Schmidt
  * options are tested.
- * In the series of runs, CVodeInit, SUNLinSol_SPGMR, and 
- * CVDlsSetLinearSolver are called only for the first run, whereas 
+ * In the series of runs, CVodeInit, SUNLinSol_SPGMR, and
+ * CVodeSetLinearSolver are called only for the first run, whereas
  * CVodeReInit, SUNLinSol_SPGMRSetPrecType, and SUNSLinSol_PGMRSetGSType
  * are called for each of the remaining three runs.
  *
@@ -81,8 +81,10 @@
  * but there should be no such messages.
  *
  * Note: This program requires the dense linear solver functions
- * newDenseMat, newIndexArray, denseAddIdentity, denseGETRF, denseGETRS, 
- * destroyMat and destroyArray.
+ * SUNDlsMat_newDenseMat, SUNDlsMat_newIndexArray,
+ * SUNDlsMat_denseAddIdentity, SUNDlsMat_denseGETRF,
+ * SUNDlsMat_denseGETRS, SUNDlsMat_destroyMat and
+ * SUNDlsMat_destroyArray.
  *
  * Note: This program assumes the sequential implementation for the
  * type N_Vector and uses the N_VGetArrayPointer function to gain
@@ -104,7 +106,16 @@
 #include <nvector/nvector_serial.h>     /* serial N_Vector types, fct. and macros      */
 #include <sundials/sundials_dense.h>    /* use generic DENSE solver in preconditioning */
 #include <sundials/sundials_types.h>    /* definition of realtype                      */
-#include <sundials/sundials_math.h>     /* contains the macros ABS and SUNSQR          */
+
+/* helpful macros */
+
+#ifndef MAX
+#define MAX(A, B) ((A) > (B) ? (A) : (B))
+#endif
+
+#ifndef SQR
+#define SQR(A) ((A)*(A))
+#endif
 
 /* Constants */
 
@@ -210,7 +221,7 @@ static void v_zero(realtype u[], int n);
 
 static int f(realtype t, N_Vector y, N_Vector ydot, void *user_data);
 
-static int Precond(realtype tn, N_Vector c, N_Vector fc, booleantype jok, 
+static int Precond(realtype tn, N_Vector c, N_Vector fc, booleantype jok,
                    booleantype *jcurPtr, realtype gamma, void *user_data);
 
 static int PSolve(realtype tn, N_Vector c, N_Vector fc, N_Vector r, N_Vector z,
@@ -220,6 +231,9 @@ static int PSolve(realtype tn, N_Vector c, N_Vector fc, N_Vector r, N_Vector z,
 
 static int check_retval(void *returnvalue, const char *funcname, int opt);
 
+/* SUNDIALS simulation context */
+static SUNContext sunctx;
+
 /* Implementation */
 
 int main()
@@ -238,8 +252,12 @@ int main()
   LS = NULL;
   cvode_mem = NULL;
 
+  /* Create the SUNDIALS simulation context that all SUNDIALS objects require */
+  retval = SUNContext_Create(NULL, &sunctx);
+  if (check_retval(&retval, "SUNContext_Create", 1)) return(1);
+
   /* Initializations */
-  c = N_VNew_Serial(NEQ);
+  c = N_VNew_Serial(NEQ, sunctx);
   if(check_retval((void *)c, "N_VNew_Serial", 0)) return(1);
   wdata = AllocUserData();
   if(check_retval((void *)wdata, "AllocUserData", 2)) return(1);
@@ -251,8 +269,8 @@ int main()
   PrintIntro();
 
   /* Loop over jpre and gstype (four cases) */
-  for (jpre = PREC_LEFT; jpre <= PREC_RIGHT; jpre++) {
-    for (gstype = MODIFIED_GS; gstype <= CLASSICAL_GS; gstype++) {
+  for (jpre = SUN_PREC_LEFT; jpre <= SUN_PREC_RIGHT; jpre++) {
+    for (gstype = SUN_MODIFIED_GS; gstype <= SUN_CLASSICAL_GS; gstype++) {
 
       /* Initialize c and print heading */
       CInit(c, wdata);
@@ -260,9 +278,9 @@ int main()
 
       /* Call CVodeInit or CVodeReInit, then SUNLinSol_SPGMR to set up problem */
 
-      firstrun = (jpre == PREC_LEFT) && (gstype == MODIFIED_GS);
+      firstrun = (jpre == SUN_PREC_LEFT) && (gstype == SUN_MODIFIED_GS);
       if (firstrun) {
-        cvode_mem = CVodeCreate(CV_BDF);
+        cvode_mem = CVodeCreate(CV_BDF, sunctx);
         if(check_retval((void *)cvode_mem, "CVodeCreate", 0)) return(1);
 
         wdata->cvode_mem = cvode_mem;
@@ -276,7 +294,7 @@ int main()
         retval = CVodeSStolerances(cvode_mem, reltol, abstol);
         if(check_retval(&retval, "CVodeSStolerances", 1)) return(1);
 
-        LS = SUNLinSol_SPGMR(c, jpre, MAXL);
+        LS = SUNLinSol_SPGMR(c, jpre, MAXL, sunctx);
         if(check_retval((void *)LS, "SUNLinSol_SPGMR", 0)) return(1);
 
         retval = CVodeSetLinearSolver(cvode_mem, LS, NULL);
@@ -302,10 +320,10 @@ int main()
         if(check_retval(&retval, "SUNLinSol_SPGMRSetGSType", 1)) return(1);
 
       }
-      
+
       /* Print initial values */
       if (firstrun) PrintAllSpecies(c, ns, mxns, T0);
-      
+
       /* Loop over output points, call CVode, print sample solution values. */
       tout = T1;
       for (iout = 1; iout <= NOUT; iout++) {
@@ -313,12 +331,12 @@ int main()
         PrintOutput(cvode_mem, t);
         if (firstrun && (iout % 3 == 0)) PrintAllSpecies(c, ns, mxns, t);
         if(check_retval(&retval, "CVode", 1)) break;
-        if (tout > RCONST(0.9)) tout += DTOUT; else tout *= TOUT_MULT; 
+        if (tout > RCONST(0.9)) tout += DTOUT; else tout *= TOUT_MULT;
       }
-      
+
       /* Print final statistics, and loop for next case */
       PrintFinalStats(cvode_mem);
-      
+
     }
   }
 
@@ -327,6 +345,7 @@ int main()
   N_VDestroy(c);
   SUNLinSolFree(LS);
   FreeUserData(wdata);
+  SUNContext_Free(&sunctx);
 
   return(0);
 }
@@ -336,14 +355,14 @@ static WebData AllocUserData(void)
   int i, ngrp = NGRP;
   sunindextype ns = NS;
   WebData wdata;
-  
+
   wdata = (WebData) malloc(sizeof *wdata);
   for(i=0; i < ngrp; i++) {
-    (wdata->P)[i] = newDenseMat(ns, ns);
-    (wdata->pivot)[i] = newIndexArray(ns);
+    (wdata->P)[i] = SUNDlsMat_newDenseMat(ns, ns);
+    (wdata->pivot)[i] = SUNDlsMat_newIndexArray(ns);
   }
-  wdata->rewt = N_VNew_Serial(NEQ);
-  wdata->tmp = N_VNew_Serial(NEQ);
+  wdata->rewt = N_VNew_Serial(NEQ, sunctx);
+  wdata->tmp = N_VNew_Serial(NEQ, sunctx);
   return(wdata);
 }
 
@@ -352,14 +371,14 @@ static void InitUserData(WebData wdata)
   int i, j, ns;
   realtype *bcoef, *diff, *cox, *coy, dx, dy;
   realtype (*acoef)[NS];
-  
+
   acoef = wdata->acoef;
   bcoef = wdata->bcoef;
   diff = wdata->diff;
   cox = wdata->cox;
   coy = wdata->coy;
   ns = wdata->ns = NS;
-  
+
   for (j = 0; j < NS; j++) { for (i = 0; i < NS; i++) acoef[i][j] = 0.; }
   for (j = 0; j < NP; j++) {
     for (i = 0; i < NP; i++) {
@@ -380,8 +399,8 @@ static void InitUserData(WebData wdata)
   dx = wdata->dx = DX;
   dy = wdata->dy = DY;
   for (i = 0; i < ns; i++) {
-    cox[i] = diff[i]/SUNSQR(dx);
-    coy[i] = diff[i]/SUNSQR(dy);
+    cox[i] = diff[i]/SQR(dx);
+    coy[i] = diff[i]/SQR(dy);
   }
 
   /* Set remaining method parameters */
@@ -390,7 +409,7 @@ static void InitUserData(WebData wdata)
   wdata->mq = MQ;
   wdata->mx = MX;
   wdata->my = MY;
-  wdata->srur = SUNRsqrt(UNIT_ROUNDOFF);
+  wdata->srur = sqrt(UNIT_ROUNDOFF);
   wdata->mxmp = MXMP;
   wdata->ngrp = NGRP;
   wdata->ngx = NGX;
@@ -417,7 +436,7 @@ static void SetGroups(int m, int ng, int jg[], int jig[], int jr[])
   mper = m/ng; /* does integer division */
   for (ig=0; ig < ng; ig++) jg[ig] = ig*mper;
   jg[ng] = m;
-  
+
   ngm1 = ng - 1;
   len1 = ngm1*mper;
   for (j = 0; j < len1; j++) jig[j] = j/mper;
@@ -432,22 +451,22 @@ static void CInit(N_Vector c, WebData wdata)
 {
   int jx, jy, ns, mxns, ioff, iyoff, i, ici;
   realtype argx, argy, x, y, dx, dy, x_factor, y_factor, *cdata;
-  
+
   cdata = N_VGetArrayPointer(c);
   ns = wdata->ns;
   mxns = wdata->mxns;
   dx = wdata->dx;
   dy = wdata->dy;
 
-  x_factor = RCONST(4.0)/SUNSQR(AX);
-  y_factor = RCONST(4.0)/SUNSQR(AY);
+  x_factor = RCONST(4.0)/SQR(AX);
+  y_factor = RCONST(4.0)/SQR(AY);
   for (jy = 0; jy < MY; jy++) {
     y = jy*dy;
-    argy = SUNSQR(y_factor*y*(AY-y));
+    argy = SQR(y_factor*y*(AY-y));
     iyoff = mxns*jy;
     for (jx = 0; jx < MX; jx++) {
       x = jx*dx;
-      argx = SUNSQR(x_factor*x*(AX-x));
+      argx = SQR(x_factor*x*(AX-x));
       ioff = iyoff + ns*jx;
       for (i = 1; i <= ns; i++) {
         ici = ioff + i-1;
@@ -509,22 +528,22 @@ static void PrintIntro(void)
 
 static void PrintHeader(int jpre, int gstype)
 {
-  if(jpre == PREC_LEFT)
-    printf("\n\nPreconditioner type is           jpre = %s\n", "PREC_LEFT");
+  if(jpre == SUN_PREC_LEFT)
+    printf("\n\nPreconditioner type is           jpre = %s\n", "SUN_PREC_LEFT");
   else
-    printf("\n\nPreconditioner type is           jpre = %s\n", "PREC_RIGHT");
+    printf("\n\nPreconditioner type is           jpre = %s\n", "SUN_PREC_RIGHT");
 
-  if(gstype == MODIFIED_GS)
-    printf("\nGram-Schmidt method type is    gstype = %s\n\n\n", "MODIFIED_GS");
+  if(gstype == SUN_MODIFIED_GS)
+    printf("\nGram-Schmidt method type is    gstype = %s\n\n\n", "SUN_MODIFIED_GS");
   else
-    printf("\nGram-Schmidt method type is    gstype = %s\n\n\n", "CLASSICAL_GS");
+    printf("\nGram-Schmidt method type is    gstype = %s\n\n\n", "SUN_CLASSICAL_GS");
 }
 
 static void PrintAllSpecies(N_Vector c, int ns, int mxns, realtype t)
 {
   int i, jx ,jy;
   realtype *cdata;
-  
+
   cdata = N_VGetArrayPointer(c);
 #if defined(SUNDIALS_EXTENDED_PRECISION)
   printf("c values at t = %Lg:\n\n", t);
@@ -588,7 +607,7 @@ static void PrintFinalStats(void *cvode_mem)
   long int nli, npe, nps, ncfl, nfeLS;
   int retval;
   realtype avdim;
-  
+
   retval = CVodeGetWorkSpace(cvode_mem, &lenrw, &leniw);
   check_retval(&retval, "CVodeGetWorkSpace", 1);
   retval = CVodeGetNumSteps(cvode_mem, &nst);
@@ -652,8 +671,8 @@ static void FreeUserData(WebData wdata)
 
   ngrp = wdata->ngrp;
   for(i=0; i < ngrp; i++) {
-    destroyMat((wdata->P)[i]);
-    destroyArray((wdata->pivot)[i]);
+    SUNDlsMat_destroyMat((wdata->P)[i]);
+    SUNDlsMat_destroyArray((wdata->pivot)[i]);
   }
   N_VDestroy(wdata->rewt);
   N_VDestroy(wdata->tmp);
@@ -671,11 +690,11 @@ static int f(realtype t, N_Vector c, N_Vector cdot,void *user_data)
   realtype dcxli, dcxui, dcyli, dcyui, x, y, *cox, *coy, *fsave, dx, dy;
   realtype *cdata, *cdotdata;
   WebData wdata;
-  
+
   wdata = (WebData) user_data;
   cdata = N_VGetArrayPointer(c);
   cdotdata = N_VGetArrayPointer(cdot);
-  
+
   mxns = wdata->mxns;
   ns = wdata->ns;
   fsave = wdata->fsave;
@@ -684,7 +703,7 @@ static int f(realtype t, N_Vector c, N_Vector cdot,void *user_data)
   mxns = wdata->mxns;
   dx = wdata->dx;
   dy = wdata->dy;
-  
+
   for (jy = 0; jy < MY; jy++) {
     y = jy*dy;
     iyoff = mxns*jy;
@@ -717,7 +736,7 @@ static int f(realtype t, N_Vector c, N_Vector cdot,void *user_data)
 
 /*
   This routine computes the interaction rates for the species
-  c_1, ... ,c_ns (stored in c[0],...,c[ns-1]), at one spatial point 
+  c_1, ... ,c_ns (stored in c[0],...,c[ns-1]), at one spatial point
   and at time t.
 */
 static void WebRates(realtype x, realtype y, realtype t, realtype c[],
@@ -726,27 +745,27 @@ static void WebRates(realtype x, realtype y, realtype t, realtype c[],
   int i, j, ns;
   realtype fac, *bcoef;
   realtype (*acoef)[NS];
-  
+
   ns = wdata->ns;
   acoef = wdata->acoef;
   bcoef = wdata->bcoef;
-  
+
   for (i = 0; i < ns; i++)
     rate[i] = ZERO;
-  
-  for (j = 0; j < ns; j++) 
-    for (i = 0; i < ns; i++) 
+
+  for (j = 0; j < ns; j++)
+    for (i = 0; i < ns; i++)
       rate[i] += c[j] * acoef[i][j];
-  
+
   fac = ONE + ALPH*x*y;
-  for (i = 0; i < ns; i++) 
+  for (i = 0; i < ns; i++)
     rate[i] = c[i]*(bcoef[i]*fac + rate[i]);
 }
 
 /*
  This routine generates the block-diagonal part of the Jacobian
  corresponding to the interaction rates, multiplies by -gamma, adds
- the identity matrix, and calls denseGETRF to do the LU decomposition of
+ the identity matrix, and calls SUNDlsMat_denseGETRF to do the LU decomposition of
  each diagonal block. The computation of the diagonal blocks uses
  the preset block and grouping information. One block per group is
  computed. The Jacobian elements are generated by difference
@@ -755,21 +774,21 @@ static void WebRates(realtype x, realtype y, realtype t, realtype c[],
  This routine can be regarded as a prototype for the general case
  of a block-diagonal preconditioner. The blocks are of size mp, and
  there are ngrp=ngx*ngy blocks computed in the block-grouping scheme.
-*/ 
+*/
 static int Precond(realtype t, N_Vector c, N_Vector fc, booleantype jok,
                    booleantype *jcurPtr, realtype gamma, void *user_data)
 {
   realtype ***P;
   sunindextype **pivot;
   int i, if0, if00, ig, igx, igy, j, jj, jx, jy;
-  int *jxr, *jyr, ngrp, ngx, ngy, mxmp, retval;
-  sunindextype mp, denseretval;
+  int *jxr, *jyr, ngrp, ngx, ngy, mxmp, mp, retval;
+  sunindextype denseretval;
   realtype uround, fac, r, r0, save, srur;
   realtype *f1, *fsave, *cdata, *rewtdata;
   WebData wdata;
   void *cvode_mem;
   N_Vector rewt;
-  
+
   wdata = (WebData) user_data;
   cvode_mem = wdata->cvode_mem;
   cdata = N_VGetArrayPointer(c);
@@ -791,30 +810,30 @@ static int Precond(realtype t, N_Vector c, N_Vector fc, booleantype jok,
   ngy = wdata->ngy;
   mxmp = wdata->mxmp;
   fsave = wdata->fsave;
-  
+
   /* Make mp calls to fblock to approximate each diagonal block of Jacobian.
-     Here, fsave contains the base value of the rate vector and 
+     Here, fsave contains the base value of the rate vector and
      r0 is a minimum increment factor for the difference quotient. */
-  
+
   f1 = N_VGetArrayPointer(wdata->tmp);
-  
+
   fac = N_VWrmsNorm (fc, rewt);
-  r0 = RCONST(1000.0)*SUNRabs(gamma)*uround*NEQ*fac;
+  r0 = RCONST(1000.0)*fabs(gamma)*uround*NEQ*fac;
   if (r0 == ZERO) r0 = ONE;
-  
+
   for (igy = 0; igy < ngy; igy++) {
     jy = jyr[igy];
     if00 = jy*mxmp;
-    for (igx = 0; igx < ngx; igx++) { 
+    for (igx = 0; igx < ngx; igx++) {
       jx = jxr[igx];
       if0 = if00 + jx*mp;
-      ig = igx + igy*ngx; 
+      ig = igx + igy*ngx;
       /* Generate ig-th diagonal block */
       for (j = 0; j < mp; j++) {
         /* Generate the jth column as a difference quotient */
-        jj = if0 + j; 
+        jj = if0 + j;
         save = cdata[jj];
-        r = SUNMAX(srur*SUNRabs(save),r0/rewtdata[jj]);
+        r = MAX(srur*fabs(save),r0/rewtdata[jj]);
         cdata[jj] += r;
         fac = -gamma/r;
         fblock (t, cdata, jx, jy, f1, wdata);
@@ -825,15 +844,15 @@ static int Precond(realtype t, N_Vector c, N_Vector fc, booleantype jok,
       }
     }
   }
-  
+
   /* Add identity matrix and do LU decompositions on blocks. */
-  
+
   for (ig = 0; ig < ngrp; ig++) {
-    denseAddIdentity(P[ig], mp);
-    denseretval = denseGETRF(P[ig], mp, mp, pivot[ig]);
+    SUNDlsMat_denseAddIdentity(P[ig], mp);
+    denseretval = SUNDlsMat_denseGETRF(P[ig], mp, mp, pivot[ig]);
     if (denseretval != 0) return(1);
   }
-  
+
   *jcurPtr = SUNTRUE;
   return(0);
 }
@@ -848,7 +867,7 @@ static void fblock(realtype t, realtype cdata[], int jx, int jy,
 {
   int iblok, ic;
   realtype x, y;
-  
+
   iblok = jx + jy*(wdata->mx);
   y = jy*(wdata->dy);
   x = jx*(wdata->dx);
@@ -871,20 +890,19 @@ static int PSolve(realtype tn, N_Vector c, N_Vector fc, N_Vector r, N_Vector z,
 {
   realtype   ***P;
   sunindextype **pivot;
-  int jx, jy, igx, igy, iv, ig, *jigx, *jigy, mx, my, ngx;
-  sunindextype mp;
+  int jx, jy, igx, igy, iv, ig, *jigx, *jigy, mx, my, ngx, mp;
   WebData wdata;
-  
+
   wdata = (WebData) user_data;
-  
+
   N_VScale(ONE, r, z);
-  
+
   /* call GSIter for Gauss-Seidel iterations */
-  
+
   GSIter(gamma, z, wdata->tmp, wdata);
-  
+
   /* Do backsolves for inverse of block-diagonal preconditioner factor */
-  
+
   P = wdata->P;
   pivot = wdata->pivot;
   mx = wdata->mx;
@@ -893,18 +911,18 @@ static int PSolve(realtype tn, N_Vector c, N_Vector fc, N_Vector r, N_Vector z,
   mp = wdata->mp;
   jigx = wdata->jigx;
   jigy = wdata->jigy;
-  
+
   iv = 0;
   for (jy = 0; jy < my; jy++) {
     igy = jigy[jy];
     for (jx = 0; jx < mx; jx++) {
       igx = jigx[jx];
       ig = igx + igy*ngx;
-      denseGETRS(P[ig], mp, pivot[ig], &(N_VGetArrayPointer(z)[iv]));
+      SUNDlsMat_denseGETRS(P[ig], mp, pivot[ig], &(N_VGetArrayPointer(z)[iv]));
       iv += mp;
     }
   }
-  
+
   return(0);
 }
 
@@ -923,7 +941,7 @@ static void GSIter(realtype gamma, N_Vector z, N_Vector x, WebData wdata)
   int ns, mxns, i, iyoff, ic, iter;
   realtype beta[NS], beta2[NS], cof1[NS], gam[NS], gam2[NS];
   realtype temp, *cox, *coy, *xd, *zd;
-  
+
   xd = N_VGetArrayPointer(x);
   zd = N_VGetArrayPointer(z);
   ns = wdata->ns;
@@ -932,10 +950,10 @@ static void GSIter(realtype gamma, N_Vector z, N_Vector x, WebData wdata)
   mxns = wdata->mxns;
   cox = wdata->cox;
   coy = wdata->coy;
-  
+
   /* Write matrix as P = D - L - U.
      Load local arrays beta, beta2, gam, gam2, and cof1. */
-  
+
   for (i = 0; i < ns; i++) {
     temp = ONE/(ONE + RCONST(2.0)*gamma*(cox[i] + coy[i]));
     beta[i] = gamma*cox[i]*temp;
@@ -944,10 +962,10 @@ static void GSIter(realtype gamma, N_Vector z, N_Vector x, WebData wdata)
     gam2[i] = RCONST(2.0)*gam[i];
     cof1[i] = temp;
   }
-  
+
   /* Begin iteration loop.
      Load vector x with (D-inverse)*z for first iteration. */
-  
+
   for (jy = 0; jy < my; jy++) {
     iyoff = mxns*jy;
     for (jx = 0; jx < mx; jx++) {
@@ -956,13 +974,13 @@ static void GSIter(realtype gamma, N_Vector z, N_Vector x, WebData wdata)
     }
   }
   N_VConst(ZERO, z);
-  
+
   /* Looping point for iterations. */
-  
+
   for (iter=1; iter <= ITMAX; iter++) {
-    
+
     /* Calculate (D-inverse)*U*x if not the first iteration. */
-    
+
     if (iter > 1) {
       for (jy=0; jy < my; jy++) {
         iyoff = mxns*jy;
@@ -971,47 +989,47 @@ static void GSIter(realtype gamma, N_Vector z, N_Vector x, WebData wdata)
           x_loc = (jx == 0) ? 0 : ((jx == mx-1) ? 2 : 1);
           y_loc = (jy == 0) ? 0 : ((jy == my-1) ? 2 : 1);
           switch (3*y_loc+x_loc) {
-          case 0 : 
+          case 0 :
             /* jx == 0, jy == 0 */
             /* x[ic+i] = beta2[i]x[ic+ns+i] + gam2[i]x[ic+mxns+i] */
             v_sum_prods(xd+ic, beta2, xd+ic+ns, gam2, xd+ic+mxns, ns);
             break;
-          case 1 : 
+          case 1 :
             /* 1 <= jx <= mx-2, jy == 0 */
             /* x[ic+i] = beta[i]x[ic+ns+i] + gam2[i]x[ic+mxns+i] */
             v_sum_prods(xd+ic, beta, xd+ic+ns, gam2, xd+ic+mxns, ns);
             break;
-          case 2 : 
+          case 2 :
             /* jx == mx-1, jy == 0 */
             /* x[ic+i] = gam2[i]x[ic+mxns+i] */
             v_prod(xd+ic, gam2, xd+ic+mxns, ns);
             break;
-          case 3 : 
+          case 3 :
             /* jx == 0, 1 <= jy <= my-2 */
             /* x[ic+i] = beta2[i]x[ic+ns+i] + gam[i]x[ic+mxns+i] */
             v_sum_prods(xd+ic, beta2, xd+ic+ns, gam, xd+ic+mxns, ns);
             break;
-          case 4 : 
+          case 4 :
             /* 1 <= jx <= mx-2, 1 <= jy <= my-2 */
             /* x[ic+i] = beta[i]x[ic+ns+i] + gam[i]x[ic+mxns+i] */
             v_sum_prods(xd+ic, beta, xd+ic+ns, gam, xd+ic+mxns, ns);
             break;
-          case 5 : 
+          case 5 :
             /* jx == mx-1, 1 <= jy <= my-2 */
             /* x[ic+i] = gam[i]x[ic+mxns+i] */
             v_prod(xd+ic, gam, xd+ic+mxns, ns);
             break;
-          case 6 : 
+          case 6 :
             /* jx == 0, jy == my-1 */
             /* x[ic+i] = beta2[i]x[ic+ns+i] */
             v_prod(xd+ic, beta2, xd+ic+ns, ns);
             break;
-          case 7 : 
+          case 7 :
             /* 1 <= jx <= mx-2, jy == my-1 */
             /* x[ic+i] = beta[i]x[ic+ns+i] */
             v_prod(xd+ic, beta, xd+ic+ns, ns);
             break;
-          case 8 : 
+          case 8 :
             /* jx == mx-1, jy == my-1 */
             /* x[ic+i] = 0.0 */
             v_zero(xd+ic, ns);
@@ -1020,9 +1038,9 @@ static void GSIter(realtype gamma, N_Vector z, N_Vector x, WebData wdata)
         }
       }
     }  /* end if (iter > 1) */
-    
+
     /* Overwrite x with [(I - (D-inverse)*L)-inverse]*x. */
-    
+
     for (jy=0; jy < my; jy++) {
       iyoff = mxns*jy;
       for (jx=0; jx < mx; jx++) { /* order of loops matters */
@@ -1030,48 +1048,48 @@ static void GSIter(realtype gamma, N_Vector z, N_Vector x, WebData wdata)
         x_loc = (jx == 0) ? 0 : ((jx == mx-1) ? 2 : 1);
         y_loc = (jy == 0) ? 0 : ((jy == my-1) ? 2 : 1);
         switch (3*y_loc+x_loc) {
-        case 0 : 
+        case 0 :
           /* jx == 0, jy == 0 */
           break;
-        case 1 : 
+        case 1 :
           /* 1 <= jx <= mx-2, jy == 0 */
           /* x[ic+i] += beta[i]x[ic-ns+i] */
           v_inc_by_prod(xd+ic, beta, xd+ic-ns, ns);
           break;
-        case 2 : 
+        case 2 :
           /* jx == mx-1, jy == 0 */
           /* x[ic+i] += beta2[i]x[ic-ns+i] */
           v_inc_by_prod(xd+ic, beta2, xd+ic-ns, ns);
           break;
-        case 3 : 
+        case 3 :
           /* jx == 0, 1 <= jy <= my-2 */
           /* x[ic+i] += gam[i]x[ic-mxns+i] */
           v_inc_by_prod(xd+ic, gam, xd+ic-mxns, ns);
           break;
-        case 4 : 
+        case 4 :
           /* 1 <= jx <= mx-2, 1 <= jy <= my-2 */
           /* x[ic+i] += beta[i]x[ic-ns+i] + gam[i]x[ic-mxns+i] */
           v_inc_by_prod(xd+ic, beta, xd+ic-ns, ns);
           v_inc_by_prod(xd+ic, gam, xd+ic-mxns, ns);
           break;
-        case 5 : 
+        case 5 :
           /* jx == mx-1, 1 <= jy <= my-2 */
           /* x[ic+i] += beta2[i]x[ic-ns+i] + gam[i]x[ic-mxns+i] */
           v_inc_by_prod(xd+ic, beta2, xd+ic-ns, ns);
           v_inc_by_prod(xd+ic, gam, xd+ic-mxns, ns);
           break;
-        case 6 : 
+        case 6 :
           /* jx == 0, jy == my-1 */
           /* x[ic+i] += gam2[i]x[ic-mxns+i] */
           v_inc_by_prod(xd+ic, gam2, xd+ic-mxns, ns);
           break;
-        case 7 : 
+        case 7 :
           /* 1 <= jx <= mx-2, jy == my-1 */
           /* x[ic+i] += beta[i]x[ic-ns+i] + gam2[i]x[ic-mxns+i] */
           v_inc_by_prod(xd+ic, beta, xd+ic-ns, ns);
           v_inc_by_prod(xd+ic, gam2, xd+ic-mxns, ns);
           break;
-        case 8 : 
+        case 8 :
           /* jx == mx-1, jy == my-1 */
           /* x[ic+i] += beta2[i]x[ic-ns+i] + gam2[i]x[ic-mxns+i] */
           v_inc_by_prod(xd+ic, beta2, xd+ic-ns, ns);
@@ -1080,36 +1098,36 @@ static void GSIter(realtype gamma, N_Vector z, N_Vector x, WebData wdata)
         }
       }
     }
-    
+
     /* Add increment x to z : z <- z+x */
-    
+
     N_VLinearSum(ONE, z, ONE, x, z);
-    
+
   }
 }
 
 static void v_inc_by_prod(realtype u[], realtype v[], realtype w[], int n)
 {
-  int i;  
+  int i;
   for (i=0; i < n; i++) u[i] += v[i]*w[i];
 }
 
 static void v_sum_prods(realtype u[], realtype p[], realtype q[],
                         realtype v[], realtype w[], int n)
 {
-  int i;  
+  int i;
   for (i=0; i < n; i++) u[i] = p[i]*q[i] + v[i]*w[i];
 }
 
 static void v_prod(realtype u[], realtype v[], realtype w[], int n)
-{ 
+{
   int i;
   for (i=0; i < n; i++) u[i] = v[i]*w[i];
 }
 
 static void v_zero(realtype u[], int n)
 {
-  int i;  
+  int i;
   for (i=0; i < n; i++) u[i] = ZERO;
 }
 
