@@ -3,7 +3,7 @@
  *      Based on cvRoberts_dns.c and modified to use SUPERLU_MT.
  * -----------------------------------------------------------------
  * SUNDIALS Copyright Start
- * Copyright (c) 2002-2021, Lawrence Livermore National Security
+ * Copyright (c) 2002-2022, Lawrence Livermore National Security
  * and Southern Methodist University.
  * All rights reserved.
  *
@@ -13,11 +13,11 @@
  * SUNDIALS Copyright End
  * -----------------------------------------------------------------
  * Example problem:
- * 
+ *
  * The following is a simple example problem, with the coding
  * needed for its solution by CVODE. The problem is from
  * chemical kinetics, and consists of the following three rate
- * equations:         
+ * equations:
  *    dy1/dt = -.04*y1 + 1.e4*y2*y3
  *    dy2/dt = .04*y1 - 1.e4*y2*y3 - 3.e7*(y2)^2
  *    dy3/dt = 3.e7*(y2)^2
@@ -39,9 +39,8 @@
 #include <nvector/nvector_serial.h>         /* access to serial N_Vector            */
 #include <sunmatrix/sunmatrix_sparse.h>     /* access to sparse SUNMatrix           */
 #include <sunlinsol/sunlinsol_superlumt.h>  /* access to SuperLUMT linear solver    */
-#include <sundials/sundials_types.h>        /* defs. of realtype, sunindextype      */
 
-/* User-defined vector and matrix accessor macro: Ith */
+/* User-defined vector accessor macro: Ith */
 
 /* These macros are defined in order to write code which exactly matches
    the mathematical problem description given above.
@@ -51,12 +50,13 @@
    using the N_VIth macro in nvector.h. N_VIth numbers the components of
    a vector starting from 0. */
 
-#define Ith(v,i)    NV_Ith_S(v,i-1)         /* Ith numbers components 1..NEQ */
+#define Ith(v,i)    NV_Ith_S(v,i-1)         /* i-th vector component, i=1..NEQ */
 
 
 /* Problem Constants */
 
 #define NEQ   3                /* number of equations  */
+#define NNZ   7                /* number of non-zero entries in the Jacobian */
 #define Y1    RCONST(1.0)      /* initial y components */
 #define Y2    RCONST(0.0)
 #define Y3    RCONST(0.0)
@@ -77,7 +77,7 @@ static int f(realtype t, N_Vector y, N_Vector ydot, void *user_data);
 
 static int g(realtype t, N_Vector y, realtype *gout, void *user_data);
 
-static int Jac(realtype t, N_Vector y, N_Vector fy, SUNMatrix J, 
+static int Jac(realtype t, N_Vector y, N_Vector fy, SUNMatrix J,
                void *user_data, N_Vector tmp1, N_Vector tmp2, N_Vector tmp3);
 
 /* Private functions to output results */
@@ -102,51 +102,58 @@ static int check_retval(void *returnvalue, const char *funcname, int opt);
 
 int main()
 {
-  realtype reltol, t, tout;
-  N_Vector y, abstol;
+  SUNContext sunctx;
+  realtype t, tout;
+  N_Vector y;
+  N_Vector abstol;
   SUNMatrix A;
   SUNLinearSolver LS;
   void *cvode_mem;
-  int retval, retvalr, iout, nnz;
+  int retval, iout;
+  int retvalr;
   int rootsfound[2];
 
-  y = abstol = NULL;
+  y = NULL;
+  abstol = NULL;
   A = NULL;
   LS = NULL;
   cvode_mem = NULL;
 
-  /* Create serial vector of length NEQ for I.C. and abstol */
-  y = N_VNew_Serial(NEQ);
+  /* Create the SUNDIALS context */
+  retval = SUNContext_Create(NULL, &sunctx);
+  if (check_retval(&retval, "SUNContext_Create", 1)) return(1);
+
+  /* Initial conditions */
+  y = N_VNew_Serial(NEQ, sunctx);
   if (check_retval((void *)y, "N_VNew_Serial", 0)) return(1);
-  abstol = N_VNew_Serial(NEQ); 
-  if (check_retval((void *)abstol, "N_VNew_Serial", 0)) return(1);
 
   /* Initialize y */
   Ith(y,1) = Y1;
   Ith(y,2) = Y2;
   Ith(y,3) = Y3;
 
-  /* Set the scalar relative tolerance */
-  reltol = RTOL;
   /* Set the vector absolute tolerance */
+  abstol = N_VNew_Serial(NEQ, sunctx);
+  if (check_retval((void *)abstol, "N_VNew_Serial", 0)) return(1);
+
   Ith(abstol,1) = ATOL1;
   Ith(abstol,2) = ATOL2;
   Ith(abstol,3) = ATOL3;
 
-  /* Call CVodeCreate to create the solver memory and specify the 
+  /* Call CVodeCreate to create the solver memory and specify the
    * Backward Differentiation Formula */
-  cvode_mem = CVodeCreate(CV_BDF);
+  cvode_mem = CVodeCreate(CV_BDF, sunctx);
   if (check_retval((void *)cvode_mem, "CVodeCreate", 0)) return(1);
-  
+
   /* Call CVodeInit to initialize the integrator memory and specify the
-   * user's right hand side function in y'=f(t,y), the inital time T0, and
+   * user's right hand side function in y'=f(t,y), the initial time T0, and
    * the initial dependent variable vector y. */
   retval = CVodeInit(cvode_mem, f, T0, y);
   if (check_retval(&retval, "CVodeInit", 1)) return(1);
 
   /* Call CVodeSVtolerances to specify the scalar relative tolerance
    * and vector absolute tolerances */
-  retval = CVodeSVtolerances(cvode_mem, reltol, abstol);
+  retval = CVodeSVtolerances(cvode_mem, RTOL, abstol);
   if (check_retval(&retval, "CVodeSVtolerances", 1)) return(1);
 
   /* Call CVodeRootInit to specify the root function g with 2 components */
@@ -154,21 +161,20 @@ int main()
   if (check_retval(&retval, "CVodeRootInit", 1)) return(1);
 
   /* Create sparse SUNMatrix for use in linear solves */
-  nnz = NEQ * NEQ;
-  A = SUNSparseMatrix(NEQ, NEQ, nnz, CSC_MAT);
-  if(check_retval((void *)A, "SUNSparseMatrix", 0)) return(1);
+  A = SUNSparseMatrix(NEQ, NEQ, NNZ, CSC_MAT, sunctx);
+  if (check_retval((void *)A, "SUNSparseMatrix", 0)) return(1);
 
   /* Create SuperLUMT solver object for use by CVode (one thread) */
-  LS = SUNLinSol_SuperLUMT(y, A, 1);
-  if(check_retval((void *)LS, "SUNLinSol_SuperLUMT", 0)) return(1);
+  LS = SUNLinSol_SuperLUMT(y, A, 1, sunctx);
+  if (check_retval((void *)LS, "SUNLinSol_SuperLUMT", 0)) return(1);
 
-  /* Call CVodeSetLinearSolver to attach the matrix and linear solver to CVode */
+  /* Attach the matrix and linear solver */
   retval = CVodeSetLinearSolver(cvode_mem, LS, A);
-  if(check_retval(&retval, "CVodeSetLinearSolver", 1)) return(1);
+  if (check_retval(&retval, "CVodeSetLinearSolver", 1)) return(1);
 
   /* Set the user-supplied Jacobian routine Jac */
   retval = CVodeSetJacFn(cvode_mem, Jac);
-  if(check_retval(&retval, "CVodeSetJacFn", 1)) return(1);
+  if (check_retval(&retval, "CVodeSetJacFn", 1)) return(1);
 
   /* In loop, call CVode, print results, and test for error.
      Break out of loop when NOUT preset output times have been reached.  */
@@ -197,18 +203,13 @@ int main()
   /* Print some final statistics */
   PrintFinalStats(cvode_mem);
 
-  /* Free y and abstol vectors */
-  N_VDestroy(y);
-  N_VDestroy(abstol);
-
-  /* Free integrator memory */
-  CVodeFree(&cvode_mem);
-
-  /* Free the linear solver memory */
-  SUNLinSolFree(LS);
-
-  /* Free the matrix memory */
-  SUNMatDestroy(A);
+  /* Free memory */
+  N_VDestroy(y);                            /* Free y vector */
+  N_VDestroy(abstol);                       /* Free abstol vector */
+  CVodeFree(&cvode_mem);                    /* Free CVODES memory */
+  SUNLinSolFree(LS);                        /* Free the linear solver memory */
+  SUNMatDestroy(A);                         /* Free the matrix memory */
+  SUNContext_Free(&sunctx);                 /* Free the SUNDIALS context */
 
   return(0);
 }
@@ -221,7 +222,7 @@ int main()
  */
 
 /*
- * f routine. Compute function f(t,y). 
+ * f routine. Compute function f(t,y).
  */
 
 static int f(realtype t, N_Vector y, N_Vector ydot, void *user_data)
@@ -238,7 +239,7 @@ static int f(realtype t, N_Vector y, N_Vector ydot, void *user_data)
 }
 
 /*
- * g routine. Compute functions g_i(t,y) for i = 0,1. 
+ * g routine. Compute functions g_i(t,y) for i = 0,1.
  */
 
 static int g(realtype t, N_Vector y, realtype *gout, void *user_data)
@@ -256,43 +257,53 @@ static int g(realtype t, N_Vector y, realtype *gout, void *user_data)
  * Jacobian routine. Compute J(t,y) = df/dy. *
  */
 
-static int Jac(realtype t, N_Vector y, N_Vector fy, SUNMatrix J, 
+static int Jac(realtype t, N_Vector y, N_Vector fy, SUNMatrix J,
                void *user_data, N_Vector tmp1, N_Vector tmp2, N_Vector tmp3)
 {
-  realtype *yval;
-  sunindextype *colptrs = SUNSparseMatrix_IndexPointers(J);
+  /* State at which to evaluate the Jacobian */
+  realtype *yval = N_VGetArrayPointer(y);
+
+  /* J is stored in CSC format:
+     data    = non-zero matrix entries stored column-wise (length NNZ)
+     rowvals = row index for each non-zero matrix entry (length NNZ)
+     colptrs = i-th entry is the index in data where the first non-zero matrix
+               entry of the i-th column is stored (length NEQ + 1) */
+  realtype     *data    = SUNSparseMatrix_Data(J);
   sunindextype *rowvals = SUNSparseMatrix_IndexValues(J);
-  realtype *data = SUNSparseMatrix_Data(J);
-  
-  yval = N_VGetArrayPointer(y);
-
-  SUNMatZero(J);
+  sunindextype *colptrs = SUNSparseMatrix_IndexPointers(J);
 
+  /* first column entries start at data[0], two entries (rows 0 and 1) */
   colptrs[0] = 0;
-  colptrs[1] = 3;
-  colptrs[2] = 6;
-  colptrs[3] = 9;
 
-  data[0] = RCONST(-0.04);
   rowvals[0] = 0;
-  data[1] = RCONST(0.04);
+  data[0]    = RCONST(-0.04);
+
   rowvals[1] = 1;
-  data[2] = ZERO;
-  rowvals[2] = 2;
+  data[1]    = RCONST(0.04);
 
-  data[3] = RCONST(1.0e4)*yval[2];
-  rowvals[3] = 0;
-  data[4] = (RCONST(-1.0e4)*yval[2]) - (RCONST(6.0e7)*yval[1]);
-  rowvals[4] = 1;
-  data[5] = RCONST(6.0e7)*yval[1];
-  rowvals[5] = 2;
+  /* second column entries start at data[2], three entries (rows 0, 1, and 2) */
+  colptrs[1] = 2;
 
-  data[6] = RCONST(1.0e4)*yval[1];
-  rowvals[6] = 0;
-  data[7] = RCONST(-1.0e4)*yval[1];
-  rowvals[7] = 1;
-  data[8] = ZERO;
-  rowvals[8] = 2;
+  rowvals[2] = 0;
+  data[2]    = RCONST(1.0e4) * yval[2];
+
+  rowvals[3] = 1;
+  data[3]    = (RCONST(-1.0e4) * yval[2]) - (RCONST(6.0e7) * yval[1]);
+
+  rowvals[4] = 2;
+  data[4]    = RCONST(6.0e7) * yval[1];
+
+  /* third column entries start at data[5], two entries (rows 0 and 1) */
+  colptrs[2] = 5;
+
+  rowvals[5] = 0;
+  data[5]    = RCONST(1.0e4) * yval[1];
+
+  rowvals[6] = 1;
+  data[6]    = RCONST(-1.0e4) * yval[1];
+
+  /* number of non-zeros */
+  colptrs[3] = 7;
 
   return(0);
 }
@@ -323,13 +334,13 @@ static void PrintRootInfo(int root_f1, int root_f2)
   return;
 }
 
-/* 
+/*
  * Get and print some final statistics
  */
 
 static void PrintFinalStats(void *cvode_mem)
 {
-  long int nst, nfe, nsetups, nje, nni, ncfn, netf, nge;
+  long int nst, nfe, nsetups, nje, nni, nnf, ncfn, netf, nge;
   int retval;
 
   retval = CVodeGetNumSteps(cvode_mem, &nst);
@@ -342,8 +353,10 @@ static void PrintFinalStats(void *cvode_mem)
   check_retval(&retval, "CVodeGetNumErrTestFails", 1);
   retval = CVodeGetNumNonlinSolvIters(cvode_mem, &nni);
   check_retval(&retval, "CVodeGetNumNonlinSolvIters", 1);
-  retval = CVodeGetNumNonlinSolvConvFails(cvode_mem, &ncfn);
+  retval = CVodeGetNumNonlinSolvConvFails(cvode_mem, &nnf);
   check_retval(&retval, "CVodeGetNumNonlinSolvConvFails", 1);
+  retval = CVodeGetNumStepSolveFails(cvode_mem, &ncfn);
+  check_retval(&retval, "CVodeGetNumStepSolveFails", 1);
 
   retval = CVodeGetNumJacEvals(cvode_mem, &nje);
   check_retval(&retval, "CVodeGetNumJacEvals", 1);
@@ -352,10 +365,10 @@ static void PrintFinalStats(void *cvode_mem)
   check_retval(&retval, "CVodeGetNumGEvals", 1);
 
   printf("\nFinal Statistics:\n");
-  printf("nst = %-6ld nfe  = %-6ld nsetups = %-6ld nje = %ld\n",
-	 nst, nfe, nsetups, nje);
-  printf("nni = %-6ld ncfn = %-6ld netf = %-6ld    nge = %ld\n \n",
-	 nni, ncfn, netf, nge);
+  printf("nst = %-6ld nfe = %-6ld nsetups = %-6ld nje = %ld\n",
+         nst, nfe, nsetups, nje);
+  printf("nni = %-6ld nnf = %-6ld netf = %-6ld    ncfn = %-6ld nge = %ld\n\n",
+         nni, nnf, netf, ncfn, nge);
 }
 
 /*
@@ -365,7 +378,7 @@ static void PrintFinalStats(void *cvode_mem)
  *   opt == 1 means SUNDIALS function returns an integer value so check if
  *            retval < 0
  *   opt == 2 means function allocates memory so check if returned
- *            NULL pointer 
+ *            NULL pointer
  */
 
 static int check_retval(void *returnvalue, const char *funcname, int opt)
@@ -375,7 +388,7 @@ static int check_retval(void *returnvalue, const char *funcname, int opt)
   /* Check if SUNDIALS function returned NULL pointer - no memory allocated */
   if (opt == 0 && returnvalue == NULL) {
     fprintf(stderr, "\nSUNDIALS_ERROR: %s() failed - returned NULL pointer\n\n",
-	    funcname);
+            funcname);
     return(1); }
 
   /* Check if retval < 0 */
@@ -383,13 +396,13 @@ static int check_retval(void *returnvalue, const char *funcname, int opt)
     retval = (int *) returnvalue;
     if (*retval < 0) {
       fprintf(stderr, "\nSUNDIALS_ERROR: %s() failed with retval = %d\n\n",
-	      funcname, *retval);
+              funcname, *retval);
       return(1); }}
 
   /* Check if function returned NULL pointer - no memory allocated */
   else if (opt == 2 && returnvalue == NULL) {
     fprintf(stderr, "\nMEMORY_ERROR: %s() failed - returned NULL pointer\n\n",
-	    funcname);
+            funcname);
     return(1); }
 
   return(0);
