@@ -3,7 +3,7 @@
  * Programmer(s): Daniel R. Reynolds and Ting Yan @ SMU
  * -----------------------------------------------------------------
  * SUNDIALS Copyright Start
- * Copyright (c) 2002-2019, Lawrence Livermore National Security
+ * Copyright (c) 2002-2022, Lawrence Livermore National Security
  * and Southern Methodist University.
  * All rights reserved.
  *
@@ -114,12 +114,17 @@
 #include <nvector/nvector_openmp.h>
 #include <sundials/sundials_dense.h>
 #include <sundials/sundials_types.h>
-#include <sundials/sundials_math.h>
 
 #ifdef _OPENMP
 #include <omp.h>
 #endif
 
+/* helpful macros */
+
+#ifndef MAX
+#define MAX(A, B) ((A) > (B) ? (A) : (B))
+#endif
+
 /* Problem Constants. */
 
 #define NPREY       1              /* No. of prey (= no. of predators). */
@@ -216,6 +221,7 @@ int main(int argc, char *argv[])
   int maxl;
   realtype rtol, atol, t0, tout, tret;
   int num_threads;
+  SUNContext ctx;
 
   ida_mem = NULL;
   LS = NULL;
@@ -228,18 +234,22 @@ int main(int argc, char *argv[])
   num_threads = omp_get_max_threads();    /* overwrite with OMP_NUM_THREADS */
 #endif
   if (argc > 1)      /* overwrithe with command line value, if supplied */
-    num_threads = strtol(argv[1], NULL, 0);
+    num_threads = (int) strtol(argv[1], NULL, 0);
+
+  /* Create the SUNDIALS context object for this simulation */
+  retval = SUNContext_Create(NULL, &ctx);
+  if (check_retval(&retval, "SUNContext_Create", 1)) return 1;
 
   /* Allocate and initialize user data block webdata. */
 
   webdata = (UserData) malloc(sizeof *webdata);
-  webdata->rates = N_VNew_OpenMP(NEQ, num_threads);
-  webdata->acoef = newDenseMat(NUM_SPECIES, NUM_SPECIES);
-  webdata->ewt = N_VNew_OpenMP(NEQ, num_threads);
+  webdata->rates = N_VNew_OpenMP(NEQ, num_threads, ctx);
+  webdata->acoef = SUNDlsMat_newDenseMat(NUM_SPECIES, NUM_SPECIES);
+  webdata->ewt = N_VNew_OpenMP(NEQ, num_threads, ctx);
   for (jx = 0; jx < MX; jx++) {
     for (jy = 0; jy < MY; jy++) {
-      (webdata->pivot)[jx][jy] = newIndexArray(NUM_SPECIES);
-      (webdata->PP)[jx][jy] = newDenseMat(NUM_SPECIES, NUM_SPECIES);
+      (webdata->pivot)[jx][jy] = SUNDlsMat_newIndexArray(NUM_SPECIES);
+      (webdata->PP)[jx][jy] = SUNDlsMat_newDenseMat(NUM_SPECIES, NUM_SPECIES);
     }
   }
   webdata->nthreads = num_threads;
@@ -248,13 +258,13 @@ int main(int argc, char *argv[])
 
   /* Allocate N-vectors and initialize cc, cp, and id. */
 
-  cc  = N_VNew_OpenMP(NEQ, num_threads);
+  cc  = N_VNew_OpenMP(NEQ, num_threads, ctx);
   if(check_retval((void *)cc, "N_VNew_OpenMP", 0)) return(1);
 
-  cp  = N_VNew_OpenMP(NEQ, num_threads);
+  cp  = N_VNew_OpenMP(NEQ, num_threads, ctx);
   if(check_retval((void *)cp, "N_VNew_OpenMP", 0)) return(1);
 
-  id  = N_VNew_OpenMP(NEQ, num_threads);
+  id  = N_VNew_OpenMP(NEQ, num_threads, ctx);
   if(check_retval((void *)id, "N_VNew_OpenMP", 0)) return(1);
 
   SetInitialProfiles(cc, cp, id, webdata);
@@ -267,7 +277,7 @@ int main(int argc, char *argv[])
 
   /* Call IDACreate and IDAMalloc to initialize IDA. */
 
-  ida_mem = IDACreate();
+  ida_mem = IDACreate(ctx);
   if(check_retval((void *)ida_mem, "IDACreate", 0)) return(1);
 
   retval = IDASetUserData(ida_mem, webdata);
@@ -288,7 +298,7 @@ int main(int argc, char *argv[])
      preconditioning routines. */
 
   maxl = 16;                               /* max dimension of the Krylov subspace */
-  LS = SUNLinSol_SPGMR(cc, PREC_LEFT, maxl);      /* IDA only allows left preconditioning */
+  LS = SUNLinSol_SPGMR(cc, SUN_PREC_LEFT, maxl, ctx);      /* IDA only allows left preconditioning */
   if(check_retval((void *)LS, "SUNLinSol_SPGMR", 0)) return(1);
 
   retval = IDASetLinearSolver(ida_mem, LS, NULL);
@@ -331,22 +341,24 @@ int main(int argc, char *argv[])
   IDAFree(&ida_mem);
   SUNLinSolFree(LS);
 
-  N_VDestroy_OpenMP(cc);
-  N_VDestroy_OpenMP(cp);
-  N_VDestroy_OpenMP(id);
+  N_VDestroy(cc);
+  N_VDestroy(cp);
+  N_VDestroy(id);
 
 
-  destroyMat(webdata->acoef);
-  N_VDestroy_OpenMP(webdata->rates);
-  N_VDestroy_OpenMP(webdata->ewt);
+  SUNDlsMat_destroyMat(webdata->acoef);
+  N_VDestroy(webdata->rates);
+  N_VDestroy(webdata->ewt);
   for (jx = 0; jx < MX; jx++) {
     for (jy = 0; jy < MY; jy ++) {
-      destroyArray((webdata->pivot)[jx][jy]);
-      destroyMat((webdata->PP)[jx][jy]);
+      SUNDlsMat_destroyArray((webdata->pivot)[jx][jy]);
+      SUNDlsMat_destroyMat((webdata->PP)[jx][jy]);
     }
   }
   free(webdata);
 
+  SUNContext_Free(&ctx);
+
   return(0);
 }
 
@@ -377,11 +389,13 @@ static int resweb(realtype tt, N_Vector cc, N_Vector cp,
   realtype *resv, *cpv;
   UserData webdata;
 
+  jx = jy = is = 0;
+
   webdata = (UserData)user_data;
 
-  cpv = NV_DATA_OMP(cp);
+  cpv  = NV_DATA_OMP(cp);
   resv = NV_DATA_OMP(res);
-  np = webdata->np;
+  np   = webdata->np;
 
   /* Call Fweb to set res to vector of right-hand sides. */
   Fweb(tt, cc, res, webdata);
@@ -411,10 +425,11 @@ static int Precond(realtype tt, N_Vector cc, N_Vector cp,
                    N_Vector rr, realtype cj, void *user_data)
 {
   int retval;
+  sunindextype ret;
   realtype uround, xx, yy, del_x, del_y;
   realtype **Pxy, *ratesxy, *Pxycol, *cxy, *cpxy, *ewtxy, cctmp;
   realtype inc, fac, sqru, perturb_rates[NUM_SPECIES];
-  int is, js, jx, jy, ret;
+  int is, js, jx, jy;
   void *ida_mem;
   N_Vector ewt;
   realtype hh;
@@ -425,7 +440,7 @@ static int Precond(realtype tt, N_Vector cc, N_Vector cp,
   del_y = webdata->dy;
 
   uround = UNIT_ROUNDOFF;
-  sqru = SUNRsqrt(uround);
+  sqru = sqrt(uround);
 
   ida_mem = webdata->ida_mem;
   ewt = webdata->ewt;
@@ -446,7 +461,7 @@ static int Precond(realtype tt, N_Vector cc, N_Vector cp,
       ratesxy = IJ_Vptr((webdata->rates), jx, jy);
 
       for (js = 0; js < NUM_SPECIES; js++) {
-	inc = sqru*(SUNMAX(SUNRabs(cxy[js]), SUNMAX(hh*SUNRabs(cpxy[js]), ONE/ewtxy[js])));
+	inc = sqru*(MAX(fabs(cxy[js]), MAX(hh*fabs(cpxy[js]), ONE/ewtxy[js])));
 	cctmp = cxy[js];
 	cxy[js] += inc;
 	fac = -ONE/inc;
@@ -463,7 +478,7 @@ static int Precond(realtype tt, N_Vector cc, N_Vector cp,
 	cxy[js] = cctmp;
       }
 
-      ret = denseGETRF(Pxy, NUM_SPECIES, NUM_SPECIES, (webdata->pivot)[jx][jy]);
+      ret = SUNDlsMat_denseGETRF(Pxy, NUM_SPECIES, NUM_SPECIES, (webdata->pivot)[jx][jy]);
 
       if (ret != 0) return(1);
     }
@@ -480,9 +495,11 @@ static int PSolve(realtype tt, N_Vector cc, N_Vector cp,
 {
   realtype **Pxy, *zxy;
   sunindextype *pivot;
-  int jx, jy;
+  sunindextype jx, jy;
   UserData webdata;
 
+  jx = jy = 0;
+
   webdata = (UserData) user_data;
 
   N_VScale(ONE, rvec, zvec);
@@ -494,7 +511,7 @@ static int PSolve(realtype tt, N_Vector cc, N_Vector cp,
       zxy = IJ_Vptr(zvec, jx, jy);
       Pxy = (webdata->PP)[jx][jy];
       pivot = (webdata->pivot)[jx][jy];
-      denseGETRS(Pxy, NUM_SPECIES, pivot, zxy);
+      SUNDlsMat_denseGETRS(Pxy, NUM_SPECIES, pivot, zxy);
     }
   }
 
@@ -515,7 +532,7 @@ static int PSolve(realtype tt, N_Vector cc, N_Vector cp,
 
 static void InitUserData(UserData webdata)
 {
-  int i, j, np;
+  sunindextype i, j, np;
   realtype *a1,*a2, *a3, *a4, dx2, dy2;
 
   webdata->mx = MX;
@@ -732,6 +749,8 @@ static void Fweb(realtype tcalc, N_Vector cc, N_Vector crate,
   /* Loop over grid points, evaluate interaction vector (length ns),
      form diffusion difference terms, and load crate.                    */
 
+  jx = jy = is = 0;
+
   for (jy = 0; jy < MY; jy++) {
     yy = (webdata->dy) * jy ;
     idyu = (jy!=MY-1) ? NSMX : -NSMX;
