@@ -2,6 +2,16 @@
  * Programmer(s): Allan Taylor, Alan Hindmarsh and
  *                Radu Serban @ LLNL
  * -----------------------------------------------------------------
+ * SUNDIALS Copyright Start
+ * Copyright (c) 2002-2022, Lawrence Livermore National Security
+ * and Southern Methodist University.
+ * All rights reserved.
+ *
+ * See the top-level LICENSE and NOTICE files for details.
+ *
+ * SPDX-License-Identifier: BSD-3-Clause
+ * SUNDIALS Copyright End
+ * -----------------------------------------------------------------
  * Example problem for IDA: 2D heat equation, serial, banded.
  *
  * This example solves a discretized 2D heat equation problem.
@@ -34,7 +44,6 @@
 #include <nvector/nvector_serial.h>    /* access to serial N_Vector            */
 #include <sunmatrix/sunmatrix_band.h>  /* access to band SUNMatrix             */
 #include <sunlinsol/sunlinsol_band.h>  /* access to band SUNLinearSolver       */
-#include <ida/ida_direct.h>            /* access to IDADls interface           */
 #include <sundials/sundials_types.h>   /* definition of type realtype          */
 
 /* Problem Constants */
@@ -63,10 +72,10 @@ int heatres(realtype tres, N_Vector uu, N_Vector up, N_Vector resval, void *user
 
 static void PrintHeader(realtype rtol, realtype atol);
 static void PrintOutput(void *mem, realtype t, N_Vector u);
-static int SetInitialProfile(UserData data, N_Vector uu, N_Vector up, 
+static int SetInitialProfile(UserData data, N_Vector uu, N_Vector up,
                              N_Vector id, N_Vector res);
 
-static int check_flag(void *flagvalue, const char *funcname, int opt);
+static int check_retval(void *returnvalue, const char *funcname, int opt);
 
 /*
  *--------------------------------------------------------------------
@@ -79,34 +88,39 @@ int main(void)
   void *mem;
   UserData data;
   N_Vector uu, up, constraints, id, res;
-  int ier, iout;
+  int retval, iout;
   long int netf, ncfn;
   sunindextype mu, ml;
   realtype rtol, atol, t0, t1, tout, tret;
   SUNMatrix A;
   SUNLinearSolver LS;
-  
+  SUNContext ctx;
+
   mem = NULL;
   data = NULL;
   uu = up = constraints = id = res = NULL;
   A = NULL;
   LS = NULL;
 
+  /* Create the SUNDIALS context object for this simulation */
+  retval = SUNContext_Create(NULL, &ctx);
+  if (check_retval(&retval, "SUNContext_Create", 1)) return 1;
+
   /* Create vectors uu, up, res, constraints, id. */
-  uu = N_VNew_Serial(NEQ);
-  if(check_flag((void *)uu, "N_VNew_Serial", 0)) return(1);
-  up = N_VNew_Serial(NEQ);
-  if(check_flag((void *)up, "N_VNew_Serial", 0)) return(1);
-  res = N_VNew_Serial(NEQ);
-  if(check_flag((void *)res, "N_VNew_Serial", 0)) return(1);
-  constraints = N_VNew_Serial(NEQ);
-  if(check_flag((void *)constraints, "N_VNew_Serial", 0)) return(1);
-  id = N_VNew_Serial(NEQ);
-  if(check_flag((void *)id, "N_VNew_Serial", 0)) return(1);
+  uu = N_VNew_Serial(NEQ, ctx);
+  if(check_retval((void *)uu, "N_VNew_Serial", 0)) return(1);
+  up = N_VClone(uu);
+  if(check_retval((void *)up, "N_VNew_Serial", 0)) return(1);
+  res = N_VClone(uu);
+  if(check_retval((void *)res, "N_VNew_Serial", 0)) return(1);
+  constraints = N_VClone(uu);
+  if(check_retval((void *)constraints, "N_VNew_Serial", 0)) return(1);
+  id = N_VClone(uu);
+  if(check_retval((void *)id, "N_VNew_Serial", 0)) return(1);
 
   /* Create and load problem data block. */
   data = (UserData) malloc(sizeof *data);
-  if(check_flag((void *)data, "malloc", 2)) return(1);
+  if(check_retval((void *)data, "malloc", 2)) return(1);
   data->mm = MGRID;
   data->dx = ONE/(MGRID - ONE);
   data->coeff = ONE/( (data->dx) * (data->dx) );
@@ -124,66 +138,66 @@ int main(void)
   atol = RCONST(1.0e-3);
 
   /* Call IDACreate and IDAMalloc to initialize solution */
-  mem = IDACreate();
-  if(check_flag((void *)mem, "IDACreate", 0)) return(1);
+  mem = IDACreate(ctx);
+  if(check_retval((void *)mem, "IDACreate", 0)) return(1);
 
-  ier = IDASetUserData(mem, data);
-  if(check_flag(&ier, "IDASetUserData", 1)) return(1);
+  retval = IDASetUserData(mem, data);
+  if(check_retval(&retval, "IDASetUserData", 1)) return(1);
 
   /* Set which components are algebraic or differential */
-  ier = IDASetId(mem, id);
-  if(check_flag(&ier, "IDASetId", 1)) return(1);
+  retval = IDASetId(mem, id);
+  if(check_retval(&retval, "IDASetId", 1)) return(1);
 
-  ier = IDASetConstraints(mem, constraints);
-  if(check_flag(&ier, "IDASetConstraints", 1)) return(1);
+  retval = IDASetConstraints(mem, constraints);
+  if(check_retval(&retval, "IDASetConstraints", 1)) return(1);
   N_VDestroy(constraints);
 
-  ier = IDAInit(mem, heatres, t0, uu, up);
-  if(check_flag(&ier, "IDAInit", 1)) return(1);
+  retval = IDAInit(mem, heatres, t0, uu, up);
+  if(check_retval(&retval, "IDAInit", 1)) return(1);
 
-  ier = IDASStolerances(mem, rtol, atol);
-  if(check_flag(&ier, "IDASStolerances", 1)) return(1);
+  retval = IDASStolerances(mem, rtol, atol);
+  if(check_retval(&retval, "IDASStolerances", 1)) return(1);
 
   /* Create banded SUNMatrix for use in linear solves */
   mu = MGRID; ml = MGRID;
-  A = SUNBandMatrix(NEQ, mu, ml, mu+ml);
-  if(check_flag((void *)A, "SUNBandMatrix", 0)) return(1);
+  A = SUNBandMatrix(NEQ, mu, ml, ctx);
+  if(check_retval((void *)A, "SUNBandMatrix", 0)) return(1);
 
   /* Create banded SUNLinearSolver object */
-  LS = SUNBandLinearSolver(uu, A);
-  if(check_flag((void *)LS, "SUNBandLinearSolver", 0)) return(1);
+  LS = SUNLinSol_Band(uu, A, ctx);
+  if(check_retval((void *)LS, "SUNLinSol_Band", 0)) return(1);
 
   /* Attach the matrix and linear solver */
-  ier = IDADlsSetLinearSolver(mem, LS, A);
-  if(check_flag(&ier, "IDADlsSetLinearSolver", 1)) return(1);
+  retval = IDASetLinearSolver(mem, LS, A);
+  if(check_retval(&retval, "IDASetLinearSolver", 1)) return(1);
 
   /* Call IDACalcIC to correct the initial values. */
 
-  ier = IDACalcIC(mem, IDA_YA_YDP_INIT, t1);
-  if(check_flag(&ier, "IDACalcIC", 1)) return(1);
+  retval = IDACalcIC(mem, IDA_YA_YDP_INIT, t1);
+  if(check_retval(&retval, "IDACalcIC", 1)) return(1);
 
   /* Print output heading. */
   PrintHeader(rtol, atol);
-  
+
   PrintOutput(mem, t0, uu);
 
 
   /* Loop over output times, call IDASolve, and print results. */
-  
+
   for (tout = t1, iout = 1; iout <= NOUT; iout++, tout *= TWO) {
-    
-    ier = IDASolve(mem, tout, &tret, uu, up, IDA_NORMAL);
-    if(check_flag(&ier, "IDASolve", 1)) return(1);
+
+    retval = IDASolve(mem, tout, &tret, uu, up, IDA_NORMAL);
+    if(check_retval(&retval, "IDASolve", 1)) return(1);
 
     PrintOutput(mem, tret, uu);
-  
+
   }
-  
+
   /* Print remaining counters and free memory. */
-  ier = IDAGetNumErrTestFails(mem, &netf);
-  check_flag(&ier, "IDAGetNumErrTestFails", 1);
-  ier = IDAGetNumNonlinSolvConvFails(mem, &ncfn);
-  check_flag(&ier, "IDAGetNumNonlinSolvConvFails", 1);
+  retval = IDAGetNumErrTestFails(mem, &netf);
+  check_retval(&retval, "IDAGetNumErrTestFails", 1);
+  retval = IDAGetNumNonlinSolvConvFails(mem, &ncfn);
+  check_retval(&retval, "IDAGetNumNonlinSolvConvFails", 1);
   printf("\n netf = %ld,   ncfn = %ld \n", netf, ncfn);
 
   IDAFree(&mem);
@@ -194,6 +208,7 @@ int main(void)
   N_VDestroy(id);
   N_VDestroy(res);
   free(data);
+  SUNContext_Free(&ctx);
 
   return(0);
 }
@@ -205,40 +220,40 @@ int main(void)
  */
 
 /*
- * heatres: heat equation system residual function                       
- * This uses 5-point central differencing on the interior points, and    
- * includes algebraic equations for the boundary values.                 
- * So for each interior point, the residual component has the form       
- *    res_i = u'_i - (central difference)_i                              
- * while for each boundary point, it is res_i = u_i.                     
+ * heatres: heat equation system residual function
+ * This uses 5-point central differencing on the interior points, and
+ * includes algebraic equations for the boundary values.
+ * So for each interior point, the residual component has the form
+ *    res_i = u'_i - (central difference)_i
+ * while for each boundary point, it is res_i = u_i.
  */
 
-int heatres(realtype tres, N_Vector uu, N_Vector up, N_Vector resval, 
+int heatres(realtype tres, N_Vector uu, N_Vector up, N_Vector resval,
             void *user_data)
 {
   sunindextype mm, i, j, offset, loc;
   realtype *uv, *upv, *resv, coeff;
   UserData data;
-  
+
   uv = N_VGetArrayPointer(uu); upv = N_VGetArrayPointer(up); resv = N_VGetArrayPointer(resval);
 
   data = (UserData)user_data;
   mm = data->mm;
   coeff = data->coeff;
-  
+
   /* Initialize resval to uu, to take care of boundary equations. */
   N_VScale(ONE, uu, resval);
-  
+
   /* Loop over interior points; set res = up - (central difference). */
   for (j = 1; j < mm-1; j++) {
     offset = mm*j;
     for (i = 1; i < mm-1; i++) {
       loc = offset + i;
-      resv[loc] = upv[loc] - coeff * 
+      resv[loc] = upv[loc] - coeff *
 	  (uv[loc-1] + uv[loc+1] + uv[loc-mm] + uv[loc+mm] - RCONST(4.0)*uv[loc]);
     }
   }
-  
+
   return(0);
 
 }
@@ -250,18 +265,18 @@ int heatres(realtype tres, N_Vector uu, N_Vector up, N_Vector resval,
  */
 
 /*
- * SetInitialProfile: routine to initialize u, up, and id vectors.       
+ * SetInitialProfile: routine to initialize u, up, and id vectors.
  */
 
-static int SetInitialProfile(UserData data, N_Vector uu, N_Vector up, 
+static int SetInitialProfile(UserData data, N_Vector uu, N_Vector up,
                              N_Vector id, N_Vector res)
 {
   realtype xfact, yfact, *udata, *updata, *iddata;
   sunindextype mm, mm1, i, j, offset, loc;
-  
+
   mm = data->mm;
   mm1 = mm - 1;
-  
+
   udata = N_VGetArrayPointer(uu);
   updata = N_VGetArrayPointer(up);
   iddata = N_VGetArrayPointer(id);
@@ -269,7 +284,7 @@ static int SetInitialProfile(UserData data, N_Vector uu, N_Vector up,
   /* Initialize id to 1's. */
   N_VConst(ONE, id);
 
-  /* Initialize uu on all grid points. */ 
+  /* Initialize uu on all grid points. */
   for (j = 0; j < mm; j++) {
     yfact = data->dx * j;
     offset = mm*j;
@@ -279,13 +294,13 @@ static int SetInitialProfile(UserData data, N_Vector uu, N_Vector up,
       udata[loc] = RCONST(16.0) * xfact * (ONE - xfact) * yfact * (ONE - yfact);
     }
   }
-  
+
   /* Initialize up vector to 0. */
   N_VConst(ZERO, up);
 
   /* heatres sets res to negative of ODE RHS values at interior points. */
   heatres(ZERO, uu, up, res, data);
-  
+
   /* Copy -res into up to get correct interior initial up values. */
   N_VScale(-ONE, res, up);
 
@@ -298,12 +313,12 @@ static int SetInitialProfile(UserData data, N_Vector uu, N_Vector up,
         udata[loc] = BVAL; updata[loc] = ZERO; iddata[loc] = ZERO; }
     }
   }
-  
+
   return(0);
 
 }
 
-/* 
+/*
  * Print first lines of output (problem description)
  */
 
@@ -315,9 +330,9 @@ static void PrintHeader(realtype rtol, realtype atol)
   printf(" polynomial initial conditions.\n");
   printf("          Mesh dimensions: %d x %d", MGRID, MGRID);
   printf("        Total system size: %d\n\n", NEQ);
-#if defined(SUNDIALS_EXTENDED_PRECISION) 
+#if defined(SUNDIALS_EXTENDED_PRECISION)
   printf("Tolerance parameters:  rtol = %Lg   atol = %Lg\n", rtol, atol);
-#elif defined(SUNDIALS_DOUBLE_PRECISION) 
+#elif defined(SUNDIALS_DOUBLE_PRECISION)
   printf("Tolerance parameters:  rtol = %g   atol = %g\n", rtol, atol);
 #else
   printf("Tolerance parameters:  rtol = %g   atol = %g\n", rtol, atol);
@@ -344,32 +359,32 @@ static void PrintHeader(realtype rtol, realtype atol)
 
 static void PrintOutput(void *mem, realtype t, N_Vector uu)
 {
-  int ier;
+  int retval;
   realtype umax, hused;
   long int nst, nni, nje, nre, nreLS;
   int kused;
 
   umax = N_VMaxNorm(uu);
-  
-  ier = IDAGetLastOrder(mem, &kused);
-  check_flag(&ier, "IDAGetLastOrder", 1);
-  ier = IDAGetNumSteps(mem, &nst);
-  check_flag(&ier, "IDAGetNumSteps", 1);
-  ier = IDAGetNumNonlinSolvIters(mem, &nni);
-  check_flag(&ier, "IDAGetNumNonlinSolvIters", 1);
-  ier = IDAGetNumResEvals(mem, &nre);
-  check_flag(&ier, "IDAGetNumResEvals", 1);
-  ier = IDAGetLastStep(mem, &hused);
-  check_flag(&ier, "IDAGetLastStep", 1);
-  ier = IDADlsGetNumJacEvals(mem, &nje);
-  check_flag(&ier, "IDADlsGetNumJacEvals", 1);
-  ier = IDADlsGetNumResEvals(mem, &nreLS);
-  check_flag(&ier, "IDADlsGetNumResEvals", 1);
 
-#if defined(SUNDIALS_EXTENDED_PRECISION) 
+  retval = IDAGetLastOrder(mem, &kused);
+  check_retval(&retval, "IDAGetLastOrder", 1);
+  retval = IDAGetNumSteps(mem, &nst);
+  check_retval(&retval, "IDAGetNumSteps", 1);
+  retval = IDAGetNumNonlinSolvIters(mem, &nni);
+  check_retval(&retval, "IDAGetNumNonlinSolvIters", 1);
+  retval = IDAGetNumResEvals(mem, &nre);
+  check_retval(&retval, "IDAGetNumResEvals", 1);
+  retval = IDAGetLastStep(mem, &hused);
+  check_retval(&retval, "IDAGetLastStep", 1);
+  retval = IDAGetNumJacEvals(mem, &nje);
+  check_retval(&retval, "IDAGetNumJacEvals", 1);
+  retval = IDAGetNumLinResEvals(mem, &nreLS);
+  check_retval(&retval, "IDAGetNumLinResEvals", 1);
+
+#if defined(SUNDIALS_EXTENDED_PRECISION)
   printf(" %5.2Lf %13.5Le  %d  %3ld  %3ld  %3ld  %4ld  %4ld  %9.2Le \n",
          t, umax, kused, nst, nni, nje, nre, nreLS, hused);
-#elif defined(SUNDIALS_DOUBLE_PRECISION) 
+#elif defined(SUNDIALS_DOUBLE_PRECISION)
   printf(" %5.2f %13.5e  %d  %3ld  %3ld  %3ld  %4ld  %4ld  %9.2e \n",
          t, umax, kused, nst, nni, nje, nre, nreLS, hused);
 #else
@@ -379,39 +394,39 @@ static void PrintOutput(void *mem, realtype t, N_Vector uu)
 
 }
 
-/* 
+/*
  * Check function return value...
  *   opt == 0 means SUNDIALS function allocates memory so check if
  *            returned NULL pointer
- *   opt == 1 means SUNDIALS function returns a flag so check if
- *            flag >= 0
+ *   opt == 1 means SUNDIALS function returns an integer value so check if
+ *            retval < 0
  *   opt == 2 means function allocates memory so check if returned
- *            NULL pointer 
+ *            NULL pointer
  */
 
-static int check_flag(void *flagvalue, const char *funcname, int opt)
+static int check_retval(void *returnvalue, const char *funcname, int opt)
 {
-  int *errflag;
+  int *retval;
 
   /* Check if SUNDIALS function returned NULL pointer - no memory allocated */
-  if (opt == 0 && flagvalue == NULL) {
-    fprintf(stderr, 
-            "\nSUNDIALS_ERROR: %s() failed - returned NULL pointer\n\n", 
+  if (opt == 0 && returnvalue == NULL) {
+    fprintf(stderr,
+            "\nSUNDIALS_ERROR: %s() failed - returned NULL pointer\n\n",
             funcname);
     return(1);
   } else if (opt == 1) {
-    /* Check if flag < 0 */
-    errflag = (int *) flagvalue;
-    if (*errflag < 0) {
-      fprintf(stderr, 
-              "\nSUNDIALS_ERROR: %s() failed with flag = %d\n\n", 
-              funcname, *errflag);
-      return(1); 
+    /* Check if retval < 0 */
+    retval = (int *) returnvalue;
+    if (*retval < 0) {
+      fprintf(stderr,
+              "\nSUNDIALS_ERROR: %s() failed with retval = %d\n\n",
+              funcname, *retval);
+      return(1);
     }
-  } else if (opt == 2 && flagvalue == NULL) {
+  } else if (opt == 2 && returnvalue == NULL) {
     /* Check if function returned NULL pointer - no memory allocated */
-    fprintf(stderr, 
-            "\nMEMORY_ERROR: %s() failed - returned NULL pointer\n\n", 
+    fprintf(stderr,
+            "\nMEMORY_ERROR: %s() failed - returned NULL pointer\n\n",
             funcname);
     return(1);
   }
