@@ -3,7 +3,7 @@
  *      Allan Taylor, Alan Hindmarsh and Radu Serban @ LLNL
  * -----------------------------------------------------------------
  * SUNDIALS Copyright Start
- * Copyright (c) 2002-2021, Lawrence Livermore National Security
+ * Copyright (c) 2002-2022, Lawrence Livermore National Security
  * and Southern Methodist University.
  * All rights reserved.
  *
@@ -219,7 +219,7 @@ static realtype dotprod(int size, realtype *x1, realtype *x2);
 /* Prototypes for private Helper Functions. */
 
 static UserData AllocUserData(MPI_Comm comm, sunindextype local_N,
-                              sunindextype SystemSize);
+                              sunindextype SystemSize, SUNContext ctx);
 
 static void InitUserData(UserData webdata, int thispe, int npes,
                          MPI_Comm comm);
@@ -248,6 +248,7 @@ static int check_retval(void *returnvalue, const char *funcname, int opt, int id
 int main(int argc, char *argv[])
 {
   MPI_Comm comm;
+  SUNContext ctx;
   void *ida_mem;
   SUNLinearSolver LS;
   UserData webdata;
@@ -272,11 +273,15 @@ int main(int argc, char *argv[])
     if (thispe == 0)
       fprintf(stderr,
               "\nMPI_ERROR(0): npes = %d not equal to NPEX*NPEY = %d\n",
-	      npes, NPEX*NPEY);
+              npes, NPEX*NPEY);
     MPI_Finalize();
     return(1);
   }
 
+  /* Create the SUNDIALS context object for this simulation. */
+  retval = SUNContext_Create((void*) &comm, &ctx);
+  if (check_retval(&retval, "SUNContext_Create", 1, thispe)) MPI_Abort(comm, 1);
+
   /* Set local length (local_N) and global length (SystemSize). */
 
   local_N = MXSUB*MYSUB*NUM_SPECIES;
@@ -284,7 +289,7 @@ int main(int argc, char *argv[])
 
   /* Set up user data block webdata. */
 
-  webdata = AllocUserData(comm, local_N, SystemSize);
+  webdata = AllocUserData(comm, local_N, SystemSize, ctx);
   if (check_retval((void *)webdata, "AllocUserData", 0, thispe)) MPI_Abort(comm, 1);
 
   InitUserData(webdata, thispe, npes, comm);
@@ -292,16 +297,16 @@ int main(int argc, char *argv[])
   /* Create needed vectors, and load initial values.
      The vector res is used temporarily only.        */
 
-  cc  = N_VNew_Parallel(comm, local_N, SystemSize);
+  cc = N_VNew_Parallel(comm, local_N, SystemSize, ctx);
   if (check_retval((void *)cc, "N_VNew_Parallel", 0, thispe)) MPI_Abort(comm, 1);
 
-  cp  = N_VNew_Parallel(comm, local_N, SystemSize);
+  cp = N_VClone(cc);
   if (check_retval((void *)cp, "N_VNew_Parallel", 0, thispe)) MPI_Abort(comm, 1);
 
-  res = N_VNew_Parallel(comm, local_N, SystemSize);
+  res = N_VClone(cc);
   if (check_retval((void *)res, "N_VNew_Parallel", 0, thispe)) MPI_Abort(comm, 1);
 
-  id  = N_VNew_Parallel(comm, local_N, SystemSize);
+  id = N_VClone(cc);
   if (check_retval((void *)id, "N_VNew_Parallel", 0, thispe)) MPI_Abort(comm, 1);
 
   SetInitialProfiles(cc, cp, id, res, webdata);
@@ -317,7 +322,7 @@ int main(int argc, char *argv[])
   /* Call IDACreate and IDAMalloc to initialize IDA.
      A pointer to IDA problem memory is returned and stored in idamem. */
 
-  ida_mem = IDACreate();
+  ida_mem = IDACreate(ctx);
   if (check_retval((void *)ida_mem, "IDACreate", 0, thispe)) MPI_Abort(comm, 1);
 
   retval = IDASetUserData(ida_mem, webdata);
@@ -339,7 +344,7 @@ int main(int argc, char *argv[])
      (Precondbd & PSolvebd).  maxl (Krylov subspace dim.) is set to 16. */
 
   maxl = 16;
-  LS = SUNLinSol_SPGMR(cc, PREC_LEFT, maxl);
+  LS = SUNLinSol_SPGMR(cc, SUN_PREC_LEFT, maxl, ctx);
   if (check_retval((void *)LS, "SUNLinSol_SPGMR", 0, thispe)) MPI_Abort(comm, 1);
 
   retval = SUNLinSol_SPGMRSetMaxRestarts(LS, 5);  /* IDA recommends allowing up to 5 restarts */
@@ -393,6 +398,8 @@ int main(int argc, char *argv[])
 
   FreeUserData(webdata);
 
+  SUNContext_Free(&ctx);
+
   MPI_Finalize();
 
   return(0);
@@ -409,24 +416,25 @@ int main(int argc, char *argv[])
  * AllocUserData: Allocate memory for data structure of type UserData.
  */
 
-static UserData AllocUserData(MPI_Comm comm, sunindextype local_N, sunindextype SystemSize)
+static UserData AllocUserData(MPI_Comm comm, sunindextype local_N,
+                              sunindextype SystemSize, SUNContext ctx)
 {
   int ix, jy;
   UserData webdata;
 
   webdata = (UserData) malloc(sizeof *webdata);
 
-  webdata->rates = N_VNew_Parallel(comm, local_N, SystemSize);
+  webdata->rates = N_VNew_Parallel(comm, local_N, SystemSize, ctx);
 
   for (ix = 0; ix < MXSUB; ix++) {
     for (jy = 0; jy < MYSUB; jy++) {
-      (webdata->PP)[ix][jy] = newDenseMat(NUM_SPECIES, NUM_SPECIES);
-      (webdata->pivot)[ix][jy] = newIndexArray(NUM_SPECIES);
+      (webdata->PP)[ix][jy] = SUNDlsMat_newDenseMat(NUM_SPECIES, NUM_SPECIES);
+      (webdata->pivot)[ix][jy] = SUNDlsMat_newIndexArray(NUM_SPECIES);
     }
   }
 
-  webdata->acoef = newDenseMat(NUM_SPECIES, NUM_SPECIES);
-  webdata->ewt = N_VNew_Parallel(comm, local_N, SystemSize);
+  webdata->acoef = SUNDlsMat_newDenseMat(NUM_SPECIES, NUM_SPECIES);
+  webdata->ewt = N_VNew_Parallel(comm, local_N, SystemSize, ctx);
   return(webdata);
 
 }
@@ -501,12 +509,12 @@ static void FreeUserData(UserData webdata)
 
   for (ix = 0; ix < MXSUB; ix++) {
     for (jy = 0; jy < MYSUB; jy++) {
-      destroyMat((webdata->PP)[ix][jy]);
-      destroyArray((webdata->pivot)[ix][jy]);
+      SUNDlsMat_destroyMat((webdata->PP)[ix][jy]);
+      SUNDlsMat_destroyArray((webdata->pivot)[ix][jy]);
     }
   }
 
-  destroyMat(webdata->acoef);
+  SUNDlsMat_destroyMat(webdata->acoef);
   N_VDestroy(webdata->rates);
   N_VDestroy(webdata->ewt);
   free(webdata);
@@ -547,7 +555,7 @@ static void SetInitialProfiles(N_Vector cc, N_Vector cp, N_Vector id,
       cxy = IJ_Vptr(cc,ix,jy);
       idxy = IJ_Vptr(id,ix,jy);
       for (is = 0; is < NUM_SPECIES; is++) {
-	if (is < np) { cxy[is] = RCONST(10.0) + (realtype)(is+1)*xyfactor; idxy[is] = ONE; }
+        if (is < np) { cxy[is] = RCONST(10.0) + (realtype)(is+1)*xyfactor; idxy[is] = ONE; }
         else { cxy[is] = 1.0e5; idxy[is] = ZERO; }
       }
     }
@@ -1196,7 +1204,7 @@ static int Precondbd(realtype tt, N_Vector cc, N_Vector cp,
       } /* End of js loop. */
 
       /* Do LU decomposition of matrix block for grid point (ix,jy). */
-      ret = denseGETRF(Pxy, ns, ns, (webdata->pivot)[ix][jy]);
+      ret = SUNDlsMat_denseGETRF(Pxy, ns, ns, (webdata->pivot)[ix][jy]);
 
       if (ret != 0) return(1);
 
@@ -1235,7 +1243,7 @@ static int PSolvebd(realtype tt, N_Vector cc, N_Vector cp,
       zxy = IJ_Vptr(zvec,ix,jy);
       Pxy = (webdata->PP)[ix][jy];
       pivot = (webdata->pivot)[ix][jy];
-      denseGETRS(Pxy, ns, pivot, zxy);
+      SUNDlsMat_denseGETRS(Pxy, ns, pivot, zxy);
 
     } /* End of jy loop. */
   } /* End of ix loop. */
