@@ -2,9 +2,19 @@
  * Programmer(s): Allan Taylor, Alan Hindmarsh and
  *                Radu Serban @ LLNL
  * -----------------------------------------------------------------
+ * SUNDIALS Copyright Start
+ * Copyright (c) 2002-2022, Lawrence Livermore National Security
+ * and Southern Methodist University.
+ * All rights reserved.
+ *
+ * See the top-level LICENSE and NOTICE files for details.
+ *
+ * SPDX-License-Identifier: BSD-3-Clause
+ * SUNDIALS Copyright End
+ * -----------------------------------------------------------------
  * Example program for IDA: Food web problem.
  *
- * This example program (serial version) uses the banded linear 
+ * This example program (serial version) uses the banded linear
  * solver, and IDACalcIC for initial condition calculation.
  *
  * The mathematical problem solved in this example is a DAE system
@@ -86,7 +96,6 @@
 #include <nvector/nvector_serial.h>    /* access to serial N_Vector            */
 #include <sunmatrix/sunmatrix_band.h>  /* access to band SUNMatrix             */
 #include <sunlinsol/sunlinsol_band.h>  /* access to band SUNLinearSolver       */
-#include <idas/idas_direct.h>          /* access to IDADls interface           */
 #include <sundials/sundials_types.h>   /* definition of type realtype          */
 
 /* Problem Constants. */
@@ -116,15 +125,15 @@
 #define NOUT        6              /* Number of output times            */
 #define TMULT       RCONST(10.0)   /* Multiplier for tout values        */
 #define TADD        RCONST(0.3)    /* Increment for tout values         */
-#define ZERO        RCONST(0.)     
-#define ONE         RCONST(1.0)    
+#define ZERO        RCONST(0.)
+#define ONE         RCONST(1.0)
 
-/* 
+/*
  * User-defined vector and accessor macro: IJ_Vptr.
- * IJ_Vptr is defined in order to express the underlying 3-D structure of 
+ * IJ_Vptr is defined in order to express the underlying 3-D structure of
  * the dependent variable vector from its underlying 1-D storage (an N_Vector).
- * IJ_Vptr(vv,i,j) returns a pointer to the location in vv corresponding to 
- * species index is = 0, x-index ix = i, and y-index jy = j.                
+ * IJ_Vptr(vv,i,j) returns a pointer to the location in vv corresponding to
+ * species index is = 0, x-index ix = i, and y-index jy = j.
  */
 
 #define IJ_Vptr(vv,i,j) (&NV_Ith_S(vv, (i)*NUM_SPECIES + (j)*NSMX))
@@ -140,7 +149,7 @@ typedef struct {
 
 /* Prototypes for functions called by the IDA Solver. */
 
-static int resweb(realtype time, N_Vector cc, N_Vector cp, N_Vector resval, 
+static int resweb(realtype time, N_Vector cc, N_Vector cp, N_Vector resval,
                   void *user_data);
 
 /* Prototypes for private Helper Functions. */
@@ -152,10 +161,10 @@ static void PrintHeader(sunindextype mu, sunindextype ml, realtype rtol, realtyp
 static void PrintOutput(void *mem, N_Vector c, realtype t);
 static void PrintFinalStats(void *mem);
 static void Fweb(realtype tcalc, N_Vector cc, N_Vector crate, UserData webdata);
-static void WebRates(realtype xx, realtype yy, realtype *cxy, realtype *ratesxy, 
+static void WebRates(realtype xx, realtype yy, realtype *cxy, realtype *ratesxy,
                      UserData webdata);
 static realtype dotprod(sunindextype size, realtype *x1, realtype *x2);
-static int check_flag(void *flagvalue, const char *funcname, int opt);
+static int check_retval(void *returnvalue, const char *funcname, int opt);
 
 /*
  *--------------------------------------------------------------------
@@ -164,7 +173,7 @@ static int check_flag(void *flagvalue, const char *funcname, int opt);
  */
 
 int main()
-{ 
+{
   void *mem;
   UserData webdata;
   N_Vector cc, cp, id;
@@ -173,6 +182,7 @@ int main()
   realtype rtol, atol, t0, tout, tret;
   SUNMatrix A;
   SUNLinearSolver LS;
+  SUNContext ctx;
 
   mem = NULL;
   webdata = NULL;
@@ -180,89 +190,93 @@ int main()
   A = NULL;
   LS = NULL;
 
+  /* Create the SUNDIALS context object for this simulation */
+  retval = SUNContext_Create(NULL, &ctx);
+  if (check_retval(&retval, "SUNContext_Create", 1)) return 1;
+
   /* Allocate and initialize user data block webdata. */
 
   webdata = (UserData) malloc(sizeof *webdata);
-  webdata->rates = N_VNew_Serial(NEQ);
-  webdata->acoef = newDenseMat(NUM_SPECIES, NUM_SPECIES);
+  webdata->rates = N_VNew_Serial(NEQ, ctx);
+  webdata->acoef = SUNDlsMat_newDenseMat(NUM_SPECIES, NUM_SPECIES);
 
   InitUserData(webdata);
 
   /* Allocate N-vectors and initialize cc, cp, and id. */
 
-  cc  = N_VNew_Serial(NEQ);
-  if(check_flag((void *)cc, "N_VNew_Serial", 0)) return(1);
+  cc = N_VNew_Serial(NEQ, ctx);
+  if(check_retval((void *)cc, "N_VNew_Serial", 0)) return(1);
 
-  cp  = N_VNew_Serial(NEQ);
-  if(check_flag((void *)cp, "N_VNew_Serial", 0)) return(1);
+  cp = N_VClone(cc);
+  if(check_retval((void *)cp, "N_VNew_Serial", 0)) return(1);
+
+  id = N_VClone(cc);
+  if(check_retval((void *)id, "N_VNew_Serial", 0)) return(1);
 
-  id  = N_VNew_Serial(NEQ);
-  if(check_flag((void *)id, "N_VNew_Serial", 0)) return(1);
-  
   SetInitialProfiles(cc, cp, id, webdata);
-  
+
   /* Set remaining inputs to IDAMalloc. */
-  
+
   t0 = ZERO;
-  rtol = RTOL; 
+  rtol = RTOL;
   atol = ATOL;
 
   /* Call IDACreate and IDAMalloc to initialize IDA. */
-  
-  mem = IDACreate();
-  if(check_flag((void *)mem, "IDACreate", 0)) return(1);
+
+  mem = IDACreate(ctx);
+  if(check_retval((void *)mem, "IDACreate", 0)) return(1);
 
   retval = IDASetUserData(mem, webdata);
-  if(check_flag(&retval, "IDASetUserData", 1)) return(1);
+  if(check_retval(&retval, "IDASetUserData", 1)) return(1);
 
   retval = IDASetId(mem, id);
-  if(check_flag(&retval, "IDASetId", 1)) return(1);
+  if(check_retval(&retval, "IDASetId", 1)) return(1);
 
   retval = IDAInit(mem, resweb, t0, cc, cp);
-  if(check_flag(&retval, "IDAInit", 1)) return(1);
+  if(check_retval(&retval, "IDAInit", 1)) return(1);
 
   retval = IDASStolerances(mem, rtol, atol);
-  if(check_flag(&retval, "IDASStolerances", 1)) return(1);
+  if(check_retval(&retval, "IDASStolerances", 1)) return(1);
 
   /* Create banded SUNMatrix for use in linear solves */
   mu = ml = NSMX;
-  A = SUNBandMatrix(NEQ, mu, ml, mu+ml);
-  if(check_flag((void *)A, "SUNBandMatrix", 0)) return(1);
+  A = SUNBandMatrix(NEQ, mu, ml, ctx);
+  if(check_retval((void *)A, "SUNBandMatrix", 0)) return(1);
 
   /* Create banded SUNLinearSolver object */
-  LS = SUNBandLinearSolver(cc, A);
-  if(check_flag((void *)LS, "SUNBandLinearSolver", 0)) return(1);
+  LS = SUNLinSol_Band(cc, A, ctx);
+  if(check_retval((void *)LS, "SUNLinSol_Band", 0)) return(1);
 
   /* Attach the matrix and linear solver */
-  retval = IDADlsSetLinearSolver(mem, LS, A);
-  if(check_flag(&retval, "IDADlsSetLinearSolver", 1)) return(1);
+  retval = IDASetLinearSolver(mem, LS, A);
+  if(check_retval(&retval, "IDASetLinearSolver", 1)) return(1);
 
   /* Call IDACalcIC (with default options) to correct the initial values. */
 
   tout = RCONST(0.001);
   retval = IDACalcIC(mem, IDA_YA_YDP_INIT, tout);
-  if(check_flag(&retval, "IDACalcIC", 1)) return(1);
-  
+  if(check_retval(&retval, "IDACalcIC", 1)) return(1);
+
   /* Print heading, basic parameters, and initial values. */
 
   PrintHeader(mu, ml, rtol, atol);
   PrintOutput(mem, cc, ZERO);
-  
+
   /* Loop over iout, call IDASolve (normal mode), print selected output. */
-  
+
   for (iout = 1; iout <= NOUT; iout++) {
-    
+
     retval = IDASolve(mem, tout, &tret, cc, cp, IDA_NORMAL);
-    if(check_flag(&retval, "IDASolve", 1)) return(retval);
-    
+    if(check_retval(&retval, "IDASolve", 1)) return(retval);
+
     PrintOutput(mem, cc, tret);
-    
+
     if (iout < 3) tout *= TMULT; else tout += TADD;
-    
+
   }
-  
-  /* Print final statistics and free memory. */  
-  
+
+  /* Print final statistics and free memory. */
+
   PrintFinalStats(mem);
 
   /* Free memory */
@@ -276,14 +290,16 @@ int main()
   N_VDestroy(id);
 
 
-  destroyMat(webdata->acoef);
+  SUNDlsMat_destroyMat(webdata->acoef);
   N_VDestroy(webdata->rates);
   free(webdata);
 
+  SUNContext_Free(&ctx);
+
   return(0);
 }
 
-/* Define lines for readability in later routines */ 
+/* Define lines for readability in later routines */
 
 #define acoef  (webdata->acoef)
 #define bcoef  (webdata->bcoef)
@@ -296,32 +312,32 @@ int main()
  *--------------------------------------------------------------------
  */
 
-/* 
+/*
  * resweb: System residual function for predator-prey system.
  * This routine calls Fweb to get all the right-hand sides of the
  * equations, then loads the residual vector accordingly,
- * using cp in the case of prey species.                 
+ * using cp in the case of prey species.
  */
 
-static int resweb(realtype tt, N_Vector cc, N_Vector cp, 
+static int resweb(realtype tt, N_Vector cc, N_Vector cp,
                   N_Vector res,  void *user_data)
 {
   sunindextype jx, jy, is, yloc, loc, np;
   realtype *resv, *cpv;
   UserData webdata;
-  
+
   webdata = (UserData)user_data;
-  
+
   cpv = N_VGetArrayPointer(cp);
   resv = N_VGetArrayPointer(res);
   np = webdata->np;
-  
+
   /* Call Fweb to set res to vector of right-hand sides. */
   Fweb(tt, cc, res, webdata);
-  
+
   /* Loop over all grid points, setting residual values appropriately
      for differential or algebraic components.                        */
-  
+
   for (jy = 0; jy < MY; jy++) {
     yloc = NSMX * jy;
     for (jx = 0; jx < MX; jx++) {
@@ -334,9 +350,9 @@ static int resweb(realtype tt, N_Vector cc, N_Vector cp,
       }
     }
   }
-  
+
   return(0);
-  
+
 }
 
 /*
@@ -346,12 +362,12 @@ static int resweb(realtype tt, N_Vector cc, N_Vector cp,
  */
 
 /*
- * InitUserData: Load problem constants in webdata (of type UserData).   
+ * InitUserData: Load problem constants in webdata (of type UserData).
  */
 
 static void InitUserData(UserData webdata)
 {
-  int i, j, np;
+  sunindextype i, j, np;
   realtype *a1,*a2, *a3, *a4, dx2, dy2;
 
   webdata->mx = MX;
@@ -361,11 +377,11 @@ static void InitUserData(UserData webdata)
   webdata->dx = AX/(MX-1);
   webdata->dy = AY/(MY-1);
   webdata->Neq= NEQ;
-  
+
   /* Set up the coefficients a and b, and others found in the equations. */
   np = webdata->np;
   dx2 = (webdata->dx)*(webdata->dx); dy2 = (webdata->dy)*(webdata->dy);
-  
+
   for (i = 0; i < np; i++) {
     a1 = &(acoef[i][np]);
     a2 = &(acoef[i+np][0]);
@@ -378,25 +394,25 @@ static void InitUserData(UserData webdata)
       *a3++ = ZERO;
       *a4++ = ZERO;
     }
-    
+
     /* Reset the diagonal elements of acoef to -AA. */
     acoef[i][i] = -AA; acoef[i+np][i+np] = -AA;
-    
+
     /* Set coefficients for b and diffusion terms. */
     bcoef[i] = BB; bcoef[i+np] = -BB;
     cox[i] = DPREY/dx2; cox[i+np] = DPRED/dx2;
     coy[i] = DPREY/dy2; coy[i+np] = DPRED/dy2;
   }
-  
+
 }
 
-/* 
+/*
  * SetInitialProfiles: Set initial conditions in cc, cp, and id.
  * A polynomial profile is used for the prey cc values, and a constant
  * (1.0e5) is loaded as the initial guess for the predator cc values.
  * The id values are set to 1 for the prey and 0 for the predators.
  * The prey cp values are set according to the given system, and
- * the predator cp values are set to zero.                               
+ * the predator cp values are set to zero.
  */
 
 static void SetInitialProfiles(N_Vector cc, N_Vector cp, N_Vector id,
@@ -405,12 +421,12 @@ static void SetInitialProfiles(N_Vector cc, N_Vector cp, N_Vector id,
   sunindextype loc, yloc, is, jx, jy, np;
   realtype xx, yy, xyfactor;
   realtype *ccv, *cpv, *idv;
-  
+
   ccv = N_VGetArrayPointer(cc);
   cpv = N_VGetArrayPointer(cp);
   idv = N_VGetArrayPointer(id);
   np = webdata->np;
-  
+
   /* Loop over grid, load cc values and id values. */
   for (jy = 0; jy < MY; jy++) {
     yy = jy * webdata->dy;
@@ -433,10 +449,10 @@ static void SetInitialProfiles(N_Vector cc, N_Vector cp, N_Vector id,
       }
     }
   }
-  
+
   /* Set c' for the prey by calling the function Fweb. */
   Fweb(ZERO, cc, cp, webdata);
-  
+
   /* Set c' for predators to 0. */
   for (jy = 0; jy < MY; jy++) {
     yloc = NSMX * jy;
@@ -459,9 +475,9 @@ static void PrintHeader(sunindextype mu, sunindextype ml, realtype rtol, realtyp
   printf("Number of species ns: %d", NUM_SPECIES);
   printf("     Mesh dimensions: %d x %d", MX, MY);
   printf("     System size: %d\n", NEQ);
-#if defined(SUNDIALS_EXTENDED_PRECISION) 
+#if defined(SUNDIALS_EXTENDED_PRECISION)
   printf("Tolerance parameters:  rtol = %Lg   atol = %Lg\n", rtol, atol);
-#elif defined(SUNDIALS_DOUBLE_PRECISION) 
+#elif defined(SUNDIALS_DOUBLE_PRECISION)
   printf("Tolerance parameters:  rtol = %g   atol = %g\n", rtol, atol);
 #else
   printf("Tolerance parameters:  rtol = %g   atol = %g\n", rtol, atol);
@@ -472,43 +488,43 @@ static void PrintHeader(sunindextype mu, sunindextype ml, realtype rtol, realtyp
   printf("  t        bottom-left  top-right");
   printf("    | nst  k      h\n");
   printf("-----------------------------------------------------------\n\n");
-  
+
 }
 
-/* 
+/*
  * PrintOutput: Print output values at output time t = tt.
  * Selected run statistics are printed.  Then values of the concentrations
- * are printed for the bottom left and top right grid points only.  
+ * are printed for the bottom left and top right grid points only.
  */
 
 static void PrintOutput(void *mem, N_Vector c, realtype t)
 {
-  int i, kused, flag;
+  int i, kused, retval;
   long int nst;
   realtype *c_bl, *c_tr, hused;
 
-  flag = IDAGetLastOrder(mem, &kused);
-  check_flag(&flag, "IDAGetLastOrder", 1);
-  flag = IDAGetNumSteps(mem, &nst);
-  check_flag(&flag, "IDAGetNumSteps", 1);
-  flag = IDAGetLastStep(mem, &hused);
-  check_flag(&flag, "IDAGetLastStep", 1);
-  
+  retval = IDAGetLastOrder(mem, &kused);
+  check_retval(&retval, "IDAGetLastOrder", 1);
+  retval = IDAGetNumSteps(mem, &nst);
+  check_retval(&retval, "IDAGetNumSteps", 1);
+  retval = IDAGetLastStep(mem, &hused);
+  check_retval(&retval, "IDAGetLastStep", 1);
+
   c_bl = IJ_Vptr(c,0,0);
   c_tr = IJ_Vptr(c,MX-1,MY-1);
 
-#if defined(SUNDIALS_EXTENDED_PRECISION) 
-  printf("%8.2Le %12.4Le %12.4Le   | %3ld  %1d %12.4Le\n", 
+#if defined(SUNDIALS_EXTENDED_PRECISION)
+  printf("%8.2Le %12.4Le %12.4Le   | %3ld  %1d %12.4Le\n",
          t, c_bl[0], c_tr[0], nst, kused, hused);
   for (i=1;i<NUM_SPECIES;i++)
     printf("         %12.4Le %12.4Le   |\n",c_bl[i],c_tr[i]);
-#elif defined(SUNDIALS_DOUBLE_PRECISION) 
-  printf("%8.2e %12.4e %12.4e   | %3ld  %1d %12.4e\n", 
+#elif defined(SUNDIALS_DOUBLE_PRECISION)
+  printf("%8.2e %12.4e %12.4e   | %3ld  %1d %12.4e\n",
          t, c_bl[0], c_tr[0], nst, kused, hused);
   for (i=1;i<NUM_SPECIES;i++)
     printf("         %12.4e %12.4e   |\n",c_bl[i],c_tr[i]);
 #else
-  printf("%8.2e %12.4e %12.4e   | %3ld  %1d %12.4e\n", 
+  printf("%8.2e %12.4e %12.4e   | %3ld  %1d %12.4e\n",
          t, c_bl[0], c_tr[0], nst, kused, hused);
   for (i=1;i<NUM_SPECIES;i++)
     printf("         %12.4e %12.4e   |\n",c_bl[i],c_tr[i]);
@@ -517,29 +533,31 @@ static void PrintOutput(void *mem, N_Vector c, realtype t)
   printf("\n");
 }
 
-/* 
- * PrintFinalStats: Print final run data contained in iopt.              
+/*
+ * PrintFinalStats: Print final run data contained in iopt.
  */
 
 static void PrintFinalStats(void *mem)
-{ 
-  long int nst, nre, nreLS, nni, nje, netf, ncfn;
-  int flag;
+{
+  long int nst, nre, nreLS, nni, nnf, nje, netf, ncfn;
+  int retval;
 
-  flag = IDAGetNumSteps(mem, &nst);
-  check_flag(&flag, "IDAGetNumSteps", 1);
-  flag = IDAGetNumNonlinSolvIters(mem, &nni);
-  check_flag(&flag, "IDAGetNumNonlinSolvIters", 1);
-  flag = IDAGetNumResEvals(mem, &nre);
-  check_flag(&flag, "IDAGetNumResEvals", 1);
-  flag = IDAGetNumErrTestFails(mem, &netf);
-  check_flag(&flag, "IDAGetNumErrTestFails", 1);
-  flag = IDAGetNumNonlinSolvConvFails(mem, &ncfn);
-  check_flag(&flag, "IDAGetNumNonlinSolvConvFails", 1);
-  flag = IDADlsGetNumJacEvals(mem, &nje);
-  check_flag(&flag, "IDADlsGetNumJacEvals", 1);
-  flag = IDADlsGetNumResEvals(mem, &nreLS);
-  check_flag(&flag, "IDADlsGetNumResEvals", 1);
+  retval = IDAGetNumSteps(mem, &nst);
+  check_retval(&retval, "IDAGetNumSteps", 1);
+  retval = IDAGetNumNonlinSolvIters(mem, &nni);
+  check_retval(&retval, "IDAGetNumNonlinSolvIters", 1);
+  retval = IDAGetNumResEvals(mem, &nre);
+  check_retval(&retval, "IDAGetNumResEvals", 1);
+  retval = IDAGetNumErrTestFails(mem, &netf);
+  check_retval(&retval, "IDAGetNumErrTestFails", 1);
+  retval = IDAGetNumNonlinSolvConvFails(mem, &nnf);
+  check_retval(&retval, "IDAGetNumNonlinSolvConvFails", 1);
+  retval = IDAGetNumStepSolveFails(mem, &ncfn);
+  check_retval(&retval, "IDAGetNumStepSolveFails", 1);
+  retval = IDAGetNumJacEvals(mem, &nje);
+  check_retval(&retval, "IDAGetNumJacEvals", 1);
+  retval = IDAGetNumLinResEvals(mem, &nreLS);
+  check_retval(&retval, "IDAGetNumLinResEvals", 1);
 
   printf("-----------------------------------------------------------\n");
   printf("Final run statistics: \n\n");
@@ -548,31 +566,32 @@ static void PrintFinalStats(void *mem)
   printf("Number of Jacobian evaluations     = %ld\n", nje);
   printf("Number of nonlinear iterations     = %ld\n", nni);
   printf("Number of error test failures      = %ld\n", netf);
-  printf("Number of nonlinear conv. failures = %ld\n", ncfn);
+  printf("Number of nonlinear conv. failures = %ld\n", nnf);
+  printf("Number of step solver failures     = %ld\n", ncfn);
 
 }
 
-/* 
- * Fweb: Rate function for the food-web problem.                        
- * This routine computes the right-hand sides of the system equations,   
- * consisting of the diffusion term and interaction term.                
- * The interaction term is computed by the function WebRates.            
+/*
+ * Fweb: Rate function for the food-web problem.
+ * This routine computes the right-hand sides of the system equations,
+ * consisting of the diffusion term and interaction term.
+ * The interaction term is computed by the function WebRates.
  */
 
-static void Fweb(realtype tcalc, N_Vector cc, N_Vector crate,  
+static void Fweb(realtype tcalc, N_Vector cc, N_Vector crate,
                  UserData webdata)
-{ 
+{
   sunindextype jx, jy, is, idyu, idyl, idxu, idxl;
   realtype xx, yy, *cxy, *ratesxy, *cratexy, dcyli, dcyui, dcxli, dcxui;
-  
+
   /* Loop over grid points, evaluate interaction vector (length ns),
      form diffusion difference terms, and load crate.                    */
-  
+
   for (jy = 0; jy < MY; jy++) {
     yy = (webdata->dy) * jy ;
     idyu = (jy!=MY-1) ? NSMX : -NSMX;
     idyl = (jy!= 0  ) ? NSMX : -NSMX;
-    
+
     for (jx = 0; jx < MX; jx++) {
       xx = (webdata->dx) * jx;
       idxu = (jx!= MX-1) ?  NUM_SPECIES : -NUM_SPECIES;
@@ -580,32 +599,32 @@ static void Fweb(realtype tcalc, N_Vector cc, N_Vector crate,
       cxy = IJ_Vptr(cc,jx,jy);
       ratesxy = IJ_Vptr(webdata->rates,jx,jy);
       cratexy = IJ_Vptr(crate,jx,jy);
-      
+
       /* Get interaction vector at this grid point. */
       WebRates(xx, yy, cxy, ratesxy, webdata);
-      
+
       /* Loop over species, do differencing, load crate segment. */
       for (is = 0; is < NUM_SPECIES; is++) {
-        
+
         /* Differencing in y. */
         dcyli = *(cxy+is) - *(cxy - idyl + is) ;
         dcyui = *(cxy + idyu + is) - *(cxy+is);
-        
+
         /* Differencing in x. */
         dcxli = *(cxy+is) - *(cxy - idxl + is);
         dcxui = *(cxy + idxu +is) - *(cxy+is);
-        
+
         /* Compute the crate values at (xx,yy). */
         cratexy[is] = coy[is] * (dcyui - dcyli) +
           cox[is] * (dcxui - dcxli) + ratesxy[is];
-        
+
       } /* End is loop */
     } /* End of jx loop */
   } /* End of jy loop */
-  
+
 }
 
-/* 
+/*
  * WebRates: Evaluate reaction rates at a given spatial point.
  * At a given (x,y), evaluate the array of ns reaction terms R.
  */
@@ -615,67 +634,67 @@ static void WebRates(realtype xx, realtype yy, realtype *cxy, realtype *ratesxy,
 {
   int is;
   realtype fac;
-  
+
   for (is = 0; is < NUM_SPECIES; is++)
     ratesxy[is] = dotprod(NUM_SPECIES, cxy, acoef[is]);
-  
+
   fac = ONE + ALPHA*xx*yy + BETA*sin(FOURPI*xx)*sin(FOURPI*yy);
-  
-  for (is = 0; is < NUM_SPECIES; is++)  
+
+  for (is = 0; is < NUM_SPECIES; is++)
     ratesxy[is] = cxy[is]*( bcoef[is]*fac + ratesxy[is] );
-  
+
 }
 
 /*
- * dotprod: dot product routine for realtype arrays, for use by WebRates.    
+ * dotprod: dot product routine for realtype arrays, for use by WebRates.
  */
 
 static realtype dotprod(sunindextype size, realtype *x1, realtype *x2)
 {
   sunindextype i;
   realtype *xx1, *xx2, temp = ZERO;
-  
+
   xx1 = x1; xx2 = x2;
   for (i = 0; i < size; i++) temp += (*xx1++) * (*xx2++);
   return(temp);
-  
+
 }
 
 /*
  * Check function return value...
  *   opt == 0 means SUNDIALS function allocates memory so check if
  *            returned NULL pointer
- *   opt == 1 means SUNDIALS function returns a flag so check if
- *            flag >= 0
+ *   opt == 1 means SUNDIALS function returns an integer value so check if
+ *            retval < 0
  *   opt == 2 means function allocates memory so check if returned
- *            NULL pointer 
+ *            NULL pointer
  */
 
-static int check_flag(void *flagvalue, const char *funcname, int opt)
+static int check_retval(void *returnvalue, const char *funcname, int opt)
 {
-  int *errflag;
+  int *retval;
 
-  if (opt == 0 && flagvalue == NULL) {
+  if (opt == 0 && returnvalue == NULL) {
     /* Check if SUNDIALS function returned NULL pointer - no memory allocated */
-    fprintf(stderr, 
-            "\nSUNDIALS_ERROR: %s() failed - returned NULL pointer\n\n", 
+    fprintf(stderr,
+            "\nSUNDIALS_ERROR: %s() failed - returned NULL pointer\n\n",
             funcname);
     return(1);
   } else if (opt == 1) {
-    /* Check if flag < 0 */
-    errflag = (int *) flagvalue;
-    if (*errflag < 0) {
-      fprintf(stderr, 
-              "\nSUNDIALS_ERROR: %s() failed with flag = %d\n\n", 
-              funcname, *errflag);
-      return(1); 
+    /* Check if retval < 0 */
+    retval = (int *) returnvalue;
+    if (*retval < 0) {
+      fprintf(stderr,
+              "\nSUNDIALS_ERROR: %s() failed with retval = %d\n\n",
+              funcname, *retval);
+      return(1);
     }
-  } else if (opt == 2 && flagvalue == NULL) {
+  } else if (opt == 2 && returnvalue == NULL) {
     /* Check if function returned NULL pointer - no memory allocated */
-    fprintf(stderr, 
-            "\nMEMORY_ERROR: %s() failed - returned NULL pointer\n\n", 
+    fprintf(stderr,
+            "\nMEMORY_ERROR: %s() failed - returned NULL pointer\n\n",
             funcname);
-    return(1); 
+    return(1);
   }
 
   return(0);
