@@ -1,8 +1,8 @@
-/* ----------------------------------------------------------------- 
+/* -----------------------------------------------------------------
  * Programmer(s): Radu Serban and Cosmin Petra @ LLNL
  * -----------------------------------------------------------------
  * SUNDIALS Copyright Start
- * Copyright (c) 2002-2021, Lawrence Livermore National Security
+ * Copyright (c) 2002-2022, Lawrence Livermore National Security
  * and Southern Methodist University.
  * All rights reserved.
  *
@@ -12,22 +12,22 @@
  * SUNDIALS Copyright End
  * -----------------------------------------------------------------
  *
- * Hessian using adjoint sensitivity example problem. 
- * 
- * This simple example problem for IDAS, due to Robertson, 
- * is from chemical kinetics, and consists of the following three 
+ * Hessian using adjoint sensitivity example problem.
+ *
+ * This simple example problem for IDAS, due to Robertson,
+ * is from chemical kinetics, and consists of the following three
  * equations:
  *
- *   [ y1' + p1 * y1 - p2 * y2 * y3              = 0 
- *   [ y2' - p1 * y1 + p2 * y2 * y3 + p3 * y2^2  = 0 
- *   [ y1 + y2 + y3 -1                               = 0 
- * 
+ *   y1' + p1 * y1 - p2 * y2 * y3             = 0
+ *   y2' - p1 * y1 + p2 * y2 * y3 + p3 * y2^2 = 0
+ *   y1 + y2 + y3 - 1                         = 0
+ *
  *        [1]        [-p1]
- *   y(0)=[0]  y'(0)=[ p1]   p1 = 0.04   p2 = 1e4   p3 = 1e07   
+ *   y(0)=[0]  y'(0)=[ p1]   p1 = 0.04   p2 = 1e4   p3 = 1e07
  *        [0]        [ 0 ]
  *
  *       80
- *      / 
+ *      /
  *  G = | 0.5 * (y1^2 + y2^2 + y3^2) dt
  *      /
  *      0
@@ -79,22 +79,22 @@
 #define TWO  RCONST(2.0)
 
 /* User defined struct */
-typedef struct { 
+typedef struct {
   realtype p[3];
 } *UserData;
 
 /* residual for forward problem */
-static int res(realtype t, N_Vector yy, N_Vector yp, 
+static int res(realtype t, N_Vector yy, N_Vector yp,
                N_Vector resval, void *user_data);
 
-static int resS(int Ns, realtype t, 
+static int resS(int Ns, realtype t,
                 N_Vector yy, N_Vector yp, N_Vector resval,
                 N_Vector *yyS, N_Vector *ypS, N_Vector *resvalS,
                 void *user_data, N_Vector tmp1, N_Vector tmp2, N_Vector tmp3);
 
 static int rhsQ(realtype t, N_Vector yy, N_Vector yp, N_Vector qdot, void *user_data);
 
-static int rhsQS(int Ns, realtype t, N_Vector yy, N_Vector yp, 
+static int rhsQS(int Ns, realtype t, N_Vector yy, N_Vector yp,
                  N_Vector *yyS, N_Vector *ypS, N_Vector rrQ, N_Vector *rhsvalQS,
                  void *user_data,  N_Vector yytmp, N_Vector yptmp, N_Vector tmpQS);
 
@@ -119,7 +119,7 @@ static int check_retval(void *returnvalue, const char *funcname, int opt);
 
 int main(int argc, char *argv[])
 {
-  N_Vector yy, yp, q, *yyS, *ypS, *qS; 
+  N_Vector yy, yp, q, *yyS, *ypS, *qS;
   N_Vector yyB1, ypB1, qB1, yyB2, ypB2, qB2;
   void *ida_mem;
   UserData data;
@@ -130,6 +130,10 @@ int main(int argc, char *argv[])
   SUNMatrix A, AB1, AB2;
   SUNLinearSolver LS, LSB1, LSB2;
 
+  /* Create the SUNDIALS context object for this simulation. */
+  SUNContext ctx = NULL;
+  SUNContext_Create(NULL, &ctx);
+
   /* Print problem description */
   printf("\nAdjoint Sensitivity Example for Chemical Kinetics\n");
   printf("---------------------------------------------------------\n");
@@ -145,12 +149,12 @@ int main(int argc, char *argv[])
   data->p[0] = P1; data->p[1] = P2; data->p[2] = P3;
 
   /* Consistent IC */
-  yy = N_VNew_Serial(NEQ);
-  yp = N_VNew_Serial(NEQ);
+  yy = N_VNew_Serial(NEQ, ctx);
+  yp = N_VClone(yy);
   Ith(yy,1) = ONE; Ith(yy,2) = ZERO; Ith(yy,3) = ZERO;
   Ith(yp,1) = -P1; Ith(yp,2) = P1; Ith(yp,3) = 0;
 
-  q = N_VNew_Serial(1);
+  q = N_VNew_Serial(1, ctx);
   N_VConst(ZERO, q);
 
   yyS = N_VCloneVectorArray(NP, yy);
@@ -161,7 +165,7 @@ int main(int argc, char *argv[])
   qS = N_VCloneVectorArray(NP, q);
   for(is=0; is<NP; is++) N_VConst(ZERO, qS[is]);
 
-  ida_mem = IDACreate();
+  ida_mem = IDACreate(ctx);
 
   ti = T0;
   retval = IDAInit(ida_mem, res, ti, yy, yp);
@@ -170,11 +174,11 @@ int main(int argc, char *argv[])
   retval = IDASStolerances(ida_mem, RTOL, ATOL);
 
   /* Create dense SUNMatrix for use in linear solves */
-  A = SUNDenseMatrix(NEQ, NEQ);
+  A = SUNDenseMatrix(NEQ, NEQ, ctx);
   if(check_retval((void *)A, "SUNDenseMatrix", 0)) return(1);
 
   /* Create dense SUNLinearSolver object */
-  LS = SUNLinSol_Dense(yy, A);
+  LS = SUNLinSol_Dense(yy, A, ctx);
   if(check_retval((void *)LS, "SUNLinSol_Dense", 0)) return(1);
 
   /* Attach the matrix and linear solver */
@@ -197,8 +201,8 @@ int main(int argc, char *argv[])
   /* Setup of quadrature's sensitivities */
   retval = IDAQuadSensInit(ida_mem, rhsQS, qS);
   retval = IDAQuadSensEEtolerances(ida_mem);
-  retval = IDASetQuadSensErrCon(ida_mem, SUNTRUE); 
-  
+  retval = IDASetQuadSensErrCon(ida_mem, SUNTRUE);
+
   /* Initialize ASA. */
   retval = IDAAdjInit(ida_mem, 100, IDA_HERMITE);
 
@@ -215,7 +219,7 @@ int main(int argc, char *argv[])
   printf("     G:    %12.4Le\n", G);
 #else
   printf("     G:    %12.4e\n", G);
-#endif  
+#endif
 
   /* Sensitivities are needed for IC of backward problems. */
   IDAGetSensDky(ida_mem, tf, 0, yyS);
@@ -226,15 +230,15 @@ int main(int argc, char *argv[])
   printf("   dG/dp:  %12.4Le %12.4Le\n", Ith(qS[0],1), Ith(qS[1],1));
 #else
   printf("   dG/dp:  %12.4e %12.4e\n", Ith(qS[0],1), Ith(qS[1],1));
-#endif  
+#endif
   printf("\n");
   /******************************
   * BACKWARD PROBLEM #1
   *******************************/
 
   /* Consistent IC. */
-  yyB1 = N_VNew_Serial(2*NEQ);
-  ypB1 = N_VNew_Serial(2*NEQ);
+  yyB1 = N_VNew_Serial(2*NEQ, ctx);
+  ypB1 = N_VClone(yyB1);
 
   N_VConst(ZERO, yyB1);
   Ith(yyB1,3) = Ith(yy,3);
@@ -245,22 +249,22 @@ int main(int argc, char *argv[])
   Ith(ypB1,2) = Ith(yy,3)-Ith(yy,2);
   Ith(ypB1,4) = Ith(yyS[0],3) - Ith(yyS[0],1);
   Ith(ypB1,5) = Ith(yyS[0],3) - Ith(yyS[0],2);
-  
-  qB1 = N_VNew_Serial(2*NP);
+
+  qB1 = N_VNew_Serial(2*NP, ctx);
   N_VConst(ZERO, qB1);
 
   retval = IDACreateB(ida_mem, &indexB1);
   retval = IDAInitBS(ida_mem, indexB1, resBS1, tf, yyB1, ypB1);
-  retval = IDASStolerancesB(ida_mem, indexB1, RTOLA, ATOLA);   
+  retval = IDASStolerancesB(ida_mem, indexB1, RTOLA, ATOLA);
   retval = IDASetUserDataB(ida_mem, indexB1, data);
   retval = IDASetMaxNumStepsB(ida_mem, indexB1, 5000);
 
   /* Create dense SUNMatrix for use in linear solves */
-  AB1 = SUNDenseMatrix(2*NEQ, 2*NEQ);
+  AB1 = SUNDenseMatrix(2*NEQ, 2*NEQ, ctx);
   if(check_retval((void *)AB1, "SUNDenseMatrix", 0)) return(1);
 
   /* Create dense SUNLinearSolver object */
-  LSB1 = SUNLinSol_Dense(yyB1, AB1);
+  LSB1 = SUNLinSol_Dense(yyB1, AB1, ctx);
   if(check_retval((void *)LSB1, "SUNLinSol_Dense", 0)) return(1);
 
   /* Attach the matrix and linear solver */
@@ -270,12 +274,12 @@ int main(int argc, char *argv[])
   retval = IDAQuadInitBS(ida_mem, indexB1, rhsQBS1, qB1);
 
   /******************************
-  * BACKWARD PROBLEM #2  
+  * BACKWARD PROBLEM #2
   *******************************/
 
   /* Consistent IC. */
-  yyB2 = N_VNew_Serial(2*NEQ);
-  ypB2 = N_VNew_Serial(2*NEQ);
+  yyB2 = N_VNew_Serial(2*NEQ, ctx);
+  ypB2 = N_VNew_Serial(2*NEQ, ctx);
 
   N_VConst(ZERO, yyB2);
   Ith(yyB2,3) = Ith(yy,3);
@@ -286,22 +290,22 @@ int main(int argc, char *argv[])
   Ith(ypB2,2) = Ith(yy,3)-Ith(yy,2);
   Ith(ypB2,4) = Ith(yyS[1],3) - Ith(yyS[1],1);
   Ith(ypB2,5) = Ith(yyS[1],3) - Ith(yyS[1],2);
-  
-  qB2 = N_VNew_Serial(2*NP);
+
+  qB2 = N_VNew_Serial(2*NP, ctx);
   N_VConst(ZERO, qB2);
 
   retval = IDACreateB(ida_mem, &indexB2);
   retval = IDAInitBS(ida_mem, indexB2, resBS2, tf, yyB2, ypB2);
-  retval = IDASStolerancesB(ida_mem, indexB2, RTOLA, ATOLA);   
+  retval = IDASStolerancesB(ida_mem, indexB2, RTOLA, ATOLA);
   retval = IDASetUserDataB(ida_mem, indexB2, data);
   retval = IDASetMaxNumStepsB(ida_mem, indexB2, 2500);
 
   /* Create dense SUNMatrix for use in linear solves */
-  AB2 = SUNDenseMatrix(2*NEQ, 2*NEQ);
+  AB2 = SUNDenseMatrix(2*NEQ, 2*NEQ, ctx);
   if(check_retval((void *)AB2, "SUNDenseMatrix", 0)) return(1);
 
   /* Create dense SUNLinearSolver object */
-  LSB2 = SUNLinSol_Dense(yyB2, AB2);
+  LSB2 = SUNLinSol_Dense(yyB2, AB2, ctx);
   if(check_retval((void *)LSB2, "SUNLinSol_Dense", 0)) return(1);
 
   /* Attach the matrix and linear solver */
@@ -313,16 +317,16 @@ int main(int argc, char *argv[])
   /* Integrate backward problems. */
   printf("---------------------------------------------------------\n");
   printf("Backward integration \n");
-  printf("---------------------------------------------------------\n\n"); 
+  printf("---------------------------------------------------------\n\n");
 
   retval = IDASolveB(ida_mem, ti, IDA_NORMAL);
 
   retval = IDAGetB(ida_mem, indexB1, &time, yyB1, ypB1);
-  /* 
+  /*
      retval = IDAGetNumSteps(IDAGetAdjIDABmem(ida_mem, indexB1), &nst);
-     printf("at time=%g \tpb 1 Num steps:%d\n", time, nst); 
+     printf("at time=%g \tpb 1 Num steps:%d\n", time, nst);
      retval = IDAGetNumSteps(IDAGetAdjIDABmem(ida_mem, indexB2), &nst);
-     printf("at time=%g \tpb 2 Num steps:%d\n\n", time, nst); 
+     printf("at time=%g \tpb 2 Num steps:%d\n\n", time, nst);
   */
 
   retval = IDAGetQuadB(ida_mem, indexB1, &time, qB1);
@@ -333,7 +337,7 @@ int main(int argc, char *argv[])
 #else
   printf("   dG/dp:  %12.4e %12.4e   (from backward pb. 1)\n", Ith(qB1,1), Ith(qB1,2));
   printf("   dG/dp:  %12.4e %12.4e   (from backward pb. 2)\n", Ith(qB2,1), Ith(qB2,2));
-#endif  
+#endif
 
   printf("\n");
   printf("   H = d2G/dp2:\n");
@@ -344,7 +348,7 @@ int main(int argc, char *argv[])
 #else
   printf("  %12.4e  %12.4e\n", Ith(qB1,3), Ith(qB2,3));
   printf("  %12.4e  %12.4e\n", Ith(qB1,4), Ith(qB2,4));
-#endif  
+#endif
 
   IDAFree(&ida_mem);
   SUNLinSolFree(LS);
@@ -369,10 +373,10 @@ int main(int argc, char *argv[])
   printf("Finite Differences ( dp1=%6.1Le and dp2 = %6.1Le )\n", dp1, dp2);
 #else
   printf("Finite Differences ( dp1=%6.1e and dp2 = %6.1e )\n", dp1, dp2);
-#endif  
+#endif
   printf("---------------------------------------------------------\n\n");
 
-  ida_mem = IDACreate();
+  ida_mem = IDACreate(ctx);
 
   /********************
   * Forward FD for p1
@@ -393,11 +397,11 @@ int main(int argc, char *argv[])
   retval = IDASStolerances(ida_mem, rtolFD, atolFD);
 
   /* Create dense SUNMatrix for use in linear solves */
-  A = SUNDenseMatrix(NEQ, NEQ);
+  A = SUNDenseMatrix(NEQ, NEQ, ctx);
   if(check_retval((void *)A, "SUNDenseMatrix", 0)) return(1);
 
   /* Create dense SUNLinearSolver object */
-  LS = SUNLinSol_Dense(yy, A);
+  LS = SUNLinSol_Dense(yy, A, ctx);
   if(check_retval((void *)LS, "SUNLinSol_Dense", 0)) return(1);
 
   /* Attach the matrix and linear solver */
@@ -423,7 +427,7 @@ int main(int argc, char *argv[])
   Ith(yy,1) = ONE; Ith(yy,2) = ZERO; Ith(yy,3) = ZERO;
   Ith(yp,1) = -data->p[0]; Ith(yp,2) = -Ith(yp,1); Ith(yp,3) = 0;
   N_VConst(ZERO, q);
-  
+
   retval = IDAReInit(ida_mem, ti, yy, yp);
   retval = IDAQuadReInit(ida_mem, q);
 
@@ -441,7 +445,7 @@ int main(int argc, char *argv[])
   * Forward FD for p2
   ********************/
   /*restore p1*/
-  data->p[0] += dp1; 
+  data->p[0] += dp1;
   data->p[1] += dp2;
 
   Ith(yy,1) = ONE; Ith(yy,2) = ZERO; Ith(yy,3) = ZERO;
@@ -463,7 +467,7 @@ int main(int argc, char *argv[])
   Ith(yy,1) = ONE; Ith(yy,2) = ZERO; Ith(yy,3) = ZERO;
   Ith(yp,1) = -data->p[0]; Ith(yp,2) = -Ith(yp,1); Ith(yp,3) = 0;
   N_VConst(ZERO, q);
-  
+
   retval = IDAReInit(ida_mem, ti, yy, yp);
   retval = IDAQuadReInit(ida_mem, q);
 
@@ -493,7 +497,7 @@ int main(int argc, char *argv[])
   printf("\n");
   printf("  H(1,1):  %12.4e\n", H11);
   printf("  H(2,2):  %12.4e\n", H22);
-#endif  
+#endif
 
   IDAFree(&ida_mem);
   SUNLinSolFree(LS);
@@ -515,6 +519,8 @@ int main(int argc, char *argv[])
   N_VDestroyVectorArray(qS, NP);
 
   free(data);
+
+  SUNContext_Free(&ctx);
   return 0;
 }
 
@@ -526,7 +532,7 @@ static int res(realtype tres, N_Vector yy, N_Vector yp, N_Vector rr, void *user_
   UserData data;
   realtype p1, p2, p3;
 
-  y1  = Ith(yy,1); y2  = Ith(yy,2); y3  = Ith(yy,3); 
+  y1  = Ith(yy,1); y2  = Ith(yy,2); y3  = Ith(yy,3);
   yp1 = Ith(yp,1); yp2 = Ith(yp,2);
   rval = N_VGetArrayPointer(rr);
 
@@ -541,10 +547,10 @@ static int res(realtype tres, N_Vector yy, N_Vector yp, N_Vector rr, void *user_
   return(0);
 }
 
-static int resS(int Ns, realtype t, 
+static int resS(int Ns, realtype t,
                 N_Vector yy, N_Vector yp, N_Vector resval,
                 N_Vector *yyS, N_Vector *ypS, N_Vector *resvalS,
-                void *user_data, 
+                void *user_data,
                 N_Vector tmp1, N_Vector tmp2, N_Vector tmp3)
 {
   UserData data;
@@ -587,7 +593,7 @@ static int resS(int Ns, realtype t,
       rs2 += y2*y3;
       break;
     }
-  
+
     Ith(resvalS[is],1) = rs1;
     Ith(resvalS[is],2) = rs2;
     Ith(resvalS[is],3) = rs3;
@@ -602,15 +608,15 @@ static int rhsQ(realtype t, N_Vector yy, N_Vector yp, N_Vector qdot, void *user_
 
   realtype y1, y2, y3;
 
-  y1 = Ith(yy,1); y2 = Ith(yy,2); y3 = Ith(yy,3); 
+  y1 = Ith(yy,1); y2 = Ith(yy,2); y3 = Ith(yy,3);
   Ith(qdot,1) = HALF*(y1*y1+y2*y2+y3*y3);
 
   return(0);
 }
 
 static int rhsQS(int Ns, realtype t,
-                 N_Vector yy, N_Vector yp, 
-                 N_Vector *yyS, N_Vector *ypS, 
+                 N_Vector yy, N_Vector yp,
+                 N_Vector *yyS, N_Vector *ypS,
                  N_Vector rrQ, N_Vector *rhsvalQS,
                  void *user_data,
                  N_Vector yytmp, N_Vector yptmp, N_Vector tmpQS)
@@ -619,8 +625,8 @@ static int rhsQS(int Ns, realtype t,
   realtype y1, y2, y3;
   realtype s1, s2, s3;
 
-  y1 = Ith(yy,1); 
-  y2 = Ith(yy,2); 
+  y1 = Ith(yy,1);
+  y2 = Ith(yy,2);
   y3 = Ith(yy,3);
 
   /* 1st sensitivity RHS */
@@ -639,8 +645,8 @@ static int rhsQS(int Ns, realtype t,
 }
 
 /* Residuals for adjoint model. */
-static int resBS1(realtype tt, 
-                  N_Vector yy, N_Vector yp, 
+static int resBS1(realtype tt,
+                  N_Vector yy, N_Vector yp,
                   N_Vector *yyS, N_Vector *ypS,
                   N_Vector yyB, N_Vector ypB,
                   N_Vector rrBS, void *user_dataB)
@@ -653,7 +659,7 @@ static int resBS1(realtype tt,
   realtype lp1, lp2, mp1, mp2;
   realtype s1, s2, s3;
   realtype l21;
-  
+
   data = (UserData) user_dataB;
 
   /* The parameters. */
@@ -689,7 +695,7 @@ static int resBS1(realtype tt,
   return(0);
 }
 
-static int rhsQBS1(realtype tt, 
+static int rhsQBS1(realtype tt,
                  N_Vector yy, N_Vector yp,
                  N_Vector *yyS, N_Vector *ypS,
                  N_Vector yyB, N_Vector ypB,
@@ -702,7 +708,7 @@ static int rhsQBS1(realtype tt,
 
   /* The y vector */
   y1 = Ith(yy,1); y2 = Ith(yy,2); y3 = Ith(yy,3);
-  
+
   /* The lambda vector. */
   l1 = Ith(yyB,1); l2 = Ith(yyB,2);
 
@@ -711,21 +717,21 @@ static int rhsQBS1(realtype tt,
 
   /* The sensitivity with respect to p1 */
   s1 = Ith(yyS[0],1); s2 = Ith(yyS[0],2); s3 = Ith(yyS[0],3);
-  
+
   /* Temporary variables */
   l21 = l2-l1;
 
   Ith(rhsBQS,1) = -y1*l21;
   Ith(rhsBQS,2) = y2*y3*l21;
 
-  Ith(rhsBQS,3) = y1*(m1-m2) - s1*l21; 
+  Ith(rhsBQS,3) = y1*(m1-m2) - s1*l21;
   Ith(rhsBQS,4) = y2*y3*(m2-m1) + (y3*s2+y2*s3)*l21;
 
   return(0);
 }
 
-static int resBS2(realtype tt, 
-                  N_Vector yy, N_Vector yp, 
+static int resBS2(realtype tt,
+                  N_Vector yy, N_Vector yp,
                   N_Vector *yyS, N_Vector *ypS,
                   N_Vector yyB, N_Vector ypB,
                   N_Vector rrBS, void *user_dataB)
@@ -738,7 +744,7 @@ static int resBS2(realtype tt,
   realtype lp1, lp2, mp1, mp2;
   realtype s1, s2, s3;
   realtype l21;
-  
+
   data = (UserData) user_dataB;
 
   /* The parameters. */
@@ -775,7 +781,7 @@ static int resBS2(realtype tt,
   return(0);
 }
 
-static int rhsQBS2(realtype tt, 
+static int rhsQBS2(realtype tt,
                  N_Vector yy, N_Vector yp,
                  N_Vector *yyS, N_Vector *ypS,
                  N_Vector yyB, N_Vector ypB,
@@ -788,7 +794,7 @@ static int rhsQBS2(realtype tt,
 
   /* The y vector */
   y1 = Ith(yy,1); y2 = Ith(yy,2); y3 = Ith(yy,3);
-  
+
   /* The lambda vector. */
   l1 = Ith(yyB,1); l2 = Ith(yyB,2);
 
@@ -797,27 +803,27 @@ static int rhsQBS2(realtype tt,
 
   /* The sensitivity with respect to p2 */
   s1 = Ith(yyS[1],1); s2 = Ith(yyS[1],2); s3 = Ith(yyS[1],3);
-  
+
   /* Temporary variables */
   l21 = l2-l1;
 
   Ith(rhsBQS,1) = -y1*l21;
   Ith(rhsBQS,2) =  y2*y3*l21;
 
-  Ith(rhsBQS,3) = y1*(m1-m2) - s1*l21; 
+  Ith(rhsBQS,3) = y1*(m1-m2) - s1*l21;
   Ith(rhsBQS,4) = y2*y3*(m2-m1) + (y3*s2+y2*s3)*l21;
 
   return(0);
 }
 
-/* 
+/*
  * Check function return value.
  *    opt == 0 means SUNDIALS function allocates memory so check if
  *             returned NULL pointer
  *    opt == 1 means SUNDIALS function returns an integer value so check if
  *             retval < 0
  *    opt == 2 means function allocates memory so check if returned
- *             NULL pointer 
+ *             NULL pointer
  */
 
 static int check_retval(void *returnvalue, const char *funcname, int opt)
