@@ -1,10 +1,20 @@
 /* -----------------------------------------------------------------
  * Programmer(s): Cosmin Petra and Radu Serban @ LLNL
  * -----------------------------------------------------------------
+ * SUNDIALS Copyright Start
+ * Copyright (c) 2002-2022, Lawrence Livermore National Security
+ * and Southern Methodist University.
+ * All rights reserved.
+ *
+ * See the top-level LICENSE and NOTICE files for details.
+ *
+ * SPDX-License-Identifier: BSD-3-Clause
+ * SUNDIALS Copyright End
+ * -----------------------------------------------------------------
  * Example problem:
  *
- * This simple example problem for IDA, due to Robertson, 
- * is from chemical kinetics, and consists of the following three 
+ * This simple example problem for IDA, due to Robertson,
+ * is from chemical kinetics, and consists of the following three
  * equations:
  *
  *      dy1/dt = -p1*y1 + p2*y2*y3
@@ -20,7 +30,7 @@
  * The sensitivity right hand side is given analytically through the
  * user routine fS (of type SensRhs1Fn).
  * Any of two sensitivity methods (SIMULTANEOUS and STAGGERED can be
- * used and sensitivities may be included in the error test or not 
+ * used and sensitivities may be included in the error test or not
  *(error control set on SUNTRUE or SUNFALSE, respectively).
  *
  * Execution:
@@ -41,7 +51,6 @@
 #include <nvector/nvector_serial.h>    /* access to serial N_Vector            */
 #include <sunmatrix/sunmatrix_dense.h> /* access to dense SUNMatrix            */
 #include <sunlinsol/sunlinsol_dense.h> /* access to dense SUNLinearSolver      */
-#include <idas/idas_direct.h>          /* access to IDADls interface           */
 #include <sundials/sundials_types.h>   /* defs. of realtype, sunindextype      */
 #include <sundials/sundials_math.h>    /* defs. of SUNRabs, SUNRexp, etc.      */
 
@@ -74,10 +83,10 @@ typedef struct {
 
 static int res(realtype t, N_Vector y, N_Vector yp, N_Vector resval, void *user_data);
 
-static int resS(int Ns, realtype t, 
+static int resS(int Ns, realtype t,
                 N_Vector y, N_Vector yp, N_Vector resval,
                 N_Vector *yyS, N_Vector *ypS, N_Vector *resvalS,
-                void *user_data, 
+                void *user_data,
                 N_Vector tmp1, N_Vector tmp2, N_Vector tmp3);
 
 static int rhsQ(realtype tres, N_Vector yy, N_Vector yp,
@@ -86,19 +95,17 @@ static int rhsQ(realtype tres, N_Vector yy, N_Vector yp,
 /* Prototypes of private functions */
 
 static void ProcessArgs(int argc, char *argv[],
-                        booleantype *sensi, int *sensi_meth, 
+                        booleantype *sensi, int *sensi_meth,
                         booleantype *err_con);
 static void WrongArgs(char *name);
 
 static void PrintIC(N_Vector y, N_Vector yp);
-static void PrintSensIC(N_Vector y, N_Vector yp, N_Vector* yS, N_Vector* ypS); 
+static void PrintSensIC(N_Vector y, N_Vector yp, N_Vector* yS, N_Vector* ypS);
 
 static void PrintOutput(void *ida_mem, realtype t, N_Vector u);
 static void PrintSensOutput(N_Vector *uS);
 
-static void PrintFinalStats(void *ida_mem, booleantype sensi);
-
-static int check_flag(void *flagvalue, const char *funcname, int opt);
+static int check_retval(void *returnvalue, const char *funcname, int opt);
 /*
  *--------------------------------------------------------------------
  * MAIN PROGRAM
@@ -107,16 +114,19 @@ static int check_flag(void *flagvalue, const char *funcname, int opt);
 
 int main(int argc, char *argv[])
 {
+  SUNContext ctx;
   void *ida_mem;
   SUNMatrix A;
   SUNLinearSolver LS;
   UserData data;
   realtype reltol, t, tout;
   N_Vector y, yp, abstol, id;
-  int iout, flag;
+  int iout, retval;
+  FILE* FID;
+  char fname[256];
 
   realtype pbar[NS];
-  int is; 
+  int is;
   N_Vector *yS, *ypS;
   booleantype sensi, err_con;
   int sensi_meth;
@@ -135,70 +145,74 @@ int main(int argc, char *argv[])
   /* Process arguments */
   ProcessArgs(argc, argv, &sensi, &sensi_meth, &err_con);
 
+  /* Create the SUNDIALS context object for this simulation */
+  retval = SUNContext_Create(NULL, &ctx);
+  if (check_retval(&retval, "SUNContext_Create", 1)) return 1;
+
   /* User data structure */
   data = (UserData) malloc(sizeof *data);
-  if (check_flag((void *)data, "malloc", 2)) return(1);
+  if (check_retval((void *)data, "malloc", 2)) return(1);
   data->p[0] = RCONST(0.040);
   data->p[1] = RCONST(1.0e4);
   data->p[2] = RCONST(3.0e7);
-  data->coef = 0.5;
+  data->coef = RCONST(0.5);
 
   /* Initial conditions */
-  y = N_VNew_Serial(NEQ);
-  if (check_flag((void *)y, "N_VNew_Serial", 0)) return(1);
+  y = N_VNew_Serial(NEQ, ctx);
+  if (check_retval((void *)y, "N_VNew_Serial", 0)) return(1);
 
   Ith(y,1) = ONE;
   Ith(y,2) = ZERO;
   Ith(y,3) = ZERO;
 
-  yp = N_VNew_Serial(NEQ);
-  if(check_flag((void *)yp, "N_VNew_Serial", 0)) return(1);
+  yp = N_VClone(y);
+  if(check_retval((void *)yp, "N_VNew_Serial", 0)) return(1);
 
   /* These initial conditions are NOT consistent. See IDACalcIC below. */
   Ith(yp,1) = RCONST(0.1);
   Ith(yp,2) = ZERO;
-  Ith(yp,3) = ZERO;  
+  Ith(yp,3) = ZERO;
 
   /* Create IDAS object */
-  ida_mem = IDACreate();
-  if (check_flag((void *)ida_mem, "IDACreate", 0)) return(1);
+  ida_mem = IDACreate(ctx);
+  if (check_retval((void *)ida_mem, "IDACreate", 0)) return(1);
 
   /* Allocate space for IDAS */
-  flag = IDAInit(ida_mem, res, T0, y, yp);
-  if (check_flag(&flag, "IDAInit", 1)) return(1);
+  retval = IDAInit(ida_mem, res, T0, y, yp);
+  if (check_retval(&retval, "IDAInit", 1)) return(1);
 
   /* Specify scalar relative tol. and vector absolute tol. */
   reltol = RCONST(1.0e-6);
-  abstol = N_VNew_Serial(NEQ);
+  abstol = N_VClone(y);
   Ith(abstol,1) = RCONST(1.0e-8);
   Ith(abstol,2) = RCONST(1.0e-14);
   Ith(abstol,3) = RCONST(1.0e-6);
-  flag = IDASVtolerances(ida_mem, reltol, abstol);
-  if (check_flag(&flag, "IDASVtolerances", 1)) return(1);
+  retval = IDASVtolerances(ida_mem, reltol, abstol);
+  if (check_retval(&retval, "IDASVtolerances", 1)) return(1);
 
   /* Set ID vector */
-  id = N_VNew_Serial(NEQ);
+  id = N_VClone(y);
   Ith(id,1) = 1.0;
   Ith(id,2) = 1.0;
   Ith(id,3) = 0.0;
-  flag = IDASetId(ida_mem, id);
-  if (check_flag(&flag, "IDASetId", 1)) return(1);
+  retval = IDASetId(ida_mem, id);
+  if (check_retval(&retval, "IDASetId", 1)) return(1);
 
   /* Attach user data */
-  flag = IDASetUserData(ida_mem, data);
-  if (check_flag(&flag, "IDASetUserData", 1)) return(1);
+  retval = IDASetUserData(ida_mem, data);
+  if (check_retval(&retval, "IDASetUserData", 1)) return(1);
 
   /* Create dense SUNMatrix for use in linear solves */
-  A = SUNDenseMatrix(NEQ, NEQ);
-  if(check_flag((void *)A, "SUNDenseMatrix", 0)) return(1);
+  A = SUNDenseMatrix(NEQ, NEQ, ctx);
+  if(check_retval((void *)A, "SUNDenseMatrix", 0)) return(1);
 
   /* Create dense SUNLinearSolver object */
-  LS = SUNDenseLinearSolver(y, A);
-  if(check_flag((void *)LS, "SUNDenseLinearSolver", 0)) return(1);
+  LS = SUNLinSol_Dense(y, A, ctx);
+  if(check_retval((void *)LS, "SUNLinSol_Dense", 0)) return(1);
 
   /* Attach the matrix and linear solver */
-  flag = IDADlsSetLinearSolver(ida_mem, LS, A);
-  if(check_flag(&flag, "IDADlsSetLinearSolver", 1)) return(1);
+  retval = IDASetLinearSolver(ida_mem, LS, A);
+  if(check_retval(&retval, "IDASetLinearSolver", 1)) return(1);
 
   printf("\n3-species chemical kinetics problem\n");
 
@@ -210,38 +224,38 @@ int main(int argc, char *argv[])
     pbar[2] = data->p[2];
 
     yS = N_VCloneVectorArray(NS, y);
-    if (check_flag((void *)yS, "N_VCloneVectorArray", 0)) return(1);
+    if (check_retval((void *)yS, "N_VCloneVectorArray", 0)) return(1);
     for (is=0;is<NS;is++) N_VConst(ZERO, yS[is]);
-    
+
     ypS = N_VCloneVectorArray(NS, y);
-    if (check_flag((void *)ypS, "N_VCloneVectorArray", 0)) return(1);
+    if (check_retval((void *)ypS, "N_VCloneVectorArray", 0)) return(1);
     for (is=0;is<NS;is++) N_VConst(ZERO, ypS[is]);
 
-    /* 
-    * Only non-zero sensitivity I.C. are ypS[0]: 
+    /*
+    * Only non-zero sensitivity I.C. are ypS[0]:
     * - Ith(ypS[0],1) = -ONE;
     * - Ith(ypS[0],2) =  ONE;
     *
     * They are not set. IDACalcIC also computes consistent IC for sensitivities.
     */
-    
-    flag = IDASensInit(ida_mem, NS, sensi_meth, resS, yS, ypS);
-    if(check_flag(&flag, "IDASensInit", 1)) return(1);
 
-    flag = IDASensEEtolerances(ida_mem);
-    if(check_flag(&flag, "IDASensEEtolerances", 1)) return(1);
+    retval = IDASensInit(ida_mem, NS, sensi_meth, resS, yS, ypS);
+    if(check_retval(&retval, "IDASensInit", 1)) return(1);
 
-    flag = IDASetSensErrCon(ida_mem, err_con);
-    if (check_flag(&flag, "IDASetSensErrCon", 1)) return(1);
+    retval = IDASensEEtolerances(ida_mem);
+    if(check_retval(&retval, "IDASensEEtolerances", 1)) return(1);
 
-    flag = IDASetSensParams(ida_mem, data->p, pbar, NULL);
-    if (check_flag(&flag, "IDASetSensParams", 1)) return(1);
+    retval = IDASetSensErrCon(ida_mem, err_con);
+    if (check_retval(&retval, "IDASetSensErrCon", 1)) return(1);
+
+    retval = IDASetSensParams(ida_mem, data->p, pbar, NULL);
+    if (check_retval(&retval, "IDASetSensParams", 1)) return(1);
 
     printf("Sensitivity: YES ");
-    if(sensi_meth == IDA_SIMULTANEOUS)   
+    if(sensi_meth == IDA_SIMULTANEOUS)
       printf("( SIMULTANEOUS +");
-    else 
-      printf("( STAGGERED +");   
+    else
+      printf("( STAGGERED +");
     if(err_con) printf(" FULL ERROR CONTROL )");
     else        printf(" PARTIAL ERROR CONTROL )");
 
@@ -254,7 +268,7 @@ int main(int argc, char *argv[])
   /*----------------------------------------------------------
    *               Q U A D R A T U R E S
    * ---------------------------------------------------------*/
-  yQ = N_VNew_Serial(2);
+  yQ = N_VNew_Serial(2, ctx);
 
   Ith(yQ,1) = 0;
   Ith(yQ,2) = 0;
@@ -263,7 +277,7 @@ int main(int argc, char *argv[])
 
   if (sensi) {
     yQS = N_VCloneVectorArray(NS, yQ);
-    for (is=0;is<NS;is++) N_VConst(ZERO, yQS[is]);  
+    for (is=0;is<NS;is++) N_VConst(ZERO, yQS[is]);
 
     IDAQuadSensInit(ida_mem, NULL, yQS);
   }
@@ -271,21 +285,21 @@ int main(int argc, char *argv[])
   /* Call IDACalcIC to compute consistent initial conditions. If sensitivity is
      enabled, this function also try to find consistent IC for the sensitivities. */
 
-  flag = IDACalcIC(ida_mem, IDA_YA_YDP_INIT, T1);;
-  if (check_flag(&flag, "IDACalcIC", 1)) return(1);
+  retval = IDACalcIC(ida_mem, IDA_YA_YDP_INIT, T1);;
+  if (check_retval(&retval, "IDACalcIC", 1)) return(1);
 
-  flag = IDAGetConsistentIC(ida_mem, y, yp);
-  if (check_flag(&flag, "IDAGetConsistentIC", 1)) return(1);
+  retval = IDAGetConsistentIC(ida_mem, y, yp);
+  if (check_retval(&retval, "IDAGetConsistentIC", 1)) return(1);
 
   PrintIC(y, yp);
 
   if(sensi) {
       IDAGetSensConsistentIC(ida_mem, yS, ypS);
-      PrintSensIC(y, yp, yS, ypS); 
+      PrintSensIC(y, yp, yS, ypS);
     }
-      
+
   /* In loop over output points, call IDA, print results, test for error */
-  
+
   printf("\n\n");
   printf("===========================================");
   printf("============================\n");
@@ -295,17 +309,17 @@ int main(int argc, char *argv[])
   printf("============================\n");
 
   for (iout=1, tout=T1; iout <= NOUT; iout++, tout *= TMULT) {
-    
-    flag = IDASolve(ida_mem, tout, &t, y, yp, IDA_NORMAL);
-    if (check_flag(&flag, "IDASolve", 1)) break;
+
+    retval = IDASolve(ida_mem, tout, &t, y, yp, IDA_NORMAL);
+    if (check_retval(&retval, "IDASolve", 1)) break;
 
     PrintOutput(ida_mem, t, y);
-    
+
     if (sensi) {
-      flag = IDAGetSens(ida_mem, &t, yS);
-      if (check_flag(&flag, "IDAGetSens", 1)) break;
+      retval = IDAGetSens(ida_mem, &t, yS);
+      if (check_retval(&retval, "IDAGetSens", 1)) break;
       PrintSensOutput(yS);
-    } 
+    }
     printf("-----------------------------------------");
     printf("------------------------------\n");
 
@@ -314,7 +328,7 @@ int main(int argc, char *argv[])
   printf("\nQuadrature:\n");
   IDAGetQuad(ida_mem, &t, yQ);
 #if defined(SUNDIALS_EXTENDED_PRECISION)
-  printf("G:      %10.4Le\n", Ith(yQ,1)); 
+  printf("G:      %10.4Le\n", Ith(yQ,1));
 #else
   printf("G:      %10.4e\n", Ith(yQ,1));
 #endif
@@ -334,8 +348,27 @@ int main(int argc, char *argv[])
 #endif
   }
 
-  /* Print final statistics */
-  PrintFinalStats(ida_mem, sensi);
+  /* Print final statistics to the screen */
+  printf("\nFinal Statistics:\n");
+  retval = IDAPrintAllStats(ida_mem, stdout, SUN_OUTPUTFORMAT_TABLE);
+
+  /* Print final statistics to a file in CSV format */
+  strcpy(fname, "idasRoberts_FSA_dns_stats");
+  if (sensi)
+  {
+    if(sensi_meth == IDA_SIMULTANEOUS)
+      strcat(fname, "_-sensi_sim");
+    else
+      strcat(fname, "_-sensi_stg");
+    if(err_con)
+      strcat(fname, "_t");
+    else
+      strcat(fname, "_f");
+  }
+  strcat(fname, ".csv");
+  FID = fopen(fname, "w");
+  retval = IDAPrintAllStats(ida_mem, FID, SUN_OUTPUTFORMAT_CSV);
+  fclose(FID);
 
   /* Free memory */
   N_VDestroy(y);
@@ -352,6 +385,7 @@ int main(int argc, char *argv[])
   IDAFree(&ida_mem);
   SUNLinSolFree(LS);
   SUNMatDestroy(A);
+  SUNContext_Free(&ctx);
 
   return(0);
 }
@@ -363,7 +397,7 @@ int main(int argc, char *argv[])
  */
 
 /*
- * Residual routine. Compute F(t,y,y',p). 
+ * Residual routine. Compute F(t,y,y',p).
  */
 static int res(realtype t, N_Vector yy, N_Vector yp, N_Vector resval, void *user_data)
 {
@@ -391,15 +425,15 @@ static int res(realtype t, N_Vector yy, N_Vector yp, N_Vector resval, void *user
   return(0);
 }
 
- 
-/* 
- * resS routine. Compute sensitivity r.h.s. 
+
+/*
+ * resS routine. Compute sensitivity r.h.s.
  */
 
-static int resS(int Ns, realtype t, 
+static int resS(int Ns, realtype t,
                 N_Vector yy, N_Vector yp, N_Vector resval,
                 N_Vector *yyS, N_Vector *ypS, N_Vector *resvalS,
-                void *user_data, 
+                void *user_data,
                 N_Vector tmp1, N_Vector tmp2, N_Vector tmp3)
 {
   UserData data;
@@ -445,7 +479,7 @@ static int resS(int Ns, realtype t,
       rs2 += y2*y2;
       break;
     }
-  
+
     Ith(resvalS[is],1) = rs1;
     Ith(resvalS[is],2) = rs2;
     Ith(resvalS[is],3) = rs3;
@@ -455,7 +489,7 @@ static int resS(int Ns, realtype t,
   return(0);
 }
 
-static int rhsQ(realtype t, N_Vector y, N_Vector yp, 
+static int rhsQ(realtype t, N_Vector y, N_Vector yp,
               N_Vector ypQ, void* user_data)
 {
   UserData data;
@@ -463,11 +497,11 @@ static int rhsQ(realtype t, N_Vector y, N_Vector yp,
   data = (UserData) user_data;
 
   Ith(ypQ,1) = Ith(y,3);
-  
-  Ith(ypQ,2) = data->coef*( Ith(y,1)*Ith(y,1)+ 
-                            Ith(y,2)*Ith(y,2)+ 
+
+  Ith(ypQ,2) = data->coef*( Ith(y,1)*Ith(y,1)+
+                            Ith(y,2)*Ith(y,2)+
                             Ith(y,3)*Ith(y,3) );
-                    
+
   return(0);
 }
 
@@ -482,7 +516,7 @@ static int rhsQ(realtype t, N_Vector y, N_Vector yp,
  * Process and verify arguments to idasfwddenx.
  */
 
-static void ProcessArgs(int argc, char *argv[], 
+static void ProcessArgs(int argc, char *argv[],
                         booleantype *sensi, int *sensi_meth, booleantype *err_con)
 {
   *sensi = SUNFALSE;
@@ -497,7 +531,7 @@ static void ProcessArgs(int argc, char *argv[],
     *sensi = SUNTRUE;
   else
     WrongArgs(argv[0]);
-  
+
   if (*sensi) {
 
     if (argc != 4)
@@ -507,7 +541,7 @@ static void ProcessArgs(int argc, char *argv[],
       *sensi_meth = IDA_SIMULTANEOUS;
     else if (strcmp(argv[2],"stg") == 0)
       *sensi_meth = IDA_STAGGERED;
-    else 
+    else
       WrongArgs(argv[0]);
 
     if (strcmp(argv[3],"t") == 0)
@@ -525,7 +559,7 @@ static void WrongArgs(char *name)
     printf("\nUsage: %s [-nosensi] [-sensi sensi_meth err_con]\n",name);
     printf("         sensi_meth = sim or stg\n");
     printf("         err_con    = t or f\n");
-    
+
     exit(0);
 }
 
@@ -633,17 +667,17 @@ static void PrintSensIC(N_Vector y, N_Vector yp, N_Vector* yS, N_Vector* ypS)
 static void PrintOutput(void *ida_mem, realtype t, N_Vector u)
 {
   long int nst;
-  int qu, flag;
+  int qu, retval;
   realtype hu, *udata;
-  
+
   udata = N_VGetArrayPointer(u);
 
-  flag = IDAGetNumSteps(ida_mem, &nst);
-  check_flag(&flag, "IDAGetNumSteps", 1);
-  flag = IDAGetLastOrder(ida_mem, &qu);
-  check_flag(&flag, "IDAGetLastOrder", 1);
-  flag = IDAGetLastStep(ida_mem, &hu);
-  check_flag(&flag, "IDAGetLastStep", 1);
+  retval = IDAGetNumSteps(ida_mem, &nst);
+  check_retval(&retval, "IDAGetNumSteps", 1);
+  retval = IDAGetLastOrder(ida_mem, &qu);
+  check_retval(&retval, "IDAGetLastOrder", 1);
+  retval = IDAGetLastStep(ida_mem, &hu);
+  check_retval(&retval, "IDAGetLastStep", 1);
 
 #if defined(SUNDIALS_EXTENDED_PRECISION)
   printf("%8.3Le %2d  %8.3Le %5ld\n", t, qu, hu, nst);
@@ -665,7 +699,7 @@ static void PrintOutput(void *ida_mem, realtype t, N_Vector u)
 
 }
 
-/* 
+/*
  * Print sensitivities.
 */
 
@@ -683,7 +717,7 @@ static void PrintSensOutput(N_Vector *uS)
 #else
   printf("%12.4e %12.4e %12.4e \n", sdata[0], sdata[1], sdata[2]);
 #endif
-  
+
   sdata = N_VGetArrayPointer(uS[1]);
   printf("                  Sensitivity 2  ");
 
@@ -707,105 +741,41 @@ static void PrintSensOutput(N_Vector *uS)
 #endif
 }
 
-/* 
- * Print some final statistics from the IDAS memory.
+/*
+ * Check function return value...
+ *   opt == 0 means SUNDIALS function allocates memory so check if
+ *            returned NULL pointer
+ *   opt == 1 means SUNDIALS function returns an integer value so check if
+ *            retval < 0
+ *   opt == 2 means function allocates memory so check if returned
+ *            NULL pointer
  */
 
-static void PrintFinalStats(void *ida_mem, booleantype sensi)
+static int check_retval(void *returnvalue, const char *funcname, int opt)
 {
-  long int nst;
-  long int nfe, nsetups, nni, ncfn, netf;
-  long int nfSe, nfeS, nsetupsS, nniS, ncfnS, netfS;
-  long int nje, nfeLS;
-  int flag;
-
-  flag = IDAGetNumSteps(ida_mem, &nst);
-  check_flag(&flag, "IDAGetNumSteps", 1);
-  flag = IDAGetNumResEvals(ida_mem, &nfe);
-  check_flag(&flag, "IDAGetNumRhsEvals", 1);
-  flag = IDAGetNumLinSolvSetups(ida_mem, &nsetups);
-  check_flag(&flag, "IDAGetNumLinSolvSetups", 1);
-  flag = IDAGetNumErrTestFails(ida_mem, &netf);
-  check_flag(&flag, "IDAGetNumErrTestFails", 1);
-  flag = IDAGetNumNonlinSolvIters(ida_mem, &nni);
-  check_flag(&flag, "IDAGetNumNonlinSolvIters", 1);
-  flag = IDAGetNumNonlinSolvConvFails(ida_mem, &ncfn);
-  check_flag(&flag, "IDAGetNumNonlinSolvConvFails", 1);
-
-  if (sensi) {
-    flag = IDAGetSensNumResEvals(ida_mem, &nfSe);
-    check_flag(&flag, "IDAGetSensNumRhsEvals", 1);
-    flag = IDAGetNumResEvalsSens(ida_mem, &nfeS);
-    check_flag(&flag, "IDAGetNumResEvalsSens", 1);
-    flag = IDAGetSensNumLinSolvSetups(ida_mem, &nsetupsS);
-    check_flag(&flag, "IDAGetSensNumLinSolvSetups", 1);
-    flag = IDAGetSensNumErrTestFails(ida_mem, &netfS);
-    check_flag(&flag, "IDAGetSensNumErrTestFails", 1);
-    flag = IDAGetSensNumNonlinSolvIters(ida_mem, &nniS);
-    check_flag(&flag, "IDAGetSensNumNonlinSolvIters", 1);
-    flag = IDAGetSensNumNonlinSolvConvFails(ida_mem, &ncfnS);
-    check_flag(&flag, "IDAGetSensNumNonlinSolvConvFails", 1);
-  }
-
-  flag = IDADlsGetNumJacEvals(ida_mem, &nje);
-  check_flag(&flag, "IDADlsGetNumJacEvals", 1);
-  flag = IDADlsGetNumResEvals(ida_mem, &nfeLS);
-  check_flag(&flag, "IDADlsGetNumResEvals", 1);
-
-  printf("\nFinal Statistics\n\n");
-  printf("nst     = %5ld\n\n", nst);
-  printf("nfe     = %5ld\n",   nfe);
-  printf("netf    = %5ld    nsetups  = %5ld\n", netf, nsetups);
-  printf("nni     = %5ld    ncfn     = %5ld\n", nni, ncfn);
-
-  if(sensi) {
-    printf("\n");
-    printf("nfSe    = %5ld    nfeS     = %5ld\n", nfSe, nfeS);
-    printf("netfs   = %5ld    nsetupsS = %5ld\n", netfS, nsetupsS);
-    printf("nniS    = %5ld    ncfnS    = %5ld\n", nniS, ncfnS);
-  }
-
-  printf("\n");
-  printf("nje    = %5ld    nfeLS     = %5ld\n", nje, nfeLS);
-
-}
-
-/* 
- * Check function return value.
- *    opt == 0 means SUNDIALS function allocates memory so check if
- *             returned NULL pointer
- *    opt == 1 means SUNDIALS function returns a flag so check if
- *             flag >= 0
- *    opt == 2 means function allocates memory so check if returned
- *             NULL pointer 
- */
-
-static int check_flag(void *flagvalue, const char *funcname, int opt)
-{
-  int *errflag;
-
+  int *retval;
   /* Check if SUNDIALS function returned NULL pointer - no memory allocated */
-  if (opt == 0 && flagvalue == NULL) {
-    fprintf(stderr, 
+  if (opt == 0 && returnvalue == NULL) {
+    fprintf(stderr,
             "\nSUNDIALS_ERROR: %s() failed - returned NULL pointer\n\n",
-	    funcname);
-    return(1); }
-
-  /* Check if flag < 0 */
-  else if (opt == 1) {
-    errflag = (int *) flagvalue;
-    if (*errflag < 0) {
-      fprintf(stderr, 
-              "\nSUNDIALS_ERROR: %s() failed with flag = %d\n\n",
-	      funcname, *errflag);
-      return(1); }}
-
-  /* Check if function returned NULL pointer - no memory allocated */
-  else if (opt == 2 && flagvalue == NULL) {
-    fprintf(stderr, 
+            funcname);
+    return(1);
+  } else if (opt == 1) {
+    /* Check if retval < 0 */
+    retval = (int *) returnvalue;
+    if (*retval < 0) {
+      fprintf(stderr,
+              "\nSUNDIALS_ERROR: %s() failed with retval = %d\n\n",
+              funcname, *retval);
+      return(1);
+    }
+  } else if (opt == 2 && returnvalue == NULL) {
+    /* Check if function returned NULL pointer - no memory allocated */
+    fprintf(stderr,
             "\nMEMORY_ERROR: %s() failed - returned NULL pointer\n\n",
-	    funcname);
-    return(1); }
+            funcname);
+    return(1);
+  }
 
   return(0);
 }
