@@ -2,7 +2,7 @@
  * Programmer(s): Radu Serban @ LLNL
  * -----------------------------------------------------------------
  * SUNDIALS Copyright Start
- * Copyright (c) 2002-2021, Lawrence Livermore National Security
+ * Copyright (c) 2002-2022, Lawrence Livermore National Security
  * and Southern Methodist University.
  * All rights reserved.
  *
@@ -95,7 +95,7 @@ static int SolveIt(void *kmem, N_Vector u, N_Vector s, int glstr, int mset);
 static void PrintHeader(realtype fnormtol, realtype scsteptol);
 static void PrintOutput(N_Vector u);
 static void PrintFinalStats(void *kmem);
-static int check_flag(void *flagvalue, const char *funcname, int opt);
+static int check_retval(void *retvalvalue, const char *funcname, int opt);
 
 /*
  *--------------------------------------------------------------------
@@ -105,10 +105,11 @@ static int check_flag(void *flagvalue, const char *funcname, int opt);
 
 int main()
 {
+  SUNContext sunctx;
   UserData data;
   realtype fnormtol, scsteptol;
   N_Vector u1, u2, u, s, c;
-  int glstr, mset, flag;
+  int glstr, mset, retval;
   void *kmem;
   SUNMatrix J;
   SUNLinearSolver LS;
@@ -120,6 +121,10 @@ int main()
   LS = NULL;
   data = NULL;
 
+  /* Create the SUNDIALS context that all SUNDIALS objects require */
+  retval = SUNContext_Create(NULL, &sunctx);
+  if (check_retval(&retval, "SUNContext_Create", 1)) return(1);
+
   /* User data */
 
   data = (UserData)malloc(sizeof *data);
@@ -127,20 +132,20 @@ int main()
   data->lb[1] = ONEPT5;     data->ub[1] = TWO*PI;
 
   /* Create serial vectors of length NEQ */
-  u1 = N_VNew_Serial(NEQ);
-  if (check_flag((void *)u1, "N_VNew_Serial", 0)) return(1);
+  u1 = N_VNew_Serial(NEQ, sunctx);
+  if (check_retval((void *)u1, "N_VNew_Serial", 0)) return(1);
 
-  u2 = N_VNew_Serial(NEQ);
-  if (check_flag((void *)u2, "N_VNew_Serial", 0)) return(1);
+  u2 = N_VNew_Serial(NEQ, sunctx);
+  if (check_retval((void *)u2, "N_VNew_Serial", 0)) return(1);
 
-  u = N_VNew_Serial(NEQ);
-  if (check_flag((void *)u, "N_VNew_Serial", 0)) return(1);
+  u = N_VNew_Serial(NEQ, sunctx);
+  if (check_retval((void *)u, "N_VNew_Serial", 0)) return(1);
 
-  s = N_VNew_Serial(NEQ);
-  if (check_flag((void *)s, "N_VNew_Serial", 0)) return(1);
+  s = N_VNew_Serial(NEQ, sunctx);
+  if (check_retval((void *)s, "N_VNew_Serial", 0)) return(1);
 
-  c = N_VNew_Serial(NEQ);
-  if (check_flag((void *)c, "N_VNew_Serial", 0)) return(1);
+  c = N_VNew_Serial(NEQ, sunctx);
+  if (check_retval((void *)c, "N_VNew_Serial", 0)) return(1);
 
   SetInitialGuess1(u1,data);
   SetInitialGuess2(u2,data);
@@ -157,32 +162,32 @@ int main()
   fnormtol=FTOL; scsteptol=STOL;
 
 
-  kmem = KINCreate();
-  if (check_flag((void *)kmem, "KINCreate", 0)) return(1);
+  kmem = KINCreate(sunctx);
+  if (check_retval((void *)kmem, "KINCreate", 0)) return(1);
 
-  flag = KINSetUserData(kmem, data);
-  if (check_flag(&flag, "KINSetUserData", 1)) return(1);
-  flag = KINSetConstraints(kmem, c);
-  if (check_flag(&flag, "KINSetConstraints", 1)) return(1);
-  flag = KINSetFuncNormTol(kmem, fnormtol);
-  if (check_flag(&flag, "KINSetFuncNormTol", 1)) return(1);
-  flag = KINSetScaledStepTol(kmem, scsteptol);
-  if (check_flag(&flag, "KINSetScaledStepTol", 1)) return(1);
+  retval = KINSetUserData(kmem, data);
+  if (check_retval(&retval, "KINSetUserData", 1)) return(1);
+  retval = KINSetConstraints(kmem, c);
+  if (check_retval(&retval, "KINSetConstraints", 1)) return(1);
+  retval = KINSetFuncNormTol(kmem, fnormtol);
+  if (check_retval(&retval, "KINSetFuncNormTol", 1)) return(1);
+  retval = KINSetScaledStepTol(kmem, scsteptol);
+  if (check_retval(&retval, "KINSetScaledStepTol", 1)) return(1);
 
-  flag = KINInit(kmem, func, u);
-  if (check_flag(&flag, "KINInit", 1)) return(1);
+  retval = KINInit(kmem, func, u);
+  if (check_retval(&retval, "KINInit", 1)) return(1);
 
   /* Create dense SUNMatrix */
-  J = SUNDenseMatrix(NEQ, NEQ);
-  if(check_flag((void *)J, "SUNDenseMatrix", 0)) return(1);
+  J = SUNDenseMatrix(NEQ, NEQ, sunctx);
+  if(check_retval((void *)J, "SUNDenseMatrix", 0)) return(1);
 
   /* Create dense SUNLinearSolver object */
-  LS = SUNLinSol_Dense(u, J);
-  if(check_flag((void *)LS, "SUNLinSol_Dense", 0)) return(1);
+  LS = SUNLinSol_Dense(u, J, sunctx);
+  if(check_retval((void *)LS, "SUNLinSol_Dense", 0)) return(1);
 
   /* Attach the matrix and linear solver to KINSOL */
-  flag = KINSetLinearSolver(kmem, LS, J);
-  if(check_flag(&flag, "KINSetLinearSolver", 1)) return(1);
+  retval = KINSetLinearSolver(kmem, LS, J);
+  if(check_retval(&retval, "KINSetLinearSolver", 1)) return(1);
 
   /* Print out the problem size, solution parameters, initial guess. */
   PrintHeader(fnormtol, scsteptol);
@@ -269,6 +274,7 @@ int main()
   SUNLinSolFree(LS);
   SUNMatDestroy(J);
   free(data);
+  SUNContext_Free(&sunctx);
 
   return(0);
 }
@@ -276,7 +282,7 @@ int main()
 
 static int SolveIt(void *kmem, N_Vector u, N_Vector s, int glstr, int mset)
 {
-  int flag;
+  int retval;
 
   printf("\n");
 
@@ -290,11 +296,11 @@ static int SolveIt(void *kmem, N_Vector u, N_Vector s, int glstr, int mset)
   else
     printf(" with line search\n");
 
-  flag = KINSetMaxSetupCalls(kmem, mset);
-  if (check_flag(&flag, "KINSetMaxSetupCalls", 1)) return(1);
+  retval = KINSetMaxSetupCalls(kmem, mset);
+  if (check_retval(&retval, "KINSetMaxSetupCalls", 1)) return(1);
 
-  flag = KINSol(kmem, u, glstr, s, s);
-  if (check_flag(&flag, "KINSol", 1)) return(1);
+  retval = KINSol(kmem, u, glstr, s, s);
+  if (check_retval(&retval, "KINSol", 1)) return(1);
 
   printf("Solution:\n  [x1,x2] = ");
   PrintOutput(u);
@@ -449,17 +455,17 @@ static void PrintOutput(N_Vector u)
 static void PrintFinalStats(void *kmem)
 {
   long int nni, nfe, nje, nfeD;
-  int flag;
+  int retval;
 
-  flag = KINGetNumNonlinSolvIters(kmem, &nni);
-  check_flag(&flag, "KINGetNumNonlinSolvIters", 1);
-  flag = KINGetNumFuncEvals(kmem, &nfe);
-  check_flag(&flag, "KINGetNumFuncEvals", 1);
+  retval = KINGetNumNonlinSolvIters(kmem, &nni);
+  check_retval(&retval, "KINGetNumNonlinSolvIters", 1);
+  retval = KINGetNumFuncEvals(kmem, &nfe);
+  check_retval(&retval, "KINGetNumFuncEvals", 1);
 
-  flag = KINGetNumJacEvals(kmem, &nje);
-  check_flag(&flag, "KINGetNumJacEvals", 1);
-  flag = KINGetNumLinFuncEvals(kmem, &nfeD);
-  check_flag(&flag, "KINGetNumLinFuncEvals", 1);
+  retval = KINGetNumJacEvals(kmem, &nje);
+  check_retval(&retval, "KINGetNumJacEvals", 1);
+  retval = KINGetNumLinFuncEvals(kmem, &nfeD);
+  check_retval(&retval, "KINGetNumLinFuncEvals", 1);
 
   printf("Final Statistics:\n");
   printf("  nni = %5ld    nfe  = %5ld \n", nni, nfe);
@@ -470,37 +476,37 @@ static void PrintFinalStats(void *kmem)
  * Check function return value...
  *    opt == 0 means SUNDIALS function allocates memory so check if
  *             returned NULL pointer
- *    opt == 1 means SUNDIALS function returns a flag so check if
- *             flag >= 0
+ *    opt == 1 means SUNDIALS function returns a retval so check if
+ *             retval >= 0
  *    opt == 2 means function allocates memory so check if returned
  *             NULL pointer
  */
 
-static int check_flag(void *flagvalue, const char *funcname, int opt)
+static int check_retval(void *retvalvalue, const char *funcname, int opt)
 {
-  int *errflag;
+  int *errretval;
 
   /* Check if SUNDIALS function returned NULL pointer - no memory allocated */
-  if (opt == 0 && flagvalue == NULL) {
+  if (opt == 0 && retvalvalue == NULL) {
     fprintf(stderr,
             "\nSUNDIALS_ERROR: %s() failed - returned NULL pointer\n\n",
 	    funcname);
     return(1);
   }
 
-  /* Check if flag < 0 */
+  /* Check if retval < 0 */
   else if (opt == 1) {
-    errflag = (int *) flagvalue;
-    if (*errflag < 0) {
+    errretval = (int *) retvalvalue;
+    if (*errretval < 0) {
       fprintf(stderr,
-              "\nSUNDIALS_ERROR: %s() failed with flag = %d\n\n",
-	      funcname, *errflag);
+              "\nSUNDIALS_ERROR: %s() failed with retval = %d\n\n",
+	      funcname, *errretval);
       return(1);
     }
   }
 
   /* Check if function returned NULL pointer - no memory allocated */
-  else if (opt == 2 && flagvalue == NULL) {
+  else if (opt == 2 && retvalvalue == NULL) {
     fprintf(stderr,
             "\nMEMORY_ERROR: %s() failed - returned NULL pointer\n\n",
 	    funcname);
