@@ -2,7 +2,7 @@
  * Programmer(s): Radu Serban @ LLNL
  * -----------------------------------------------------------------
  * SUNDIALS Copyright Start
- * Copyright (c) 2002-2021, Lawrence Livermore National Security
+ * Copyright (c) 2002-2022, Lawrence Livermore National Security
  * and Southern Methodist University.
  * All rights reserved.
  *
@@ -65,7 +65,7 @@
 static int func(N_Vector u, N_Vector f, void *user_data);
 static void PrintOutput(N_Vector u);
 static void PrintFinalStats(void *kmem);
-static int check_flag(void *flagvalue, const char *funcname, int opt);
+static int check_retval(void *retvalvalue, const char *funcname, int opt);
 
 /*
  *--------------------------------------------------------------------
@@ -75,9 +75,10 @@ static int check_flag(void *flagvalue, const char *funcname, int opt);
 
 int main()
 {
+  SUNContext sunctx;
   realtype fnormtol, fnorm;
   N_Vector y, scale;
-  int mset, msubset, flag;
+  int mset, msubset, retval;
   void *kmem;
   SUNMatrix J;
   SUNLinearSolver LS;
@@ -97,27 +98,31 @@ int main()
   printf("Solution method: Modified Newton with band linear solver\n");
   printf("Problem size: %2ld x %2ld = %4ld\n", (long int) NX, (long int) NY, (long int) NEQ);
 
+  /* Create the SUNDIALS context that all SUNDIALS objects require */
+  retval = SUNContext_Create(NULL, &sunctx);
+  if (check_retval(&retval, "SUNContext_Create", 1)) return(1);
+
   /* --------------------------------------
    * Create vectors for solution and scales
    * -------------------------------------- */
 
-  y = N_VNew_Serial(NEQ);
-  if (check_flag((void *)y, "N_VNew_Serial", 0)) return(1);
+  y = N_VNew_Serial(NEQ, sunctx);
+  if (check_retval((void *)y, "N_VNew_Serial", 0)) return(1);
 
-  scale = N_VNew_Serial(NEQ);
-  if (check_flag((void *)scale, "N_VNew_Serial", 0)) return(1);
+  scale = N_VNew_Serial(NEQ, sunctx);
+  if (check_retval((void *)scale, "N_VNew_Serial", 0)) return(1);
 
   /* -----------------------------------------
    * Initialize and allocate memory for KINSOL
    * ----------------------------------------- */
 
-  kmem = KINCreate();
-  if (check_flag((void *)kmem, "KINCreate", 0)) return(1);
+  kmem = KINCreate(sunctx);
+  if (check_retval((void *)kmem, "KINCreate", 0)) return(1);
 
   /* y is used as a template */
 
-  flag = KINInit(kmem, func, y);
-  if (check_flag(&flag, "KINInit", 1)) return(1);
+  retval = KINInit(kmem, func, y);
+  if (check_retval(&retval, "KINInit", 1)) return(1);
 
   /* -------------------
    * Set optional inputs
@@ -126,29 +131,29 @@ int main()
   /* Specify stopping tolerance based on residual */
 
   fnormtol  = FTOL;
-  flag = KINSetFuncNormTol(kmem, fnormtol);
-  if (check_flag(&flag, "KINSetFuncNormTol", 1)) return(1);
+  retval = KINSetFuncNormTol(kmem, fnormtol);
+  if (check_retval(&retval, "KINSetFuncNormTol", 1)) return(1);
 
   /* -------------------------
    * Create band SUNMatrix
    * ------------------------- */
 
-  J = SUNBandMatrix(NEQ, NX, NX);
-  if(check_flag((void *)J, "SUNBandMatrix", 0)) return(1);
+  J = SUNBandMatrix(NEQ, NX, NX, sunctx);
+  if(check_retval((void *)J, "SUNBandMatrix", 0)) return(1);
 
   /* ---------------------------
    * Create band SUNLinearSolver
    * --------------------------- */
 
-  LS = SUNLinSol_Band(y, J);
-  if(check_flag((void *)LS, "SUNLinSol_Band", 0)) return(1);
+  LS = SUNLinSol_Band(y, J, sunctx);
+  if(check_retval((void *)LS, "SUNLinSol_Band", 0)) return(1);
 
   /* -------------------------
    * Attach band linear solver
    * ------------------------- */
 
-  flag = KINSetLinearSolver(kmem, LS, J);
-  if(check_flag(&flag, "KINSetLinearSolver", 1)) return(1);
+  retval = KINSetLinearSolver(kmem, LS, J);
+  if(check_retval(&retval, "KINSetLinearSolver", 1)) return(1);
 
   /* ------------------------------
    * Parameters for Modified Newton
@@ -156,14 +161,14 @@ int main()
 
   /* Force a Jacobian re-evaluation every mset iterations */
   mset = 100;
-  flag = KINSetMaxSetupCalls(kmem, mset);
-  if (check_flag(&flag, "KINSetMaxSetupCalls", 1)) return(1);
+  retval = KINSetMaxSetupCalls(kmem, mset);
+  if (check_retval(&retval, "KINSetMaxSetupCalls", 1)) return(1);
 
   /* Every msubset iterations, test if a Jacobian evaluation
      is necessary */
   msubset = 1;
-  flag = KINSetMaxSubSetupCalls(kmem, msubset);
-  if (check_flag(&flag, "KINSetMaxSubSetupCalls", 1)) return(1);
+  retval = KINSetMaxSubSetupCalls(kmem, msubset);
+  if (check_retval(&retval, "KINSetMaxSubSetupCalls", 1)) return(1);
 
   /* -------------
    * Initial guess
@@ -179,12 +184,12 @@ int main()
   N_VConst(ONE,scale);
 
   /* Call main solver */
-  flag = KINSol(kmem,           /* KINSol memory block */
+  retval = KINSol(kmem,           /* KINSol memory block */
                 y,              /* initial guess on input; solution vector */
                 KIN_LINESEARCH, /* global strategy choice */
                 scale,          /* scaling vector, for the variable cc */
                 scale);         /* scaling vector for function values fval */
-  if (check_flag(&flag, "KINSol", 1)) return(1);
+  if (check_retval(&retval, "KINSol", 1)) return(1);
 
 
   /* ------------------------------------
@@ -193,8 +198,8 @@ int main()
 
   /* Get scaled norm of the system function */
 
-  flag = KINGetFuncNorm(kmem, &fnorm);
-  if (check_flag(&flag, "KINGetfuncNorm", 1)) return(1);
+  retval = KINGetFuncNorm(kmem, &fnorm);
+  if (check_retval(&retval, "KINGetfuncNorm", 1)) return(1);
 
 #if defined(SUNDIALS_EXTENDED_PRECISION)
   printf("\nComputed solution (||F|| = %Lg):\n\n",fnorm);
@@ -214,6 +219,7 @@ int main()
   KINFree(&kmem);
   SUNLinSolFree(LS);
   SUNMatDestroy(J);
+  SUNContext_Free(&sunctx);
 
   return(0);
 }
@@ -330,38 +336,38 @@ static void PrintFinalStats(void *kmem)
   long int nni, nfe, nje, nfeD;
   long int lenrw, leniw, lenrwB, leniwB;
   long int nbcfails, nbacktr;
-  int flag;
+  int retval;
 
   /* Main solver statistics */
 
-  flag = KINGetNumNonlinSolvIters(kmem, &nni);
-  check_flag(&flag, "KINGetNumNonlinSolvIters", 1);
-  flag = KINGetNumFuncEvals(kmem, &nfe);
-  check_flag(&flag, "KINGetNumFuncEvals", 1);
+  retval = KINGetNumNonlinSolvIters(kmem, &nni);
+  check_retval(&retval, "KINGetNumNonlinSolvIters", 1);
+  retval = KINGetNumFuncEvals(kmem, &nfe);
+  check_retval(&retval, "KINGetNumFuncEvals", 1);
 
   /* Linesearch statistics */
 
-  flag = KINGetNumBetaCondFails(kmem, &nbcfails);
-  check_flag(&flag, "KINGetNumBetacondFails", 1);
-  flag = KINGetNumBacktrackOps(kmem, &nbacktr);
-  check_flag(&flag, "KINGetNumBacktrackOps", 1);
+  retval = KINGetNumBetaCondFails(kmem, &nbcfails);
+  check_retval(&retval, "KINGetNumBetacondFails", 1);
+  retval = KINGetNumBacktrackOps(kmem, &nbacktr);
+  check_retval(&retval, "KINGetNumBacktrackOps", 1);
 
   /* Main solver workspace size */
 
-  flag = KINGetWorkSpace(kmem, &lenrw, &leniw);
-  check_flag(&flag, "KINGetWorkSpace", 1);
+  retval = KINGetWorkSpace(kmem, &lenrw, &leniw);
+  check_retval(&retval, "KINGetWorkSpace", 1);
 
   /* Band linear solver statistics */
 
-  flag = KINGetNumJacEvals(kmem, &nje);
-  check_flag(&flag, "KINGetNumJacEvals", 1);
-  flag = KINGetNumLinFuncEvals(kmem, &nfeD);
-  check_flag(&flag, "KINGetNumLinFuncEvals", 1);
+  retval = KINGetNumJacEvals(kmem, &nje);
+  check_retval(&retval, "KINGetNumJacEvals", 1);
+  retval = KINGetNumLinFuncEvals(kmem, &nfeD);
+  check_retval(&retval, "KINGetNumLinFuncEvals", 1);
 
   /* Band linear solver workspace size */
 
-  flag = KINGetLinWorkSpace(kmem, &lenrwB, &leniwB);
-  check_flag(&flag, "KINGetLinWorkSpace", 1);
+  retval = KINGetLinWorkSpace(kmem, &lenrwB, &leniwB);
+  check_retval(&retval, "KINGetLinWorkSpace", 1);
 
   printf("\nFinal Statistics.. \n\n");
   printf("nni      = %6ld    nfe     = %6ld \n", nni, nfe);
@@ -377,37 +383,37 @@ static void PrintFinalStats(void *kmem)
  * Check function return value...
  *    opt == 0 means SUNDIALS function allocates memory so check if
  *             returned NULL pointer
- *    opt == 1 means SUNDIALS function returns a flag so check if
- *             flag >= 0
+ *    opt == 1 means SUNDIALS function returns a retval so check if
+ *             retval >= 0
  *    opt == 2 means function allocates memory so check if returned
  *             NULL pointer
  */
 
-static int check_flag(void *flagvalue, const char *funcname, int opt)
+static int check_retval(void *retvalvalue, const char *funcname, int opt)
 {
-  int *errflag;
+  int *errretval;
 
   /* Check if SUNDIALS function returned NULL pointer - no memory allocated */
-  if (opt == 0 && flagvalue == NULL) {
+  if (opt == 0 && retvalvalue == NULL) {
     fprintf(stderr,
             "\nSUNDIALS_ERROR: %s() failed - returned NULL pointer\n\n",
 	    funcname);
     return(1);
   }
 
-  /* Check if flag < 0 */
+  /* Check if retval < 0 */
   else if (opt == 1) {
-    errflag = (int *) flagvalue;
-    if (*errflag < 0) {
+    errretval = (int *) retvalvalue;
+    if (*errretval < 0) {
       fprintf(stderr,
-              "\nSUNDIALS_ERROR: %s() failed with flag = %d\n\n",
-	      funcname, *errflag);
+              "\nSUNDIALS_ERROR: %s() failed with retval = %d\n\n",
+	      funcname, *errretval);
       return(1);
     }
   }
 
   /* Check if function returned NULL pointer - no memory allocated */
-  else if (opt == 2 && flagvalue == NULL) {
+  else if (opt == 2 && retvalvalue == NULL) {
     fprintf(stderr,
             "\nMEMORY_ERROR: %s() failed - returned NULL pointer\n\n",
 	    funcname);
