@@ -1,18 +1,28 @@
 /* -----------------------------------------------------------------
  * Programmer(s): Carol Woodward @ LLNL
  * -----------------------------------------------------------------
+ * SUNDIALS Copyright Start
+ * Copyright (c) 2002-2022, Lawrence Livermore National Security
+ * and Southern Methodist University.
+ * All rights reserved.
+ *
+ * See the top-level LICENSE and NOTICE files for details.
+ *
+ * SPDX-License-Identifier: BSD-3-Clause
+ * SUNDIALS Copyright End
+ * -----------------------------------------------------------------
  * This example solves a 2D elliptic PDE
  *
  *    d^2 u / dx^2 + d^2 u / dy^2 = u^3 - u - 2.0
  *
- * subject to homogeneous Dirichelt boundary conditions.
+ * subject to homogeneous Dirichlet boundary conditions.
  * The PDE is discretized on a uniform NX+2 by NY+2 grid with
  * central differencing, and with boundary values eliminated,
  * leaving a system of size NEQ = NX*NY.
  * The nonlinear system is solved by KINSOL using the Picard
  * iteration and the SUNBAND linear solver.
  *
- * This file is strongly based on the kinLaplace_bnd.c file 
+ * This file is strongly based on the kinLaplace_bnd.c file
  * developed by Radu Serban.
  * -----------------------------------------------------------------
  */
@@ -24,7 +34,6 @@
 #include <kinsol/kinsol.h>             /* access to KINSOL func., consts. */
 #include <nvector/nvector_serial.h>    /* access to serial N_Vector       */
 #include <sunmatrix/sunmatrix_band.h>  /* access to band SUNMatrix        */
-#include <kinsol/kinsol_direct.h>      /* access to KINDls interface      */
 #include <sunlinsol/sunlinsol_band.h>  /* access to band SUNLinearSolver  */
 #include <sundials/sundials_types.h>   /* defs. of realtype, sunindextype */
 #include <sundials/sundials_math.h>    /* access to SUNRexp               */
@@ -45,11 +54,11 @@
 
 /* IJth is defined in order to isolate the translation from the
    mathematical 2-dimensional structure of the dependent variable vector
-   to the underlying 1-dimensional storage. 
+   to the underlying 1-dimensional storage.
    IJth(vdata,i,j) references the element in the vdata array for
    u at mesh point (i,j), where 1 <= i <= NX, 1 <= j <= NY.
-   The vdata array is obtained via the call vdata = N_VGetArrayPointer_Serial(v),
-   where v is an N_Vector. 
+   The vdata array is obtained via the call vdata = N_VGetArrayPointer(v),
+   where v is an N_Vector.
    The variables are ordered by the y index j, then by the x index i. */
 
 #define IJth(vdata,i,j) (vdata[(j-1) + (i-1)*NY])
@@ -61,7 +70,7 @@ static int jac(N_Vector u, N_Vector f, SUNMatrix J,
                void *user_data, N_Vector tmp1, N_Vector tmp2);
 static void PrintOutput(N_Vector u);
 static void PrintFinalStats(void *kmem);
-static int check_flag(void *flagvalue, const char *funcname, int opt);
+static int check_retval(void *retvalvalue, const char *funcname, int opt);
 
 /*
  *--------------------------------------------------------------------
@@ -71,9 +80,10 @@ static int check_flag(void *flagvalue, const char *funcname, int opt);
 
 int main()
 {
+  SUNContext sunctx;
   realtype fnormtol, fnorm;
   N_Vector y, scale;
-  int flag;
+  int retval;
   void *kmem;
   SUNMatrix J;
   SUNLinearSolver LS;
@@ -86,127 +96,132 @@ int main()
   /* -------------------------
    * Print problem description
    * ------------------------- */
-  
+
   printf("\n2D elliptic PDE on unit square\n");
   printf("   d^2 u / dx^2 + d^2 u / dy^2 = u^3 - u + 2.0\n");
   printf(" + homogeneous Dirichlet boundary conditions\n\n");
   printf("Solution method: Anderson accelerated Picard iteration with band linear solver.\n");
   printf("Problem size: %2ld x %2ld = %4ld\n", (long int) NX, (long int) NY, (long int) NEQ);
 
+  /* Create the SUNDIALS context that all SUNDIALS objects require */
+  retval = SUNContext_Create(NULL, &sunctx);
+  if (check_retval(&retval, "SUNContext_Create", 1)) return(1);
+
   /* --------------------------------------
    * Create vectors for solution and scales
    * -------------------------------------- */
 
-  y = N_VNew_Serial(NEQ);
-  if (check_flag((void *)y, "N_VNew_Serial", 0)) return(1);
+  y = N_VNew_Serial(NEQ, sunctx);
+  if (check_retval((void *)y, "N_VNew_Serial", 0)) return(1);
 
-  scale = N_VNew_Serial(NEQ);
-  if (check_flag((void *)scale, "N_VNew_Serial", 0)) return(1);
+  scale = N_VNew_Serial(NEQ, sunctx);
+  if (check_retval((void *)scale, "N_VNew_Serial", 0)) return(1);
 
   /* ----------------------------------------------------------------------------------
    * Initialize and allocate memory for KINSOL, set parametrs for Anderson acceleration
    * ---------------------------------------------------------------------------------- */
 
-  kmem = KINCreate();
-  if (check_flag((void *)kmem, "KINCreate", 0)) return(1);
+  kmem = KINCreate(sunctx);
+  if (check_retval((void *)kmem, "KINCreate", 0)) return(1);
 
   /* y is used as a template */
 
   /* Use acceleration with up to 3 prior residuals */
-  flag = KINSetMAA(kmem, 3);
-  if (check_flag(&flag, "KINSetMAA", 1)) return(1);
+  retval = KINSetMAA(kmem, 3);
+  if (check_retval(&retval, "KINSetMAA", 1)) return(1);
 
-  flag = KINInit(kmem, func, y);
-  if (check_flag(&flag, "KINInit", 1)) return(1);
+  retval = KINInit(kmem, func, y);
+  if (check_retval(&retval, "KINInit", 1)) return(1);
 
   /* -------------------
-   * Set optional inputs 
+   * Set optional inputs
    * ------------------- */
 
   /* Specify stopping tolerance based on residual */
 
-  fnormtol  = FTOL; 
-  flag = KINSetFuncNormTol(kmem, fnormtol);
-  if (check_flag(&flag, "KINSetFuncNormTol", 1)) return(1);
+  fnormtol  = FTOL;
+  retval = KINSetFuncNormTol(kmem, fnormtol);
+  if (check_retval(&retval, "KINSetFuncNormTol", 1)) return(1);
 
   /* -------------------------
    * Create band SUNMatrix
    * ------------------------- */
 
-  J = SUNBandMatrix(NEQ, NX, NX, 2*NX);
-  if(check_flag((void *)J, "SUNBandMatrix", 0)) return(1);
+  J = SUNBandMatrix(NEQ, NX, NX, sunctx);
+  if(check_retval((void *)J, "SUNBandMatrix", 0)) return(1);
 
   /* ---------------------------
    * Create band SUNLinearSolver
    * --------------------------- */
 
-  LS = SUNBandLinearSolver(y, J);
-  if(check_flag((void *)LS, "SUNDenseLinearSolver", 0)) return(1);
+  LS = SUNLinSol_Band(y, J, sunctx);
+  if(check_retval((void *)LS, "SUNLinSol_Band", 0)) return(1);
 
   /* -------------------------
-   * Attach band linear solver 
+   * Attach band linear solver
    * ------------------------- */
 
-  flag = KINDlsSetLinearSolver(kmem, LS, J);
-  if(check_flag(&flag, "KINDlsSetLinearSolver", 1)) return(1);
+  retval = KINSetLinearSolver(kmem, LS, J);
+  if(check_retval(&retval, "KINSetLinearSolver", 1)) return(1);
 
   /* -------------------------
    * Set Jacobian function
    * ------------------------- */
 
-  flag = KINDlsSetJacFn(kmem, jac);
-  if (check_flag(&flag, "KINDlsSetJacFn", 1)) return(1);
+  retval = KINSetJacFn(kmem, jac);
+  if (check_retval(&retval, "KINSetJacFn", 1)) return(1);
 
   /* -------------
-   * Initial guess 
+   * Initial guess
    * ------------- */
 
-  N_VConst_Serial(ZERO, y);
-  IJth(N_VGetArrayPointer_Serial(y), 2, 2) = ONE;
+  N_VConst(ZERO, y);
+  IJth(N_VGetArrayPointer(y), 2, 2) = ONE;
 
   /* ----------------------------
-   * Call KINSol to solve problem 
+   * Call KINSol to solve problem
    * ---------------------------- */
 
   /* No scaling used */
-  N_VConst_Serial(ONE,scale);
+  N_VConst(ONE,scale);
 
   /* Call main solver */
-  flag = KINSol(kmem,           /* KINSol memory block */
+  retval = KINSol(kmem,           /* KINSol memory block */
                 y,              /* initial guess on input; solution vector */
                 KIN_PICARD,     /* global strategy choice */
                 scale,          /* scaling vector, for the variable cc */
                 scale);         /* scaling vector for function values fval */
-  if (check_flag(&flag, "KINSol", 1)) return(1);
+  if (check_retval(&retval, "KINSol", 1)) return(1);
 
 
   /* ------------------------------------
-   * Print solution and solver statistics 
+   * Print solution and solver statistics
    * ------------------------------------ */
 
   /* Get scaled norm of the system function */
 
-  flag = KINGetFuncNorm(kmem, &fnorm);
-  if (check_flag(&flag, "KINGetfuncNorm", 1)) return(1);
+  retval = KINGetFuncNorm(kmem, &fnorm);
+  if (check_retval(&retval, "KINGetfuncNorm", 1)) return(1);
 
 #if defined(SUNDIALS_EXTENDED_PRECISION)
   printf("\nComputed solution (||F|| = %Lg):\n\n",fnorm);
 #else
   printf("\nComputed solution (||F|| = %g):\n\n",fnorm);
-#endif  
+#endif
   PrintOutput(y);
 
   PrintFinalStats(kmem);
 
   /* -----------
-   * Free memory 
+   * Free memory
    * ----------- */
-  
-  N_VDestroy_Serial(y);
-  N_VDestroy_Serial(scale);
+
+  N_VDestroy(y);
+  N_VDestroy(scale);
   KINFree(&kmem);
   SUNLinSolFree(LS);
   SUNMatDestroy(J);
+  SUNContext_Free(&sunctx);
 
   return(0);
 }
@@ -217,8 +232,8 @@ int main()
  *--------------------------------------------------------------------
  */
 
-/* 
- * System function 
+/*
+ * System function
  */
 
 static int func(N_Vector u, N_Vector f, void *user_data)
@@ -230,13 +245,13 @@ static int func(N_Vector u, N_Vector f, void *user_data)
 
   int i, j;
 
-  dx = ONE/(NX+1);  
+  dx = ONE/(NX+1);
   dy = ONE/(NY+1);
   hdc = ONE/(dx*dx);
   vdc = ONE/(dy*dy);
 
-  udata = N_VGetArrayPointer_Serial(u);
-  fdata = N_VGetArrayPointer_Serial(f);
+  udata = N_VGetArrayPointer(u);
+  fdata = N_VGetArrayPointer(f);
 
   for (j=1; j <= NY; j++) {
     for (i=1; i <= NX; i++) {
@@ -264,8 +279,8 @@ static int func(N_Vector u, N_Vector f, void *user_data)
   return(0);
 }
 
-/* 
- * Jacobian function 
+/*
+ * Jacobian function
  */
 
 static int jac(N_Vector u, N_Vector f, SUNMatrix J,
@@ -277,7 +292,7 @@ static int jac(N_Vector u, N_Vector f, SUNMatrix J,
 
   int i, j, k;
 
-  dx  = ONE/(NX+1);  
+  dx  = ONE/(NX+1);
   dy  = ONE/(NY+1);
   hdc = ONE/(dx*dx);
   vdc = ONE/(dy*dy);
@@ -313,7 +328,7 @@ static int jac(N_Vector u, N_Vector f, SUNMatrix J,
   return(0);
 }
 
-/* 
+/*
  * Print solution at selected points
  */
 
@@ -326,7 +341,7 @@ static void PrintOutput(N_Vector u)
   dx = ONE/(NX+1);
   dy = ONE/(NY+1);
 
-  udata = N_VGetArrayPointer_Serial(u);
+  udata = N_VGetArrayPointer(u);
 
   printf("            ");
   for (i=1; i<=NX; i+= SKIP) {
@@ -363,7 +378,7 @@ static void PrintOutput(N_Vector u)
   }
 }
 
-/* 
+/*
  * Print final statistics
  */
 
@@ -371,70 +386,70 @@ static void PrintFinalStats(void *kmem)
 {
   long int nni, nfe, nje, nfeD;
   long int lenrwB, leniwB;
-  int flag;
-  
+  int retval;
+
   /* Main solver statistics */
 
-  flag = KINGetNumNonlinSolvIters(kmem, &nni);
-  check_flag(&flag, "KINGetNumNonlinSolvIters", 1);
-  flag = KINGetNumFuncEvals(kmem, &nfe);
-  check_flag(&flag, "KINGetNumFuncEvals", 1);
+  retval = KINGetNumNonlinSolvIters(kmem, &nni);
+  check_retval(&retval, "KINGetNumNonlinSolvIters", 1);
+  retval = KINGetNumFuncEvals(kmem, &nfe);
+  check_retval(&retval, "KINGetNumFuncEvals", 1);
 
   /* Band linear solver statistics */
 
-  flag = KINDlsGetNumJacEvals(kmem, &nje);
-  check_flag(&flag, "KINDlsGetNumJacEvals", 1);
-  flag = KINDlsGetNumFuncEvals(kmem, &nfeD);
-  check_flag(&flag, "KINDlsGetNumFuncEvals", 1);
+  retval = KINGetNumJacEvals(kmem, &nje);
+  check_retval(&retval, "KINGetNumJacEvals", 1);
+  retval = KINGetNumLinFuncEvals(kmem, &nfeD);
+  check_retval(&retval, "KINGetNumLinFuncEvals", 1);
 
   /* Band linear solver workspace size */
 
-  flag = KINDlsGetWorkSpace(kmem, &lenrwB, &leniwB);
-  check_flag(&flag, "KINDlsGetWorkSpace", 1);
+  retval = KINGetLinWorkSpace(kmem, &lenrwB, &leniwB);
+  check_retval(&retval, "KINGetLinWorkSpace", 1);
 
   printf("\nFinal Statistics.. \n\n");
   printf("nni      = %6ld    nfe     = %6ld \n", nni, nfe);
   printf("nje      = %6ld    nfeB    = %6ld \n", nje, nfeD);
   printf("\n");
   printf("lenrwB   = %6ld    leniwB  = %6ld \n", lenrwB, leniwB);
-  
+
 }
 
 /*
  * Check function return value...
  *    opt == 0 means SUNDIALS function allocates memory so check if
  *             returned NULL pointer
- *    opt == 1 means SUNDIALS function returns a flag so check if
- *             flag >= 0
+ *    opt == 1 means SUNDIALS function returns a retval so check if
+ *             retval >= 0
  *    opt == 2 means function allocates memory so check if returned
- *             NULL pointer 
+ *             NULL pointer
  */
 
-static int check_flag(void *flagvalue, const char *funcname, int opt)
+static int check_retval(void *retvalvalue, const char *funcname, int opt)
 {
-  int *errflag;
+  int *errretval;
 
   /* Check if SUNDIALS function returned NULL pointer - no memory allocated */
-  if (opt == 0 && flagvalue == NULL) {
-    fprintf(stderr, 
+  if (opt == 0 && retvalvalue == NULL) {
+    fprintf(stderr,
             "\nSUNDIALS_ERROR: %s() failed - returned NULL pointer\n\n",
 	    funcname);
     return(1);
   }
 
-  /* Check if flag < 0 */
+  /* Check if retval < 0 */
   else if (opt == 1) {
-    errflag = (int *) flagvalue;
-    if (*errflag < 0) {
+    errretval = (int *) retvalvalue;
+    if (*errretval < 0) {
       fprintf(stderr,
-              "\nSUNDIALS_ERROR: %s() failed with flag = %d\n\n",
-	      funcname, *errflag);
-      return(1); 
+              "\nSUNDIALS_ERROR: %s() failed with retval = %d\n\n",
+	      funcname, *errretval);
+      return(1);
     }
   }
 
   /* Check if function returned NULL pointer - no memory allocated */
-  else if (opt == 2 && flagvalue == NULL) {
+  else if (opt == 2 && retvalvalue == NULL) {
     fprintf(stderr,
             "\nMEMORY_ERROR: %s() failed - returned NULL pointer\n\n",
 	    funcname);
