@@ -4,7 +4,7 @@
  * Based on kinRoboKin_dns.c by Radu Serban @ LLNL
  * -----------------------------------------------------------------
  * SUNDIALS Copyright Start
- * Copyright (c) 2002-2021, Lawrence Livermore National Security
+ * Copyright (c) 2002-2022, Lawrence Livermore National Security
  * and Southern Methodist University.
  * All rights reserved.
  *
@@ -58,7 +58,7 @@ static int jac(N_Vector y, N_Vector f, SUNMatrix J,
                void *user_data, N_Vector tmp1, N_Vector tmp2);
 static void PrintOutput(N_Vector y);
 static void PrintFinalStats(void *kmem);
-static int check_flag(void *flagvalue, const char *funcname, int opt);
+static int check_retval(void *retvalvalue, const char *funcname, int opt);
 
 /*
  *--------------------------------------------------------------------
@@ -68,9 +68,10 @@ static int check_flag(void *flagvalue, const char *funcname, int opt);
 
 int main()
 {
+  SUNContext sunctx;
   realtype fnormtol, scsteptol;
   N_Vector y, scale, constraints;
-  int mset, flag, i;
+  int mset, retval, i;
   void *kmem;
   SUNMatrix J;
   SUNLinearSolver LS;
@@ -86,64 +87,68 @@ int main()
   printf("8 variables; -1 <= x_i <= 1\n");
   printf("KINSOL problem size: 8 + 2*8 = 24 \n\n");
 
+  /* Create the SUNDIALS context that all SUNDIALS objects require */
+  retval = SUNContext_Create(NULL, &sunctx);
+  if (check_retval(&retval, "SUNContext_Create", 1)) return(1);
+
   /* Create vectors for solution, scales, and constraints */
 
-  y = N_VNew_Serial(NEQ);
-  if (check_flag((void *)y, "N_VNew_Serial", 0)) return(1);
+  y = N_VNew_Serial(NEQ, sunctx);
+  if (check_retval((void *)y, "N_VNew_Serial", 0)) return(1);
 
-  scale = N_VNew_Serial(NEQ);
-  if (check_flag((void *)scale, "N_VNew_Serial", 0)) return(1);
+  scale = N_VNew_Serial(NEQ, sunctx);
+  if (check_retval((void *)scale, "N_VNew_Serial", 0)) return(1);
 
-  constraints = N_VNew_Serial(NEQ);
-  if (check_flag((void *)constraints, "N_VNew_Serial", 0)) return(1);
+  constraints = N_VNew_Serial(NEQ, sunctx);
+  if (check_retval((void *)constraints, "N_VNew_Serial", 0)) return(1);
 
   /* Initialize and allocate memory for KINSOL */
 
-  kmem = KINCreate();
-  if (check_flag((void *)kmem, "KINCreate", 0)) return(1);
+  kmem = KINCreate(sunctx);
+  if (check_retval((void *)kmem, "KINCreate", 0)) return(1);
 
-  flag = KINInit(kmem, func, y); /* y passed as a template */
-  if (check_flag(&flag, "KINInit", 1)) return(1);
+  retval = KINInit(kmem, func, y); /* y passed as a template */
+  if (check_retval(&retval, "KINInit", 1)) return(1);
 
   /* Set optional inputs */
 
   N_VConst(ZERO,constraints);
   for (i = NVAR+1; i <= NEQ; i++) Ith(constraints, i) = ONE;
 
-  flag = KINSetConstraints(kmem, constraints);
-  if (check_flag(&flag, "KINSetConstraints", 1)) return(1);
+  retval = KINSetConstraints(kmem, constraints);
+  if (check_retval(&retval, "KINSetConstraints", 1)) return(1);
 
   fnormtol  = FTOL;
-  flag = KINSetFuncNormTol(kmem, fnormtol);
-  if (check_flag(&flag, "KINSetFuncNormTol", 1)) return(1);
+  retval = KINSetFuncNormTol(kmem, fnormtol);
+  if (check_retval(&retval, "KINSetFuncNormTol", 1)) return(1);
 
   scsteptol = STOL;
-  flag = KINSetScaledStepTol(kmem, scsteptol);
-  if (check_flag(&flag, "KINSetScaledStepTol", 1)) return(1);
+  retval = KINSetScaledStepTol(kmem, scsteptol);
+  if (check_retval(&retval, "KINSetScaledStepTol", 1)) return(1);
 
   /* Create sparse SUNMatrix */
   nnz = 56; /* number of nonzeros in the Jacobian */
-  J = SUNSparseMatrix(NEQ, NEQ, nnz, CSC_MAT);
-  if(check_flag((void *)J, "SUNSparseMatrix", 0)) return(1);
+  J = SUNSparseMatrix(NEQ, NEQ, nnz, CSC_MAT, sunctx);
+  if(check_retval((void *)J, "SUNSparseMatrix", 0)) return(1);
 
   /* Create SuperLUMT solver object */
   num_threads = 2; /* number fo threads to use */
-  LS = SUNLinSol_SuperLUMT(y, J, num_threads);
-  if(check_flag((void *)LS, "SUNLinSol_SuperLUMT", 0)) return(1);
+  LS = SUNLinSol_SuperLUMT(y, J, num_threads, sunctx);
+  if(check_retval((void *)LS, "SUNLinSol_SuperLUMT", 0)) return(1);
 
   /* Attach the SuperLU_MT linear solver */
-  flag = KINSetLinearSolver(kmem, LS, J);
-  if(check_flag(&flag, "KINSetLinearSolver", 1)) return(1);
+  retval = KINSetLinearSolver(kmem, LS, J);
+  if(check_retval(&retval, "KINSetLinearSolver", 1)) return(1);
 
   /* Set the Jacobian function */
-  flag = KINSetJacFn(kmem, jac);
-  if (check_flag(&flag, "KINSetJacFn", 1)) return(1);
+  retval = KINSetJacFn(kmem, jac);
+  if (check_retval(&retval, "KINSetJacFn", 1)) return(1);
 
   /* Indicate exact Newton */
 
   mset = 1;
-  flag = KINSetMaxSetupCalls(kmem, mset);
-  if (check_flag(&flag, "KINSetMaxSetupCalls", 1)) return(1);
+  retval = KINSetMaxSetupCalls(kmem, mset);
+  if (check_retval(&retval, "KINSetMaxSetupCalls", 1)) return(1);
 
   /* Initial guess */
 
@@ -156,12 +161,12 @@ int main()
   /* Call KINSol to solve problem */
 
   N_VConst(ONE,scale);
-  flag = KINSol(kmem,           /* KINSol memory block */
+  retval = KINSol(kmem,           /* KINSol memory block */
                 y,              /* initial guess on input; solution vector */
                 KIN_LINESEARCH, /* global strategy choice */
                 scale,          /* scaling vector, for the variable cc */
                 scale);         /* scaling vector for function values fval */
-  if (check_flag(&flag, "KINSol", 1)) return(1);
+  if (check_retval(&retval, "KINSol", 1)) return(1);
 
   printf("\nComputed solution:\n");
   PrintOutput(y);
@@ -176,6 +181,7 @@ int main()
   KINFree(&kmem);
   SUNLinSolFree(LS);
   SUNMatDestroy(J);
+  SUNContext_Free(&sunctx);
 
   return(0);
 }
@@ -573,15 +579,15 @@ static void PrintOutput(N_Vector y)
 static void PrintFinalStats(void *kmem)
 {
   long int nni, nfe, nje;
-  int flag;
+  int retval;
 
-  flag = KINGetNumNonlinSolvIters(kmem, &nni);
-  check_flag(&flag, "KINGetNumNonlinSolvIters", 1);
-  flag = KINGetNumFuncEvals(kmem, &nfe);
-  check_flag(&flag, "KINGetNumFuncEvals", 1);
+  retval = KINGetNumNonlinSolvIters(kmem, &nni);
+  check_retval(&retval, "KINGetNumNonlinSolvIters", 1);
+  retval = KINGetNumFuncEvals(kmem, &nfe);
+  check_retval(&retval, "KINGetNumFuncEvals", 1);
 
-  flag = KINGetNumJacEvals(kmem, &nje);
-  check_flag(&flag, "KINGetNumJacEvals", 1);
+  retval = KINGetNumJacEvals(kmem, &nje);
+  check_retval(&retval, "KINGetNumJacEvals", 1);
 
   printf("\nFinal Statistics.. \n");
   printf("nni    = %5ld    nfe   = %5ld \n", nni, nfe);
@@ -592,37 +598,37 @@ static void PrintFinalStats(void *kmem)
  * Check function return value...
  *    opt == 0 means SUNDIALS function allocates memory so check if
  *             returned NULL pointer
- *    opt == 1 means SUNDIALS function returns a flag so check if
- *             flag >= 0
+ *    opt == 1 means SUNDIALS function returns a retval so check if
+ *             retval >= 0
  *    opt == 2 means function allocates memory so check if returned
  *             NULL pointer
  */
 
-static int check_flag(void *flagvalue, const char *funcname, int opt)
+static int check_retval(void *retvalvalue, const char *funcname, int opt)
 {
-  int *errflag;
+  int *errretval;
 
   /* Check if SUNDIALS function returned NULL pointer - no memory allocated */
-  if (opt == 0 && flagvalue == NULL) {
+  if (opt == 0 && retvalvalue == NULL) {
     fprintf(stderr,
             "\nSUNDIALS_ERROR: %s() failed - returned NULL pointer\n\n",
 	    funcname);
     return(1);
   }
 
-  /* Check if flag < 0 */
+  /* Check if retval < 0 */
   else if (opt == 1) {
-    errflag = (int *) flagvalue;
-    if (*errflag < 0) {
+    errretval = (int *) retvalvalue;
+    if (*errretval < 0) {
       fprintf(stderr,
-              "\nSUNDIALS_ERROR: %s() failed with flag = %d\n\n",
-	      funcname, *errflag);
+              "\nSUNDIALS_ERROR: %s() failed with retval = %d\n\n",
+	      funcname, *errretval);
       return(1);
     }
   }
 
   /* Check if function returned NULL pointer - no memory allocated */
-  else if (opt == 2 && flagvalue == NULL) {
+  else if (opt == 2 && retvalvalue == NULL) {
     fprintf(stderr,
             "\nMEMORY_ERROR: %s() failed - returned NULL pointer\n\n",
 	    funcname);
