@@ -2,7 +2,7 @@
  * Programmer(s): David J. Gardner @ LLNL
  * -----------------------------------------------------------------
  * SUNDIALS Copyright Start
- * Copyright (c) 2002-2021, Lawrence Livermore National Security
+ * Copyright (c) 2002-2022, Lawrence Livermore National Security
  * and Southern Methodist University.
  * All rights reserved.
  *
@@ -39,22 +39,24 @@ int main(int argc, char *argv[])
   N_Vector     U, V, W, X, Y, Z;  /* test vectors              */
   int          print_timing;      /* turn timing on/off        */
 
+  Test_Init(NULL);
+
   /* check input and set vector length */
   if (argc < 3){
     printf("ERROR: TWO (2) Inputs required: vector length and print timing \n");
-    return(-1);
+    Test_Abort(-1);
   }
 
   length = (sunindextype) atol(argv[1]);
   if (length <= 0) {
     printf("ERROR: length of vector must be a positive integer \n");
-    return(-1);
+    Test_Abort(-1);
   }
 
   print_timing = atoi(argv[2]);
   SetTiming(print_timing, 0);
 
-  printf("Testing the OpenMP DEV N_Vector \n");
+  printf("Testing the OpenMPDEV N_Vector \n");
   printf("Vector length %ld \n", (long int) length);
   printf("\n omp_get_default_device = %d \n", omp_get_default_device());
   printf("\n omp_get_num_devices    = %d \n", omp_get_num_devices());
@@ -62,17 +64,17 @@ int main(int argc, char *argv[])
   printf("\n omp_is_initial_device  = %d \n", omp_is_initial_device());
 
   /* Create new vectors */
-  W = N_VNewEmpty_OpenMPDEV(length);
+  W = N_VNewEmpty_OpenMPDEV(length, sunctx);
   if (W == NULL) {
     printf("FAIL: Unable to create a new empty vector \n\n");
-    return(1);
+    Test_Abort(1);
   }
 
-  X = N_VNew_OpenMPDEV(length);
+  X = N_VNew_OpenMPDEV(length, sunctx);
   if (X == NULL) {
     N_VDestroy(W);
     printf("FAIL: Unable to create a new vector \n\n");
-    return(1);
+    Test_Abort(1);
   }
 
   /* Check vector ID */
@@ -96,7 +98,7 @@ int main(int argc, char *argv[])
     N_VDestroy(W);
     N_VDestroy(X);
     printf("FAIL: Unable to create a new vector \n\n");
-    return(1);
+    Test_Abort(1);
   }
 
   Z = N_VClone(X);
@@ -105,7 +107,7 @@ int main(int argc, char *argv[])
     N_VDestroy(X);
     N_VDestroy(Y);
     printf("FAIL: Unable to create a new vector \n\n");
-    return(1);
+    Test_Abort(1);
   }
 
   /* Standard vector operation tests */
@@ -135,7 +137,7 @@ int main(int argc, char *argv[])
   printf("\nTesting fused and vector array operations (disabled):\n\n");
 
   /* create vector and disable all fused and vector array operations */
-  U = N_VNew_OpenMPDEV(length);
+  U = N_VNew_OpenMPDEV(length, sunctx);
   retval = N_VEnableFusedOps_OpenMPDEV(U, SUNFALSE);
   if (U == NULL || retval != 0) {
     N_VDestroy(W);
@@ -143,7 +145,7 @@ int main(int argc, char *argv[])
     N_VDestroy(Y);
     N_VDestroy(Z);
     printf("FAIL: Unable to create a new vector \n\n");
-    return(1);
+    Test_Abort(1);
   }
 
   /* fused operations */
@@ -164,7 +166,7 @@ int main(int argc, char *argv[])
   printf("\nTesting fused and vector array operations (enabled):\n\n");
 
   /* create vector and enable all fused and vector array operations */
-  V = N_VNew_OpenMPDEV(length);
+  V = N_VNew_OpenMPDEV(length, sunctx);
   retval = N_VEnableFusedOps_OpenMPDEV(V, SUNTRUE);
   if (V == NULL || retval != 0) {
     N_VDestroy(W);
@@ -173,7 +175,7 @@ int main(int argc, char *argv[])
     N_VDestroy(Z);
     N_VDestroy(U);
     printf("FAIL: Unable to create a new vector \n\n");
-    return(1);
+    Test_Abort(1);
   }
 
   /* fused operations */
@@ -203,6 +205,10 @@ int main(int argc, char *argv[])
   fails += Test_N_VConstrMaskLocal(X, Y, Z, length, 0);
   fails += Test_N_VMinQuotientLocal(X, Y, length, 0);
 
+  /* local fused reduction operations */
+  printf("\nTesting local fused reduction operations:\n\n");
+  fails += Test_N_VDotProdMultiLocal(V, length, 0);
+
   /* Free vectors */
   N_VDestroy(U);
   N_VDestroy(V);
@@ -218,6 +224,7 @@ int main(int argc, char *argv[])
     printf("SUCCESS: NVector module passed all tests \n\n");
   }
 
+  Test_Finalize();
   return(fails);
 }
 
