@@ -2,7 +2,7 @@
  * Programmer(s): Slaven Peles @ LLNL
  * -----------------------------------------------------------------
  * SUNDIALS Copyright Start
- * Copyright (c) 2002-2021, Lawrence Livermore National Security
+ * Copyright (c) 2002-2022, Lawrence Livermore National Security
  * and Southern Methodist University.
  * All rights reserved.
  *
@@ -46,7 +46,10 @@ int main(int argc, char *argv[])
 
   /* Get processor number and total number of processes */
   MPI_Init(&argc, &argv);
+
   comm = MPI_COMM_WORLD;
+  Test_Init(&comm);
+
   MPI_Comm_size(comm, &nprocs);
   MPI_Comm_rank(comm, &myid);
 
@@ -54,14 +57,14 @@ int main(int argc, char *argv[])
   if (argc < 3) {
     if (myid == 0)
       printf("ERROR: TWO (2) Inputs required: vector length, print timing \n");
-    MPI_Abort(comm, -1);
+    Test_AbortMPI(&comm, -1);
   }
 
   local_length = (sunindextype) atol(argv[1]);
   if (local_length < 1) {
     if (myid == 0)
       printf("ERROR: local vector length must be a positive integer \n");
-    MPI_Abort(comm, -1);
+    Test_AbortMPI(&comm, -1);
   }
 
   print_timing = atoi(argv[2]);
@@ -86,13 +89,13 @@ int main(int argc, char *argv[])
   VecSetFromOptions(xvec);
 
   /* Create PETSc N_Vector wrapper and test */
-  X = N_VMake_Petsc(xvec);
+  X = N_VMake_Petsc(xvec, sunctx);
   fails += Test_N_VMake(X, local_length, myid);
   if (fails != 0) {
     N_VDestroy(X);
     VecDestroy(&xvec);
     if (myid == 0) printf("FAIL: Unable to create a new vector \n\n");
-    MPI_Abort(comm, 1);
+    Test_AbortMPI(&comm, 1);
   }
 
   /* Check vector ID */
@@ -119,7 +122,7 @@ int main(int argc, char *argv[])
     N_VDestroy(X);
     VecDestroy(&xvec);
     if (myid == 0) printf("FAIL: Unable to create a new vector \n\n");
-    MPI_Abort(comm, 1);
+    Test_AbortMPI(&comm, 1);
   }
 
   Z = N_VClone(X);
@@ -128,7 +131,7 @@ int main(int argc, char *argv[])
     N_VDestroy(Y);
     VecDestroy(&xvec);
     if (myid == 0) printf("FAIL: Unable to create a new vector \n\n");
-    MPI_Abort(comm, 1);
+    Test_AbortMPI(&comm, 1);
   }
 
   /* Standard vector operation tests */
@@ -158,7 +161,7 @@ int main(int argc, char *argv[])
   if (myid == 0) printf("\nTesting fused and vector array operations (disabled):\n\n");
 
   /* create vector and disable all fused and vector array operations */
-  U = N_VMake_Petsc(xvec);
+  U = N_VMake_Petsc(xvec, sunctx);
   retval = N_VEnableFusedOps_Petsc(U, SUNFALSE);
   if (U == NULL || retval != 0) {
     N_VDestroy(X);
@@ -166,7 +169,7 @@ int main(int argc, char *argv[])
     N_VDestroy(Z);
     VecDestroy(&xvec);
     if (myid == 0) printf("FAIL: Unable to create a new vector \n\n");
-    MPI_Abort(comm, 1);
+    Test_AbortMPI(&comm, 1);
   }
 
   /* fused operations */
@@ -187,7 +190,7 @@ int main(int argc, char *argv[])
   if (myid == 0) printf("\nTesting fused and vector array operations (enabled):\n\n");
 
   /* create vector and enable all fused and vector array operations */
-  V = N_VMake_Petsc(xvec);
+  V = N_VMake_Petsc(xvec, sunctx);
   retval = N_VEnableFusedOps_Petsc(V, SUNTRUE);
   if (V == NULL || retval != 0) {
     N_VDestroy(X);
@@ -196,7 +199,7 @@ int main(int argc, char *argv[])
     N_VDestroy(U);
     VecDestroy(&xvec);
     if (myid == 0) printf("FAIL: Unable to create a new vector \n\n");
-    MPI_Abort(comm, 1);
+    Test_AbortMPI(&comm, 1);
   }
 
   /* fused operations */
@@ -226,8 +229,13 @@ int main(int argc, char *argv[])
   fails += Test_N_VConstrMaskLocal(X, Y, Z, local_length, myid);
   fails += Test_N_VMinQuotientLocal(X, Y, local_length, myid);
 
+  /* local fused reduction operations */
+  if (myid == 0) printf("\nTesting local fused reduction operations:\n\n");
+  fails += Test_N_VDotProdMultiLocal(V, local_length, myid);
+  fails += Test_N_VDotProdMultiAllReduce(V, local_length, myid);
+
   /* XBraid interface operations */
-  printf("\nTesting XBraid interface operations:\n\n");
+  if (myid == 0) printf("\nTesting XBraid interface operations:\n\n");
 
   fails += Test_N_VBufSize(X, local_length, myid);
   fails += Test_N_VBufPack(X, local_length, myid);
@@ -256,8 +264,8 @@ int main(int argc, char *argv[])
   ierr = PetscFinalize();
   CHKERRQ(ierr);
 
+  Test_Finalize();
   MPI_Finalize();
-
   return(globfails);
 }
 
