@@ -1,9 +1,9 @@
 /*
- * ----------------------------------------------------------------- 
+ * -----------------------------------------------------------------
  * Programmer(s): Daniel Reynolds @ SMU
  * -----------------------------------------------------------------
  * SUNDIALS Copyright Start
- * Copyright (c) 2002-2019, Lawrence Livermore National Security
+ * Copyright (c) 2002-2022, Lawrence Livermore National Security
  * and Southern Methodist University.
  * All rights reserved.
  *
@@ -12,8 +12,8 @@
  * SPDX-License-Identifier: BSD-3-Clause
  * SUNDIALS Copyright End
  * -----------------------------------------------------------------
- * This is the testing routine to check the SUNLinSol KLU module 
- * implementation. 
+ * This is the testing routine to check the SUNLinSol KLU module
+ * implementation.
  * -----------------------------------------------------------------
  */
 
@@ -31,7 +31,7 @@
 /* ----------------------------------------------------------------------
  * SUNLinSol_KLU Linear Solver Testing Routine
  * --------------------------------------------------------------------*/
-int main(int argc, char *argv[]) 
+int main(int argc, char *argv[])
 {
   int             fails = 0;          /* counter for test failures  */
   sunindextype    N;                  /* matrix columns, rows       */
@@ -41,6 +41,15 @@ int main(int argc, char *argv[])
   realtype        *matdata, *xdata;
   int             mattype, print_timing;
   sunindextype    i, j, k;
+  sun_klu_symbolic *symbolic;
+  sun_klu_numeric  *numeric;
+  sun_klu_common   *common;
+  SUNContext      sunctx;
+
+  if (SUNContext_Create(NULL, &sunctx)) {
+    printf("ERROR: SUNContext_Create failed\n");
+    return(-1);
+  }
 
   /* check input and set matrix dimensions */
   if (argc < 4){
@@ -48,19 +57,19 @@ int main(int argc, char *argv[])
     return(-1);
   }
 
-  N = atol(argv[1]); 
+  N = (sunindextype) atol(argv[1]);
   if (N <= 0) {
     printf("ERROR: matrix size must be a positive integer \n");
-    return(-1); 
+    return(-1);
   }
 
   mattype = atoi(argv[2]);
   if ((mattype != 0) && (mattype != 1)) {
     printf("ERROR: matrix type must be 0 or 1 \n");
-    return(-1); 
+    return(-1);
   }
   mattype = (mattype == 0) ? CSC_MAT : CSR_MAT;
-  
+
   print_timing = atoi(argv[3]);
   SetTiming(print_timing);
 
@@ -68,10 +77,10 @@ int main(int argc, char *argv[])
          (long int) N, mattype);
 
   /* Create matrices and vectors */
-  B = SUNDenseMatrix(N, N);
-  x = N_VNew_Serial(N);
-  y = N_VNew_Serial(N);
-  b = N_VNew_Serial(N);
+  B = SUNDenseMatrix(N, N, sunctx);
+  x = N_VNew_Serial(N, sunctx);
+  y = N_VNew_Serial(N, sunctx);
+  b = N_VNew_Serial(N, sunctx);
 
   /* Fill matrix with uniform random data in [0,1/N] */
   for (k=0; k<5*N; k++) {
@@ -106,19 +115,43 @@ int main(int argc, char *argv[])
     printf("FAIL: SUNLinSol SUNMatMatvec failure\n");
     return(1);
   }
-  
+
   /* Create KLU linear solver */
-  LS = SUNLinSol_KLU(x, A);
-  
+  LS = SUNLinSol_KLU(x, A, sunctx);
+
   /* Run Tests */
   fails += Test_SUNLinSolInitialize(LS, 0);
   fails += Test_SUNLinSolSetup(LS, A, 0);
-  fails += Test_SUNLinSolSolve(LS, A, x, b, 1000*UNIT_ROUNDOFF, 0);
- 
+  fails += Test_SUNLinSolSolve(LS, A, x, b, 1000*UNIT_ROUNDOFF, SUNTRUE, 0);
+
   fails += Test_SUNLinSolGetType(LS, SUNLINEARSOLVER_DIRECT, 0);
+  fails += Test_SUNLinSolGetID(LS, SUNLINEARSOLVER_KLU, 0);
   fails += Test_SUNLinSolLastFlag(LS, 0);
   fails += Test_SUNLinSolSpace(LS, 0);
 
+  /* Test 'Get' routines */
+  symbolic = SUNLinSol_KLUGetSymbolic(LS);
+  if (symbolic->n != N) {
+    printf("FAIL: SUNLinSol_KLUGetSymbolic failure\n");
+    fails += 1;
+  } else {
+    printf("    PASSED test -- SUNLinSol_KLUGetSymbolic \n");
+  }
+  numeric = SUNLinSol_KLUGetNumeric(LS);
+  if (numeric->n != N) {
+    printf("FAIL: SUNLinSol_KLUGetNumeric failure\n");
+    fails += 1;
+  } else {
+    printf("    PASSED test -- SUNLinSol_KLUGetNumeric \n");
+  }
+  common = SUNLinSol_KLUGetCommon(LS);
+  if (common->singular_col != N) {
+    printf("FAIL: SUNLinSol_KLUGetCommon failure\n");
+    fails += 1;
+  } else {
+    printf("    PASSED test -- SUNLinSol_KLUGetCommon \n");
+  }
+
   /* Print result */
   if (fails) {
     printf("FAIL: SUNLinSol module failed %i tests \n \n", fails);
@@ -141,6 +174,8 @@ int main(int argc, char *argv[])
   N_VDestroy(y);
   N_VDestroy(b);
 
+  SUNContext_Free(&sunctx);
+
   return(fails);
 }
 
@@ -152,14 +187,14 @@ int check_vector(N_Vector X, N_Vector Y, realtype tol)
   int failure = 0;
   sunindextype i, local_length, maxloc;
   realtype *Xdata, *Ydata, maxerr;
-  
+
   Xdata = N_VGetArrayPointer(X);
   Ydata = N_VGetArrayPointer(Y);
   local_length = N_VGetLength_Serial(X);
-  
+
   /* check vector data */
   for(i=0; i < local_length; i++)
-    failure += FNEQ(Xdata[i], Ydata[i], tol);
+    failure += SUNRCompareTol(Xdata[i], Ydata[i], tol);
 
   if (failure > ZERO) {
     maxerr = ZERO;
@@ -177,3 +212,7 @@ int check_vector(N_Vector X, N_Vector Y, realtype tol)
   else
     return(0);
 }
+
+void sync_device()
+{
+}
