@@ -4,7 +4,7 @@
  *                David J. Gardner @ LLNL
  * -----------------------------------------------------------------
  * SUNDIALS Copyright Start
- * Copyright (c) 2002-2021, Lawrence Livermore National Security
+ * Copyright (c) 2002-2022, Lawrence Livermore National Security
  * and Southern Methodist University.
  * All rights reserved.
  *
@@ -231,7 +231,7 @@ int Test_SUNLinSolResid(SUNLinearSolver S, int myid)
  * SUNLinSolSetATimes Test
  * --------------------------------------------------------------------*/
 int Test_SUNLinSolSetATimes(SUNLinearSolver S, void *ATdata,
-                            ATimesFn ATimes, int myid)
+                            SUNATimesFn ATimes, int myid)
 {
   int     failure;
   double  start_time, stop_time;
@@ -260,7 +260,7 @@ int Test_SUNLinSolSetATimes(SUNLinearSolver S, void *ATdata,
  * SUNLinSolSetPreconditioner
  * --------------------------------------------------------------------*/
 int Test_SUNLinSolSetPreconditioner(SUNLinearSolver S, void *Pdata,
-                                    PSetupFn PSetup, PSolveFn PSolve, int myid)
+                                    SUNPSetupFn PSetup, SUNPSolveFn PSolve, int myid)
 {
   int       failure;
   double    start_time, stop_time;
@@ -496,7 +496,7 @@ void SetTiming(int onoff)
 
 #if defined( SUNDIALS_HAVE_POSIX_TIMERS) && defined(_POSIX_TIMERS)
   struct timespec spec;
-  clock_gettime( CLOCK_MONOTONIC_RAW, &spec );
+  clock_gettime( CLOCK_MONOTONIC, &spec );
   base_time_tv_sec = spec.tv_sec;
 #endif
 }
@@ -508,7 +508,7 @@ static double get_time()
 {
 #if defined( SUNDIALS_HAVE_POSIX_TIMERS) && defined(_POSIX_TIMERS)
   struct timespec spec;
-  clock_gettime( CLOCK_MONOTONIC_RAW, &spec );
+  clock_gettime( CLOCK_MONOTONIC, &spec );
   double time = (double)(spec.tv_sec - base_time_tv_sec) + ((double)(spec.tv_nsec) / 1E9);
 #else
   double time = 0;
