@@ -4,7 +4,7 @@
  *                Radu Serban @ LLNL
  * ----------------------------------------------------------------
  * SUNDIALS Copyright Start
- * Copyright (c) 2002-2021, Lawrence Livermore National Security
+ * Copyright (c) 2002-2022, Lawrence Livermore National Security
  * and Southern Methodist University.
  * All rights reserved.
  *
@@ -92,6 +92,8 @@ SUNDIALS_EXPORT int CVodeSetJacEvalFrequency(void *cvode_mem,
                                              long int msbj);
 SUNDIALS_EXPORT int CVodeSetLinearSolutionScaling(void *cvode_mem,
                                                   booleantype onoff);
+SUNDIALS_EXPORT int CVodeSetDeltaGammaMaxBadJac(void *cvode_mem,
+                                                realtype dgmax_jbad);
 SUNDIALS_EXPORT int CVodeSetEpsLin(void *cvode_mem, realtype eplifac);
 SUNDIALS_EXPORT int CVodeSetLSNormFactor(void *arkode_mem,
                                          realtype nrmfac);
@@ -135,11 +137,6 @@ SUNDIALS_EXPORT int CVodeGetLastLinFlag(void *cvode_mem,
                                         long int *flag);
 SUNDIALS_EXPORT char *CVodeGetLinReturnFlagName(long int flag);
 
-
-/* Deprecated functions */
-SUNDIALS_DEPRECATED_EXPORT
-int CVodeSetMaxStepsBetweenJac(void *cvode_mem, long int msbj);
-
 #ifdef __cplusplus
 }
 #endif
