@@ -2,7 +2,7 @@
  * Programmer(s): Radu Serban @ LLNL
  * -----------------------------------------------------------------
  * SUNDIALS Copyright Start
- * Copyright (c) 2002-2021, Lawrence Livermore National Security
+ * Copyright (c) 2002-2022, Lawrence Livermore National Security
  * and Southern Methodist University.
  * All rights reserved.
  *
@@ -18,9 +18,11 @@
 #define _CVODES_H
 
 #include <stdio.h>
+#include <sundials/sundials_context.h>
 #include <sundials/sundials_nvector.h>
 #include <sundials/sundials_nonlinearsolver.h>
 #include <cvodes/cvodes_ls.h>
+#include <cvodes/cvodes_proj.h>
 
 #ifdef __cplusplus  /* wrapper to enable C++ usage */
 extern "C" {
@@ -107,6 +109,11 @@ extern "C" {
 #define CV_REPTD_QSRHSFUNC_ERR  -53
 #define CV_UNREC_QSRHSFUNC_ERR  -54
 
+#define CV_CONTEXT_ERR          -55
+
+#define CV_PROJ_MEM_NULL        -56
+#define CV_PROJFUNC_FAIL        -57
+#define CV_REPTD_PROJFUNC_ERR   -58
 
 #define CV_UNRECOGNIZED_ERR     -99
 
@@ -136,6 +143,8 @@ typedef void (*CVErrHandlerFn)(int error_code,
                                const char *module, const char *function,
                                char *msg, void *user_data);
 
+typedef int (*CVMonitorFn)(void *cvode_mem, void *user_data);
+
 typedef int (*CVQuadRhsFn)(realtype t, N_Vector y,
                            N_Vector yQdot, void *user_data);
 
@@ -176,7 +185,7 @@ typedef int (*CVQuadRhsFnBS)(realtype t, N_Vector y, N_Vector *yS,
  * --------------------------------------- */
 
 /* Initialization functions */
-SUNDIALS_EXPORT void *CVodeCreate(int lmm);
+SUNDIALS_EXPORT void *CVodeCreate(int lmm, SUNContext sunctx);
 
 SUNDIALS_EXPORT int CVodeInit(void *cvode_mem, CVRhsFn f, realtype t0,
                               N_Vector y0);
@@ -190,27 +199,53 @@ SUNDIALS_EXPORT int CVodeSVtolerances(void *cvode_mem, realtype reltol,
 SUNDIALS_EXPORT int CVodeWFtolerances(void *cvode_mem, CVEwtFn efun);
 
 /* Optional input functions */
-SUNDIALS_EXPORT int CVodeSetErrHandlerFn(void *cvode_mem, CVErrHandlerFn ehfun,
-                                         void *eh_data);
-SUNDIALS_EXPORT int CVodeSetErrFile(void *cvode_mem, FILE *errfp);
-SUNDIALS_EXPORT int CVodeSetUserData(void *cvode_mem, void *user_data);
-SUNDIALS_EXPORT int CVodeSetMaxOrd(void *cvode_mem, int maxord);
-SUNDIALS_EXPORT int CVodeSetMaxNumSteps(void *cvode_mem, long int mxsteps);
-SUNDIALS_EXPORT int CVodeSetMaxHnilWarns(void *cvode_mem, int mxhnil);
-SUNDIALS_EXPORT int CVodeSetStabLimDet(void *cvode_mem, booleantype stldet);
-SUNDIALS_EXPORT int CVodeSetInitStep(void *cvode_mem, realtype hin);
-SUNDIALS_EXPORT int CVodeSetMinStep(void *cvode_mem, realtype hmin);
-SUNDIALS_EXPORT int CVodeSetMaxStep(void *cvode_mem, realtype hmax);
-SUNDIALS_EXPORT int CVodeSetStopTime(void *cvode_mem, realtype tstop);
-SUNDIALS_EXPORT int CVodeSetMaxErrTestFails(void *cvode_mem, int maxnef);
-SUNDIALS_EXPORT int CVodeSetMaxNonlinIters(void *cvode_mem, int maxcor);
-SUNDIALS_EXPORT int CVodeSetMaxConvFails(void *cvode_mem, int maxncf);
-SUNDIALS_EXPORT int CVodeSetNonlinConvCoef(void *cvode_mem, realtype nlscoef);
-SUNDIALS_EXPORT int CVodeSetLSetupFrequency(void *cvode_mem, long int msbp);
+
 SUNDIALS_EXPORT int CVodeSetConstraints(void *cvode_mem, N_Vector constraints);
-SUNDIALS_EXPORT int CVodeSetNonlinearSolver(void *cvode_mem,
-                                            SUNNonlinearSolver NLS);
+SUNDIALS_EXPORT int CVodeSetDeltaGammaMaxLSetup(void *cvode_mem,
+                                                realtype dgmax_lsetup);
+SUNDIALS_EXPORT int CVodeSetErrFile(void *cvode_mem, FILE *errfp);
+SUNDIALS_EXPORT int CVodeSetErrHandlerFn(void *cvode_mem, CVErrHandlerFn ehfun, void *eh_data);
+SUNDIALS_EXPORT int CVodeSetInitStep(void *cvode_mem, realtype hin);
+SUNDIALS_EXPORT int CVodeSetLSetupFrequency(void *cvode_mem, long int msbp);
+SUNDIALS_EXPORT int CVodeSetMaxConvFails(void *cvode_mem, int maxncf);
+SUNDIALS_EXPORT int CVodeSetMaxErrTestFails(void *cvode_mem, int maxnef);
+SUNDIALS_EXPORT int CVodeSetMaxHnilWarns(void *cvode_mem, int mxhnil);
+SUNDIALS_EXPORT int CVodeSetMaxNonlinIters(void *cvode_mem, int maxcor);
+SUNDIALS_EXPORT int CVodeSetMaxNumSteps(void *cvode_mem, long int mxsteps);
+SUNDIALS_EXPORT int CVodeSetMaxOrd(void *cvode_mem, int maxord);
+SUNDIALS_EXPORT int CVodeSetMaxStep(void *cvode_mem, realtype hmax);
+SUNDIALS_EXPORT int CVodeSetMinStep(void *cvode_mem, realtype hmin);
+SUNDIALS_EXPORT int CVodeSetMonitorFn(void *cvode_mem, CVMonitorFn fn);
+SUNDIALS_EXPORT int CVodeSetMonitorFrequency(void *cvode_mem, long int nst);
 SUNDIALS_EXPORT int CVodeSetNlsRhsFn(void *cvode_mem, CVRhsFn f);
+SUNDIALS_EXPORT int CVodeSetNonlinConvCoef(void *cvode_mem, realtype nlscoef);
+SUNDIALS_EXPORT int CVodeSetNonlinearSolver(void *cvode_mem, SUNNonlinearSolver NLS);
+SUNDIALS_EXPORT int CVodeSetStabLimDet(void *cvode_mem, booleantype stldet);
+SUNDIALS_EXPORT int CVodeSetStopTime(void *cvode_mem, realtype tstop);
+SUNDIALS_EXPORT int CVodeSetUserData(void *cvode_mem, void *user_data);
+
+/* Optional step adaptivity input functions */
+SUNDIALS_EXPORT
+int CVodeSetEtaFixedStepBounds(void* cvode_mem, realtype eta_min_fx,
+                               realtype eta_max_fx);
+SUNDIALS_EXPORT
+int CVodeSetEtaMaxFirstStep(void* cvode_mem, realtype eta_max_fs);
+SUNDIALS_EXPORT
+int CVodeSetEtaMaxEarlyStep(void* cvode_mem, realtype eta_max_es);
+SUNDIALS_EXPORT
+int CVodeSetNumStepsEtaMaxEarlyStep(void* cvode_mem, long int small_nst);
+SUNDIALS_EXPORT
+int CVodeSetEtaMax(void* cvode_mem, realtype eta_max_gs);
+SUNDIALS_EXPORT
+int CVodeSetEtaMin(void* cvode_mem, realtype eta_min);
+SUNDIALS_EXPORT
+int CVodeSetEtaMinErrFail(void *cvode_mem, realtype eta_min_ef);
+SUNDIALS_EXPORT
+int CVodeSetEtaMaxErrFail(void* cvode_mem, realtype eta_max_ef);
+SUNDIALS_EXPORT
+int CVodeSetNumFailsEtaMaxErrFail(void *cvode_mem, int small_nef);
+SUNDIALS_EXPORT
+int CVodeSetEtaConvFail(void* cvode_mem, realtype eta_cf);
 
 /* Rootfinding initialization function */
 SUNDIALS_EXPORT int CVodeRootInit(void *cvode_mem, int nrtfn, CVRootFn g);
@@ -284,9 +319,14 @@ SUNDIALS_EXPORT int CVodeGetNonlinearSystemDataSens(void *cvode_mem,
 SUNDIALS_EXPORT int CVodeGetNumNonlinSolvIters(void *cvode_mem,
                                                long int *nniters);
 SUNDIALS_EXPORT int CVodeGetNumNonlinSolvConvFails(void *cvode_mem,
-                                                   long int *nncfails);
+                                                   long int *nnfails);
 SUNDIALS_EXPORT int CVodeGetNonlinSolvStats(void *cvode_mem, long int *nniters,
-                                            long int *nncfails);
+                                            long int *nnfails);
+SUNDIALS_EXPORT int CVodeGetNumStepSolveFails(void *cvode_mem,
+                                              long int *nncfails);
+SUNDIALS_EXPORT int CVodeGetUserData(void *cvode_mem, void **user_data);
+SUNDIALS_EXPORT int CVodePrintAllStats(void *cvode_mem, FILE *outfile,
+                                       SUNOutputFormat fmt);
 SUNDIALS_EXPORT char *CVodeGetReturnFlagName(long int flag);
 
 /* Free function */
@@ -398,17 +438,21 @@ SUNDIALS_EXPORT int CVodeGetSensStats(void *cvode_mem, long int *nfSevals,
 SUNDIALS_EXPORT int CVodeGetSensNumNonlinSolvIters(void *cvode_mem,
                                                    long int *nSniters);
 SUNDIALS_EXPORT int CVodeGetSensNumNonlinSolvConvFails(void *cvode_mem,
-                                                       long int *nSncfails);
+                                                       long int *nSnfails);
 SUNDIALS_EXPORT int CVodeGetSensNonlinSolvStats(void *cvode_mem,
                                                 long int *nSniters,
-                                                long int *nSncfails);
+                                                long int *nSnfails);
+SUNDIALS_EXPORT int CVodeGetNumStepSensSolveFails(void *cvode_mem,
+                                                  long int *nSncfails);
 SUNDIALS_EXPORT int CVodeGetStgrSensNumNonlinSolvIters(void *cvode_mem,
                                                        long int *nSTGR1niters);
 SUNDIALS_EXPORT int CVodeGetStgrSensNumNonlinSolvConvFails(void *cvode_mem,
-                                                           long int *nSTGR1ncfails);
+                                                           long int *nSTGR1nfails);
 SUNDIALS_EXPORT int CVodeGetStgrSensNonlinSolvStats(void *cvode_mem,
                                                     long int *nSTGR1niters,
-                                                    long int *nSTGR1ncfails);
+                                                    long int *nSTGR1nfails);
+SUNDIALS_EXPORT int CVodeGetNumStepStgrSensSolveFails(void *cvode_mem,
+                                                      long int *nSTGR1ncfails);
 
 /* Free function */
 SUNDIALS_EXPORT void CVodeSensFree(void *cvode_mem);
