@@ -2,7 +2,7 @@
  * Programmer(s): Radu Serban @ LLNL
  * -----------------------------------------------------------------
  * SUNDIALS Copyright Start
- * Copyright (c) 2002-2021, Lawrence Livermore National Security
+ * Copyright (c) 2002-2022, Lawrence Livermore National Security
  * and Southern Methodist University.
  * All rights reserved.
  *
@@ -18,6 +18,7 @@
 #define _IDAS_H
 
 #include <stdio.h>
+#include <sundials/sundials_context.h>
 #include <sundials/sundials_nvector.h>
 #include <sundials/sundials_nonlinearsolver.h>
 #include <idas/idas_ls.h>
@@ -88,6 +89,8 @@ extern "C" {
 #define IDA_BAD_DKY         -27
 #define IDA_VECTOROP_ERR    -28
 
+#define IDA_CONTEXT_ERR     -29
+
 #define IDA_NO_QUAD         -30
 #define IDA_QRHS_FAIL       -31
 #define IDA_FIRST_QRHS_ERR  -32
@@ -175,7 +178,7 @@ typedef int (*IDAQuadRhsFnBS)(realtype t,
  * --------------------------------------- */
 
 /* Initialization functions */
-SUNDIALS_EXPORT void *IDACreate(void);
+SUNDIALS_EXPORT void *IDACreate(SUNContext sunctx);
 
 SUNDIALS_EXPORT int IDAInit(void *ida_mem, IDAResFn res, realtype t0,
                             N_Vector yy0, N_Vector yp0);
@@ -202,6 +205,7 @@ SUNDIALS_EXPORT int IDASetStepToleranceIC(void *ida_mem, realtype steptol);
 SUNDIALS_EXPORT int IDASetMaxBacksIC(void *ida_mem, int maxbacks);
 
 /* Optional input functions */
+SUNDIALS_EXPORT int IDASetDeltaCjLSetup(void *ida_max, realtype dcj);
 SUNDIALS_EXPORT int IDASetErrHandlerFn(void *ida_mem, IDAErrHandlerFn ehfun,
                                        void *eh_data);
 SUNDIALS_EXPORT int IDASetErrFile(void *ida_mem, FILE *errfp);
@@ -210,18 +214,35 @@ SUNDIALS_EXPORT int IDASetMaxOrd(void *ida_mem, int maxord);
 SUNDIALS_EXPORT int IDASetMaxNumSteps(void *ida_mem, long int mxsteps);
 SUNDIALS_EXPORT int IDASetInitStep(void *ida_mem, realtype hin);
 SUNDIALS_EXPORT int IDASetMaxStep(void *ida_mem, realtype hmax);
+SUNDIALS_EXPORT int IDASetMinStep(void *ida_mem, realtype hmin);
 SUNDIALS_EXPORT int IDASetStopTime(void *ida_mem, realtype tstop);
-SUNDIALS_EXPORT int IDASetNonlinConvCoef(void *ida_mem, realtype epcon);
 SUNDIALS_EXPORT int IDASetMaxErrTestFails(void *ida_mem, int maxnef);
-SUNDIALS_EXPORT int IDASetMaxNonlinIters(void *ida_mem, int maxcor);
-SUNDIALS_EXPORT int IDASetMaxConvFails(void *ida_mem, int maxncf);
 SUNDIALS_EXPORT int IDASetSuppressAlg(void *ida_mem, booleantype suppressalg);
 SUNDIALS_EXPORT int IDASetId(void *ida_mem, N_Vector id);
 SUNDIALS_EXPORT int IDASetConstraints(void *ida_mem, N_Vector constraints);
 
+/* Optional step adaptivity input functions */
+SUNDIALS_EXPORT
+int IDASetEtaFixedStepBounds(void *ida_mem, realtype eta_min_fx,
+                             realtype eta_max_fx);
+SUNDIALS_EXPORT
+int IDASetEtaMin(void *ida_mem, realtype eta_min);
+SUNDIALS_EXPORT
+int IDASetEtaMax(void *ida_mem, realtype eta_max);
+SUNDIALS_EXPORT
+int IDASetEtaLow(void *ida_mem, realtype eta_low);
+SUNDIALS_EXPORT
+int IDASetEtaMinErrFail(void *ida_mem, realtype eta_min_ef);
+SUNDIALS_EXPORT
+int IDASetEtaConvFail(void *ida_mem, realtype eta_cf);
+
+/* Nonlinear solve input functions */
+SUNDIALS_EXPORT int IDASetMaxConvFails(void *ida_mem, int maxncf);
+SUNDIALS_EXPORT int IDASetMaxNonlinIters(void *ida_mem, int maxcor);
+SUNDIALS_EXPORT int IDASetNlsResFn(void *IDA_mem, IDAResFn res);
+SUNDIALS_EXPORT int IDASetNonlinConvCoef(void *ida_mem, realtype epcon);
 SUNDIALS_EXPORT int IDASetNonlinearSolver(void *ida_mem,
                                           SUNNonlinearSolver NLS);
-SUNDIALS_EXPORT int IDASetNlsResFn(void *IDA_mem, IDAResFn res);
 
 /* Rootfinding initialization function */
 SUNDIALS_EXPORT int IDARootInit(void *ida_mem, int nrtfn, IDARootFn g);
@@ -291,9 +312,14 @@ SUNDIALS_EXPORT int IDAGetNonlinearSystemDataSens(void *ida_mem, realtype *tcur,
                                                   void **user_data);
 SUNDIALS_EXPORT int IDAGetNumNonlinSolvIters(void *ida_mem, long int *nniters);
 SUNDIALS_EXPORT int IDAGetNumNonlinSolvConvFails(void *ida_mem,
-                                                 long int *nncfails);
+                                                 long int *nnfails);
 SUNDIALS_EXPORT int IDAGetNonlinSolvStats(void *ida_mem, long int *nniters,
-                                          long int *nncfails);
+                                          long int *nnfails);
+SUNDIALS_EXPORT int IDAGetNumStepSolveFails(void *ida_mem,
+                                            long int *nncfails);
+SUNDIALS_EXPORT int IDAGetUserData(void *ida_mem, void **user_data);
+SUNDIALS_EXPORT int IDAPrintAllStats(void *ida_mem, FILE *outfile,
+                                     SUNOutputFormat fmt);
 SUNDIALS_EXPORT char *IDAGetReturnFlagName(long int flag);
 
 /* Free function */
@@ -401,10 +427,12 @@ SUNDIALS_EXPORT int IDAGetSensStats(void *ida_mem, long int *nresSevals,
 SUNDIALS_EXPORT int IDAGetSensNumNonlinSolvIters(void *ida_mem,
                                                  long int *nSniters);
 SUNDIALS_EXPORT int IDAGetSensNumNonlinSolvConvFails(void *ida_mem,
-                                                     long int *nSncfails);
+                                                     long int *nSnfails);
 SUNDIALS_EXPORT int IDAGetSensNonlinSolvStats(void *ida_mem,
                                               long int *nSniters,
-                                              long int *nSncfails);
+                                              long int *nSnfails);
+SUNDIALS_EXPORT int IDAGetNumStepSensSolveFails(void *ida_mem,
+                                                long int *nSncfails);
 
 /* Free function */
 SUNDIALS_EXPORT void IDASensFree(void *ida_mem);
