@@ -3,7 +3,7 @@
  * Programmer(s): Cody J. Balos @ LLNL
  * ----------------------------------------------------------------------------
  * SUNDIALS Copyright Start
- * Copyright (c) 2002-2021, Lawrence Livermore National Security
+ * Copyright (c) 2002-2022, Lawrence Livermore National Security
  * and Southern Methodist University.
  * All rights reserved.
  *
@@ -40,7 +40,6 @@
 #include <sundials/sundials_nvector.h>
 #include <sunmatrix/sunmatrix_slunrloc.h>
 
-#if (SUPERLU_DIST_MAJOR_VERSION >= 7) || ((SUPERLU_DIST_MAJOR_VERSION == 6) && (SUPERLU_DIST_MINOR_VERSION >= 3))
 #define xLUstructInit dLUstructInit
 #define xScalePermstructInit dScalePermstructInit
 #define xScalePermstructFree dScalePermstructFree
@@ -49,17 +48,6 @@
 #define xScalePermstruct_t dScalePermstruct_t
 #define xLUstruct_t dLUstruct_t
 #define xSOLVEstruct_t dSOLVEstruct_t
-#else
-#define xLUstructInit LUstructInit
-#define xScalePermstructInit ScalePermstructInit
-#define xScalePermstructFree ScalePermstructFree
-#define xLUstructFree LUstructFree
-#define xDestroy_LU Destroy_LU
-#define xScalePermstruct_t ScalePermstruct_t
-#define xLUstruct_t LUstruct_t
-#define xSOLVEstruct_t SOLVEstruct_t
-#endif
-
 
 #ifdef __cplusplus
 extern "C" {
@@ -100,7 +88,8 @@ SUNDIALS_EXPORT SUNLinearSolver SUNLinSol_SuperLUDIST(N_Vector y, SUNMatrix A,
                                                       xScalePermstruct_t *scaleperm,
                                                       xSOLVEstruct_t *solve,
                                                       SuperLUStat_t *stat,
-                                                      superlu_dist_options_t *options);
+                                                      superlu_dist_options_t *options,
+                                                      SUNContext sunctx);
 
 /*
  * ----------------------------------------------------------------------------
