@@ -2,7 +2,7 @@
  * Programmer(s): Daniel R. Reynolds @ SMU
  *---------------------------------------------------------------
  * SUNDIALS Copyright Start
- * Copyright (c) 2002-2021, Lawrence Livermore National Security
+ * Copyright (c) 2002-2022, Lawrence Livermore National Security
  * and Southern Methodist University.
  * All rights reserved.
  *
@@ -12,7 +12,7 @@
  * SUNDIALS Copyright End
  *---------------------------------------------------------------
  * This is the implementation file for the optional input and
- * output functions for the ARKode ARKStep time stepper module.
+ * output functions for the ARKODE ARKStep time stepper module.
  *
  * NOTE: many functions currently in arkode_io.c will move here,
  * with slightly different names.  The code transition will be
@@ -26,15 +26,9 @@
 #include <sundials/sundials_math.h>
 #include <sundials/sundials_types.h>
 
-#if defined(SUNDIALS_EXTENDED_PRECISION)
-#define RSYM "Lg"
-#else
-#define RSYM "g"
-#endif
-
 
 /*===============================================================
-  ARKStep Optional input functions (wrappers for generic ARKode
+  ARKStep Optional input functions (wrappers for generic ARKODE
   utility routines).  All are documented in arkode_io.c.
   ===============================================================*/
 int ARKStepSetDenseOrder(void *arkode_mem, int dord) {
@@ -155,13 +149,8 @@ int ARKStepSetMassTimes(void *arkode_mem, ARKLsMassTimesSetupFn msetup,
 int ARKStepSetLinSysFn(void *arkode_mem, ARKLsLinSysFn linsys) {
   return(arkLSSetLinSysFn(arkode_mem, linsys)); }
 
-/* deprecated */
-int ARKStepSetMaxStepsBetweenJac(void *arkode_mem, long int msbj) {
-  return(arkLSSetJacEvalFrequency(arkode_mem, msbj)); }
-
-
 /*===============================================================
-  ARKStep Optional output functions (wrappers for generic ARKode
+  ARKStep Optional output functions (wrappers for generic ARKODE
   utility routines).  All are documented in arkode_io.c.
   ===============================================================*/
 int ARKStepGetNumStepAttempts(void *arkode_mem, long int *nstep_attempts) {
@@ -202,6 +191,10 @@ int ARKStepGetNumAccSteps(void *arkode_mem, long int *nsteps) {
   return(arkGetNumAccSteps(arkode_mem, nsteps)); }
 int ARKStepGetNumErrTestFails(void *arkode_mem, long int *netfails) {
   return(arkGetNumErrTestFails(arkode_mem, netfails)); }
+int ARKStepGetNumStepSolveFails(void *arkode_mem, long int *nncfails) {
+  return(arkGetNumStepSolveFails(arkode_mem, nncfails)); }
+int ARKStepGetUserData(void *arkode_mem, void** user_data) {
+  return(arkGetUserData(arkode_mem, user_data)); }
 char *ARKStepGetReturnFlagName(long int flag) {
   return(arkGetReturnFlagName(flag)); }
 
@@ -280,17 +273,17 @@ int ARKStepSetUserData(void *arkode_mem, void *user_data)
                                  &ark_mem, &step_mem);
   if (retval != ARK_SUCCESS) return(retval);
 
-  /* set user_data in ARKode mem */
+  /* set user_data in ARKODE mem */
   retval = arkSetUserData(arkode_mem, user_data);
   if (retval != ARK_SUCCESS) return(retval);
 
-  /* set user data in ARKodeLS mem */
+  /* set user data in ARKODE LS mem */
   if (step_mem->lmem != NULL) {
     retval = arkLSSetUserData(arkode_mem, user_data);
     if (retval != ARKLS_SUCCESS) return(retval);
   }
 
-  /* set user data in ARKodeLSMass mem */
+  /* set user data in ARKODE LSMass mem */
   if (step_mem->mass_mem != NULL) {
     retval = arkLSSetMassUserData(arkode_mem, user_data);
     if (retval != ARKLS_SUCCESS) return(retval);
@@ -306,7 +299,7 @@ int ARKStepSetUserData(void *arkode_mem, void *user_data)
   Resets all ARKStep optional inputs to their default values.
   Does not change problem-defining function pointers or
   user_data pointer.  Also leaves alone any data
-  structures/options related to the ARKode infrastructure itself
+  structures/options related to the ARKODE infrastructure itself
   (e.g., root-finding and post-process step).
   ---------------------------------------------------------------*/
 int ARKStepSetDefaults(void* arkode_mem)
@@ -320,12 +313,12 @@ int ARKStepSetDefaults(void* arkode_mem)
                                  &ark_mem, &step_mem);
   if (retval != ARK_SUCCESS)  return(retval);
 
-  /* Set default ARKode infrastructure parameters */
+  /* Set default ARKODE infrastructure parameters */
   retval = arkSetDefaults(ark_mem);
   if (retval != ARK_SUCCESS) {
-    arkProcessError(NULL, ARK_MEM_NULL, "ARKode::ARKStep",
+    arkProcessError(NULL, ARK_MEM_NULL, "ARKODE::ARKStep",
                     "ARKStepSetDefaults",
-                    "Error setting ARKode infrastructure defaults");
+                    "Error setting ARKODE infrastructure defaults");
     return(retval);
   }
 
@@ -337,6 +330,7 @@ int ARKStepSetDefaults(void* arkode_mem)
   step_mem->linear_timedep   = SUNTRUE;        /* dfi/dy depends on t */
   step_mem->explicit         = SUNTRUE;        /* fe(t,y) will be used */
   step_mem->implicit         = SUNTRUE;        /* fi(t,y) will be used */
+  step_mem->deduce_rhs       = SUNFALSE;       /* deduce fi on result of NLS */
   step_mem->maxcor           = MAXCOR;         /* max nonlinear iters/stage */
   step_mem->nlscoef          = NLSCOEF;        /* nonlinear tolerance coefficient */
   step_mem->crdown           = CRDOWN;         /* nonlinear convergence estimate coeff. */
@@ -379,7 +373,7 @@ int ARKStepSetOptimalParams(void *arkode_mem)
 
   /* access ARKodeHAdaptMem structure */
   if (ark_mem->hadapt_mem == NULL) {
-    arkProcessError(ark_mem, ARK_MEM_NULL, "ARKode::ARKStep",
+    arkProcessError(ark_mem, ARK_MEM_NULL, "ARKODE::ARKStep",
                     "ARKStepSetOptimalParams",
                     MSG_ARKADAPT_NO_MEM);
     return(ARK_MEM_NULL);
@@ -659,7 +653,7 @@ int ARKStepSetExplicit(void *arkode_mem)
 
   /* ensure that fe is defined */
   if (step_mem->fe == NULL) {
-    arkProcessError(ark_mem, ARK_ILL_INPUT, "ARKode::ARKStep",
+    arkProcessError(ark_mem, ARK_ILL_INPUT, "ARKODE::ARKStep",
                     "ARKStepSetExplicit", MSG_ARK_MISSING_FE);
     return(ARK_ILL_INPUT);
   }
@@ -691,7 +685,7 @@ int ARKStepSetImplicit(void *arkode_mem)
 
   /* ensure that fi is defined */
   if (step_mem->fi == NULL) {
-    arkProcessError(ark_mem, ARK_ILL_INPUT, "ARKode::ARKStep",
+    arkProcessError(ark_mem, ARK_ILL_INPUT, "ARKODE::ARKStep",
                     "ARKStepSetImplicit", MSG_ARK_MISSING_FI);
     return(ARK_ILL_INPUT);
   }
@@ -732,12 +726,12 @@ int ARKStepSetImEx(void *arkode_mem)
 
   /* ensure that fe and fi are defined */
   if (step_mem->fe == NULL) {
-    arkProcessError(ark_mem, ARK_ILL_INPUT, "ARKode::ARKStep",
+    arkProcessError(ark_mem, ARK_ILL_INPUT, "ARKODE::ARKStep",
                     "ARKStepSetImEx", MSG_ARK_MISSING_FE);
     return(ARK_ILL_INPUT);
   }
   if (step_mem->fi == NULL) {
-    arkProcessError(ark_mem, ARK_ILL_INPUT, "ARKode::ARKStep",
+    arkProcessError(ark_mem, ARK_ILL_INPUT, "ARKODE::ARKStep",
                     "ARKStepSetImEx", MSG_ARK_MISSING_FI);
     return(ARK_ILL_INPUT);
   }
@@ -785,7 +779,7 @@ int ARKStepSetTables(void *arkode_mem, int q, int p,
 
   /* check for illegal inputs */
   if ((Bi == NULL) && (Be == NULL)) {
-    arkProcessError(ark_mem, ARK_MEM_NULL, "ARKode::ARKStep",
+    arkProcessError(ark_mem, ARK_MEM_NULL, "ARKODE::ARKStep",
                     "ARKStepSetTables",
                     "At least one complete table must be supplied");
     return(ARK_ILL_INPUT);
@@ -794,7 +788,7 @@ int ARKStepSetTables(void *arkode_mem, int q, int p,
   /* if both tables are set, check that they have the same number of stages */
   if ((Bi != NULL) && (Be != NULL)) {
     if (Bi->stages != Be->stages) {
-      arkProcessError(ark_mem, ARK_MEM_NULL, "ARKode::ARKStep",
+      arkProcessError(ark_mem, ARK_MEM_NULL, "ARKODE::ARKStep",
                       "ARKStepSetTables",
                       "Both tables must have the same number of stages");
       return(ARK_ILL_INPUT);
@@ -833,7 +827,7 @@ int ARKStepSetTables(void *arkode_mem, int q, int p,
     /* copy the table in step memory */
     step_mem->Be = ARKodeButcherTable_Copy(Be);
     if (step_mem->Be == NULL) {
-      arkProcessError(ark_mem, ARK_MEM_NULL, "ARKode::ARKStep",
+      arkProcessError(ark_mem, ARK_MEM_NULL, "ARKODE::ARKStep",
                       "ARKStepSetTables", MSG_ARK_NO_MEM);
       return(ARK_MEM_NULL);
     }
@@ -841,7 +835,7 @@ int ARKStepSetTables(void *arkode_mem, int q, int p,
     /* set method as purely explicit */
     retval = ARKStepSetExplicit(arkode_mem);
     if (retval != ARK_SUCCESS) {
-      arkProcessError(ark_mem, ARK_ILL_INPUT, "ARKode::ARKStep",
+      arkProcessError(ark_mem, ARK_ILL_INPUT, "ARKODE::ARKStep",
                       "ARKStepSetTables",
                       "Error in ARKStepSetExplicit");
       return(retval);
@@ -858,7 +852,7 @@ int ARKStepSetTables(void *arkode_mem, int q, int p,
     /* copy the table in step memory */
     step_mem->Bi = ARKodeButcherTable_Copy(Bi);
     if (step_mem->Bi == NULL) {
-      arkProcessError(ark_mem, ARK_MEM_NULL, "ARKode::ARKStep",
+      arkProcessError(ark_mem, ARK_MEM_NULL, "ARKODE::ARKStep",
                       "ARKStepSetTables", MSG_ARK_NO_MEM);
       return(ARK_MEM_NULL);
     }
@@ -866,7 +860,7 @@ int ARKStepSetTables(void *arkode_mem, int q, int p,
     /* set method as purely implicit */
     retval = ARKStepSetImplicit(arkode_mem);
     if (retval != ARK_SUCCESS) {
-      arkProcessError(ark_mem, ARK_ILL_INPUT, "ARKode::ARKStep",
+      arkProcessError(ark_mem, ARK_ILL_INPUT, "ARKODE::ARKStep",
                       "ARKStepSetTables",
                       "Error in ARKStepSetImplicit");
       return(ARK_ILL_INPUT);
@@ -883,7 +877,7 @@ int ARKStepSetTables(void *arkode_mem, int q, int p,
     /* copy the explicit table into step memory */
     step_mem->Be = ARKodeButcherTable_Copy(Be);
     if (step_mem->Be == NULL) {
-      arkProcessError(ark_mem, ARK_MEM_NULL, "ARKode::ARKStep",
+      arkProcessError(ark_mem, ARK_MEM_NULL, "ARKODE::ARKStep",
                       "ARKStepSetTables", MSG_ARK_NO_MEM);
       return(ARK_MEM_NULL);
     }
@@ -891,7 +885,7 @@ int ARKStepSetTables(void *arkode_mem, int q, int p,
     /* copy the implicit table into step memory */
     step_mem->Bi = ARKodeButcherTable_Copy(Bi);
     if (step_mem->Bi == NULL) {
-      arkProcessError(ark_mem, ARK_MEM_NULL, "ARKode::ARKStep",
+      arkProcessError(ark_mem, ARK_MEM_NULL, "ARKODE::ARKStep",
                       "ARKStepSetTables", MSG_ARK_NO_MEM);
       return(ARK_MEM_NULL);
     }
@@ -899,7 +893,7 @@ int ARKStepSetTables(void *arkode_mem, int q, int p,
     /* set method as ImEx */
     retval = ARKStepSetImEx(arkode_mem);
     if (retval != ARK_SUCCESS) {
-      arkProcessError(ark_mem, ARK_ILL_INPUT, "ARKode::ARKStep",
+      arkProcessError(ark_mem, ARK_ILL_INPUT, "ARKODE::ARKStep",
                       "ARKStepSetTables",
                       "Error in ARKStepSetImEx");
       return(ARK_ILL_INPUT);
@@ -931,7 +925,7 @@ int ARKStepSetTables(void *arkode_mem, int q, int p,
   If either argument is negative (illegal), then this disables the
   corresponding table (e.g. itable = -1  ->  explicit)
   ---------------------------------------------------------------*/
-int ARKStepSetTableNum(void *arkode_mem, int itable, int etable)
+int ARKStepSetTableNum(void *arkode_mem, ARKODE_DIRKTableID itable, ARKODE_ERKTableID etable)
 {
   int flag, retval;
   ARKodeMem ark_mem;
@@ -965,7 +959,7 @@ int ARKStepSetTableNum(void *arkode_mem, int itable, int etable)
 
   /*     illegal inputs */
   if ((itable < 0) && (etable < 0)) {
-    arkProcessError(ark_mem, ARK_MEM_NULL, "ARKode::ARKStep",
+    arkProcessError(ark_mem, ARK_MEM_NULL, "ARKODE::ARKStep",
                     "ARKStepSetTableNum",
                     "At least one valid table number must be supplied");
     return(ARK_ILL_INPUT);
@@ -975,8 +969,8 @@ int ARKStepSetTableNum(void *arkode_mem, int itable, int etable)
   } else if (itable < 0) {
 
     /* check that argument specifies an explicit table */
-    if (etable<MIN_ERK_NUM || etable>MAX_ERK_NUM) {
-      arkProcessError(ark_mem, ARK_MEM_NULL, "ARKode::ARKStep",
+    if (etable<ARKODE_MIN_ERK_NUM || etable>ARKODE_MAX_ERK_NUM) {
+      arkProcessError(ark_mem, ARK_MEM_NULL, "ARKODE::ARKStep",
                       "ARKStepSetTableNum",
                       "Illegal ERK table number");
       return(ARK_ILL_INPUT);
@@ -985,7 +979,7 @@ int ARKStepSetTableNum(void *arkode_mem, int itable, int etable)
     /* fill in table based on argument */
     step_mem->Be = ARKodeButcherTable_LoadERK(etable);
     if (step_mem->Be == NULL) {
-      arkProcessError(ark_mem, ARK_MEM_NULL, "ARKode::ARKStep",
+      arkProcessError(ark_mem, ARK_MEM_NULL, "ARKODE::ARKStep",
                       "ARKStepSetTableNum",
                       "Error setting explicit table with that index");
       return(ARK_ILL_INPUT);
@@ -997,7 +991,7 @@ int ARKStepSetTableNum(void *arkode_mem, int itable, int etable)
     /* set method as purely explicit */
     flag = ARKStepSetExplicit(arkode_mem);
     if (flag != ARK_SUCCESS) {
-      arkProcessError(ark_mem, ARK_ILL_INPUT, "ARKode::ARKStep",
+      arkProcessError(ark_mem, ARK_ILL_INPUT, "ARKODE::ARKStep",
                       "ARKStepSetTableNum",
                       "Error in ARKStepSetExplicit");
       return(flag);
@@ -1008,8 +1002,8 @@ int ARKStepSetTableNum(void *arkode_mem, int itable, int etable)
   } else if (etable < 0) {
 
     /* check that argument specifies an implicit table */
-    if (itable<MIN_DIRK_NUM || itable>MAX_DIRK_NUM) {
-      arkProcessError(ark_mem, ARK_MEM_NULL, "ARKode::ARKStep",
+    if (itable<ARKODE_MIN_DIRK_NUM || itable>ARKODE_MAX_DIRK_NUM) {
+      arkProcessError(ark_mem, ARK_MEM_NULL, "ARKODE::ARKStep",
                       "ARKStepSetTableNum",
                       "Illegal IRK table number");
       return(ARK_ILL_INPUT);
@@ -1018,7 +1012,7 @@ int ARKStepSetTableNum(void *arkode_mem, int itable, int etable)
     /* fill in table based on argument */
     step_mem->Bi = ARKodeButcherTable_LoadDIRK(itable);
     if (step_mem->Bi == NULL) {
-      arkProcessError(ark_mem, ARK_MEM_NULL, "ARKode::ARKStep",
+      arkProcessError(ark_mem, ARK_MEM_NULL, "ARKODE::ARKStep",
                       "ARKStepSetTableNum",
                       "Error setting table with that index");
       return(ARK_ILL_INPUT);
@@ -1030,9 +1024,9 @@ int ARKStepSetTableNum(void *arkode_mem, int itable, int etable)
     /* set method as purely implicit */
     flag = ARKStepSetImplicit(arkode_mem);
     if (flag != ARK_SUCCESS) {
-      arkProcessError(ark_mem, ARK_ILL_INPUT, "ARKode::ARKStep",
+      arkProcessError(ark_mem, ARK_ILL_INPUT, "ARKODE::ARKStep",
                       "ARKStepSetTableNum",
-                      "Error in ARKStepSetIxplicit");
+                      "Error in ARKStepSetImplicit");
       return(flag);
     }
 
@@ -1041,12 +1035,12 @@ int ARKStepSetTableNum(void *arkode_mem, int itable, int etable)
   } else {
 
     /* ensure that tables match */
-    if ( !((etable == ARK324L2SA_ERK_4_2_3) && (itable == ARK324L2SA_DIRK_4_2_3)) &&
-         !((etable == ARK436L2SA_ERK_6_3_4) && (itable == ARK436L2SA_DIRK_6_3_4)) &&
-         !((etable == ARK437L2SA_ERK_7_3_4) && (itable == ARK437L2SA_DIRK_7_3_4)) &&
-         !((etable == ARK548L2SA_ERK_8_4_5) && (itable == ARK548L2SA_DIRK_8_4_5)) &&
-         !((etable == ARK548L2SAb_ERK_8_4_5) && (itable == ARK548L2SAb_DIRK_8_4_5)) ) {
-      arkProcessError(ark_mem, ARK_ILL_INPUT, "ARKode::ARKStep",
+    if ( !((etable == ARKODE_ARK324L2SA_ERK_4_2_3) && (itable == ARKODE_ARK324L2SA_DIRK_4_2_3)) &&
+         !((etable == ARKODE_ARK436L2SA_ERK_6_3_4) && (itable == ARKODE_ARK436L2SA_DIRK_6_3_4)) &&
+         !((etable == ARKODE_ARK437L2SA_ERK_7_3_4) && (itable == ARKODE_ARK437L2SA_DIRK_7_3_4)) &&
+         !((etable == ARKODE_ARK548L2SA_ERK_8_4_5) && (itable == ARKODE_ARK548L2SA_DIRK_8_4_5)) &&
+         !((etable == ARKODE_ARK548L2SAb_ERK_8_4_5) && (itable == ARKODE_ARK548L2SAb_DIRK_8_4_5)) ) {
+      arkProcessError(ark_mem, ARK_ILL_INPUT, "ARKODE::ARKStep",
                       "ARKStepSetTableNum",
                       "Incompatible Butcher tables for ARK method");
       return(ARK_ILL_INPUT);
@@ -1056,13 +1050,13 @@ int ARKStepSetTableNum(void *arkode_mem, int itable, int etable)
     step_mem->Bi = ARKodeButcherTable_LoadDIRK(itable);
     step_mem->Be = ARKodeButcherTable_LoadERK(etable);
     if (step_mem->Bi == NULL) {
-      arkProcessError(ark_mem, ARK_MEM_NULL, "ARKode::ARKStep",
+      arkProcessError(ark_mem, ARK_MEM_NULL, "ARKODE::ARKStep",
                       "ARKStepSetTableNum",
                       "Illegal IRK table number");
       return(ARK_ILL_INPUT);
     }
     if (step_mem->Be == NULL) {
-      arkProcessError(ark_mem, ARK_MEM_NULL, "ARKode::ARKStep",
+      arkProcessError(ark_mem, ARK_MEM_NULL, "ARKODE::ARKStep",
                       "ARKStepSetTableNum",
                       "Illegal ERK table number");
       return(ARK_ILL_INPUT);
@@ -1073,7 +1067,7 @@ int ARKStepSetTableNum(void *arkode_mem, int itable, int etable)
 
     /* set method as ImEx */
     if (ARKStepSetImEx(arkode_mem) != ARK_SUCCESS) {
-      arkProcessError(ark_mem, ARK_ILL_INPUT, "ARKode::ARKStep",
+      arkProcessError(ark_mem, ARK_ILL_INPUT, "ARKODE::ARKStep",
                       "ARKStepSetTableNum", MSG_ARK_MISSING_F);
       return(ARK_ILL_INPUT);
     }
@@ -1084,6 +1078,27 @@ int ARKStepSetTableNum(void *arkode_mem, int itable, int etable)
 }
 
 
+/*---------------------------------------------------------------
+  ARKStepSetTableName:
+
+  Specifies to use pre-existing Butcher tables for the system,
+  based on the string passed to
+  ARKodeButcherTable_LoadERKByName() and
+  ARKodeButcherTable_LoadDIRKByName() within the files
+  arkode_butcher_erk.c and arkode_butcher_dirk.c (automatically
+  calls ARKStepSetImEx).
+
+  If itable is "ARKODE_DIRK_NONE" or etable is "ARKODE_ERK_NONE",
+  then this disables the corresponding table.
+  ---------------------------------------------------------------*/
+int ARKStepSetTableName(void *arkode_mem, const char *itable, const char *etable)
+{
+  return ARKStepSetTableNum(arkode_mem,
+                            arkButcherTableDIRKNameToID(itable),
+                            arkButcherTableERKNameToID(etable));
+}
+
+
 /*---------------------------------------------------------------
   ARKStepSetNonlinCRDown:
 
@@ -1200,13 +1215,6 @@ int ARKStepSetLSetupFrequency(void *arkode_mem, int msbp)
   return(ARK_SUCCESS);
 }
 
-/* Deprecated */
-int ARKStepSetMaxStepsBetweenLSet(void *arkode_mem, int msbp)
-{
-  return(ARKStepSetLSetupFrequency(arkode_mem, msbp));
-}
-
-
 /*---------------------------------------------------------------
   ARKStepSetPredictorMethod:
 
@@ -1228,11 +1236,21 @@ int ARKStepSetPredictorMethod(void *arkode_mem, int pred_method)
   /* return error if pred_method==5 and a non-NULL stage predictor function
      has been supplied */
   if ((pred_method == 5) && (step_mem->stage_predict != NULL)) {
-    arkProcessError(ark_mem, ARK_ILL_INPUT, "ARKode::ARKStep", "ARKStepSetPredictorMethod",
+    arkProcessError(ark_mem, ARK_ILL_INPUT, "ARKODE::ARKStep", "ARKStepSetPredictorMethod",
                     "predictor 5 cannot be combined with user-supplied stage predictor");
     return(ARK_ILL_INPUT);
   }
 
+  /* Deprecate options 4 and 5 */
+  if (pred_method == 4) {
+    arkProcessError(ark_mem, ARK_ILL_INPUT, "ARKODE::ARKStep", "ARKStepSetPredictorMethod",
+                    "Predictor option 4 is deprecated, and will be removed in an upcoming release");
+  }
+  if (pred_method == 5) {
+    arkProcessError(ark_mem, ARK_ILL_INPUT, "ARKODE::ARKStep", "ARKStepSetPredictorMethod",
+                    "Predictor option 5 is deprecated, and will be removed in an upcoming release");
+  }
+
   /* set parameter */
   step_mem->predictor = pred_method;
 
@@ -1260,7 +1278,7 @@ int ARKStepSetMaxNonlinIters(void *arkode_mem, int maxcor)
 
   /* Return error message if no NLS module is present */
   if (step_mem->NLS == NULL) {
-    arkProcessError(ark_mem, ARK_NLS_OP_ERR, "ARKode::ARKStep",
+    arkProcessError(ark_mem, ARK_NLS_OP_ERR, "ARKODE::ARKStep",
                     "ARKStepSetMaxNonlinIters",
                     "No SUNNonlinearSolver object is present");
     return(ARK_ILL_INPUT);
@@ -1276,7 +1294,7 @@ int ARKStepSetMaxNonlinIters(void *arkode_mem, int maxcor)
   /* send argument to NLS structure */
   retval = SUNNonlinSolSetMaxIters(step_mem->NLS, step_mem->maxcor);
   if (retval != SUN_NLS_SUCCESS) {
-    arkProcessError(ark_mem, ARK_NLS_OP_ERR, "ARKode::ARKStep",
+    arkProcessError(ark_mem, ARK_NLS_OP_ERR, "ARKODE::ARKStep",
                     "ARKStepSetMaxNonlinIters",
                     "Error setting maxcor in SUNNonlinearSolver object");
     return(ARK_NLS_OP_ERR);
@@ -1333,7 +1351,7 @@ int ARKStepSetStagePredictFn(void *arkode_mem,
 
   /* override predictor method 5 if non-NULL PredictStage is supplied */
   if ((step_mem->predictor == 5) && (PredictStage != NULL)) {
-    arkProcessError(ark_mem, ARK_ILL_INPUT, "ARKode::ARKStep",
+    arkProcessError(ark_mem, ARK_ILL_INPUT, "ARKODE::ARKStep",
                     "ARKStepSetStagePredictFn",
                     "User-supplied predictor is incompatible with predictor method 5");
     return(ARK_ILL_INPUT);
@@ -1344,6 +1362,34 @@ int ARKStepSetStagePredictFn(void *arkode_mem,
 }
 
 
+/*---------------------------------------------------------------
+  ARKStepSetDeduceImplicitRhs:
+
+  Specifies if an optimization is used to avoid an evaluation of
+  fi after a nonlinear solve for an implicit stage.  If stage
+  postprocessecing in enabled, this option is ignored, and fi is
+  never deduced.
+
+  An argument of SUNTRUE indicates that fi is deduced to compute
+  fi(z_i), and SUNFALSE indicates that fi(z_i) is computed with
+  an additional evaluation of fi.
+  ---------------------------------------------------------------*/
+int ARKStepSetDeduceImplicitRhs(void *arkode_mem, sunbooleantype deduce)
+{
+  ARKodeMem        ark_mem;
+  ARKodeARKStepMem step_mem;
+  int              retval;
+
+  /* access ARKodeARKStepMem structure and set function pointer */
+  retval = arkStep_AccessStepMem(arkode_mem, "ARKStepSetDeduceImplicitRhs",
+                                 &ark_mem, &step_mem);
+  if (retval != ARK_SUCCESS) return(retval);
+
+  step_mem->deduce_rhs = deduce;
+  return(ARK_SUCCESS);
+}
+
+
 /*===============================================================
   ARKStep optional output functions -- stepper-specific
   ===============================================================*/
@@ -1523,7 +1569,7 @@ int ARKStepGetNumNonlinSolvIters(void *arkode_mem, long int *nniters)
 
   Returns the current number of nonlinear solver convergence fails
   ---------------------------------------------------------------*/
-int ARKStepGetNumNonlinSolvConvFails(void *arkode_mem, long int *nncfails)
+int ARKStepGetNumNonlinSolvConvFails(void *arkode_mem, long int *nnfails)
 {
   ARKodeMem ark_mem;
   ARKodeARKStepMem step_mem;
@@ -1535,7 +1581,7 @@ int ARKStepGetNumNonlinSolvConvFails(void *arkode_mem, long int *nncfails)
   if (retval != ARK_SUCCESS)  return(retval);
 
   /* set output from step_mem */
-  *nncfails = ark_mem->ncfn;
+  *nnfails = step_mem->nls_fails;
 
   return(ARK_SUCCESS);
 }
@@ -1547,7 +1593,7 @@ int ARKStepGetNumNonlinSolvConvFails(void *arkode_mem, long int *nncfails)
   Returns nonlinear solver statistics
   ---------------------------------------------------------------*/
 int ARKStepGetNonlinSolvStats(void *arkode_mem, long int *nniters,
-                              long int *nncfails)
+                              long int *nnfails)
 {
   ARKodeMem ark_mem;
   ARKodeARKStepMem step_mem;
@@ -1558,8 +1604,159 @@ int ARKStepGetNonlinSolvStats(void *arkode_mem, long int *nniters,
                                  &ark_mem, &step_mem);
   if (retval != ARK_SUCCESS)  return(retval);
 
-  *nniters  = step_mem->nls_iters;
-  *nncfails = ark_mem->ncfn;
+  *nniters = step_mem->nls_iters;
+  *nnfails = step_mem->nls_fails;
+
+  return(ARK_SUCCESS);
+}
+
+
+/*---------------------------------------------------------------
+  ARKStepPrintAllStats:
+
+  Prints integrator statistics
+  ---------------------------------------------------------------*/
+int ARKStepPrintAllStats(void *arkode_mem, FILE *outfile, SUNOutputFormat fmt)
+{
+  ARKodeMem ark_mem;
+  ARKodeARKStepMem step_mem;
+  ARKLsMem arkls_mem;
+  ARKLsMassMem arklsm_mem;
+  int retval;
+
+  /* access ARKodeARKStepMem structure */
+  retval = arkStep_AccessStepMem(arkode_mem, "ARKStepPrintAllStats",
+                                 &ark_mem, &step_mem);
+  if (retval != ARK_SUCCESS) return(retval);
+
+  /* step and rootfinding stats */
+  retval = arkPrintAllStats(arkode_mem, outfile, fmt);
+  if (retval != ARK_SUCCESS) return(retval);
+
+  switch(fmt)
+  {
+  case SUN_OUTPUTFORMAT_TABLE:
+    /* function evaluations */
+    fprintf(outfile, "Explicit RHS fn evals        = %ld\n", step_mem->nfe);
+    fprintf(outfile, "Implicit RHS fn evals        = %ld\n", step_mem->nfi);
+
+    /* nonlinear solver stats */
+    fprintf(outfile, "NLS iters                    = %ld\n", step_mem->nls_iters);
+    fprintf(outfile, "NLS fails                    = %ld\n", step_mem->nls_fails);
+    if (ark_mem->nst > 0)
+    {
+      fprintf(outfile, "NLS iters per step           = %"RSYM"\n",
+              (realtype) step_mem->nls_iters / (realtype) ark_mem->nst);
+    }
+
+    /* linear solver stats */
+    fprintf(outfile, "LS setups                    = %ld\n", step_mem->nsetups);
+    if (ark_mem->step_getlinmem(arkode_mem))
+    {
+      arkls_mem = (ARKLsMem) (ark_mem->step_getlinmem(arkode_mem));
+      fprintf(outfile, "Jac fn evals                 = %ld\n", arkls_mem->nje);
+      fprintf(outfile, "LS RHS fn evals              = %ld\n", arkls_mem->nfeDQ);
+      fprintf(outfile, "Prec setup evals             = %ld\n", arkls_mem->npe);
+      fprintf(outfile, "Prec solves                  = %ld\n", arkls_mem->nps);
+      fprintf(outfile, "LS iters                     = %ld\n", arkls_mem->nli);
+      fprintf(outfile, "LS fails                     = %ld\n", arkls_mem->ncfl);
+      fprintf(outfile, "Jac-times setups             = %ld\n", arkls_mem->njtsetup);
+      fprintf(outfile, "Jac-times evals              = %ld\n", arkls_mem->njtimes);
+      if (step_mem->nls_iters > 0)
+      {
+        fprintf(outfile, "LS iters per NLS iter        = %"RSYM"\n",
+                (realtype) arkls_mem->nli / (realtype) step_mem->nls_iters);
+        fprintf(outfile, "Jac evals per NLS iter       = %"RSYM"\n",
+                (realtype) arkls_mem->nje / (realtype) step_mem->nls_iters);
+        fprintf(outfile, "Prec evals per NLS iter      = %"RSYM"\n",
+                (realtype) arkls_mem->npe / (realtype) step_mem->nls_iters);
+      }
+    }
+
+    /* mass solve stats */
+    if (ark_mem->step_getmassmem(arkode_mem))
+    {
+      arklsm_mem = (ARKLsMassMem) (ark_mem->step_getmassmem(arkode_mem));
+      fprintf(outfile, "Mass setups                  = %ld\n", arklsm_mem->nmsetups);
+      fprintf(outfile, "Mass solves                  = %ld\n", arklsm_mem->nmsolves);
+      fprintf(outfile, "Mass Prec setup evals        = %ld\n", arklsm_mem->npe);
+      fprintf(outfile, "Mass Prec solves             = %ld\n", arklsm_mem->nps);
+      fprintf(outfile, "Mass LS iters                = %ld\n", arklsm_mem->nli);
+      fprintf(outfile, "Mass LS fails                = %ld\n", arklsm_mem->ncfl);
+      fprintf(outfile, "Mass-times setups            = %ld\n", arklsm_mem->nmtsetup);
+      fprintf(outfile, "Mass-times evals             = %ld\n", arklsm_mem->nmtimes);
+    }
+    break;
+
+  case SUN_OUTPUTFORMAT_CSV:
+    /* function evaluations */
+    fprintf(outfile, ",Explicit RHS fn evals,%ld", step_mem->nfe);
+    fprintf(outfile, ",Implicit RHS fn evals,%ld", step_mem->nfi);
+
+    /* nonlinear solver stats */
+    fprintf(outfile, ",NLS iters,%ld", step_mem->nls_iters);
+    fprintf(outfile, ",NLS fails,%ld", step_mem->nls_fails);
+    if (ark_mem->nst > 0)
+    {
+      fprintf(outfile, ",NLS iters per step,%"RSYM,
+              (realtype) step_mem->nls_iters / (realtype) ark_mem->nst);
+    }
+    else
+    {
+      fprintf(outfile, ",NLS iters per step,0");
+    }
+
+    /* linear solver stats */
+    fprintf(outfile, ",LS setups,%ld", step_mem->nsetups);
+    if (ark_mem->step_getlinmem(arkode_mem))
+    {
+      arkls_mem = (ARKLsMem) (ark_mem->step_getlinmem(arkode_mem));
+      fprintf(outfile, ",Jac fn evals,%ld", arkls_mem->nje);
+      fprintf(outfile, ",LS RHS fn evals,%ld", arkls_mem->nfeDQ);
+      fprintf(outfile, ",Prec setup evals,%ld", arkls_mem->npe);
+      fprintf(outfile, ",Prec solves,%ld", arkls_mem->nps);
+      fprintf(outfile, ",LS iters,%ld", arkls_mem->nli);
+      fprintf(outfile, ",LS fails,%ld", arkls_mem->ncfl);
+      fprintf(outfile, ",Jac-times setups,%ld", arkls_mem->njtsetup);
+      fprintf(outfile, ",Jac-times evals,%ld", arkls_mem->njtimes);
+      if (step_mem->nls_iters > 0)
+      {
+        fprintf(outfile, ",LS iters per NLS iter,%"RSYM,
+                (realtype) arkls_mem->nli / (realtype) step_mem->nls_iters);
+        fprintf(outfile, ",Jac evals per NLS iter,%"RSYM,
+                (realtype) arkls_mem->nje / (realtype) step_mem->nls_iters);
+        fprintf(outfile, ",Prec evals per NLS iter,%"RSYM,
+                (realtype) arkls_mem->npe / (realtype) step_mem->nls_iters);
+      }
+      else
+      {
+        fprintf(outfile, ",LS iters per NLS iter,0");
+        fprintf(outfile, ",Jac evals per NLS iter,0");
+        fprintf(outfile, ",Prec evals per NLS iter,0");
+      }
+    }
+
+    /* mass solve stats */
+    if (ark_mem->step_getmassmem(arkode_mem))
+    {
+      arklsm_mem = (ARKLsMassMem) (ark_mem->step_getmassmem(arkode_mem));
+      fprintf(outfile, ",Mass setups,%ld", arklsm_mem->nmsetups);
+      fprintf(outfile, ",Mass solves,%ld", arklsm_mem->nmsolves);
+      fprintf(outfile, ",Mass Prec setup evals,%ld", arklsm_mem->npe);
+      fprintf(outfile, ",Mass Prec solves,%ld", arklsm_mem->nps);
+      fprintf(outfile, ",Mass LS iters,%ld", arklsm_mem->nli);
+      fprintf(outfile, ",Mass LS fails,%ld", arklsm_mem->ncfl);
+      fprintf(outfile, ",Mass-times setups,%ld", arklsm_mem->nmtsetup);
+      fprintf(outfile, ",Mass-times evals,%ld", arklsm_mem->nmtimes);
+    }
+    fprintf(outfile, "\n");
+    break;
+
+  default:
+    arkProcessError(ark_mem, ARK_ILL_INPUT, "ARKODE", "ARKStepPrintAllStats",
+                    "Invalid formatting option.");
+    return(ARK_ILL_INPUT);
+  }
 
   return(ARK_SUCCESS);
 }
@@ -1585,12 +1782,12 @@ int ARKStepWriteParameters(void *arkode_mem, FILE *fp)
                                  &ark_mem, &step_mem);
   if (retval != ARK_SUCCESS)  return(retval);
 
-  /* output ARKode infrastructure parameters first */
+  /* output ARKODE infrastructure parameters first */
   flag = arkWriteParameters(ark_mem, fp);
   if (flag != ARK_SUCCESS) {
-    arkProcessError(ark_mem, ARK_MEM_NULL, "ARKode::ARKStep",
+    arkProcessError(ark_mem, ARK_MEM_NULL, "ARKODE::ARKStep",
                     "ARKStepWriteParameters",
-                    "Error writing ARKode infrastructure parameters");
+                    "Error writing ARKODE infrastructure parameters");
     return(flag);
   }
 
@@ -1646,7 +1843,7 @@ int ARKStepWriteButcher(void *arkode_mem, FILE *fp)
 
   /* check that Butcher table is non-NULL (otherwise report error) */
   if ((step_mem->Be == NULL) && (step_mem->Bi == NULL)) {
-    arkProcessError(ark_mem, ARK_MEM_NULL, "ARKode::ARKStep",
+    arkProcessError(ark_mem, ARK_MEM_NULL, "ARKODE::ARKStep",
                     "ARKStepWriteButcher", "Butcher table memory is NULL");
     return(ARK_MEM_NULL);
   }
