@@ -2,7 +2,7 @@
  * Programmer(s): Daniel R. Reynolds @ SMU
  *---------------------------------------------------------------
  * SUNDIALS Copyright Start
- * Copyright (c) 2002-2019, Lawrence Livermore National Security
+ * Copyright (c) 2002-2022, Lawrence Livermore National Security
  * and Southern Methodist University.
  * All rights reserved.
  *
@@ -44,6 +44,7 @@ realtype *ARK_rout;
 int       ARK_nrtfn;
 int       ARK_ls;
 int       ARK_mass_ls;
+int       ARK_constr;
 
 /*=============================================================*/
 
@@ -105,6 +106,7 @@ void FARK_MALLOC(realtype *t0, realtype *y0, int *imex,
   ARK_nrtfn = 0;
   ARK_ls = SUNFALSE;
   ARK_mass_ls = SUNFALSE;
+  ARK_constr = SUNFALSE;
 
   /* Set data in F2C_ARKODE_vec to y0 */
   N_VSetArrayPointer(y0, F2C_ARKODE_vec);
@@ -112,16 +114,16 @@ void FARK_MALLOC(realtype *t0, realtype *y0, int *imex,
   /* Call ARKStepCreate based on imex argument */
   switch (*imex) {
   case 0:  /* purely implicit */
-    ARK_arkodemem = ARKStepCreate(NULL, FARKfi, *t0, F2C_ARKODE_vec);
+    ARK_arkodemem = ARKStepCreate(NULL, FARKfi, *t0, F2C_ARKODE_vec, NULL);
     break;
   case 1:  /* purely explicit */
-    ARK_arkodemem = ARKStepCreate(FARKfe, NULL, *t0, F2C_ARKODE_vec);
+    ARK_arkodemem = ARKStepCreate(FARKfe, NULL, *t0, F2C_ARKODE_vec, NULL);
     FARKNullMatrix();
     FARKNullLinsol();
     FARKNullNonlinsol();
     break;
   case 2:  /* imex */
-    ARK_arkodemem = ARKStepCreate(FARKfe, FARKfi, *t0, F2C_ARKODE_vec);
+    ARK_arkodemem = ARKStepCreate(FARKfe, FARKfi, *t0, F2C_ARKODE_vec, NULL);
     break;
   }
   if (ARK_arkodemem == NULL) {
@@ -166,7 +168,6 @@ void FARK_MALLOC(realtype *t0, realtype *y0, int *imex,
       return;
     }
     N_VSetArrayPointer(atol, Vatol);
-    if (N_VMin(Vatol) <= ZERO)  N_VConst(abstol, Vatol);
     *ier = ARKStepSVtolerances(ARK_arkodemem, reltol, Vatol);
     N_VDestroy(Vatol);
     break;
@@ -250,7 +251,6 @@ void FARK_REINIT(realtype *t0, realtype *y0, int *imex, int *iatol,
       return;
     }
     N_VSetArrayPointer(atol, Vatol);
-    if (N_VMin(Vatol) <= ZERO)  N_VConst(abstol, Vatol);
     *ier = ARKStepSVtolerances(ARK_arkodemem, reltol, Vatol);
     N_VDestroy(Vatol);
     break;
@@ -360,6 +360,8 @@ void FARK_SETIIN(char key_name[], long int *ival, int *ier) {
     *ier = ARKStepSetSmallNumEFails(ARK_arkodemem, (int) *ival);
   else if (!strncmp(key_name, "LSETUP_MSBP", 11))
     *ier = ARKStepSetMaxStepsBetweenLSet(ARK_arkodemem, (int) *ival);
+  else if (!strncmp(key_name, "MAX_CONSTR_FAIL", 15))
+    *ier = ARKStepSetMaxNumConstrFails(ARK_arkodemem, (int) *ival);
   else {
     *ier = -99;
     fprintf(stderr, "FARKSETIIN: Unrecognized key.\n\n");
@@ -415,6 +417,32 @@ void FARK_SETRIN(char key_name[], realtype *rval, int *ier) {
 
 /*=============================================================*/
 
+void FARK_SETVIN(char key_name[], realtype *vval, int *ier)
+{
+  N_Vector Vec;
+
+  *ier = 0;
+
+  if (!strncmp(key_name,"CONSTR_VEC",10)) {
+    Vec = NULL;
+    Vec = N_VCloneEmpty(F2C_ARKODE_vec);
+    if (Vec == NULL) {
+      *ier = -1;
+      return;
+    }
+    N_VSetArrayPointer(vval, Vec);
+    *ier = ARKStepSetConstraints(ARK_arkodemem, Vec);
+    ARK_constr = SUNTRUE;
+    N_VDestroy(Vec);
+  }
+  else {
+    *ier = -99;
+    fprintf(stderr, "FARKSETVIN: Unrecognized key. \n\n");
+  }
+
+}
+/*=============================================================*/
+
 /* Fortran interface to C routine ARKStepSetAdaptivityMethod;
    see farkode.h for further details */
 void FARK_SETADAPTMETHOD(int *imethod, int *idefault, int *ipq,
@@ -624,7 +652,7 @@ void FARK_LSMASSINIT(int *time_dep, int *ier) {
   }
   *ier = ARKStepSetMassLinearSolver(ARK_arkodemem,
                                     F2C_ARKODE_mass_sol,
-                                    F2C_ARKODE_mass_matrix, 
+                                    F2C_ARKODE_mass_matrix,
                                     *time_dep);
   ARK_mass_ls = SUNTRUE;
   return;
@@ -698,7 +726,7 @@ void FARK_ARKODE(realtype *tout, realtype *t, realtype *y,
     ARKStepGetNumLinIters(ARK_arkodemem,     &ARK_iout[22]);            /* NLI   */
     ARKStepGetNumLinConvFails(ARK_arkodemem, &ARK_iout[23]);            /* NCFL  */
   }
-  
+
   /* Attach mass matrix linear solver outputs */
   if(ARK_mass_ls) {
     ARKStepGetMassWorkSpace(ARK_arkodemem, &ARK_iout[24], &ARK_iout[25]); /* LENRWMS, LENIWMS */
@@ -712,6 +740,11 @@ void FARK_ARKODE(realtype *tout, realtype *t, realtype *y,
     ARKStepGetNumMassIters(ARK_arkodemem,      &ARK_iout[33]);            /* NMLI   */
     ARKStepGetNumMassConvFails(ARK_arkodemem,  &ARK_iout[34]);            /* NMCFL  */
   }
+
+  /* Attach constraints output */
+  if(ARK_constr) {
+    ARKStepGetNumConstrFails(ARK_arkodemem, &ARK_iout[35]);
+  }
   return;
 }
 
@@ -828,6 +861,8 @@ void FARK_FREE() {
     SUNLinSolFree(F2C_ARKODE_linsol);
   if (F2C_ARKODE_mass_sol)
     SUNLinSolFree(F2C_ARKODE_mass_sol);
+  if (F2C_ARKODE_nonlinsol)
+    SUNNonlinSolFree(F2C_ARKODE_nonlinsol);
   return;
 }
 
