@@ -1,11 +1,11 @@
 /*
- * ----------------------------------------------------------------- 
+ * -----------------------------------------------------------------
  * Programmer(s): Daniel R. Reynolds @ SMU
- *    Michael Wittman, Alan C. Hindmarsh, Radu Serban, and 
+ *    Michael Wittman, Alan C. Hindmarsh, Radu Serban, and
  *    Aaron Collier @ LLNL
  * -----------------------------------------------------------------
  * SUNDIALS Copyright Start
- * Copyright (c) 2002-2019, Lawrence Livermore National Security
+ * Copyright (c) 2002-2022, Lawrence Livermore National Security
  * and Southern Methodist University.
  * All rights reserved.
  *
@@ -36,11 +36,11 @@
 #define TWO          RCONST(2.0)
 
 /* Prototypes of functions CVBBDPrecSetup and CVBBDPrecSolve */
-static int CVBBDPrecSetup(realtype t, N_Vector y, N_Vector fy, 
-                          booleantype jok, booleantype *jcurPtr, 
+static int CVBBDPrecSetup(realtype t, N_Vector y, N_Vector fy,
+                          booleantype jok, booleantype *jcurPtr,
                           realtype gamma, void *bbd_data);
-static int CVBBDPrecSolve(realtype t, N_Vector y, N_Vector fy, 
-                          N_Vector r, N_Vector z, 
+static int CVBBDPrecSolve(realtype t, N_Vector y, N_Vector fy,
+                          N_Vector r, N_Vector z,
                           realtype gamma, realtype delta,
                           int lr, void *bbd_data);
 
@@ -48,16 +48,16 @@ static int CVBBDPrecSolve(realtype t, N_Vector y, N_Vector fy,
 static int CVBBDPrecFree(CVodeMem cv_mem);
 
 /* Prototype for difference quotient Jacobian calculation routine */
-static int CVBBDDQJac(CVBBDPrecData pdata, realtype t, 
-                      N_Vector y, N_Vector gy, 
+static int CVBBDDQJac(CVBBDPrecData pdata, realtype t,
+                      N_Vector y, N_Vector gy,
                       N_Vector ytemp, N_Vector gtemp);
 
 /*-----------------------------------------------------------------
   User-Callable Functions: initialization, reinit and free
   -----------------------------------------------------------------*/
-int CVBBDPrecInit(void *cvode_mem, sunindextype Nlocal, 
+int CVBBDPrecInit(void *cvode_mem, sunindextype Nlocal,
                   sunindextype mudq, sunindextype mldq,
-                  sunindextype mukeep, sunindextype mlkeep, 
+                  sunindextype mukeep, sunindextype mlkeep,
                   realtype dqrely, CVLocalFn gloc, CVCommFn cfn)
 {
   CVodeMem cv_mem;
@@ -91,7 +91,7 @@ int CVBBDPrecInit(void *cvode_mem, sunindextype Nlocal,
 
   /* Allocate data memory */
   pdata = NULL;
-  pdata = (CVBBDPrecData) malloc(sizeof *pdata);  
+  pdata = (CVBBDPrecData) malloc(sizeof *pdata);
   if (pdata == NULL) {
     cvProcessError(cv_mem, CVLS_MEM_FAIL, "CVBBDPRE",
                    "CVBBDPrecInit", MSGBBD_MEM_FAIL);
@@ -110,18 +110,18 @@ int CVBBDPrecInit(void *cvode_mem, sunindextype Nlocal,
   pdata->mlkeep = mlk;
 
   /* Allocate memory for saved Jacobian */
-  pdata->savedJ = SUNBandMatrixStorage(Nlocal, muk, mlk, muk);
-  if (pdata->savedJ == NULL) { 
-    free(pdata); pdata = NULL; 
+  pdata->savedJ = SUNBandMatrixStorage(Nlocal, muk, mlk, muk, cv_mem->cv_sunctx);
+  if (pdata->savedJ == NULL) {
+    free(pdata); pdata = NULL;
     cvProcessError(cv_mem, CVLS_MEM_FAIL, "CVBBDPRE",
                    "CVBBDPrecInit", MSGBBD_MEM_FAIL);
-    return(CVLS_MEM_FAIL); 
+    return(CVLS_MEM_FAIL);
   }
 
   /* Allocate memory for preconditioner matrix */
   storage_mu = SUNMIN(Nlocal-1, muk + mlk);
   pdata->savedP = NULL;
-  pdata->savedP = SUNBandMatrixStorage(Nlocal, muk, mlk, storage_mu);
+  pdata->savedP = SUNBandMatrixStorage(Nlocal, muk, mlk, storage_mu, cv_mem->cv_sunctx);
   if (pdata->savedP == NULL) {
     SUNMatDestroy(pdata->savedJ);
     free(pdata); pdata = NULL;
@@ -132,23 +132,23 @@ int CVBBDPrecInit(void *cvode_mem, sunindextype Nlocal,
 
   /* Allocate memory for temporary N_Vectors */
   pdata->zlocal = NULL;
-  pdata->zlocal = N_VNewEmpty_Serial(Nlocal);
+  pdata->zlocal = N_VNewEmpty_Serial(Nlocal, cv_mem->cv_sunctx);
   if (pdata->zlocal == NULL) {
     SUNMatDestroy(pdata->savedP);
     SUNMatDestroy(pdata->savedJ);
     free(pdata); pdata = NULL;
-    cvProcessError(cv_mem, CVLS_MEM_FAIL, "CVBBDPRE", 
+    cvProcessError(cv_mem, CVLS_MEM_FAIL, "CVBBDPRE",
                    "CVBBDPrecInit", MSGBBD_MEM_FAIL);
     return(CVLS_MEM_FAIL);
   }
   pdata->rlocal = NULL;
-  pdata->rlocal = N_VNewEmpty_Serial(Nlocal);
+  pdata->rlocal = N_VNewEmpty_Serial(Nlocal, cv_mem->cv_sunctx);
   if (pdata->rlocal == NULL) {
     N_VDestroy(pdata->zlocal);
     SUNMatDestroy(pdata->savedP);
     SUNMatDestroy(pdata->savedJ);
     free(pdata); pdata = NULL;
-    cvProcessError(cv_mem, CVLS_MEM_FAIL, "CVBBDPRE", 
+    cvProcessError(cv_mem, CVLS_MEM_FAIL, "CVBBDPRE",
                    "CVBBDPrecInit", MSGBBD_MEM_FAIL);
     return(CVLS_MEM_FAIL);
   }
@@ -160,7 +160,7 @@ int CVBBDPrecInit(void *cvode_mem, sunindextype Nlocal,
     SUNMatDestroy(pdata->savedP);
     SUNMatDestroy(pdata->savedJ);
     free(pdata); pdata = NULL;
-    cvProcessError(cv_mem, CVLS_MEM_FAIL, "CVBBDPRE", 
+    cvProcessError(cv_mem, CVLS_MEM_FAIL, "CVBBDPRE",
                    "CVBBDPrecInit", MSGBBD_MEM_FAIL);
     return(CVLS_MEM_FAIL);
   }
@@ -173,7 +173,7 @@ int CVBBDPrecInit(void *cvode_mem, sunindextype Nlocal,
     SUNMatDestroy(pdata->savedP);
     SUNMatDestroy(pdata->savedJ);
     free(pdata); pdata = NULL;
-    cvProcessError(cv_mem, CVLS_MEM_FAIL, "CVBBDPRE", 
+    cvProcessError(cv_mem, CVLS_MEM_FAIL, "CVBBDPRE",
                    "CVBBDPrecInit", MSGBBD_MEM_FAIL);
     return(CVLS_MEM_FAIL);
   }
@@ -187,14 +187,14 @@ int CVBBDPrecInit(void *cvode_mem, sunindextype Nlocal,
     SUNMatDestroy(pdata->savedP);
     SUNMatDestroy(pdata->savedJ);
     free(pdata); pdata = NULL;
-    cvProcessError(cv_mem, CVLS_MEM_FAIL, "CVBBDPRE", 
+    cvProcessError(cv_mem, CVLS_MEM_FAIL, "CVBBDPRE",
                    "CVBBDPrecInit", MSGBBD_MEM_FAIL);
     return(CVLS_MEM_FAIL);
   }
 
   /* Allocate memory for banded linear solver */
   pdata->LS = NULL;
-  pdata->LS = SUNLinSol_Band(pdata->rlocal, pdata->savedP);
+  pdata->LS = SUNLinSol_Band(pdata->rlocal, pdata->savedP, cv_mem->cv_sunctx);
   if (pdata->LS == NULL) {
     N_VDestroy(pdata->tmp1);
     N_VDestroy(pdata->tmp2);
@@ -204,7 +204,7 @@ int CVBBDPrecInit(void *cvode_mem, sunindextype Nlocal,
     SUNMatDestroy(pdata->savedP);
     SUNMatDestroy(pdata->savedJ);
     free(pdata); pdata = NULL;
-    cvProcessError(cv_mem, CVLS_MEM_FAIL, "CVBBDPRE", 
+    cvProcessError(cv_mem, CVLS_MEM_FAIL, "CVBBDPRE",
                    "CVBBDPrecInit", MSGBBD_MEM_FAIL);
     return(CVLS_MEM_FAIL);
   }
@@ -221,7 +221,7 @@ int CVBBDPrecInit(void *cvode_mem, sunindextype Nlocal,
     SUNMatDestroy(pdata->savedJ);
     SUNLinSolFree(pdata->LS);
     free(pdata); pdata = NULL;
-    cvProcessError(cv_mem, CVLS_SUNLS_FAIL, "CVBBDPRE", 
+    cvProcessError(cv_mem, CVLS_SUNLS_FAIL, "CVBBDPRE",
                    "CVBBDPrecInit", MSGBBD_SUNLS_FAIL);
     return(CVLS_SUNLS_FAIL);
   }
@@ -309,7 +309,7 @@ int CVBBDPrecReInit(void *cvode_mem, sunindextype mudq,
     cvProcessError(cv_mem, CVLS_PMEM_NULL, "CVBBDPRE",
                    "CVBBDPrecReInit", MSGBBD_PMEM_NULL);
     return(CVLS_PMEM_NULL);
-  } 
+  }
   pdata = (CVBBDPrecData) cvls_mem->P_data;
 
   /* Load half-bandwidths */
@@ -354,7 +354,7 @@ int CVBBDPrecGetWorkSpace(void *cvode_mem,
     cvProcessError(cv_mem, CVLS_PMEM_NULL, "CVBBDPRE",
                    "CVBBDPrecGetWorkSpace", MSGBBD_PMEM_NULL);
     return(CVLS_PMEM_NULL);
-  } 
+  }
   pdata = (CVBBDPrecData) cvls_mem->P_data;
 
   *lenrwBBDP = pdata->rpwsize;
@@ -389,7 +389,7 @@ int CVBBDPrecGetNumGfnEvals(void *cvode_mem,
     cvProcessError(cv_mem, CVLS_PMEM_NULL, "CVBBDPRE",
                    "CVBBDPrecGetNumGfnEvals", MSGBBD_PMEM_NULL);
     return(CVLS_PMEM_NULL);
-  } 
+  }
   pdata = (CVBBDPrecData) cvls_mem->P_data;
 
   *ngevalsBBDP = pdata->nge;
@@ -399,25 +399,25 @@ int CVBBDPrecGetNumGfnEvals(void *cvode_mem,
 
 
 /*-----------------------------------------------------------------
-  Function : CVBBDPrecSetup                                      
+  Function : CVBBDPrecSetup
   -----------------------------------------------------------------
   CVBBDPrecSetup generates and factors a banded block of the
   preconditioner matrix on each processor, via calls to the
   user-supplied gloc and cfn functions. It uses difference
   quotient approximations to the Jacobian elements.
- 
+
   CVBBDPrecSetup calculates a new J,if necessary, then calculates
   P = I - gamma*J, and does an LU factorization of P.
- 
+
   The parameters of CVBBDPrecSetup used here are as follows:
- 
+
   t       is the current value of the independent variable.
- 
+
   y       is the current value of the dependent variable vector,
           namely the predicted value of y(t).
- 
+
   fy      is the vector f(t,y).
- 
+
   jok     is an input flag indicating whether Jacobian-related
           data needs to be recomputed, as follows:
             jok == SUNFALSE means recompute Jacobian-related data
@@ -427,28 +427,27 @@ int CVBBDPrecGetNumGfnEvals(void *cvode_mem,
                    (with the current value of gamma).
           A cvBBDPrecSetup call with jok == SUNTRUE should only occur
           after a call with jok == SUNFALSE.
- 
+
   jcurPtr is a pointer to an output integer flag which is
           set by cvBBDPrecSetup as follows:
             *jcurPtr = SUNTRUE if Jacobian data was recomputed.
             *jcurPtr = SUNFALSE if Jacobian data was not recomputed,
                        but saved data was reused.
- 
+
   gamma   is the scalar appearing in the Newton matrix.
- 
+
   bbd_data is a pointer to the preconditioner data set by
            CVBBDPrecInit
- 
+
   Return value:
   The value returned by this CVBBDPrecSetup function is the int
     0  if successful,
     1  for a recoverable error (step will be retried).
   -----------------------------------------------------------------*/
-static int CVBBDPrecSetup(realtype t, N_Vector y, N_Vector fy, 
-                          booleantype jok, booleantype *jcurPtr, 
+static int CVBBDPrecSetup(realtype t, N_Vector y, N_Vector fy,
+                          booleantype jok, booleantype *jcurPtr,
                           realtype gamma, void *bbd_data)
 {
-  sunindextype ier;
   CVBBDPrecData pdata;
   CVodeMem cv_mem;
   int retval;
@@ -461,7 +460,7 @@ static int CVBBDPrecSetup(realtype t, N_Vector y, N_Vector fy,
     *jcurPtr = SUNFALSE;
     retval = SUNMatCopy(pdata->savedJ, pdata->savedP);
     if (retval < 0) {
-      cvProcessError(cv_mem, -1, "CVBBDPRE", 
+      cvProcessError(cv_mem, -1, "CVBBDPRE",
                      "CVBBDPrecSetup", MSGBBD_SUNMAT_FAIL);
       return(-1);
     }
@@ -475,7 +474,7 @@ static int CVBBDPrecSetup(realtype t, N_Vector y, N_Vector fy,
     *jcurPtr = SUNTRUE;
     retval = SUNMatZero(pdata->savedJ);
     if (retval < 0) {
-      cvProcessError(cv_mem, -1, "CVBBDPRE", 
+      cvProcessError(cv_mem, -1, "CVBBDPRE",
                      "CVBBDPrecSetup", MSGBBD_SUNMAT_FAIL);
       return(-1);
     }
@@ -483,10 +482,10 @@ static int CVBBDPrecSetup(realtype t, N_Vector y, N_Vector fy,
       return(1);
     }
 
-    retval = CVBBDDQJac(pdata, t, y, pdata->tmp1, 
+    retval = CVBBDDQJac(pdata, t, y, pdata->tmp1,
                         pdata->tmp2, pdata->tmp3);
     if (retval < 0) {
-      cvProcessError(cv_mem, -1, "CVBBDPRE", "CVBBDPrecSetup", 
+      cvProcessError(cv_mem, -1, "CVBBDPRE", "CVBBDPrecSetup",
                      MSGBBD_FUNC_FAILED);
       return(-1);
     }
@@ -496,7 +495,7 @@ static int CVBBDPrecSetup(realtype t, N_Vector y, N_Vector fy,
 
     retval = SUNMatCopy(pdata->savedJ, pdata->savedP);
     if (retval < 0) {
-      cvProcessError(cv_mem, -1, "CVBBDPRE", 
+      cvProcessError(cv_mem, -1, "CVBBDPRE",
                      "CVBBDPrecSetup", MSGBBD_SUNMAT_FAIL);
       return(-1);
     }
@@ -505,18 +504,18 @@ static int CVBBDPrecSetup(realtype t, N_Vector y, N_Vector fy,
     }
 
   }
-  
+
   /* Scale and add I to get P = I - gamma*J */
   retval = SUNMatScaleAddI(-gamma, pdata->savedP);
   if (retval) {
-    cvProcessError(cv_mem, -1, "CVBBDPRE", 
+    cvProcessError(cv_mem, -1, "CVBBDPRE",
                    "CVBBDPrecSetup", MSGBBD_SUNMAT_FAIL);
     return(-1);
   }
- 
+
   /* Do LU factorization of matrix and return error flag */
-  ier = SUNLinSolSetup_Band(pdata->LS, pdata->savedP);
-  return(ier);
+  retval = SUNLinSolSetup_Band(pdata->LS, pdata->savedP);
+  return(retval);
 }
 
 
@@ -526,21 +525,21 @@ static int CVBBDPrecSetup(realtype t, N_Vector y, N_Vector fy,
   CVBBDPrecSolve solves a linear system P z = r, with the
   band-block-diagonal preconditioner matrix P generated and
   factored by CVBBDPrecSetup.
- 
+
   The parameters of CVBBDPrecSolve used here are as follows:
- 
+
   r is the right-hand side vector of the linear system.
- 
+
   bbd_data is a pointer to the preconditioner data set by
     CVBBDPrecInit.
- 
+
   z is the output vector computed by CVBBDPrecSolve.
- 
+
   The value returned by the CVBBDPrecSolve function is always 0,
   indicating success.
   -----------------------------------------------------------------*/
-static int CVBBDPrecSolve(realtype t, N_Vector y, N_Vector fy, 
-                          N_Vector r, N_Vector z, 
+static int CVBBDPrecSolve(realtype t, N_Vector y, N_Vector fy,
+                          N_Vector r, N_Vector z,
                           realtype gamma, realtype delta,
                           int lr, void *bbd_data)
 {
@@ -552,9 +551,9 @@ static int CVBBDPrecSolve(realtype t, N_Vector y, N_Vector fy,
   /* Attach local data arrays for r and z to rlocal and zlocal */
   N_VSetArrayPointer(N_VGetArrayPointer(r), pdata->rlocal);
   N_VSetArrayPointer(N_VGetArrayPointer(z), pdata->zlocal);
-  
+
   /* Call banded solver object to do the work */
-  retval = SUNLinSolSolve(pdata->LS, pdata->savedP, pdata->zlocal, 
+  retval = SUNLinSolSolve(pdata->LS, pdata->savedP, pdata->zlocal,
                           pdata->rlocal, ZERO);
 
   /* Detach local data arrays from rlocal and zlocal */
@@ -569,10 +568,10 @@ static int CVBBDPrecFree(CVodeMem cv_mem)
 {
   CVLsMem cvls_mem;
   CVBBDPrecData pdata;
-  
+
   if (cv_mem->cv_lmem == NULL) return(0);
   cvls_mem = (CVLsMem) cv_mem->cv_lmem;
-  
+
   if (cvls_mem->P_data == NULL) return(0);
   pdata = (CVBBDPrecData) cvls_mem->P_data;
 
@@ -597,8 +596,8 @@ static int CVBBDPrecFree(CVodeMem cv_mem)
   -----------------------------------------------------------------
   This routine generates a banded difference quotient approximation
   to the local block of the Jacobian of g(t,y). It assumes that a
-  band SUNMatrix is stored columnwise, and that elements within each 
-  column are contiguous. All matrix elements are generated as 
+  band SUNMatrix is stored columnwise, and that elements within each
+  column are contiguous. All matrix elements are generated as
   difference quotients, by way of calls to the user routine gloc.
   By virtue of the band structure, the number of these calls is
   bandwidth + 1, where bandwidth = mldq + mudq + 1.
@@ -606,7 +605,7 @@ static int CVBBDPrecFree(CVodeMem cv_mem)
   This routine also assumes that the local elements of a vector are
   stored contiguously.
   -----------------------------------------------------------------*/
-static int CVBBDDQJac(CVBBDPrecData pdata, realtype t, N_Vector y, 
+static int CVBBDDQJac(CVBBDPrecData pdata, realtype t, N_Vector y,
                       N_Vector gy, N_Vector ytemp, N_Vector gtemp)
 {
   CVodeMem cv_mem;
@@ -616,6 +615,9 @@ static int CVBBDDQJac(CVBBDPrecData pdata, realtype t, N_Vector y,
   realtype *ytemp_data, *col_j, *cns_data;
   int retval;
 
+  /* initialize cns_data to avoid compiler warning */
+  cns_data = NULL;
+
   cv_mem = (CVodeMem) pdata->cvode_mem;
 
   /* Load ytemp with y = predicted solution vector */
@@ -638,7 +640,7 @@ static int CVBBDDQJac(CVBBDPrecData pdata, realtype t, N_Vector y,
   ewt_data   =  N_VGetArrayPointer(cv_mem->cv_ewt);
   ytemp_data =  N_VGetArrayPointer(ytemp);
   gtemp_data =  N_VGetArrayPointer(gtemp);
-  if (cv_mem->cv_constraints != NULL)
+  if (cv_mem->cv_constraintsSet)
     cns_data =  N_VGetArrayPointer(cv_mem->cv_constraints);
 
   /* Set minimum increment based on uround and norm of g */
@@ -651,16 +653,16 @@ static int CVBBDDQJac(CVBBDPrecData pdata, realtype t, N_Vector y,
   width = pdata->mldq + pdata->mudq + 1;
   ngroups = SUNMIN(width, pdata->n_local);
 
-  /* Loop over groups */  
+  /* Loop over groups */
   for (group=1; group <= ngroups; group++) {
-    
+
     /* Increment all y_j in group */
     for(j=group-1; j < pdata->n_local; j+=width) {
       inc = SUNMAX(pdata->dqrely * SUNRabs(y_data[j]), minInc/ewt_data[j]);
       yj = y_data[j];
 
       /* Adjust sign(inc) again if yj has an inequality constraint. */
-      if (cv_mem->cv_constraints != NULL) {
+      if (cv_mem->cv_constraintsSet) {
         conj = cns_data[j];
         if (SUNRabs(conj) == ONE)      {if ((yj+inc)*conj < ZERO)  inc = -inc;}
         else if (SUNRabs(conj) == TWO) {if ((yj+inc)*conj <= ZERO) inc = -inc;}
@@ -683,7 +685,7 @@ static int CVBBDDQJac(CVBBDPrecData pdata, realtype t, N_Vector y,
       inc = SUNMAX(pdata->dqrely * SUNRabs(y_data[j]), minInc/ewt_data[j]);
 
       /* Adjust sign(inc) as before. */
-      if (cv_mem->cv_constraints != NULL) {
+      if (cv_mem->cv_constraintsSet) {
         conj = cns_data[j];
         if (SUNRabs(conj) == ONE)      {if ((yj+inc)*conj < ZERO)  inc = -inc;}
         else if (SUNRabs(conj) == TWO) {if ((yj+inc)*conj <= ZERO) inc = -inc;}
