@@ -1,9 +1,9 @@
-/*-----------------------------------------------------------------
+/* ----------------------------------------------------------------
  * Programmer(s): Daniel R. Reynolds @ SMU
  *                Alan C. Hindmarsh and Radu Serban @ LLNL
- *-----------------------------------------------------------------
+ * ----------------------------------------------------------------
  * SUNDIALS Copyright Start
- * Copyright (c) 2002-2019, Lawrence Livermore National Security
+ * Copyright (c) 2002-2022, Lawrence Livermore National Security
  * and Southern Methodist University.
  * All rights reserved.
  *
@@ -11,9 +11,9 @@
  *
  * SPDX-License-Identifier: BSD-3-Clause
  * SUNDIALS Copyright End
- *-----------------------------------------------------------------
+ * ----------------------------------------------------------------
  * Implementation file for CVode's linear solver interface.
- *-----------------------------------------------------------------*/
+ * ---------------------------------------------------------------- */
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -26,14 +26,22 @@
 #include <sunmatrix/sunmatrix_dense.h>
 #include <sunmatrix/sunmatrix_sparse.h>
 
-/* constants */
+/* Private constants */
 #define MIN_INC_MULT RCONST(1000.0)
-#define MAX_DQITERS  3  /* max. # of attempts to recover in DQ J*v */
+#define MAX_DQITERS  3  /* max. number of attempts to recover in DQ J*v */
 #define ZERO         RCONST(0.0)
 #define PT25         RCONST(0.25)
 #define ONE          RCONST(1.0)
 #define TWO          RCONST(2.0)
 
+/*=================================================================
+  PRIVATE FUNCTION PROTOTYPES
+  =================================================================*/
+
+static int cvLsLinSys(realtype t, N_Vector y, N_Vector fy, SUNMatrix A,
+                      booleantype jok, booleantype *jcur, realtype gamma,
+                      void *user_data, N_Vector tmp1, N_Vector tmp2,
+                      N_Vector tmp3);
 
 /*===============================================================
   CVLS Exported functions -- Required
@@ -45,9 +53,11 @@
 int CVodeSetLinearSolver(void *cvode_mem, SUNLinearSolver LS,
                          SUNMatrix A)
 {
-  CVodeMem cv_mem;
-  CVLsMem  cvls_mem;
-  int      retval, LSType;
+  CVodeMem    cv_mem;
+  CVLsMem     cvls_mem;
+  int         retval, LSType;
+  booleantype iterative;    /* is the solver iterative?    */
+  booleantype matrixbased;  /* is a matrix structure used? */
 
   /* Return immediately if either cvode_mem or LS inputs are NULL */
   if (cvode_mem == NULL) {
@@ -58,48 +68,70 @@ int CVodeSetLinearSolver(void *cvode_mem, SUNLinearSolver LS,
   if (LS == NULL) {
     cvProcessError(NULL, CVLS_ILL_INPUT, "CVLS",
                    "CVodeSetLinearSolver",
-                    "LS must be non-NULL");
+                   "LS must be non-NULL");
     return(CVLS_ILL_INPUT);
   }
   cv_mem = (CVodeMem) cvode_mem;
 
   /* Test if solver is compatible with LS interface */
-  if ( (LS->ops->gettype == NULL) ||
-       (LS->ops->initialize == NULL) ||
-       (LS->ops->setup == NULL) ||
-       (LS->ops->solve == NULL) ) {
+  if ( (LS->ops->gettype == NULL) || (LS->ops->solve == NULL) ) {
     cvProcessError(cv_mem, CVLS_ILL_INPUT, "CVLS",
                    "CVodeSetLinearSolver",
                    "LS object is missing a required operation");
     return(CVLS_ILL_INPUT);
   }
 
-  /* Test if vector is compatible with LS interface */
-  if ( (cv_mem->cv_tempv->ops->nvconst == NULL) ||
-       (cv_mem->cv_tempv->ops->nvdotprod == NULL) ) {
-    cvProcessError(cv_mem, CVLS_ILL_INPUT, "CVLS",
-                    "CVodeSetLinearSolver", MSG_LS_BAD_NVECTOR);
-    return(CVLS_ILL_INPUT);
-  }
-
   /* Retrieve the LS type */
   LSType = SUNLinSolGetType(LS);
 
+  /* Set flags based on LS type */
+  iterative   = (LSType != SUNLINEARSOLVER_DIRECT);
+  matrixbased = ((LSType != SUNLINEARSOLVER_ITERATIVE) &&
+                 (LSType != SUNLINEARSOLVER_MATRIX_EMBEDDED));
+
+  /* Test if vector is compatible with LS interface */
+  if ( (cv_mem->cv_tempv->ops->nvconst == NULL) ||
+       (cv_mem->cv_tempv->ops->nvwrmsnorm == NULL) ) {
+    cvProcessError(cv_mem, CVLS_ILL_INPUT, "CVLS",
+                   "CVodeSetLinearSolver", MSG_LS_BAD_NVECTOR);
+    return(CVLS_ILL_INPUT);
+  }
+
+  /* Ensure that A is NULL when LS is matrix-embedded */
+  if ((LSType == SUNLINEARSOLVER_MATRIX_EMBEDDED) && (A != NULL)) {
+    cvProcessError(cv_mem, CVLS_ILL_INPUT, "CVLS", "CVodeSetLinearSolver",
+                    "Incompatible inputs: matrix-embedded LS requires NULL matrix");
+    return(CVLS_ILL_INPUT);
+  }
+
   /* Check for compatible LS type, matrix and "atimes" support */
-  if ((LSType == SUNLINEARSOLVER_ITERATIVE) && (LS->ops->setatimes == NULL)) {
+  if (iterative) {
+
+    if (cv_mem->cv_tempv->ops->nvgetlength == NULL) {
+      cvProcessError(cv_mem, CVLS_ILL_INPUT, "CVLS",
+                     "CVodeSetLinearSolver", MSG_LS_BAD_NVECTOR);
+      return(CVLS_ILL_INPUT);
+    }
+
+    if (!matrixbased && (LSType != SUNLINEARSOLVER_MATRIX_EMBEDDED) &&
+        (LS->ops->setatimes == NULL)) {
+      cvProcessError(cv_mem, CVLS_ILL_INPUT, "CVLS", "CVodeSetLinearSolver",
+                     "Incompatible inputs: iterative LS must support ATimes routine");
+      return(CVLS_ILL_INPUT);
+    }
+
+    if (matrixbased && (A == NULL)) {
+      cvProcessError(cv_mem, CVLS_ILL_INPUT, "CVLS", "CVodeSetLinearSolver",
+                     "Incompatible inputs: matrix-iterative LS requires non-NULL matrix");
+      return(CVLS_ILL_INPUT);
+    }
+
+  } else if (A == NULL) {
+
     cvProcessError(cv_mem, CVLS_ILL_INPUT, "CVLS", "CVodeSetLinearSolver",
-                    "Incompatible inputs: iterative LS must support ATimes routine");
-    return(CVLS_ILL_INPUT);
-  }
-  if ((LSType == SUNLINEARSOLVER_DIRECT) && (A == NULL)) {
-    cvProcessError(cv_mem, CVLS_ILL_INPUT, "CVLS", "CVodeSetLinearSolver",
-                    "Incompatible inputs: direct LS requires non-NULL matrix");
-    return(CVLS_ILL_INPUT);
-  }
-  if ((LSType == SUNLINEARSOLVER_MATRIX_ITERATIVE) && (A == NULL)) {
-    cvProcessError(cv_mem, CVLS_ILL_INPUT, "CVLS", "CVodeSetLinearSolver",
-                    "Incompatible inputs: matrix-iterative LS requires non-NULL matrix");
+                   "Incompatible inputs: direct LS requires non-NULL matrix");
     return(CVLS_ILL_INPUT);
+
   }
 
   /* free any existing system solver attached to CVode */
@@ -124,6 +156,10 @@ int CVodeSetLinearSolver(void *cvode_mem, SUNLinearSolver LS,
   /* set SUNLinearSolver pointer */
   cvls_mem->LS = LS;
 
+  /* Linear solver type information */
+  cvls_mem->iterative   = iterative;
+  cvls_mem->matrixbased = matrixbased;
+
   /* Set defaults for Jacobian-related fields */
   if (A != NULL) {
     cvls_mem->jacDQ  = SUNTRUE;
@@ -134,11 +170,17 @@ int CVodeSetLinearSolver(void *cvode_mem, SUNLinearSolver LS,
     cvls_mem->jac    = NULL;
     cvls_mem->J_data = NULL;
   }
+
   cvls_mem->jtimesDQ = SUNTRUE;
   cvls_mem->jtsetup  = NULL;
   cvls_mem->jtimes   = cvLsDQJtimes;
+  cvls_mem->jt_f     = cv_mem->cv_f;
   cvls_mem->jt_data  = cv_mem;
 
+  cvls_mem->user_linsys = SUNFALSE;
+  cvls_mem->linsys      = cvLsLinSys;
+  cvls_mem->A_data      = cv_mem;
+
   /* Set defaults for preconditioner-related fields */
   cvls_mem->pset   = NULL;
   cvls_mem->psolve = NULL;
@@ -149,10 +191,11 @@ int CVodeSetLinearSolver(void *cvode_mem, SUNLinearSolver LS,
   cvLsInitializeCounters(cvls_mem);
 
   /* Set default values for the rest of the LS parameters */
-  cvls_mem->msbj      = CVLS_MSBJ;
-  cvls_mem->jbad      = SUNTRUE;
-  cvls_mem->eplifac   = CVLS_EPLIN;
-  cvls_mem->last_flag = CVLS_SUCCESS;
+  cvls_mem->msbj       = CVLS_MSBJ;
+  cvls_mem->jbad       = SUNTRUE;
+  cvls_mem->dgmax_jbad = CVLS_DGMAX;
+  cvls_mem->eplifac    = CVLS_EPLIN;
+  cvls_mem->last_flag  = CVLS_SUCCESS;
 
   /* If LS supports ATimes, attach CVLs routine */
   if (LS->ops->setatimes) {
@@ -178,23 +221,17 @@ int CVodeSetLinearSolver(void *cvode_mem, SUNLinearSolver LS,
     }
   }
 
-  /* When using a non-NULL SUNMatrix object, store pointer to A and create saved_J */
+  /* When using a SUNMatrix object, store pointer to A and initialize savedJ */
   if (A != NULL) {
     cvls_mem->A = A;
-    cvls_mem->savedJ = SUNMatClone(A);
-    if (cvls_mem->savedJ == NULL) {
-      cvProcessError(cv_mem, CVLS_MEM_FAIL, "CVLS",
-                     "CVodeSetLinearSolver", MSG_LS_MEM_FAIL);
-      free(cvls_mem); cvls_mem = NULL;
-      return(CVLS_MEM_FAIL);
-    }
+    cvls_mem->savedJ = NULL; /* allocated in cvLsInitialize */
   }
+
   /* Allocate memory for ytemp and x */
   cvls_mem->ytemp = N_VClone(cv_mem->cv_tempv);
   if (cvls_mem->ytemp == NULL) {
     cvProcessError(cv_mem, CVLS_MEM_FAIL, "CVLS",
                     "CVodeSetLinearSolver", MSG_LS_MEM_FAIL);
-    SUNMatDestroy(cvls_mem->savedJ);
     free(cvls_mem); cvls_mem = NULL;
     return(CVLS_MEM_FAIL);
   }
@@ -203,19 +240,20 @@ int CVodeSetLinearSolver(void *cvode_mem, SUNLinearSolver LS,
   if (cvls_mem->x == NULL) {
     cvProcessError(cv_mem, CVLS_MEM_FAIL, "CVLS",
                     "CVodeSetLinearSolver", MSG_LS_MEM_FAIL);
-    SUNMatDestroy(cvls_mem->savedJ);
     N_VDestroy(cvls_mem->ytemp);
     free(cvls_mem); cvls_mem = NULL;
     return(CVLS_MEM_FAIL);
   }
 
-  /* For iterative LS, compute sqrtN from a dot product */
-  if ( (LSType == SUNLINEARSOLVER_ITERATIVE) ||
-       (LSType == SUNLINEARSOLVER_MATRIX_ITERATIVE) ) {
-    N_VConst(ONE, cvls_mem->ytemp);
-    cvls_mem->sqrtN = SUNRsqrt( N_VDotProd(cvls_mem->ytemp,
-                                           cvls_mem->ytemp) );
-  }
+  /* For iterative LS, compute default norm conversion factor */
+  if (iterative)
+    cvls_mem->nrmfac = SUNRsqrt( N_VGetLength(cvls_mem->ytemp) );
+
+  /* Check if solution scaling should be enabled */
+  if (matrixbased && cv_mem->cv_lmm == CV_BDF)
+    cvls_mem->scalesol = SUNTRUE;
+  else
+    cvls_mem->scalesol = SUNFALSE;
 
   /* Attach linear solver memory to integrator memory */
   cv_mem->cv_lmem = cvls_mem;
@@ -248,7 +286,7 @@ int CVodeSetJacFn(void *cvode_mem, CVLsJacFn jac)
     return(CVLS_ILL_INPUT);
   }
 
-  /* set Jacobian routine pointer, and update relevant flags */
+  /* set the Jacobian routine pointer, and update relevant flags */
   if (jac != NULL) {
     cvls_mem->jacDQ  = SUNFALSE;
     cvls_mem->jac    = jac;
@@ -259,6 +297,35 @@ int CVodeSetJacFn(void *cvode_mem, CVLsJacFn jac)
     cvls_mem->J_data = cv_mem;
   }
 
+  /* ensure the internal linear system function is used */
+  cvls_mem->user_linsys = SUNFALSE;
+  cvls_mem->linsys      = cvLsLinSys;
+  cvls_mem->A_data      = cv_mem;
+
+  return(CVLS_SUCCESS);
+}
+
+
+/* CVodeSetDeltaGammaMaxBadJac specifies the maximum gamma ratio change
+ * after a NLS convergence failure with a potentially bad Jacobian. If
+ * |gamma/gammap-1| < dgmax_jbad then the Jacobian is marked as bad */
+int CVodeSetDeltaGammaMaxBadJac(void *cvode_mem, realtype dgmax_jbad)
+{
+  CVodeMem cv_mem;
+  CVLsMem  cvls_mem;
+  int      retval;
+
+  /* Access CVLsMem structure */
+  retval = cvLs_AccessLMem(cvode_mem, "CVodeSetDeltaGammaMaxBadJac",
+                           &cv_mem, &cvls_mem);
+  if (retval != CVLS_SUCCESS) return(retval);
+
+  /* Set value or use default */
+  if(dgmax_jbad <= ZERO)
+    cvls_mem->dgmax_jbad = CVLS_DGMAX;
+  else
+    cvls_mem->dgmax_jbad = dgmax_jbad;
+
   return(CVLS_SUCCESS);
 }
 
@@ -288,20 +355,79 @@ int CVodeSetEpsLin(void *cvode_mem, realtype eplifac)
 }
 
 
-/* CVodeSetMaxStepsBetweenJac specifies the maximum number of
-   time steps to wait before recomputing the Jacobian matrix
-   and/or preconditioner */
-int CVodeSetMaxStepsBetweenJac(void *cvode_mem, long int msbj)
+/* CVodeSetLSNormFactor sets or computes the factor to use when converting from
+   the integrator tolerance to the linear solver tolerance (WRMS to L2 norm). */
+int CVodeSetLSNormFactor(void *cvode_mem, realtype nrmfac)
+{
+  CVodeMem cv_mem;
+  CVLsMem  cvls_mem;
+  int      retval;
+
+  /* access CVLsMem structure */
+  retval = cvLs_AccessLMem(cvode_mem, "CVodeSetLSNormFactor",
+                           &cv_mem, &cvls_mem);
+  if (retval != CVLS_SUCCESS) return(retval);
+
+  if (nrmfac > ZERO) {
+    /* user-provided factor */
+    cvls_mem->nrmfac = nrmfac;
+  } else if (nrmfac < ZERO) {
+    /* compute factor for WRMS norm with dot product */
+    N_VConst(ONE, cvls_mem->ytemp);
+    cvls_mem->nrmfac = SUNRsqrt(N_VDotProd(cvls_mem->ytemp, cvls_mem->ytemp));
+  } else {
+    /* compute default factor for WRMS norm from vector legnth */
+    cvls_mem->nrmfac = SUNRsqrt(N_VGetLength(cvls_mem->ytemp));
+  }
+
+  return(CVLS_SUCCESS);
+}
+
+
+/* CVodeSetJacEvalFrequency specifies the frequency for recomputing the Jacobian
+   matrix and/or preconditioner */
+int CVodeSetJacEvalFrequency(void *cvode_mem, long int msbj)
 {
   CVodeMem cv_mem;
   CVLsMem  cvls_mem;
   int      retval;
 
   /* access CVLsMem structure; store input and return */
-  retval = cvLs_AccessLMem(cvode_mem, "CVodeSetMaxStepsBetweenJac",
+  retval = cvLs_AccessLMem(cvode_mem, "CVodeSetJacEvalFrequency",
                            &cv_mem, &cvls_mem);
   if (retval != CVLS_SUCCESS)  return(retval);
-  cvls_mem->msbj = (msbj <= ZERO) ? CVLS_MSBJ : msbj;
+
+  /* Check for legal msbj */
+  if(msbj < 0) {
+    cvProcessError(cv_mem, CVLS_ILL_INPUT, "CVLS", "CVodeSetJacEvalFrequency",
+                   "A negative evaluation frequency was provided.");
+    return(CVLS_ILL_INPUT);
+  }
+
+  cvls_mem->msbj = (msbj == 0) ? CVLS_MSBJ : msbj;
+
+  return(CVLS_SUCCESS);
+}
+
+/* CVodeSetLinearSolutionScaling enables or disables scaling the
+   linear solver solution to account for changes in gamma. */
+int CVodeSetLinearSolutionScaling(void *cvode_mem, booleantype onoff)
+{
+  CVodeMem cv_mem;
+  CVLsMem  cvls_mem;
+  int      retval;
+
+  /* access CVLsMem structure; store input and return */
+  retval = cvLs_AccessLMem(cvode_mem, "CVodeSetLinearSolutionScaling",
+                           &cv_mem, &cvls_mem);
+  if (retval != CVLS_SUCCESS) return(retval);
+
+  /* check for valid solver and method type */
+  if (!(cvls_mem->matrixbased) || cv_mem->cv_lmm != CV_BDF)
+    return(CVLS_ILL_INPUT);
+
+  /* set solution scaling flag */
+  cvls_mem->scalesol = onoff;
 
   return(CVLS_SUCCESS);
 }
@@ -314,8 +440,8 @@ int CVodeSetPreconditioner(void *cvode_mem, CVLsPrecSetupFn psetup,
 {
   CVodeMem cv_mem;
   CVLsMem  cvls_mem;
-  PSetupFn cvls_psetup;
-  PSolveFn cvls_psolve;
+  SUNPSetupFn cvls_psetup;
+  SUNPSolveFn cvls_psolve;
   int      retval;
 
   /* access CVLsMem structure */
@@ -384,6 +510,7 @@ int CVodeSetJacTimes(void *cvode_mem, CVLsJacTimesSetupFn jtsetup,
     cvls_mem->jtimesDQ = SUNTRUE;
     cvls_mem->jtsetup  = NULL;
     cvls_mem->jtimes   = cvLsDQJtimes;
+    cvls_mem->jt_f     = cv_mem->cv_f;
     cvls_mem->jt_data  = cv_mem;
   }
 
@@ -391,6 +518,71 @@ int CVodeSetJacTimes(void *cvode_mem, CVLsJacTimesSetupFn jtsetup,
 }
 
 
+/* CVodeSetJacTimesRhsFn specifies an alternative user-supplied ODE right-hand
+   side function to use in the internal finite difference Jacobian-vector
+   product */
+int CVodeSetJacTimesRhsFn(void *cvode_mem, CVRhsFn jtimesRhsFn)
+{
+  CVodeMem cv_mem;
+  CVLsMem  cvls_mem;
+  int      retval;
+
+  /* access CVLsMem structure */
+  retval = cvLs_AccessLMem(cvode_mem, "CVodeSetJacTimesRhsFn",
+                           &cv_mem, &cvls_mem);
+  if (retval != CVLS_SUCCESS) return(retval);
+
+  /* check if using internal finite difference approximation */
+  if (!(cvls_mem->jtimesDQ)) {
+    cvProcessError(cv_mem, CVLS_ILL_INPUT, "CVLS", "CVodeSetJacTimesRhsFn",
+                   "Internal finite-difference Jacobian-vector product is disabled.");
+    return(CVLS_ILL_INPUT);
+  }
+
+  /* store function pointers for RHS function (NULL implies use ODE RHS) */
+  if (jtimesRhsFn != NULL)
+    cvls_mem->jt_f = jtimesRhsFn;
+  else
+    cvls_mem->jt_f = cv_mem->cv_f;
+
+  return(CVLS_SUCCESS);
+}
+
+
+/* CVodeSetLinSysFn specifies the linear system setup function. */
+int CVodeSetLinSysFn(void *cvode_mem, CVLsLinSysFn linsys)
+{
+  CVodeMem cv_mem;
+  CVLsMem  cvls_mem;
+  int      retval;
+
+  /* access CVLsMem structure */
+  retval = cvLs_AccessLMem(cvode_mem, "CVodeSetLinSysFn",
+                           &cv_mem, &cvls_mem);
+  if (retval != CVLS_SUCCESS) return(retval);
+
+  /* return with failure if linsys cannot be used */
+  if ((linsys != NULL) && (cvls_mem->A == NULL)) {
+    cvProcessError(cv_mem, CVLS_ILL_INPUT, "CVLS", "CVodeSetLinSysFn",
+                   "Linear system setup routine cannot be supplied for NULL SUNMatrix");
+    return(CVLS_ILL_INPUT);
+  }
+
+  /* set the linear system routine pointer, and update relevant flags */
+  if (linsys != NULL) {
+    cvls_mem->user_linsys = SUNTRUE;
+    cvls_mem->linsys      = linsys;
+    cvls_mem->A_data      = cv_mem->cv_user_data;
+  } else {
+    cvls_mem->user_linsys = SUNFALSE;
+    cvls_mem->linsys      = cvLsLinSys;
+    cvls_mem->A_data      = cv_mem;
+  }
+
+  return(CVLS_SUCCESS);
+}
+
+
 /* CVodeGetLinWorkSpace returns the length of workspace allocated
    for the CVLS linear solver interface */
 int CVodeGetLinWorkSpace(void *cvode_mem, long int *lenrwLS,
@@ -577,6 +769,33 @@ int CVodeGetNumJtimesEvals(void *cvode_mem, long int *njvevals)
 }
 
 
+/* CVodeGetLinSolveStats returns statistics related to the linear solve. */
+int CVodeGetLinSolveStats(void* cvode_mem, long int* njevals, long int* nfevalsLS,
+                          long int* nliters, long int* nlcfails, long int* npevals,
+                          long int* npsolves, long int* njtsetups, long int* njtimes)
+{
+  CVodeMem cv_mem;
+  CVLsMem  cvls_mem;
+  int      retval;
+
+  /* access CVLsMem structure; set output value and return */
+  retval = cvLs_AccessLMem(cvode_mem, "CVodeGetLinSolveStats",
+                           &cv_mem, &cvls_mem);
+  if (retval != CVLS_SUCCESS)  return(retval);
+
+  *njevals   = cvls_mem->nje;
+  *nfevalsLS = cvls_mem->nfeDQ;
+  *nliters   = cvls_mem->nli;
+  *nlcfails  = cvls_mem->ncfl;
+  *npevals   = cvls_mem->npe;
+  *npsolves  = cvls_mem->nps;
+  *njtsetups = cvls_mem->njtsetup;
+  *njtimes   = cvls_mem->njtimes;
+
+  return(CVLS_SUCCESS);
+}
+
+
 /* CVodeGetLastLinFlag returns the last flag set in a CVLS function */
 int CVodeGetLastLinFlag(void *cvode_mem, long int *flag)
 {
@@ -822,11 +1041,14 @@ int cvLsDenseDQJac(realtype t, N_Vector y, N_Vector fy,
   CVLsMem cvls_mem;
   int retval = 0;
 
+  /* initialize cns_data to avoid compiler warning */
+  cns_data = NULL;
+
   /* access LsMem interface structure */
   cvls_mem = (CVLsMem) cv_mem->cv_lmem;
 
   /* access matrix dimension */
-  N = SUNDenseMatrix_Rows(Jac);
+  N = SUNDenseMatrix_Columns(Jac);
 
   /* Rename work vector for readibility */
   ftemp = tmp1;
@@ -837,7 +1059,7 @@ int cvLsDenseDQJac(realtype t, N_Vector y, N_Vector fy,
   /* Obtain pointers to the data for ewt, y */
   ewt_data = N_VGetArrayPointer(cv_mem->cv_ewt);
   y_data   = N_VGetArrayPointer(y);
-  if (cv_mem->cv_constraints != NULL)
+  if (cv_mem->cv_constraintsSet)
     cns_data = N_VGetArrayPointer(cv_mem->cv_constraints);
 
   /* Set minimum increment based on uround and norm of f */
@@ -855,7 +1077,7 @@ int cvLsDenseDQJac(realtype t, N_Vector y, N_Vector fy,
     inc = SUNMAX(srur*SUNRabs(yjsaved), minInc/ewt_data[j]);
 
     /* Adjust sign(inc) if y_j has an inequality constraint. */
-    if (cv_mem->cv_constraints != NULL) {
+    if (cv_mem->cv_constraintsSet) {
       conj = cns_data[j];
       if (SUNRabs(conj) == ONE)      {if ((yjsaved+inc)*conj < ZERO)  inc = -inc;}
       else if (SUNRabs(conj) == TWO) {if ((yjsaved+inc)*conj <= ZERO) inc = -inc;}
@@ -905,6 +1127,9 @@ int cvLsBandDQJac(realtype t, N_Vector y, N_Vector fy, SUNMatrix Jac,
   CVLsMem cvls_mem;
   int retval = 0;
 
+  /* initialize cns_data to avoid compiler warning */
+  cns_data = NULL;
+
   /* access LsMem interface structure */
   cvls_mem = (CVLsMem) cv_mem->cv_lmem;
 
@@ -923,7 +1148,7 @@ int cvLsBandDQJac(realtype t, N_Vector y, N_Vector fy, SUNMatrix Jac,
   ftemp_data = N_VGetArrayPointer(ftemp);
   y_data     = N_VGetArrayPointer(y);
   ytemp_data = N_VGetArrayPointer(ytemp);
-  if (cv_mem->cv_constraints != NULL)
+  if (cv_mem->cv_constraintsSet)
     cns_data = N_VGetArrayPointer(cv_mem->cv_constraints);
 
   /* Load ytemp with y = predicted y vector */
@@ -947,7 +1172,7 @@ int cvLsBandDQJac(realtype t, N_Vector y, N_Vector fy, SUNMatrix Jac,
       inc = SUNMAX(srur*SUNRabs(y_data[j]), minInc/ewt_data[j]);
 
       /* Adjust sign(inc) if yj has an inequality constraint. */
-      if (cv_mem->cv_constraints != NULL) {
+      if (cv_mem->cv_constraintsSet) {
         conj = cns_data[j];
         if (SUNRabs(conj) == ONE)      {if ((ytemp_data[j]+inc)*conj < ZERO)  inc = -inc;}
         else if (SUNRabs(conj) == TWO) {if ((ytemp_data[j]+inc)*conj <= ZERO) inc = -inc;}
@@ -968,7 +1193,7 @@ int cvLsBandDQJac(realtype t, N_Vector y, N_Vector fy, SUNMatrix Jac,
       inc = SUNMAX(srur*SUNRabs(y_data[j]), minInc/ewt_data[j]);
 
       /* Adjust sign(inc) as before. */
-      if (cv_mem->cv_constraints != NULL) {
+      if (cv_mem->cv_constraintsSet) {
         conj = cns_data[j];
         if (SUNRabs(conj) == ONE)      {if ((ytemp_data[j]+inc)*conj < ZERO)  inc = -inc;}
         else if (SUNRabs(conj) == TWO) {if ((ytemp_data[j]+inc)*conj <= ZERO) inc = -inc;}
@@ -1017,7 +1242,7 @@ int cvLsDQJtimes(N_Vector v, N_Vector Jv, realtype t,
     N_VLinearSum(sig, v, ONE, y, work);
 
     /* Set Jv = f(tn, y+sig*v) */
-    retval = cv_mem->cv_f(t, work, Jv, cv_mem->cv_user_data);
+    retval = cvls_mem->jt_f(t, work, Jv, cv_mem->cv_user_data);
     cvls_mem->nfeDQ++;
     if (retval == 0) break;
     if (retval < 0)  return(-1);
@@ -1037,6 +1262,94 @@ int cvLsDQJtimes(N_Vector v, N_Vector Jv, realtype t,
 }
 
 
+/*-----------------------------------------------------------------
+  cvLsLinSys
+
+  Setup the linear system A = I - gamma J
+  -----------------------------------------------------------------*/
+static int cvLsLinSys(realtype t, N_Vector y, N_Vector fy, SUNMatrix A,
+                      booleantype jok, booleantype *jcur, realtype gamma,
+                      void *cvode_mem, N_Vector vtemp1, N_Vector vtemp2,
+                      N_Vector vtemp3)
+{
+  CVodeMem cv_mem;
+  CVLsMem  cvls_mem;
+  int      retval;
+
+  /* access CVLsMem structure */
+  retval = cvLs_AccessLMem(cvode_mem, "cvLsLinSys",
+                           &cv_mem, &cvls_mem);
+  if (retval != CVLS_SUCCESS)  return(retval);
+
+  /* Check if Jacobian needs to be updated */
+  if (jok) {
+
+    /* Use saved copy of J */
+    *jcur = SUNFALSE;
+
+    /* Overwrite linear system matrix with saved J */
+    retval = SUNMatCopy(cvls_mem->savedJ, A);
+    if (retval) {
+      cvProcessError(cv_mem, CVLS_SUNMAT_FAIL, "CVLS",
+                     "cvLsSetup",  MSG_LS_SUNMAT_FAILED);
+      cvls_mem->last_flag = CVLS_SUNMAT_FAIL;
+      return(cvls_mem->last_flag);
+    }
+
+  } else {
+
+    /* Call jac() routine to update J */
+    *jcur = SUNTRUE;
+
+    /* Clear the linear system matrix if necessary */
+    if (SUNLinSolGetType(cvls_mem->LS) == SUNLINEARSOLVER_DIRECT) {
+      retval = SUNMatZero(A);
+      if (retval) {
+        cvProcessError(cv_mem, CVLS_SUNMAT_FAIL, "CVLS",
+                       "cvLsSetup",  MSG_LS_SUNMAT_FAILED);
+        cvls_mem->last_flag = CVLS_SUNMAT_FAIL;
+        return(cvls_mem->last_flag);
+      }
+    }
+
+    /* Compute new Jacobian matrix */
+    retval = cvls_mem->jac(t, y, fy, A, cvls_mem->J_data,
+                           vtemp1, vtemp2, vtemp3);
+    if (retval < 0) {
+      cvProcessError(cv_mem, CVLS_JACFUNC_UNRECVR, "CVLS",
+                     "cvLsSetup",  MSG_LS_JACFUNC_FAILED);
+      cvls_mem->last_flag = CVLS_JACFUNC_UNRECVR;
+      return(-1);
+    }
+    if (retval > 0) {
+      cvls_mem->last_flag = CVLS_JACFUNC_RECVR;
+      return(1);
+    }
+
+    /* Update saved copy of the Jacobian matrix */
+    retval = SUNMatCopy(A, cvls_mem->savedJ);
+    if (retval) {
+      cvProcessError(cv_mem, CVLS_SUNMAT_FAIL, "CVLS",
+                     "cvLsSetup",  MSG_LS_SUNMAT_FAILED);
+      cvls_mem->last_flag = CVLS_SUNMAT_FAIL;
+      return(cvls_mem->last_flag);
+    }
+
+  }
+
+  /* Perform linear combination A = I - gamma*J */
+  retval = SUNMatScaleAddI(-gamma, A);
+  if (retval) {
+    cvProcessError(cv_mem, CVLS_SUNMAT_FAIL, "CVLS",
+                   "cvLsSetup",  MSG_LS_SUNMAT_FAILED);
+    cvls_mem->last_flag = CVLS_SUNMAT_FAIL;
+    return(cvls_mem->last_flag);
+  }
+
+  return(CVLS_SUCCESS);
+}
+
+
 /*-----------------------------------------------------------------
   cvLsInitialize
 
@@ -1057,50 +1370,84 @@ int cvLsInitialize(CVodeMem cv_mem)
   cvls_mem = (CVLsMem) cv_mem->cv_lmem;
 
   /* Test for valid combinations of matrix & Jacobian routines: */
-  if (cvls_mem->A == NULL) {
+  if (cvls_mem->A != NULL) {
 
-    /* If SUNMatrix A is NULL: ensure 'jac' function pointer is NULL */
+    /* Matrix-based case */
+
+    if (cvls_mem->user_linsys) {
+
+      /* User-supplied linear system function, reset A_data (just in case) */
+      cvls_mem->A_data = cv_mem->cv_user_data;
+
+    } else {
+
+      /* Internal linear system function, reset pointers (just in case) */
+      cvls_mem->linsys = cvLsLinSys;
+      cvls_mem->A_data = cv_mem;
+
+      /* Check if an internal or user-supplied Jacobian function is used */
+      if (cvls_mem->jacDQ) {
+
+        /* Internal difference quotient Jacobian. Check that A is dense or band,
+           otherwise return an error */
+        retval = 0;
+        if (cvls_mem->A->ops->getid) {
+
+          if ( (SUNMatGetID(cvls_mem->A) == SUNMATRIX_DENSE) ||
+               (SUNMatGetID(cvls_mem->A) == SUNMATRIX_BAND) ) {
+            cvls_mem->jac    = cvLsDQJac;
+            cvls_mem->J_data = cv_mem;
+          } else {
+            retval++;
+          }
+
+        } else {
+          retval++;
+        }
+        if (retval) {
+          cvProcessError(cv_mem, CVLS_ILL_INPUT, "CVLS", "cvLsInitialize",
+                         "No Jacobian constructor available for SUNMatrix type");
+          cvls_mem->last_flag = CVLS_ILL_INPUT;
+          return(CVLS_ILL_INPUT);
+        }
+
+      } else {
+
+        /* User-supplied Jacobian, reset J_data pointer (just in case) */
+        cvls_mem->J_data = cv_mem->cv_user_data;
+
+      }
+
+      /* Allocate internally saved Jacobian if not already done */
+      if (cvls_mem->savedJ == NULL) {
+        cvls_mem->savedJ = SUNMatClone(cvls_mem->A);
+        if (cvls_mem->savedJ == NULL) {
+          cvProcessError(cv_mem, CVLS_MEM_FAIL, "CVLS", "cvLsInitialize",
+                         MSG_LS_MEM_FAIL);
+          cvls_mem->last_flag = CVLS_MEM_FAIL;
+          return(CVLS_MEM_FAIL);
+        }
+      }
+
+    } /* end matrix-based case */
+
+  } else {
+
+    /* Matrix-free case: ensure 'jac' and `linsys` function pointers are NULL */
     cvls_mem->jacDQ  = SUNFALSE;
     cvls_mem->jac    = NULL;
     cvls_mem->J_data = NULL;
 
-  } else if (cvls_mem->jacDQ) {
+    cvls_mem->user_linsys = SUNFALSE;
+    cvls_mem->linsys      = NULL;
+    cvls_mem->A_data      = NULL;
 
-    /* If A is non-NULL, and 'jac' is not user-supplied:
-       - if A is dense or band, ensure that our DQ approx. is used
-       - otherwise => error */
-    retval = 0;
-    if (cvls_mem->A->ops->getid) {
-
-      if ( (SUNMatGetID(cvls_mem->A) == SUNMATRIX_DENSE) ||
-           (SUNMatGetID(cvls_mem->A) == SUNMATRIX_BAND) ) {
-        cvls_mem->jac    = cvLsDQJac;
-        cvls_mem->J_data = cv_mem;
-      } else {
-        retval++;
-      }
-
-    } else {
-      retval++;
-    }
-    if (retval) {
-      cvProcessError(cv_mem, CVLS_ILL_INPUT, "CVLS", "cvLsInitialize",
-                     "No Jacobian constructor available for SUNMatrix type");
-      cvls_mem->last_flag = CVLS_ILL_INPUT;
-      return(CVLS_ILL_INPUT);
-    }
-
-  } else {
-
-    /* If A is non-NULL, and 'jac' is user-supplied,
-       reset J_data pointer (just in case) */
-    cvls_mem->J_data = cv_mem->cv_user_data;
   }
 
   /* reset counters */
   cvLsInitializeCounters(cvls_mem);
 
-  /* Set Jacobian-related fields, based on jtimesDQ */
+  /* Set Jacobian-vector product related fields, based on jtimesDQ */
   if (cvls_mem->jtimesDQ) {
     cvls_mem->jtsetup = NULL;
     cvls_mem->jtimes  = cvLsDQJtimes;
@@ -1114,6 +1461,12 @@ int cvLsInitialize(CVodeMem cv_mem)
   if ( (cvls_mem->A == NULL) && (cvls_mem->pset == NULL) )
     cv_mem->cv_lsetup = NULL;
 
+  /* When using a matrix-embedded linear solver, disable lsetup call and solution scaling */
+  if (SUNLinSolGetType(cvls_mem->LS) == SUNLINEARSOLVER_MATRIX_EMBEDDED) {
+    cv_mem->cv_lsetup = NULL;
+    cvls_mem->scalesol = SUNFALSE;
+  }
+
   /* Call LS initialize routine, and return result */
   cvls_mem->last_flag = SUNLinSolInitialize(cvls_mem->LS);
   return(cvls_mem->last_flag);
@@ -1150,6 +1503,12 @@ int cvLsSetup(CVodeMem cv_mem, int convfail, N_Vector ypred,
   }
   cvls_mem = (CVLsMem) cv_mem->cv_lmem;
 
+  /* Immediately return when using matrix-embedded linear solver */
+  if (SUNLinSolGetType(cvls_mem->LS) == SUNLINEARSOLVER_MATRIX_EMBEDDED) {
+    cvls_mem->last_flag = CVLS_SUCCESS;
+    return(cvls_mem->last_flag);
+  }
+
   /* Set CVLs N_Vector pointers to current solution and rhs */
   cvls_mem->ycur = ypred;
   cvls_mem->fcur = fpred;
@@ -1157,74 +1516,45 @@ int cvLsSetup(CVodeMem cv_mem, int convfail, N_Vector ypred,
   /* Use nst, gamma/gammap, and convfail to set J/P eval. flag jok */
   dgamma = SUNRabs((cv_mem->cv_gamma/cv_mem->cv_gammap) - ONE);
   cvls_mem->jbad = (cv_mem->cv_nst == 0) ||
-    (cv_mem->cv_nst > cvls_mem->nstlj + cvls_mem->msbj) ||
-    ((convfail == CV_FAIL_BAD_J) && (dgamma < CVLS_DGMAX)) ||
+    (cv_mem->cv_nst >= cvls_mem->nstlj + cvls_mem->msbj) ||
+    ((convfail == CV_FAIL_BAD_J) && (dgamma < cvls_mem->dgmax_jbad)) ||
     (convfail == CV_FAIL_OTHER);
 
-  /* If using a NULL SUNMatrix, set jcur to jbad; otherwise update J as appropriate */
-  if (cvls_mem->A == NULL) {
+  /* Setup the linear system if necessary */
+  if (cvls_mem->A != NULL) {
 
-    *jcurPtr = cvls_mem->jbad;
+    /* Update J if appropriate and evaluate A = I - gamma J */
+    retval = cvls_mem->linsys(cv_mem->cv_tn, ypred, fpred, cvls_mem->A,
+                              !(cvls_mem->jbad), jcurPtr, cv_mem->cv_gamma,
+                              cvls_mem->A_data, vtemp1, vtemp2, vtemp3);
+
+    /* Update J eval count and step when J was last updated */
+    if (*jcurPtr) {
+      cvls_mem->nje++;
+      cvls_mem->nstlj = cv_mem->cv_nst;
+    }
+
+    /* Check linsys() return value and return if necessary */
+    if (retval != CVLS_SUCCESS) {
+      if (cvls_mem->user_linsys) {
+        if (retval < 0) {
+          cvProcessError(cv_mem, CVLS_JACFUNC_UNRECVR, "CVLS",
+                         "cvLsSetup",  MSG_LS_JACFUNC_FAILED);
+          cvls_mem->last_flag = CVLS_JACFUNC_UNRECVR;
+          return(-1);
+        } else {
+          cvls_mem->last_flag = CVLS_JACFUNC_RECVR;
+          return(1);
+        }
+      } else {
+        return(retval);
+      }
+    }
 
   } else {
 
-    /* If jbad = SUNFALSE, use saved copy of J */
-    if (!cvls_mem->jbad) {
-
-      *jcurPtr = SUNFALSE;
-      retval = SUNMatCopy(cvls_mem->savedJ, cvls_mem->A);
-      if (retval) {
-        cvProcessError(cv_mem, CVLS_SUNMAT_FAIL, "CVLS",
-                       "cvLsSetup",  MSG_LS_SUNMAT_FAILED);
-        cvls_mem->last_flag = CVLS_SUNMAT_FAIL;
-        return(cvls_mem->last_flag);
-      }
-
-    /* If jbad = SUNTRUE, call jac routine for new J value */
-    } else {
-
-      cvls_mem->nje++;
-      cvls_mem->nstlj = cv_mem->cv_nst;
-      *jcurPtr = SUNTRUE;
-      retval = SUNMatZero(cvls_mem->A);
-      if (retval) {
-        cvProcessError(cv_mem, CVLS_SUNMAT_FAIL, "CVLS",
-                       "cvLsSetup",  MSG_LS_SUNMAT_FAILED);
-        cvls_mem->last_flag = CVLS_SUNMAT_FAIL;
-        return(cvls_mem->last_flag);
-      }
-
-      retval = cvls_mem->jac(cv_mem->cv_tn, ypred, fpred, cvls_mem->A,
-                             cvls_mem->J_data, vtemp1, vtemp2, vtemp3);
-      if (retval < 0) {
-        cvProcessError(cv_mem, CVLS_JACFUNC_UNRECVR, "CVLS",
-                       "cvLsSetup",  MSG_LS_JACFUNC_FAILED);
-        cvls_mem->last_flag = CVLS_JACFUNC_UNRECVR;
-        return(-1);
-      }
-      if (retval > 0) {
-        cvls_mem->last_flag = CVLS_JACFUNC_RECVR;
-        return(1);
-      }
-
-      retval = SUNMatCopy(cvls_mem->A, cvls_mem->savedJ);
-      if (retval) {
-        cvProcessError(cv_mem, CVLS_SUNMAT_FAIL, "CVLS",
-                       "cvLsSetup",  MSG_LS_SUNMAT_FAILED);
-        cvls_mem->last_flag = CVLS_SUNMAT_FAIL;
-        return(cvls_mem->last_flag);
-      }
-
-    }
-
-    /* Scale and add I to get A = I - gamma*J */
-    retval = SUNMatScaleAddI(-cv_mem->cv_gamma, cvls_mem->A);
-    if (retval) {
-      cvProcessError(cv_mem, CVLS_SUNMAT_FAIL, "CVLS",
-                     "cvLsSetup",  MSG_LS_SUNMAT_FAILED);
-      cvls_mem->last_flag = CVLS_SUNMAT_FAIL;
-      return(cvls_mem->last_flag);
-    }
+    /* Matrix-free case, set jcur to jbad */
+    *jcurPtr = cvls_mem->jbad;
 
   }
 
@@ -1232,7 +1562,7 @@ int cvLsSetup(CVodeMem cv_mem, int convfail, N_Vector ypred,
      pass the heuristic suggestions above to the user code(s) */
   cvls_mem->last_flag = SUNLinSolSetup(cvls_mem->LS, cvls_mem->A);
 
-  /* If the SUNMatrix was NULL, update heuristics flags */
+  /* If Matrix-free, update heuristics flags */
   if (cvls_mem->A == NULL) {
 
     /* If user set jcur to SUNTRUE, increment npe and save nst value */
@@ -1262,7 +1592,12 @@ int cvLsSolve(CVodeMem cv_mem, N_Vector b, N_Vector weight,
 {
   CVLsMem  cvls_mem;
   realtype bnorm, deltar, delta, w_mean;
-  int      curiter, nli_inc, retval, LSType;
+  int      curiter, nli_inc, retval;
+#if SUNDIALS_LOGGING_LEVEL >= SUNDIALS_LOGGING_DEBUG
+  realtype resnorm;
+  long int nps_inc;
+#endif
+
   /* access CVLsMem structure */
   if (cv_mem->cv_lmem==NULL) {
     cvProcessError(cv_mem, CVLS_LMEM_NULL, "CVLS",
@@ -1271,17 +1606,13 @@ int cvLsSolve(CVodeMem cv_mem, N_Vector b, N_Vector weight,
   }
   cvls_mem = (CVLsMem) cv_mem->cv_lmem;
 
-  /* Retrieve the LS type */
-  LSType = SUNLinSolGetType(cvls_mem->LS);
-
   /* get current nonlinear solver iteration */
   retval = SUNNonlinSolGetCurIter(cv_mem->NLS, &curiter);
 
   /* If the linear solver is iterative:
      test norm(b), if small, return x = 0 or x = b;
      set linear solver tolerance (in left/right scaled 2-norm) */
-  if ( (LSType == SUNLINEARSOLVER_ITERATIVE) ||
-       (LSType == SUNLINEARSOLVER_MATRIX_ITERATIVE) ) {
+  if (cvls_mem->iterative) {
     deltar = cvls_mem->eplifac * cv_mem->cv_tq[4];
     bnorm = N_VWrmsNorm(b, weight);
     if (bnorm <= deltar) {
@@ -1289,7 +1620,8 @@ int cvLsSolve(CVodeMem cv_mem, N_Vector b, N_Vector weight,
       cvls_mem->last_flag = CVLS_SUCCESS;
       return(cvls_mem->last_flag);
     }
-    delta = deltar * cvls_mem->sqrtN;
+    /* Adjust tolerance for 2-norm */
+    delta = deltar * cvls_mem->nrmfac;
   } else {
     delta = ZERO;
   }
@@ -1299,9 +1631,6 @@ int cvLsSolve(CVodeMem cv_mem, N_Vector b, N_Vector weight,
   cvls_mem->ycur = ynow;
   cvls_mem->fcur = fnow;
 
-  /* Set initial guess x = 0 to LS */
-  N_VConst(ZERO, cvls_mem->x);
-
   /* Set scaling vectors for LS to use (if applicable) */
   if (cvls_mem->LS->ops->setscalingvectors) {
     retval = SUNLinSolSetScalingVectors(cvls_mem->LS,
@@ -1327,16 +1656,26 @@ int cvLsSolve(CVodeMem cv_mem, N_Vector b, N_Vector weight,
        <=> w_mean^2 \sum_{i=0}^{n-1} (b - A x_i)^2 < tol^2
        <=> \sum_{i=0}^{n-1} (b - A x_i)^2 < tol^2 / w_mean^2
        <=> || b - A x ||_2 < tol / w_mean
-     So we compute w_mean = ||w||_RMS = ||w||_2 / sqrt(n), and scale
-     the desired tolerance accordingly. */
-  } else if ( (LSType == SUNLINEARSOLVER_ITERATIVE) ||
-              (LSType == SUNLINEARSOLVER_MATRIX_ITERATIVE) ) {
+     So we compute w_mean = ||w||_RMS = ||w||_2 and scale the desired tolerance accordingly. */
+  } else if (cvls_mem->iterative) {
 
-    w_mean = SUNRsqrt( N_VDotProd(weight, weight) ) / cvls_mem->sqrtN;
+    N_VConst(ONE, cvls_mem->x);
+    w_mean = N_VWrmsNorm(weight, cvls_mem->x);
     delta /= w_mean;
 
   }
 
+  /* Set initial guess x = 0 to LS */
+  N_VConst(ZERO, cvls_mem->x);
+
+  /* Set zero initial guess flag */
+  retval = SUNLinSolSetZeroGuess(cvls_mem->LS, SUNTRUE);
+  if (retval != SUNLS_SUCCESS) return(-1);
+
+#if SUNDIALS_LOGGING_LEVEL >= SUNDIALS_LOGGING_DEBUG
+  /* Store previous nps value in nps_inc */
+  nps_inc = cvls_mem->nps;
+#endif
   /* If a user-provided jtsetup routine is supplied, call that here */
   if (cvls_mem->jtsetup) {
     cvls_mem->last_flag = cvls_mem->jtsetup(cv_mem->cv_tn, ynow, fnow,
@@ -1355,18 +1694,23 @@ int cvLsSolve(CVodeMem cv_mem, N_Vector b, N_Vector weight,
 
   /* If using a direct or matrix-iterative solver, BDF method, and gamma has changed,
      scale the correction to account for change in gamma */
-  if ( ((LSType == SUNLINEARSOLVER_DIRECT) ||
-        (LSType == SUNLINEARSOLVER_MATRIX_ITERATIVE)) &&
-       (cv_mem->cv_lmm == CV_BDF) &&
-       (cv_mem->cv_gamrat != ONE) )
+  if (cvls_mem->scalesol && cv_mem->cv_gamrat != ONE)
     N_VScale(TWO/(ONE + cv_mem->cv_gamrat), b, b);
 
   /* Retrieve statistics from iterative linear solvers */
+#if SUNDIALS_LOGGING_LEVEL >= SUNDIALS_LOGGING_DEBUG
+  resnorm = ZERO;
+#endif
   nli_inc = 0;
-  if ( ((LSType == SUNLINEARSOLVER_ITERATIVE) ||
-        (LSType == SUNLINEARSOLVER_MATRIX_ITERATIVE)) &&
-       (cvls_mem->LS->ops->numiters) )
-    nli_inc = SUNLinSolNumIters(cvls_mem->LS);
+  if (cvls_mem->iterative) {
+#if SUNDIALS_LOGGING_LEVEL >= SUNDIALS_LOGGING_DEBUG
+    if (cvls_mem->LS->ops->resnorm)
+      resnorm = SUNLinSolResNorm(cvls_mem->LS);
+#endif
+    if (cvls_mem->LS->ops->numiters)
+      nli_inc = SUNLinSolNumIters(cvls_mem->LS);
+  }
+
 
   /* Increment counters nli and ncfl */
   cvls_mem->nli += nli_inc;
@@ -1375,6 +1719,13 @@ int cvLsSolve(CVodeMem cv_mem, N_Vector b, N_Vector weight,
   /* Interpret solver return value  */
   cvls_mem->last_flag = retval;
 
+#if SUNDIALS_LOGGING_LEVEL >= SUNDIALS_LOGGING_DEBUG
+  SUNLogger_QueueMsg(CV_LOGGER, SUN_LOGLEVEL_DEBUG,
+    "CVODE::cvLsSolve", "ls-stats",
+    "bnorm = %.16g, resnorm = %.16g, ls_iters = %i, prec_solves = %i",
+    bnorm, resnorm, nli_inc, (int)(cvls_mem->nps - nps_inc));
+#endif
+
   switch(retval) {
 
   case SUNLS_SUCCESS:
