@@ -3,7 +3,7 @@
  *                Alan C. Hindmarsh and Radu Serban @ LLNL
  *-----------------------------------------------------------------
  * SUNDIALS Copyright Start
- * Copyright (c) 2002-2021, Lawrence Livermore National Security
+ * Copyright (c) 2002-2022, Lawrence Livermore National Security
  * and Southern Methodist University.
  * All rights reserved.
  *
@@ -54,10 +54,12 @@ typedef struct CVLsMemRec {
   booleantype matrixbased;  /* is a matrix structure used? */
 
   /* Jacobian construction & storage */
-  booleantype jacDQ;  /* SUNTRUE if using internal DQ Jac approx.     */
-  CVLsJacFn jac;      /* Jacobian routine to be called                */
-  void *J_data;       /* user data is passed to jac                   */
-  booleantype jbad;   /* heuristic suggestion for pset                */
+  booleantype jacDQ;   /* SUNTRUE if using internal DQ Jac approx.     */
+  CVLsJacFn jac;       /* Jacobian routine to be called                */
+  void *J_data;        /* user data is passed to jac                   */
+  booleantype jbad;    /* heuristic suggestion for pset                */
+  realtype dgmax_jbad; /* if convfail = FAIL_BAD_J and the gamma ratio *
+                        * |gamma/gammap-1| < dgmax_jbad then J is bad  */
 
   /* Matrix-based solver, scale solution to account for change in gamma */
   booleantype scalesol;
