@@ -3,22 +3,18 @@
  * Programmer(s): Daniel R. Reynolds @ SMU
  *    Alan C. Hindmarsh, Radu Serban and Aaron Collier @ LLNL
  * -----------------------------------------------------------------
- * LLNS/SMU Copyright Start
- * Copyright (c) 2017, Southern Methodist University and 
- * Lawrence Livermore National Security
- *
- * This work was performed under the auspices of the U.S. Department 
- * of Energy by Southern Methodist University and Lawrence Livermore 
- * National Laboratory under Contract DE-AC52-07NA27344.
- * Produced at Southern Methodist University and the Lawrence 
- * Livermore National Laboratory.
- *
+ * SUNDIALS Copyright Start
+ * Copyright (c) 2002-2022, Lawrence Livermore National Security
+ * and Southern Methodist University.
  * All rights reserved.
- * For details, see the LICENSE file.
- * LLNS/SMU Copyright End
+ *
+ * See the top-level LICENSE and NOTICE files for details.
+ *
+ * SPDX-License-Identifier: BSD-3-Clause
+ * SUNDIALS Copyright End
  * -----------------------------------------------------------------
  * The C functions FCVJTSetup and FCVJtimes are to interface 
- * between the CVSPILS module and the user-supplied 
+ * between the CVLS module and the user-supplied 
  * Jacobian-vector product routines FCVJTSETUP and FCVJTIMES. 
  * Note the use of the generic names FCV_JTSETUP and FCV_JTIMES 
  * in the code below.
@@ -31,7 +27,7 @@
 #include "fcvode.h"     /* actual fn. names, prototypes and global vars.*/
 #include "cvode_impl.h" /* definition of CVodeMem type                  */
 
-#include <cvode/cvode_spils.h>
+#include <cvode/cvode_ls.h>
 
 /***************************************************************************/
 
@@ -56,12 +52,17 @@ extern "C" {
 
 /***************************************************************************/
 
-void FCV_SPILSSETJAC(int *flag, int *ier)
+/* ---DEPRECATED--- */
+void FCV_SPILLSSETJAC(int *flag, int *ier)
+{ FCV_LSSETJAC(flag, ier); }
+
+
+void FCV_LSSETJAC(int *flag, int *ier)
 {
   if (*flag == 0) {
-    *ier = CVSpilsSetJacTimes(CV_cvodemem, NULL, NULL);
+    *ier = CVodeSetJacTimes(CV_cvodemem, NULL, NULL);
   } else {
-    *ier = CVSpilsSetJacTimes(CV_cvodemem, FCVJTSetup, FCVJtimes);
+    *ier = CVodeSetJacTimes(CV_cvodemem, FCVJTSetup, FCVJtimes);
   }
 }
 
