@@ -1,21 +1,17 @@
 /*
- * ----------------------------------------------------------------- 
+ * -----------------------------------------------------------------
  * Programmer(s): Daniel R. Reynolds @ SMU
  *     Alan C. Hindmarsh, Radu Serban and Aaron Collier @ LLNL
  * -----------------------------------------------------------------
- * LLNS/SMU Copyright Start
- * Copyright (c) 2017, Southern Methodist University and 
- * Lawrence Livermore National Security
- *
- * This work was performed under the auspices of the U.S. Department 
- * of Energy by Southern Methodist University and Lawrence Livermore 
- * National Laboratory under Contract DE-AC52-07NA27344.
- * Produced at Southern Methodist University and the Lawrence 
- * Livermore National Laboratory.
- *
+ * SUNDIALS Copyright Start
+ * Copyright (c) 2002-2022, Lawrence Livermore National Security
+ * and Southern Methodist University.
  * All rights reserved.
- * For details, see the LICENSE file.
- * LLNS/SMU Copyright End
+ *
+ * See the top-level LICENSE and NOTICE files for details.
+ *
+ * SPDX-License-Identifier: BSD-3-Clause
+ * SUNDIALS Copyright End
  * -----------------------------------------------------------------
  * This is the implementation file for the Fortran interface to
  * the CVODE package.  See fcvode.h for usage.
@@ -29,11 +25,10 @@
 #include <stdlib.h>
 #include <string.h>
 
-#include "fcvode.h"                    /* actual function names, prototypes, global vars.*/ 
+#include "fcvode.h"                    /* actual function names, prototypes, global vars.*/
 #include "cvode_impl.h"                /* definition of CVodeMem type                    */
 #include <sundials/sundials_matrix.h>
-#include <cvode/cvode_direct.h>
-#include <cvode/cvode_spils.h>
+#include <cvode/cvode_ls.h>
 #include <cvode/cvode_diag.h>
 
 
@@ -71,14 +66,14 @@ extern "C" {
 
 /**************************************************************************/
 
-void FCV_MALLOC(realtype *t0, realtype *y0, 
-                int *meth, int *itmeth, int *iatol, 
+void FCV_MALLOC(realtype *t0, realtype *y0,
+                int *meth, int *iatol,
                 realtype *rtol, realtype *atol,
                 long int *iout, realtype *rout,
                 long int *ipar, realtype *rpar,
                 int *ier)
 {
-  int lmm, iter;
+  int lmm;
   N_Vector Vatol;
   FCVUserData CV_userdata;
 
@@ -95,27 +90,20 @@ void FCV_MALLOC(realtype *t0, realtype *y0,
   /* Initialize all pointers to NULL */
   CV_cvodemem = NULL;
   Vatol = NULL;
+  FCVNullNonlinSol();
 
   /* initialize global constants to disable each option */
   CV_nrtfn = 0;
   CV_ls = -1;
-  
+
   /* Create CVODE object */
   lmm = (*meth == 1) ? CV_ADAMS : CV_BDF;
-  iter = (*itmeth == 1) ? CV_FUNCTIONAL : CV_NEWTON;
-  CV_cvodemem = CVodeCreate(lmm, iter);
+  CV_cvodemem = CVodeCreate(lmm, NULL);
   if (CV_cvodemem == NULL) {
     *ier = -1;
     return;
   }
 
-  /* If using a functional iteration, initialize NULL F2C_CVODE_linsol 
-     and F2C_CVODE_matrix objects */
-  if (iter == CV_FUNCTIONAL) {
-    FCVNullMatrix();
-    FCVNullLinsol();
-  }
-  
   /* Set and attach user data */
   CV_userdata = NULL;
   CV_userdata = (FCVUserData) malloc(sizeof *CV_userdata);
@@ -152,7 +140,7 @@ void FCV_MALLOC(realtype *t0, realtype *y0,
   /* Set tolerances */
   switch (*iatol) {
   case 1:
-    *ier = CVodeSStolerances(CV_cvodemem, *rtol, *atol); 
+    *ier = CVodeSStolerances(CV_cvodemem, *rtol, *atol);
     break;
   case 2:
     Vatol = NULL;
@@ -187,8 +175,8 @@ void FCV_MALLOC(realtype *t0, realtype *y0,
 
 /***************************************************************************/
 
-void FCV_REINIT(realtype *t0, realtype *y0, 
-                int *iatol, realtype *rtol, realtype *atol, 
+void FCV_REINIT(realtype *t0, realtype *y0,
+                int *iatol, realtype *rtol, realtype *atol,
                 int *ier)
 {
   N_Vector Vatol;
@@ -216,7 +204,7 @@ void FCV_REINIT(realtype *t0, realtype *y0,
   /* Set tolerances */
   switch (*iatol) {
   case 1:
-    *ier = CVodeSStolerances(CV_cvodemem, *rtol, *atol); 
+    *ier = CVodeSStolerances(CV_cvodemem, *rtol, *atol);
     break;
   case 2:
     Vatol = NULL;
@@ -288,30 +276,63 @@ void FCV_SETRIN(char key_name[], realtype *rval, int *ier)
 
 /***************************************************************************/
 
-void FCV_DLSINIT(int *ier) {
-  if ( (CV_cvodemem == NULL) || (F2C_CVODE_linsol == NULL) || 
-       (F2C_CVODE_matrix == NULL) ) {
+void FCV_SETVIN(char key_name[], realtype *vval, int *ier)
+{
+  N_Vector Vec;
+
+  *ier = 0;
+
+  if (!strncmp(key_name,"CONSTR_VEC",10)) {
+    Vec = NULL;
+    Vec = N_VCloneEmpty(F2C_CVODE_vec);
+    if (Vec == NULL) {
+      *ier = -1;
+      return;
+    }
+    N_VSetArrayPointer(vval, Vec);
+    *ier = CVodeSetConstraints(CV_cvodemem, Vec);
+    N_VDestroy(Vec);
+  }
+  else {
+    *ier = -99;
+    fprintf(stderr, "FCVSETVIN: Unrecognized key. \n\n");
+  }
+
+}
+
+/***************************************************************************/
+
+void FCV_LSINIT(int *ier) {
+  if ( (CV_cvodemem == NULL) || (F2C_CVODE_linsol == NULL) ) {
     *ier = -1;
     return;
   }
-  *ier = CVDlsSetLinearSolver(CV_cvodemem, F2C_CVODE_linsol, 
+  *ier = CVodeSetLinearSolver(CV_cvodemem, F2C_CVODE_linsol,
                               F2C_CVODE_matrix);
-  CV_ls = CV_LS_DIRECT;
+  CV_ls = CV_LS_STD;
   return;
 }
 
 /***************************************************************************/
 
-void FCV_SPILSINIT(int *ier) {
-  if ( (CV_cvodemem == NULL) || (F2C_CVODE_linsol == NULL) ) {
-    *ier = -1;
-    return;
-  }
-  *ier = CVSpilsSetLinearSolver(CV_cvodemem, F2C_CVODE_linsol);
-  FCVNullMatrix();
-  CV_ls = CV_LS_ITERATIVE;
-  return;
-}
+/* ---DEPRECATED--- */
+void FCV_DLSINIT(int *ier)
+{ FCV_LSINIT(ier); }
+
+/***************************************************************************/
+
+/* ---DEPRECATED--- */
+void FCV_SPILSINIT(int *ier)
+{ FCV_LSINIT(ier); }
+
+/*=============================================================*/
+
+/* ---DEPRECATED--- */
+void FCV_SPILSSETEPSLIN(realtype *eplifac, int *ier)
+{ FCV_LSSETEPSLIN(eplifac, ier); }
+
+void FCV_LSSETEPSLIN(realtype *eplifac, int *ier)
+{ *ier = CVodeSetEpsLin(CV_cvodemem, *eplifac); }
 
 /***************************************************************************/
 
@@ -330,12 +351,12 @@ void FCV_DIAG(int *ier)
 
 void FCV_CVODE(realtype *tout, realtype *t, realtype *y, int *itask, int *ier)
 {
-  /* 
+  /*
      tout          is the t value where output is desired
      F2C_CVODE_vec is the N_Vector containing the solution on return
      t             is the returned independent variable value
-     itask         is the task indicator (1 = CV_NORMAL, 2 = CV_ONE_STEP, 
-                                          3 = CV_NORMAL_TSTOP, 4 = CV_ONE_STEP_TSTOP) 
+     itask         is the task indicator (1 = CV_NORMAL, 2 = CV_ONE_STEP,
+                                          3 = CV_NORMAL_TSTOP, 4 = CV_ONE_STEP_TSTOP)
   */
 
   int qu, qcur;
@@ -350,46 +371,42 @@ void FCV_CVODE(realtype *tout, realtype *t, realtype *y, int *itask, int *ier)
   CVodeGetWorkSpace(CV_cvodemem,
                     &CV_iout[0],                          /* LENRW   */
                     &CV_iout[1]);                         /* LENIW   */
-  CVodeGetIntegratorStats(CV_cvodemem, 
+  CVodeGetIntegratorStats(CV_cvodemem,
                           &CV_iout[2],                    /* NST     */
-                          &CV_iout[3],                    /* NFE     */ 
-                          &CV_iout[7],                    /* NSETUPS */ 
-                          &CV_iout[4],                    /* NETF    */ 
+                          &CV_iout[3],                    /* NFE     */
+                          &CV_iout[7],                    /* NSETUPS */
+                          &CV_iout[4],                    /* NETF    */
                           &qu,                            /* QU      */
                           &qcur,                          /* QCUR    */
                           &CV_rout[0],                    /* H0U     */
-                          &CV_rout[1],                    /* HU      */ 
-                          &CV_rout[2],                    /* HCUR    */ 
-                          &CV_rout[3]);                   /* TCUR    */ 
+                          &CV_rout[1],                    /* HU      */
+                          &CV_rout[2],                    /* HCUR    */
+                          &CV_rout[3]);                   /* TCUR    */
   CV_iout[8] = (long int) qu;
   CV_iout[9] = (long int) qcur;
-  CVodeGetTolScaleFactor(CV_cvodemem, 
+  CVodeGetTolScaleFactor(CV_cvodemem,
                          &CV_rout[4]);                    /* TOLSFAC */
   CVodeGetNonlinSolvStats(CV_cvodemem,
                           &CV_iout[6],                    /* NNI     */
                           &CV_iout[5]);                   /* NCFN    */
   CVodeGetNumStabLimOrderReds(CV_cvodemem, &CV_iout[10]); /* NOR     */
-  
+
   /* Root finding is on */
   if (CV_nrtfn != 0)
     CVodeGetNumGEvals(CV_cvodemem, &CV_iout[11]);         /* NGE     */
-  
+
   switch(CV_ls) {
-  case CV_LS_DIRECT:
-    CVDlsGetWorkSpace(CV_cvodemem, &CV_iout[12], &CV_iout[13]);   /* LENRWLS,LENIWLS */
-    CVDlsGetLastFlag(CV_cvodemem, &CV_iout[14]);                  /* LSTF */
-    CVDlsGetNumRhsEvals(CV_cvodemem, &CV_iout[15]);               /* NFELS */
-    CVDlsGetNumJacEvals(CV_cvodemem, &CV_iout[16]);               /* NJE */
-    break;
-  case CV_LS_ITERATIVE:
-    CVSpilsGetWorkSpace(CV_cvodemem, &CV_iout[12], &CV_iout[13]); /* LENRWLS,LENIWLS */
-    CVSpilsGetLastFlag(CV_cvodemem, &CV_iout[14]);                /* LSTF */
-    CVSpilsGetNumRhsEvals(CV_cvodemem, &CV_iout[15]);             /* NFELS */
-    CVSpilsGetNumJtimesEvals(CV_cvodemem, &CV_iout[16]);          /* NJTV */
-    CVSpilsGetNumPrecEvals(CV_cvodemem, &CV_iout[17]);            /* NPE */
-    CVSpilsGetNumPrecSolves(CV_cvodemem, &CV_iout[18]);           /* NPS */
-    CVSpilsGetNumLinIters(CV_cvodemem, &CV_iout[19]);             /* NLI */
-    CVSpilsGetNumConvFails(CV_cvodemem, &CV_iout[20]);            /* NCFL */
+  case CV_LS_STD:
+    CVodeGetLinWorkSpace(CV_cvodemem, &CV_iout[12], &CV_iout[13]);   /* LENRWLS,LENIWLS */
+    CVodeGetLastLinFlag(CV_cvodemem, &CV_iout[14]);                  /* LSTF */
+    CVodeGetNumLinRhsEvals(CV_cvodemem, &CV_iout[15]);               /* NFELS */
+    CVodeGetNumJacEvals(CV_cvodemem, &CV_iout[16]);                  /* NJE */
+    CVodeGetNumJTSetupEvals(CV_cvodemem, &CV_iout[17]);              /* NJTS */
+    CVodeGetNumJtimesEvals(CV_cvodemem, &CV_iout[18]);               /* NJTV */
+    CVodeGetNumPrecEvals(CV_cvodemem, &CV_iout[19]);                 /* NPE */
+    CVodeGetNumPrecSolves(CV_cvodemem, &CV_iout[20]);                /* NPS */
+    CVodeGetNumLinIters(CV_cvodemem, &CV_iout[21]);                  /* NLI */
+    CVodeGetNumLinConvFails(CV_cvodemem, &CV_iout[22]);              /* NCFL */
     break;
   case CV_LS_DIAG:
     CVDiagGetWorkSpace(CV_cvodemem, &CV_iout[12], &CV_iout[13]);  /* LENRWLS,LENIWLS */
@@ -402,10 +419,10 @@ void FCV_CVODE(realtype *tout, realtype *t, realtype *y, int *itask, int *ier)
 
 void FCV_DKY (realtype *t, int *k, realtype *dky, int *ier)
 {
-  /* 
+  /*
      t             is the t value where output is desired
      k             is the derivative order
-     F2C_CVODE_vec is the N_Vector containing the solution derivative on return 
+     F2C_CVODE_vec is the N_Vector containing the solution derivative on return
   */
 
   realtype *f2c_data = N_VGetArrayPointer(F2C_CVODE_vec);
@@ -463,7 +480,7 @@ void FCV_FREE ()
   if (cv_mem->cv_lfree)
     cv_mem->cv_lfree(cv_mem);
   cv_mem->cv_lmem = NULL;
-  
+
   free(cv_mem->cv_user_data); cv_mem->cv_user_data = NULL;
 
   CVodeFree(&CV_cvodemem);
@@ -474,15 +491,18 @@ void FCV_FREE ()
     SUNMatDestroy(F2C_CVODE_matrix);
   if (F2C_CVODE_linsol)
     SUNLinSolFree(F2C_CVODE_linsol);
+  if (F2C_CVODE_nonlinsol)
+    SUNNonlinSolFree(F2C_CVODE_nonlinsol);
+  return;
 }
 
 /***************************************************************************/
 
-/* 
+/*
  * C function CVf to interface between CVODE and a Fortran subroutine FCVFUN.
  * Addresses of t, y, and ydot are passed to CVFUN, using the
  * routine N_VGetArrayPointer from the NVECTOR module.
- * Auxiliary data is assumed to be communicated by Common. 
+ * Auxiliary data is assumed to be communicated by Common.
  */
 
 int FCVf(realtype t, N_Vector y, N_Vector ydot, void *user_data)
@@ -500,3 +520,19 @@ int FCVf(realtype t, N_Vector y, N_Vector ydot, void *user_data)
 
   return(ier);
 }
+
+/* Fortran interface to C routine CVodeSetNonlinearSolver; see
+   fcvode.h for further details */
+void FCV_NLSINIT(int *ier) {
+  if ( (CV_cvodemem == NULL) || (F2C_CVODE_nonlinsol == NULL) ) {
+    *ier = -1;
+    return;
+  }
+  if (((CVodeMem) CV_cvodemem)->cv_lsolve == NULL) {
+    FCVNullMatrix();
+    FCVNullLinsol();
+  }
+
+  *ier = CVodeSetNonlinearSolver(CV_cvodemem, F2C_CVODE_nonlinsol);
+  return;
+}
