@@ -3,7 +3,7 @@
  * Programmer(s): Radu Serban @ LLNL
  * -----------------------------------------------------------------
  * SUNDIALS Copyright Start
- * Copyright (c) 2002-2021, Lawrence Livermore National Security
+ * Copyright (c) 2002-2022, Lawrence Livermore National Security
  * and Southern Methodist University.
  * All rights reserved.
  *
@@ -42,6 +42,12 @@
 #define HUNDRED     RCONST(100.0)      /* real 100.0 */
 #define FUZZ_FACTOR RCONST(1000000.0)  /* fuzz factor for IMget */
 
+/*=================================================================*/
+/* Shortcuts                                                       */
+/*=================================================================*/
+
+#define CV_PROFILER cv_mem->cv_sunctx->profiler
+
 /*
  * =================================================================
  * PRIVATE FUNCTION PROTOTYPES
@@ -107,13 +113,17 @@ int CVodeAdjInit(void *cvode_mem, long int steps, int interp)
   }
   cv_mem = (CVodeMem)cvode_mem;
 
+  SUNDIALS_MARK_FUNCTION_BEGIN(CV_PROFILER);
+
   if (steps <= 0) {
     cvProcessError(cv_mem, CV_ILL_INPUT, "CVODEA", "CVodeAdjInit", MSGCV_BAD_STEPS);
+    SUNDIALS_MARK_FUNCTION_END(CV_PROFILER);
     return(CV_ILL_INPUT);
   }
 
   if ( (interp != CV_HERMITE) && (interp != CV_POLYNOMIAL) ) {
     cvProcessError(cv_mem, CV_ILL_INPUT, "CVODEA", "CVodeAdjInit", MSGCV_BAD_INTERP);
+    SUNDIALS_MARK_FUNCTION_END(CV_PROFILER);
     return(CV_ILL_INPUT);
   }
 
@@ -125,6 +135,7 @@ int CVodeAdjInit(void *cvode_mem, long int steps, int interp)
   ca_mem = (CVadjMem) malloc(sizeof(struct CVadjMemRec));
   if (ca_mem == NULL) {
     cvProcessError(cv_mem, CV_MEM_FAIL, "CVODEA", "CVodeAdjInit", MSGCV_MEM_FAIL);
+    SUNDIALS_MARK_FUNCTION_END(CV_PROFILER);
     return(CV_MEM_FAIL);
   }
 
@@ -167,6 +178,7 @@ int CVodeAdjInit(void *cvode_mem, long int steps, int interp)
   if (ca_mem->dt_mem == NULL) {
     free(ca_mem); ca_mem = NULL;
     cvProcessError(cv_mem, CV_MEM_FAIL, "CVODEA", "CVodeAdjInit", MSGCV_MEM_FAIL);
+    SUNDIALS_MARK_FUNCTION_END(CV_PROFILER);
     return(CV_MEM_FAIL);
   }
 
@@ -178,6 +190,7 @@ int CVodeAdjInit(void *cvode_mem, long int steps, int interp)
       free(ca_mem->dt_mem); ca_mem->dt_mem = NULL;
       free(ca_mem); ca_mem = NULL;
       cvProcessError(cv_mem, CV_MEM_FAIL, "CVODEA", "CVodeAdjInit", MSGCV_MEM_FAIL);
+      SUNDIALS_MARK_FUNCTION_END(CV_PROFILER);
       return(CV_MEM_FAIL);
     }
   }
@@ -245,6 +258,7 @@ int CVodeAdjInit(void *cvode_mem, long int steps, int interp)
   cv_mem->cv_adj = SUNTRUE;
   cv_mem->cv_adjMallocDone = SUNTRUE;
 
+  SUNDIALS_MARK_FUNCTION_END(CV_PROFILER);
   return(CV_SUCCESS);
 }
 
@@ -275,9 +289,12 @@ int CVodeAdjReInit(void *cvode_mem)
   }
   cv_mem = (CVodeMem) cvode_mem;
 
+  SUNDIALS_MARK_FUNCTION_BEGIN(CV_PROFILER);
+
   /* Was ASA initialized? */
   if (cv_mem->cv_adjMallocDone == SUNFALSE) {
     cvProcessError(cv_mem, CV_NO_ADJ, "CVODEA", "CVodeAdjReInit", MSGCV_NO_ADJ);
+    SUNDIALS_MARK_FUNCTION_END(CV_PROFILER);
     return(CV_NO_ADJ);
   }
 
@@ -299,6 +316,7 @@ int CVodeAdjReInit(void *cvode_mem)
   ca_mem->ca_tstopCVodeFcall = SUNFALSE;
   ca_mem->ca_firstCVodeBcall = SUNTRUE;
 
+  SUNDIALS_MARK_FUNCTION_END(CV_PROFILER);
   return(CV_SUCCESS);
 }
 
@@ -376,9 +394,12 @@ int CVodeF(void *cvode_mem, realtype tout, N_Vector yout,
   }
   cv_mem = (CVodeMem) cvode_mem;
 
+  SUNDIALS_MARK_FUNCTION_BEGIN(CV_PROFILER);
+
   /* Was ASA initialized? */
   if (cv_mem->cv_adjMallocDone == SUNFALSE) {
     cvProcessError(cv_mem, CV_NO_ADJ, "CVODEA", "CVodeF", MSGCV_NO_ADJ);
+    SUNDIALS_MARK_FUNCTION_END(CV_PROFILER);
     return(CV_NO_ADJ);
   }
 
@@ -387,18 +408,21 @@ int CVodeF(void *cvode_mem, realtype tout, N_Vector yout,
   /* Check for yout != NULL */
   if (yout == NULL) {
     cvProcessError(cv_mem, CV_ILL_INPUT, "CVODEA", "CVodeF", MSGCV_YOUT_NULL);
+    SUNDIALS_MARK_FUNCTION_END(CV_PROFILER);
     return(CV_ILL_INPUT);
   }
 
   /* Check for tret != NULL */
   if (tret == NULL) {
     cvProcessError(cv_mem, CV_ILL_INPUT, "CVODEA", "CVodeF", MSGCV_TRET_NULL);
+    SUNDIALS_MARK_FUNCTION_END(CV_PROFILER);
     return(CV_ILL_INPUT);
   }
 
   /* Check for valid itask */
   if ( (itask != CV_NORMAL) && (itask != CV_ONE_STEP) ) {
     cvProcessError(cv_mem, CV_ILL_INPUT, "CVODEA", "CVodeF", MSGCV_BAD_ITASK);
+    SUNDIALS_MARK_FUNCTION_END(CV_PROFILER);
     return(CV_ILL_INPUT);
   }
 
@@ -426,6 +450,7 @@ int CVodeF(void *cvode_mem, realtype tout, N_Vector yout,
     ca_mem->ck_mem = CVAckpntInit(cv_mem);
     if (ca_mem->ck_mem == NULL) {
       cvProcessError(cv_mem, CV_MEM_FAIL, "CVODEA", "CVodeF", MSGCV_MEM_FAIL);
+      SUNDIALS_MARK_FUNCTION_END(CV_PROFILER);
       return(CV_MEM_FAIL);
     }
 
@@ -438,6 +463,7 @@ int CVodeF(void *cvode_mem, realtype tout, N_Vector yout,
       allocOK = ca_mem->ca_IMmalloc(cv_mem);
       if (!allocOK) {
         cvProcessError(cv_mem, CV_MEM_FAIL, "CVODEA", "CVodeF", MSGCV_MEM_FAIL);
+        SUNDIALS_MARK_FUNCTION_END(CV_PROFILER);
         return(CV_MEM_FAIL);
       }
 
@@ -489,6 +515,7 @@ int CVodeF(void *cvode_mem, realtype tout, N_Vector yout,
       ca_mem->ca_IMnewData = SUNTRUE;
       ca_mem->ca_ckpntData = ca_mem->ck_mem;
       ca_mem->ca_np = cv_mem->cv_nst % ca_mem->ca_nsteps + 1;
+      SUNDIALS_MARK_FUNCTION_END(CV_PROFILER);
       return(flag);
     }
 
@@ -590,6 +617,7 @@ int CVodeF(void *cvode_mem, realtype tout, N_Vector yout,
   ca_mem->ca_ckpntData = ca_mem->ck_mem;
   ca_mem->ca_np = cv_mem->cv_nst % ca_mem->ca_nsteps + 1;
 
+  SUNDIALS_MARK_FUNCTION_END(CV_PROFILER);
   return(flag);
 }
 
@@ -634,7 +662,7 @@ int CVodeCreateB(void *cvode_mem, int lmmB, int *which)
 
   /* Create and set a new CVODES object for the backward problem */
 
-  cvodeB_mem = CVodeCreate(lmmB);
+  cvodeB_mem = CVodeCreate(lmmB, cv_mem->cv_sunctx);
   if (cvodeB_mem == NULL) {
     cvProcessError(cv_mem, CV_MEM_FAIL, "CVODEA", "CVodeCreateB", MSGCV_MEM_FAIL);
     return(CV_MEM_FAIL);
@@ -705,10 +733,13 @@ int CVodeInitB(void *cvode_mem, int which,
   }
   cv_mem = (CVodeMem) cvode_mem;
 
+  SUNDIALS_MARK_FUNCTION_BEGIN(CV_PROFILER);
+
   /* Was ASA initialized? */
 
   if (cv_mem->cv_adjMallocDone == SUNFALSE) {
     cvProcessError(cv_mem, CV_NO_ADJ, "CVODEA", "CVodeInitB", MSGCV_NO_ADJ);
+    SUNDIALS_MARK_FUNCTION_END(CV_PROFILER);
     return(CV_NO_ADJ);
   }
   ca_mem = cv_mem->cv_adj_mem;
@@ -717,6 +748,7 @@ int CVodeInitB(void *cvode_mem, int which,
 
   if ( which >= ca_mem->ca_nbckpbs ) {
     cvProcessError(cv_mem, CV_ILL_INPUT, "CVODEA", "CVodeInitB", MSGCV_BAD_WHICH);
+    SUNDIALS_MARK_FUNCTION_END(CV_PROFILER);
     return(CV_ILL_INPUT);
   }
 
@@ -734,7 +766,10 @@ int CVodeInitB(void *cvode_mem, int which,
 
   flag = CVodeInit(cvodeB_mem, CVArhs, tB0, yB0);
 
-  if (flag != CV_SUCCESS) return(flag);
+  if (flag != CV_SUCCESS) {
+    SUNDIALS_MARK_FUNCTION_END(CV_PROFILER);
+    return(flag);
+  }
 
   /* Copy fB function in cvB_mem */
 
@@ -747,6 +782,7 @@ int CVodeInitB(void *cvode_mem, int which,
   cvB_mem->cv_y = N_VClone(yB0);
   N_VScale(ONE, yB0, cvB_mem->cv_y);
 
+  SUNDIALS_MARK_FUNCTION_END(CV_PROFILER);
   return(CV_SUCCESS);
 }
 
@@ -768,10 +804,13 @@ int CVodeInitBS(void *cvode_mem, int which,
   }
   cv_mem = (CVodeMem) cvode_mem;
 
+  SUNDIALS_MARK_FUNCTION_BEGIN(CV_PROFILER);
+
   /* Was ASA initialized? */
 
   if (cv_mem->cv_adjMallocDone == SUNFALSE) {
     cvProcessError(cv_mem, CV_NO_ADJ, "CVODEA", "CVodeInitBS", MSGCV_NO_ADJ);
+    SUNDIALS_MARK_FUNCTION_END(CV_PROFILER);
     return(CV_NO_ADJ);
   }
   ca_mem = cv_mem->cv_adj_mem;
@@ -780,6 +819,7 @@ int CVodeInitBS(void *cvode_mem, int which,
 
   if ( which >= ca_mem->ca_nbckpbs ) {
     cvProcessError(cv_mem, CV_ILL_INPUT, "CVODEA", "CVodeInitBS", MSGCV_BAD_WHICH);
+    SUNDIALS_MARK_FUNCTION_END(CV_PROFILER);
     return(CV_ILL_INPUT);
   }
 
@@ -797,7 +837,10 @@ int CVodeInitBS(void *cvode_mem, int which,
 
   flag = CVodeInit(cvodeB_mem, CVArhs, tB0, yB0);
 
-  if (flag != CV_SUCCESS) return(flag);
+  if (flag != CV_SUCCESS) {
+    SUNDIALS_MARK_FUNCTION_END(CV_PROFILER);
+    return(flag);
+  }
 
   /* Copy fBs function in cvB_mem */
 
@@ -810,6 +853,7 @@ int CVodeInitBS(void *cvode_mem, int which,
   cvB_mem->cv_y = N_VClone(yB0);
   N_VScale(ONE, yB0, cvB_mem->cv_y);
 
+  SUNDIALS_MARK_FUNCTION_END(CV_PROFILER);
   return(CV_SUCCESS);
 }
 
@@ -830,9 +874,12 @@ int CVodeReInitB(void *cvode_mem, int which,
   }
   cv_mem = (CVodeMem) cvode_mem;
 
+  SUNDIALS_MARK_FUNCTION_BEGIN(CV_PROFILER);
+
   /* Was ASA initialized? */
   if (cv_mem->cv_adjMallocDone == SUNFALSE) {
     cvProcessError(cv_mem, CV_NO_ADJ, "CVODEA", "CVodeReInitB", MSGCV_NO_ADJ);
+    SUNDIALS_MARK_FUNCTION_END(CV_PROFILER);
     return(CV_NO_ADJ);
   }
   ca_mem = cv_mem->cv_adj_mem;
@@ -840,6 +887,7 @@ int CVodeReInitB(void *cvode_mem, int which,
   /* Check the value of which */
   if ( which >= ca_mem->ca_nbckpbs ) {
     cvProcessError(cv_mem, CV_ILL_INPUT, "CVODEA", "CVodeReInitB", MSGCV_BAD_WHICH);
+    SUNDIALS_MARK_FUNCTION_END(CV_PROFILER);
     return(CV_ILL_INPUT);
   }
 
@@ -856,6 +904,7 @@ int CVodeReInitB(void *cvode_mem, int which,
 
   flag = CVodeReInit(cvodeB_mem, tB0, yB0);
 
+  SUNDIALS_MARK_FUNCTION_END(CV_PROFILER);
   return(flag);
 }
 
@@ -974,9 +1023,12 @@ int CVodeQuadInitB(void *cvode_mem, int which,
   }
   cv_mem = (CVodeMem) cvode_mem;
 
+  SUNDIALS_MARK_FUNCTION_BEGIN(CV_PROFILER);
+
   /* Was ASA initialized? */
   if (cv_mem->cv_adjMallocDone == SUNFALSE) {
     cvProcessError(cv_mem, CV_NO_ADJ, "CVODEA", "CVodeQuadInitB", MSGCV_NO_ADJ);
+    SUNDIALS_MARK_FUNCTION_END(CV_PROFILER);
     return(CV_NO_ADJ);
   }
   ca_mem = cv_mem->cv_adj_mem;
@@ -984,6 +1036,7 @@ int CVodeQuadInitB(void *cvode_mem, int which,
   /* Check which */
   if ( which >= ca_mem->ca_nbckpbs ) {
     cvProcessError(cv_mem, CV_ILL_INPUT, "CVODEA", "CVodeQuadInitB", MSGCV_BAD_WHICH);
+    SUNDIALS_MARK_FUNCTION_END(CV_PROFILER);
     return(CV_ILL_INPUT);
   }
 
@@ -997,11 +1050,15 @@ int CVodeQuadInitB(void *cvode_mem, int which,
   cvodeB_mem = (void *) (cvB_mem->cv_mem);
 
   flag = CVodeQuadInit(cvodeB_mem, CVArhsQ, yQB0);
-  if (flag != CV_SUCCESS) return(flag);
+  if (flag != CV_SUCCESS) {
+    SUNDIALS_MARK_FUNCTION_END(CV_PROFILER);
+    return(flag);
+  }
 
   cvB_mem->cv_fQ_withSensi = SUNFALSE;
   cvB_mem->cv_fQ = fQB;
 
+  SUNDIALS_MARK_FUNCTION_END(CV_PROFILER);
   return(CV_SUCCESS);
 }
 
@@ -1021,9 +1078,12 @@ int CVodeQuadInitBS(void *cvode_mem, int which,
   }
   cv_mem = (CVodeMem) cvode_mem;
 
+  SUNDIALS_MARK_FUNCTION_BEGIN(CV_PROFILER);
+
   /* Was ASA initialized? */
   if (cv_mem->cv_adjMallocDone == SUNFALSE) {
     cvProcessError(cv_mem, CV_NO_ADJ, "CVODEA", "CVodeQuadInitBS", MSGCV_NO_ADJ);
+    SUNDIALS_MARK_FUNCTION_END(CV_PROFILER);
     return(CV_NO_ADJ);
   }
   ca_mem = cv_mem->cv_adj_mem;
@@ -1031,6 +1091,7 @@ int CVodeQuadInitBS(void *cvode_mem, int which,
   /* Check which */
   if ( which >= ca_mem->ca_nbckpbs ) {
     cvProcessError(cv_mem, CV_ILL_INPUT, "CVODEA", "CVodeQuadInitBS", MSGCV_BAD_WHICH);
+    SUNDIALS_MARK_FUNCTION_END(CV_PROFILER);
     return(CV_ILL_INPUT);
   }
 
@@ -1044,11 +1105,15 @@ int CVodeQuadInitBS(void *cvode_mem, int which,
   cvodeB_mem = (void *) (cvB_mem->cv_mem);
 
   flag = CVodeQuadInit(cvodeB_mem, CVArhsQ, yQB0);
-  if (flag != CV_SUCCESS) return(flag);
+  if (flag != CV_SUCCESS) {
+    SUNDIALS_MARK_FUNCTION_END(CV_PROFILER);
+    return(flag);
+  }
 
   cvB_mem->cv_fQ_withSensi = SUNTRUE;
   cvB_mem->cv_fQs = fQBs;
 
+  SUNDIALS_MARK_FUNCTION_END(CV_PROFILER);
   return(CV_SUCCESS);
 }
 
@@ -1067,9 +1132,12 @@ int CVodeQuadReInitB(void *cvode_mem, int which, N_Vector yQB0)
   }
   cv_mem = (CVodeMem) cvode_mem;
 
+  SUNDIALS_MARK_FUNCTION_BEGIN(CV_PROFILER);
+
   /* Was ASA initialized? */
   if (cv_mem->cv_adjMallocDone == SUNFALSE) {
     cvProcessError(cv_mem, CV_NO_ADJ, "CVODEA", "CVodeQuadReInitB", MSGCV_NO_ADJ);
+    SUNDIALS_MARK_FUNCTION_END(CV_PROFILER);
     return(CV_NO_ADJ);
   }
   ca_mem = cv_mem->cv_adj_mem;
@@ -1077,6 +1145,7 @@ int CVodeQuadReInitB(void *cvode_mem, int which, N_Vector yQB0)
   /* Check the value of which */
   if ( which >= ca_mem->ca_nbckpbs ) {
     cvProcessError(cv_mem, CV_ILL_INPUT, "CVODEA", "CVodeQuadReInitB", MSGCV_BAD_WHICH);
+    SUNDIALS_MARK_FUNCTION_END(CV_PROFILER);
     return(CV_ILL_INPUT);
   }
 
@@ -1090,8 +1159,12 @@ int CVodeQuadReInitB(void *cvode_mem, int which, N_Vector yQB0)
   cvodeB_mem = (void *) (cvB_mem->cv_mem);
 
   flag = CVodeQuadReInit(cvodeB_mem, yQB0);
-  if (flag != CV_SUCCESS) return(flag);
+  if (flag != CV_SUCCESS) {
+    SUNDIALS_MARK_FUNCTION_END(CV_PROFILER);
+    return(flag);
+  }
 
+  SUNDIALS_MARK_FUNCTION_END(CV_PROFILER);
   return(CV_SUCCESS);
 }
 
@@ -1110,9 +1183,12 @@ int CVodeQuadSStolerancesB(void *cvode_mem, int which, realtype reltolQB, realty
   }
   cv_mem = (CVodeMem) cvode_mem;
 
+  SUNDIALS_MARK_FUNCTION_BEGIN(CV_PROFILER);
+
   /* Was ASA initialized? */
   if (cv_mem->cv_adjMallocDone == SUNFALSE) {
     cvProcessError(cv_mem, CV_NO_ADJ, "CVODEA", "CVodeQuadSStolerancesB", MSGCV_NO_ADJ);
+    SUNDIALS_MARK_FUNCTION_END(CV_PROFILER);
     return(CV_NO_ADJ);
   }
   ca_mem = cv_mem->cv_adj_mem;
@@ -1120,6 +1196,7 @@ int CVodeQuadSStolerancesB(void *cvode_mem, int which, realtype reltolQB, realty
   /* Check which */
   if ( which >= ca_mem->ca_nbckpbs ) {
     cvProcessError(cv_mem, CV_ILL_INPUT, "CVODEA", "CVodeQuadSStolerancesB", MSGCV_BAD_WHICH);
+    SUNDIALS_MARK_FUNCTION_END(CV_PROFILER);
     return(CV_ILL_INPUT);
   }
 
@@ -1134,6 +1211,7 @@ int CVodeQuadSStolerancesB(void *cvode_mem, int which, realtype reltolQB, realty
 
   flag = CVodeQuadSStolerances(cvodeB_mem, reltolQB, abstolQB);
 
+  SUNDIALS_MARK_FUNCTION_END(CV_PROFILER);
   return(flag);
 }
 
@@ -1216,10 +1294,13 @@ int CVodeB(void *cvode_mem, realtype tBout, int itaskB)
   }
   cv_mem = (CVodeMem) cvode_mem;
 
+  SUNDIALS_MARK_FUNCTION_BEGIN(CV_PROFILER);
+
   /* Was ASA initialized? */
 
   if (cv_mem->cv_adjMallocDone == SUNFALSE) {
     cvProcessError(cv_mem, CV_NO_ADJ, "CVODEA", "CVodeB", MSGCV_NO_ADJ);
+    SUNDIALS_MARK_FUNCTION_END(CV_PROFILER);
     return(CV_NO_ADJ);
   }
   ca_mem = cv_mem->cv_adj_mem;
@@ -1228,6 +1309,7 @@ int CVodeB(void *cvode_mem, realtype tBout, int itaskB)
 
   if ( ca_mem->ca_nbckpbs == 0 ) {
     cvProcessError(cv_mem, CV_NO_BCK, "CVODEA", "CVodeB", MSGCV_NO_BCK);
+    SUNDIALS_MARK_FUNCTION_END(CV_PROFILER);
     return(CV_NO_BCK);
   }
   cvB_mem = ca_mem->cvB_mem;
@@ -1236,6 +1318,7 @@ int CVodeB(void *cvode_mem, realtype tBout, int itaskB)
 
   if ( ca_mem->ca_firstCVodeFcall ) {
     cvProcessError(cv_mem, CV_NO_FWD, "CVODEA", "CVodeB", MSGCV_NO_FWD);
+    SUNDIALS_MARK_FUNCTION_END(CV_PROFILER);
     return(CV_NO_FWD);
   }
   sign = (ca_mem->ca_tfinal - ca_mem->ca_tinitial > ZERO) ? 1 : -1;
@@ -1257,12 +1340,14 @@ int CVodeB(void *cvode_mem, realtype tBout, int itaskB)
       if ( (sign*(tBn-ca_mem->ca_tinitial) < ZERO) || (sign*(ca_mem->ca_tfinal-tBn) < ZERO) ) {
         cvProcessError(cv_mem, CV_BAD_TB0, "CVODEA", "CVodeB", MSGCV_BAD_TB0,
                        tmp_cvB_mem->cv_index);
+        SUNDIALS_MARK_FUNCTION_END(CV_PROFILER);
         return(CV_BAD_TB0);
       }
 
       if (sign*(tBn-tBout) <= ZERO) {
         cvProcessError(cv_mem, CV_ILL_INPUT, "CVODEA", "CVodeB", MSGCV_BAD_TBOUT,
                        tmp_cvB_mem->cv_index);
+        SUNDIALS_MARK_FUNCTION_END(CV_PROFILER);
         return(CV_ILL_INPUT);
       }
 
@@ -1275,6 +1360,7 @@ int CVodeB(void *cvode_mem, realtype tBout, int itaskB)
 
     if ( ca_mem->ca_IMinterpSensi && !ca_mem->ca_IMstoreSensi) {
       cvProcessError(cv_mem, CV_ILL_INPUT, "CVODEA", "CVodeB", MSGCV_BAD_SENSI);
+      SUNDIALS_MARK_FUNCTION_END(CV_PROFILER);
       return(CV_ILL_INPUT);
     }
 
@@ -1285,6 +1371,7 @@ int CVodeB(void *cvode_mem, realtype tBout, int itaskB)
 
   if ( (itaskB != CV_NORMAL) && (itaskB != CV_ONE_STEP) ) {
     cvProcessError(cv_mem, CV_ILL_INPUT, "CVODEA", "CVodeB", MSGCV_BAD_ITASKB);
+    SUNDIALS_MARK_FUNCTION_END(CV_PROFILER);
     return(CV_ILL_INPUT);
   }
 
@@ -1296,6 +1383,7 @@ int CVodeB(void *cvode_mem, realtype tBout, int itaskB)
       tBout = ca_mem->ca_tinitial;
     } else {
       cvProcessError(cv_mem, CV_ILL_INPUT, "CVODEA", "CVodeB", MSGCV_BAD_TBOUT);
+      SUNDIALS_MARK_FUNCTION_END(CV_PROFILER);
       return(CV_ILL_INPUT);
     }
   }
@@ -1395,6 +1483,7 @@ int CVodeB(void *cvode_mem, realtype tBout, int itaskB)
     if (flag <0) {
       cvProcessError(cv_mem, flag, "CVODEA", "CVodeB", MSGCV_BACK_ERROR,
                      tmp_cvB_mem->cv_index);
+      SUNDIALS_MARK_FUNCTION_END(CV_PROFILER);
       return(flag);
     }
 
@@ -1423,6 +1512,7 @@ int CVodeB(void *cvode_mem, realtype tBout, int itaskB)
 
   }
 
+  SUNDIALS_MARK_FUNCTION_END(CV_PROFILER);
   return(flag);
 }
 
