@@ -1,10 +1,10 @@
 /*
- * ----------------------------------------------------------------- 
+ * -----------------------------------------------------------------
  * Programmer(s): Daniel R. Reynolds @ SMU
  *                Radu Serban and Aaron Collier @ LLNL
  * -----------------------------------------------------------------
  * SUNDIALS Copyright Start
- * Copyright (c) 2002-2019, Lawrence Livermore National Security
+ * Copyright (c) 2002-2022, Lawrence Livermore National Security
  * and Southern Methodist University.
  * All rights reserved.
  *
@@ -33,11 +33,11 @@
 #define TWO          RCONST(2.0)
 
 /* Prototypes of cvBandPrecSetup and cvBandPrecSolve */
-static int cvBandPrecSetup(realtype t, N_Vector y, N_Vector fy, 
-                           booleantype jok, booleantype *jcurPtr, 
+static int cvBandPrecSetup(realtype t, N_Vector y, N_Vector fy,
+                           booleantype jok, booleantype *jcurPtr,
                            realtype gamma, void *bp_data);
-static int cvBandPrecSolve(realtype t, N_Vector y, N_Vector fy, 
-                           N_Vector r, N_Vector z, 
+static int cvBandPrecSolve(realtype t, N_Vector y, N_Vector fy,
+                           N_Vector r, N_Vector z,
                            realtype gamma, realtype delta,
                            int lr, void *bp_data);
 
@@ -45,7 +45,7 @@ static int cvBandPrecSolve(realtype t, N_Vector y, N_Vector fy,
 static int cvBandPrecFree(CVodeMem cv_mem);
 
 /* Prototype for difference quotient Jacobian calculation routine */
-static int cvBandPrecDQJac(CVBandPrecData pdata, realtype t, N_Vector y, 
+static int cvBandPrecDQJac(CVBandPrecData pdata, realtype t, N_Vector y,
                            N_Vector fy, N_Vector ftemp, N_Vector ytemp);
 
 
@@ -55,10 +55,10 @@ static int cvBandPrecDQJac(CVBandPrecData pdata, realtype t, N_Vector y,
 
 /*-----------------------------------------------------------------
   Initialization, Free, and Get Functions
-  NOTE: The band linear solver assumes a serial/OpenMP/Pthreads 
-        implementation of the NVECTOR package. Therefore, 
-        CVBandPrecInit will first test for a compatible N_Vector 
-        internal representation by checking that the function 
+  NOTE: The band linear solver assumes a serial/OpenMP/Pthreads
+        implementation of the NVECTOR package. Therefore,
+        CVBandPrecInit will first test for a compatible N_Vector
+        internal representation by checking that the function
         N_VGetArrayPointer exists.
   -----------------------------------------------------------------*/
 int CVBandPrecInit(void *cvode_mem, sunindextype N,
@@ -112,7 +112,7 @@ int CVBandPrecInit(void *cvode_mem, sunindextype N,
 
   /* Allocate memory for saved banded Jacobian approximation. */
   pdata->savedJ = NULL;
-  pdata->savedJ = SUNBandMatrixStorage(N, mup, mlp, mup);
+  pdata->savedJ = SUNBandMatrixStorage(N, mup, mlp, mup, cv_mem->cv_sunctx);
   if (pdata->savedJ == NULL) {
     free(pdata); pdata = NULL;
     cvProcessError(cv_mem, CVLS_MEM_FAIL, "CVSBANDPRE",
@@ -123,7 +123,7 @@ int CVBandPrecInit(void *cvode_mem, sunindextype N,
   /* Allocate memory for banded preconditioner. */
   storagemu = SUNMIN(N-1, mup+mlp);
   pdata->savedP = NULL;
-  pdata->savedP = SUNBandMatrixStorage(N, mup, mlp, storagemu);
+  pdata->savedP = SUNBandMatrixStorage(N, mup, mlp, storagemu, cv_mem->cv_sunctx);
   if (pdata->savedP == NULL) {
     SUNMatDestroy(pdata->savedJ);
     free(pdata); pdata = NULL;
@@ -134,7 +134,7 @@ int CVBandPrecInit(void *cvode_mem, sunindextype N,
 
   /* Allocate memory for banded linear solver */
   pdata->LS = NULL;
-  pdata->LS = SUNLinSol_Band(cv_mem->cv_tempv, pdata->savedP);
+  pdata->LS = SUNLinSol_Band(cv_mem->cv_tempv, pdata->savedP, cv_mem->cv_sunctx);
   if (pdata->LS == NULL) {
     SUNMatDestroy(pdata->savedP);
     SUNMatDestroy(pdata->savedJ);
@@ -152,7 +152,7 @@ int CVBandPrecInit(void *cvode_mem, sunindextype N,
     SUNMatDestroy(pdata->savedP);
     SUNMatDestroy(pdata->savedJ);
     free(pdata); pdata = NULL;
-    cvProcessError(cv_mem, CVLS_MEM_FAIL, "CVSBANDPRE", 
+    cvProcessError(cv_mem, CVLS_MEM_FAIL, "CVSBANDPRE",
                     "CVBandPrecInit", MSGBP_MEM_FAIL);
     return(CVLS_MEM_FAIL);
   }
@@ -164,7 +164,7 @@ int CVBandPrecInit(void *cvode_mem, sunindextype N,
     SUNMatDestroy(pdata->savedJ);
     N_VDestroy(pdata->tmp1);
     free(pdata); pdata = NULL;
-    cvProcessError(cv_mem, CVLS_MEM_FAIL, "CVSBANDPRE", 
+    cvProcessError(cv_mem, CVLS_MEM_FAIL, "CVSBANDPRE",
                     "CVBandPrecInit", MSGBP_MEM_FAIL);
     return(CVLS_MEM_FAIL);
   }
@@ -178,11 +178,11 @@ int CVBandPrecInit(void *cvode_mem, sunindextype N,
     N_VDestroy(pdata->tmp1);
     N_VDestroy(pdata->tmp2);
     free(pdata); pdata = NULL;
-    cvProcessError(cv_mem, CVLS_SUNLS_FAIL, "CVSBANDPRE", 
+    cvProcessError(cv_mem, CVLS_SUNLS_FAIL, "CVSBANDPRE",
                     "CVBandPrecInit", MSGBP_SUNLS_FAIL);
     return(CVLS_SUNLS_FAIL);
   }
-  
+
   /* make sure P_data is free from any previous allocations */
   if (cvls_mem->pfree)
     cvls_mem->pfree(cv_mem);
@@ -209,7 +209,7 @@ int CVBandPrecGetWorkSpace(void *cvode_mem, long int *lenrwBP,
   sunindextype lrw1, liw1;
   long int lrw, liw;
   int flag;
-  
+
   if (cvode_mem == NULL) {
     cvProcessError(NULL, CVLS_MEM_NULL, "CVSBANDPRE",
                    "CVBandPrecGetWorkSpace", MSGBP_MEM_NULL);
@@ -228,7 +228,7 @@ int CVBandPrecGetWorkSpace(void *cvode_mem, long int *lenrwBP,
     cvProcessError(cv_mem, CVLS_PMEM_NULL, "CVSBANDPRE",
                    "CVBandPrecGetWorkSpace", MSGBP_PMEM_NULL);
     return(CVLS_PMEM_NULL);
-  } 
+  }
   pdata = (CVBandPrecData) cvls_mem->P_data;
 
   /* sum space requirements for all objects in pdata */
@@ -286,7 +286,7 @@ int CVBandPrecGetNumRhsEvals(void *cvode_mem, long int *nfevalsBP)
     cvProcessError(cv_mem, CVLS_PMEM_NULL, "CVSBANDPRE",
                    "CVBandPrecGetNumRhsEvals", MSGBP_PMEM_NULL);
     return(CVLS_PMEM_NULL);
-  } 
+  }
   pdata = (CVBandPrecData) cvls_mem->P_data;
 
   *nfevalsBP = pdata->nfeBP;
@@ -302,16 +302,16 @@ int CVBandPrecGetNumRhsEvals(void *cvode_mem, long int *nfevalsBP)
   difference quotient Jacobian to create a preconditioner.
   cvBandPrecSetup calculates a new J, if necessary, then
   calculates P = I - gamma*J, and does an LU factorization of P.
- 
+
   The parameters of cvBandPrecSetup are as follows:
- 
+
   t       is the current value of the independent variable.
- 
+
   y       is the current value of the dependent variable vector,
           namely the predicted value of y(t).
- 
+
   fy      is the vector f(t,y).
- 
+
   jok     is an input flag indicating whether Jacobian-related
           data needs to be recomputed, as follows:
             jok == SUNFALSE means recompute Jacobian-related data
@@ -321,29 +321,28 @@ int CVBandPrecGetNumRhsEvals(void *cvode_mem, long int *nfevalsBP)
                    (with the current value of gamma).
           A cvBandPrecSetup call with jok == SUNTRUE should only
           occur after a call with jok == SUNFALSE.
- 
+
   *jcurPtr is a pointer to an output integer flag which is
            set by cvBandPrecSetup as follows:
              *jcurPtr = SUNTRUE if Jacobian data was recomputed.
              *jcurPtr = SUNFALSE if Jacobian data was not recomputed,
                         but saved data was reused.
- 
+
   gamma   is the scalar appearing in the Newton matrix.
- 
+
   bp_data is a pointer to preconditoner data (set by cvBandPrecInit)
- 
+
   The value to be returned by the cvBandPrecSetup function is
     0  if successful, or
     1  if the band factorization failed.
   -----------------------------------------------------------------*/
-static int cvBandPrecSetup(realtype t, N_Vector y, N_Vector fy, 
-                           booleantype jok, booleantype *jcurPtr, 
+static int cvBandPrecSetup(realtype t, N_Vector y, N_Vector fy,
+                           booleantype jok, booleantype *jcurPtr,
                            realtype gamma, void *bp_data)
 {
   CVBandPrecData pdata;
   CVodeMem cv_mem;
   int retval;
-  sunindextype ier;
 
   /* Assume matrix and lpivots have already been allocated. */
   pdata = (CVBandPrecData) bp_data;
@@ -355,7 +354,7 @@ static int cvBandPrecSetup(realtype t, N_Vector y, N_Vector fy,
     *jcurPtr = SUNFALSE;
     retval = SUNMatCopy(pdata->savedJ, pdata->savedP);
     if (retval < 0) {
-      cvProcessError(cv_mem, -1, "CVBANDPRE", 
+      cvProcessError(cv_mem, -1, "CVBANDPRE",
                      "cvBandPrecSetup", MSGBP_SUNMAT_FAIL);
       return(-1);
     }
@@ -369,7 +368,7 @@ static int cvBandPrecSetup(realtype t, N_Vector y, N_Vector fy,
     *jcurPtr = SUNTRUE;
     retval = SUNMatZero(pdata->savedJ);
     if (retval < 0) {
-      cvProcessError(cv_mem, -1, "CVBANDPRE", 
+      cvProcessError(cv_mem, -1, "CVBANDPRE",
                      "cvBandPrecSetup", MSGBP_SUNMAT_FAIL);
       return(-1);
     }
@@ -377,10 +376,10 @@ static int cvBandPrecSetup(realtype t, N_Vector y, N_Vector fy,
       return(1);
     }
 
-    retval = cvBandPrecDQJac(pdata, t, y, fy, 
+    retval = cvBandPrecDQJac(pdata, t, y, fy,
                              pdata->tmp1, pdata->tmp2);
     if (retval < 0) {
-      cvProcessError(cv_mem, -1, "CVBANDPRE", 
+      cvProcessError(cv_mem, -1, "CVBANDPRE",
                      "cvBandPrecSetup", MSGBP_RHSFUNC_FAILED);
       return(-1);
     }
@@ -390,7 +389,7 @@ static int cvBandPrecSetup(realtype t, N_Vector y, N_Vector fy,
 
     retval = SUNMatCopy(pdata->savedJ, pdata->savedP);
     if (retval < 0) {
-      cvProcessError(cv_mem, -1, "CVBANDPRE", 
+      cvProcessError(cv_mem, -1, "CVBANDPRE",
                      "cvBandPrecSetup", MSGBP_SUNMAT_FAIL);
       return(-1);
     }
@@ -399,18 +398,18 @@ static int cvBandPrecSetup(realtype t, N_Vector y, N_Vector fy,
     }
 
   }
-  
+
   /* Scale and add identity to get savedP = I - gamma*J. */
   retval = SUNMatScaleAddI(-gamma, pdata->savedP);
   if (retval) {
-    cvProcessError(cv_mem, -1, "CVBANDPRE", 
+    cvProcessError(cv_mem, -1, "CVBANDPRE",
                    "cvBandPrecSetup", MSGBP_SUNMAT_FAIL);
     return(-1);
   }
 
   /* Do LU factorization of matrix and return error flag */
-  ier = SUNLinSolSetup_Band(pdata->LS, pdata->savedP);
-  return(ier);
+  retval = SUNLinSolSetup_Band(pdata->LS, pdata->savedP);
+  return(retval);
 }
 
 
@@ -419,20 +418,20 @@ static int cvBandPrecSetup(realtype t, N_Vector y, N_Vector fy,
   -----------------------------------------------------------------
   cvBandPrecSolve solves a linear system P z = r, where P is the
   matrix computed by cvBandPrecond.
- 
+
   The parameters of cvBandPrecSolve used here are as follows:
- 
+
   r       is the right-hand side vector of the linear system.
- 
+
   bp_data is a pointer to preconditoner data (set by CVBandPrecInit)
- 
+
   z       is the output vector computed by cvBandPrecSolve.
- 
+
   The value returned by the cvBandPrecSolve function is always 0,
   indicating success.
-  -----------------------------------------------------------------*/ 
-static int cvBandPrecSolve(realtype t, N_Vector y, N_Vector fy, 
-                           N_Vector r, N_Vector z, realtype gamma, 
+  -----------------------------------------------------------------*/
+static int cvBandPrecSolve(realtype t, N_Vector y, N_Vector fy,
+                           N_Vector r, N_Vector z, realtype gamma,
                            realtype delta, int lr, void *bp_data)
 {
   CVBandPrecData pdata;
@@ -454,7 +453,7 @@ static int cvBandPrecFree(CVodeMem cv_mem)
 
   if (cv_mem->cv_lmem == NULL) return(0);
   cvls_mem = (CVLsMem) cv_mem->cv_lmem;
-  
+
   if (cvls_mem->P_data == NULL) return(0);
   pdata = (CVBandPrecData) cvls_mem->P_data;
 
@@ -474,12 +473,12 @@ static int cvBandPrecFree(CVodeMem cv_mem)
 /*-----------------------------------------------------------------
   cvBandPrecDQJac
   -----------------------------------------------------------------
-  This routine generates a banded difference quotient approximation 
-  to the Jacobian of f(t,y). It assumes that a band SUNMatrix is 
-  stored column-wise, and that elements within each column are 
+  This routine generates a banded difference quotient approximation
+  to the Jacobian of f(t,y). It assumes that a band SUNMatrix is
+  stored column-wise, and that elements within each column are
   contiguous. This makes it possible to get the address of a column
-  of J via the accessor function SUNBandMatrix_Column() and to 
-  write a simple for loop to set each of the elements of a column 
+  of J via the accessor function SUNBandMatrix_Column() and to
+  write a simple for loop to set each of the elements of a column
   in succession.
   -----------------------------------------------------------------*/
 static int cvBandPrecDQJac(CVBandPrecData pdata, realtype t, N_Vector y,
@@ -492,6 +491,9 @@ static int cvBandPrecDQJac(CVBandPrecData pdata, realtype t, N_Vector y,
   realtype *y_data, *ytemp_data, *cns_data;
   int retval;
 
+  /* initialize cns_data to avoid compiler warning */
+  cns_data = NULL;
+
   cv_mem = (CVodeMem) pdata->cvode_mem;
 
   /* Obtain pointers to the data for ewt, fy, ftemp, y, ytemp. */
@@ -500,7 +502,7 @@ static int cvBandPrecDQJac(CVBandPrecData pdata, realtype t, N_Vector y,
   ftemp_data = N_VGetArrayPointer(ftemp);
   y_data     = N_VGetArrayPointer(y);
   ytemp_data = N_VGetArrayPointer(ytemp);
-  if (cv_mem->cv_constraints != NULL)
+  if (cv_mem->cv_constraintsSet)
     cns_data  = N_VGetArrayPointer(cv_mem->cv_constraints);
 
   /* Load ytemp with y = predicted y vector. */
@@ -515,16 +517,16 @@ static int cvBandPrecDQJac(CVBandPrecData pdata, realtype t, N_Vector y,
   /* Set bandwidth and number of column groups for band differencing. */
   width = pdata->ml + pdata->mu + 1;
   ngroups = SUNMIN(width, pdata->N);
-  
+
   for (group = 1; group <= ngroups; group++) {
-    
+
     /* Increment all y_j in group. */
     for(j = group-1; j < pdata->N; j += width) {
       inc = SUNMAX(srur*SUNRabs(y_data[j]), minInc/ewt_data[j]);
       yj = y_data[j];
 
       /* Adjust sign(inc) again if yj has an inequality constraint. */
-      if (cv_mem->cv_constraints != NULL) {
+      if (cv_mem->cv_constraintsSet) {
         conj = cns_data[j];
         if (SUNRabs(conj) == ONE)      {if ((yj+inc)*conj < ZERO)  inc = -inc;}
         else if (SUNRabs(conj) == TWO) {if ((yj+inc)*conj <= ZERO) inc = -inc;}
@@ -546,7 +548,7 @@ static int cvBandPrecDQJac(CVBandPrecData pdata, realtype t, N_Vector y,
       inc = SUNMAX(srur*SUNRabs(y_data[j]), minInc/ewt_data[j]);
 
       /* Adjust sign(inc) as before. */
-      if (cv_mem->cv_constraints != NULL) {
+      if (cv_mem->cv_constraintsSet) {
         conj = cns_data[j];
         if (SUNRabs(conj) == ONE)      {if ((yj+inc)*conj < ZERO)  inc = -inc;}
         else if (SUNRabs(conj) == TWO) {if ((yj+inc)*conj <= ZERO) inc = -inc;}
@@ -594,7 +596,7 @@ int CVBandPrecInitB(void *cvode_mem, int which, sunindextype nB,
     cvProcessError(cv_mem, CVLS_NO_ADJ, "CVSBANDPRE",
                    "CVBandPrecInitB", MSGBP_NO_ADJ);
     return(CVLS_NO_ADJ);
-  } 
+  }
   ca_mem = cv_mem->cv_adj_mem;
 
   /* Check which */
@@ -616,7 +618,7 @@ int CVBandPrecInitB(void *cvode_mem, int which, sunindextype nB,
 
   /* Set pfree */
   cvB_mem->cv_pfree = NULL;
- 
+
   /* Initialize the band preconditioner for this backward problem. */
   flag = CVBandPrecInit(cvodeB_mem, nB, muB, mlB);
   return(flag);
