@@ -3,7 +3,7 @@
  *                Aaron Collier @ LLNL
  * -----------------------------------------------------------------
  * SUNDIALS Copyright Start
- * Copyright (c) 2002-2021, Lawrence Livermore National Security
+ * Copyright (c) 2002-2022, Lawrence Livermore National Security
  * and Southern Methodist University.
  * All rights reserved.
  *
@@ -20,6 +20,7 @@
 #include <stdlib.h>
 
 #include "ida_impl.h"
+#include "ida_ls_impl.h"
 #include "sundials/sundials_types.h"
 #include "sundials/sundials_math.h"
 
@@ -34,6 +35,26 @@
  * =================================================================
  */
 
+int IDASetDeltaCjLSetup(void *ida_mem, realtype dcj)
+{
+  IDAMem IDA_mem;
+
+  if (ida_mem == NULL) {
+    IDAProcessError(NULL, IDA_MEM_NULL, "IDA", "IDASetDeltaCjLSetup",
+                    MSG_NO_MEM);
+    return(IDA_MEM_NULL);
+  }
+
+  IDA_mem = (IDAMem) ida_mem;
+
+  if (dcj < ZERO || dcj >= ONE)
+    IDA_mem->ida_dcj = DCJ_DEFAULT;
+  else
+    IDA_mem->ida_dcj = dcj;
+
+  return(IDA_SUCCESS);
+}
+
 int IDASetErrHandlerFn(void *ida_mem, IDAErrHandlerFn ehfun, void *eh_data)
 {
   IDAMem IDA_mem;
@@ -88,6 +109,144 @@ int IDASetUserData(void *ida_mem, void *user_data)
 
 /*-----------------------------------------------------------------*/
 
+int IDASetEtaFixedStepBounds(void *ida_mem, realtype eta_min_fx,
+                             realtype eta_max_fx)
+{
+  IDAMem IDA_mem;
+
+  if (ida_mem == NULL) {
+    IDAProcessError(NULL, IDA_MEM_NULL, "IDA", "IDASetEtaFixedStepBounds",
+                    MSG_NO_MEM);
+    return(IDA_MEM_NULL);
+  }
+  IDA_mem = (IDAMem) ida_mem;
+
+  /* set allowed value or use default */
+  if (eta_min_fx >= ZERO && eta_min_fx <= ONE)
+    IDA_mem->ida_eta_min_fx = eta_min_fx;
+  else
+    IDA_mem->ida_eta_min_fx = ETA_MIN_FX_DEFAULT;
+
+  if (eta_max_fx >= ONE)
+    IDA_mem->ida_eta_max_fx = eta_max_fx;
+  else
+    IDA_mem->ida_eta_max_fx = ETA_MAX_FX_DEFAULT;
+
+  return(IDA_SUCCESS);
+}
+
+/*-----------------------------------------------------------------*/
+
+int IDASetEtaMax(void *ida_mem, realtype eta_max)
+{
+  IDAMem IDA_mem;
+
+  if (ida_mem == NULL) {
+    IDAProcessError(NULL, IDA_MEM_NULL, "IDA", "IDASetEtaMax",
+                    MSG_NO_MEM);
+    return(IDA_MEM_NULL);
+  }
+  IDA_mem = (IDAMem) ida_mem;
+
+  /* set allowed value or use default */
+  if (eta_max <= ONE)
+    IDA_mem->ida_eta_max = ETA_MAX_DEFAULT;
+  else
+    IDA_mem->ida_eta_max = eta_max;
+
+  return(IDA_SUCCESS);
+}
+
+/*-----------------------------------------------------------------*/
+
+int IDASetEtaMin(void *ida_mem, realtype eta_min)
+{
+  IDAMem IDA_mem;
+
+  if (ida_mem == NULL) {
+    IDAProcessError(NULL, IDA_MEM_NULL, "IDA", "IDASetEtaMin",
+                    MSG_NO_MEM);
+    return(IDA_MEM_NULL);
+  }
+  IDA_mem = (IDAMem) ida_mem;
+
+  /* set allowed value or use default */
+  if (eta_min <= ZERO || eta_min >= ONE)
+    IDA_mem->ida_eta_min = ETA_MIN_DEFAULT;
+  else
+    IDA_mem->ida_eta_min = eta_min;
+
+  return(IDA_SUCCESS);
+}
+
+/*-----------------------------------------------------------------*/
+
+int IDASetEtaLow(void *ida_mem, realtype eta_low)
+{
+  IDAMem IDA_mem;
+
+  if (ida_mem == NULL) {
+    IDAProcessError(NULL, IDA_MEM_NULL, "IDA", "IDASetEtaLow",
+                    MSG_NO_MEM);
+    return(IDA_MEM_NULL);
+  }
+  IDA_mem = (IDAMem) ida_mem;
+
+  /* set allowed value or use default */
+  if (eta_low <= ZERO || eta_low >= ONE)
+    IDA_mem->ida_eta_low = ETA_LOW_DEFAULT;
+  else
+    IDA_mem->ida_eta_low = eta_low;
+
+  return(IDA_SUCCESS);
+}
+
+/*-----------------------------------------------------------------*/
+
+int IDASetEtaMinErrFail(void *ida_mem, realtype eta_min_ef)
+{
+  IDAMem IDA_mem;
+
+  if (ida_mem == NULL) {
+    IDAProcessError(NULL, IDA_MEM_NULL, "IDA", "IDASetEtaMinErrFail",
+                    MSG_NO_MEM);
+    return(IDA_MEM_NULL);
+  }
+  IDA_mem = (IDAMem) ida_mem;
+
+  /* set allowed value or use default */
+  if (eta_min_ef <= ZERO || eta_min_ef >= ONE)
+    IDA_mem->ida_eta_min_ef = ETA_MIN_EF_DEFAULT;
+  else
+    IDA_mem->ida_eta_min_ef = eta_min_ef;
+
+  return(IDA_SUCCESS);
+}
+
+/*-----------------------------------------------------------------*/
+
+int IDASetEtaConvFail(void *ida_mem, realtype eta_cf)
+{
+  IDAMem IDA_mem;
+
+  if (ida_mem == NULL) {
+    IDAProcessError(NULL, IDA_MEM_NULL, "IDA", "IDASetEtaConvFail",
+                    MSG_NO_MEM);
+    return(IDA_MEM_NULL);
+  }
+  IDA_mem = (IDAMem) ida_mem;
+
+  /* set allowed value or use default */
+  if (eta_cf <= ZERO || eta_cf >= ONE)
+    IDA_mem->ida_eta_cf = ETA_CF_DEFAULT;
+  else
+    IDA_mem->ida_eta_cf = eta_cf;
+
+  return(IDA_SUCCESS);
+}
+
+/*-----------------------------------------------------------------*/
+
 int IDASetMaxOrd(void *ida_mem, int maxord)
 {
   IDAMem IDA_mem;
@@ -173,7 +332,7 @@ int IDASetMaxStep(void *ida_mem, realtype hmax)
 
   IDA_mem = (IDAMem) ida_mem;
 
-  if (hmax < 0) {
+  if (hmax < ZERO) {
     IDAProcessError(IDA_mem, IDA_ILL_INPUT, "IDA", "IDASetMaxStep", MSG_NEG_HMAX);
     return(IDA_ILL_INPUT);
   }
@@ -191,6 +350,36 @@ int IDASetMaxStep(void *ida_mem, realtype hmax)
 
 /*-----------------------------------------------------------------*/
 
+int IDASetMinStep(void *ida_mem, realtype hmin)
+{
+  IDAMem IDA_mem;
+
+  if (ida_mem == NULL) {
+    IDAProcessError(NULL, IDA_MEM_NULL, "IDA", "IDASetMinStep", MSG_NO_MEM);
+    return(IDA_MEM_NULL);
+  }
+
+  IDA_mem = (IDAMem) ida_mem;
+
+  if (hmin < ZERO) {
+    IDAProcessError(IDA_mem, IDA_ILL_INPUT, "IDA", "IDASetMinStep",
+                    MSG_NEG_HMIN);
+    return(IDA_ILL_INPUT);
+  }
+
+  /* Passing 0 sets hmin = zero */
+  if (hmin == ZERO) {
+    IDA_mem->ida_hmin = HMIN_DEFAULT;
+    return(IDA_SUCCESS);
+  }
+
+  IDA_mem->ida_hmin = hmin;
+
+  return(IDA_SUCCESS);
+}
+
+/*-----------------------------------------------------------------*/
+
 int IDASetStopTime(void *ida_mem, realtype tstop)
 {
   IDAMem IDA_mem;
@@ -1049,7 +1238,8 @@ int IDAGetRootInfo(void *ida_mem, int *rootsfound)
 
   nrt = IDA_mem->ida_nrtfn;
 
-  for (i=0; i<nrt; i++) rootsfound[i] = IDA_mem->ida_iroots[i];
+  for (i=0; i<nrt; i++)
+    rootsfound[i] = IDA_mem->ida_iroots[i];
 
   return(IDA_SUCCESS);
 }
@@ -1075,7 +1265,7 @@ int IDAGetNumNonlinSolvIters(void *ida_mem, long int *nniters)
 
 /*-----------------------------------------------------------------*/
 
-int IDAGetNumNonlinSolvConvFails(void *ida_mem, long int *nncfails)
+int IDAGetNumNonlinSolvConvFails(void *ida_mem, long int *nnfails)
 {
   IDAMem IDA_mem;
 
@@ -1086,6 +1276,45 @@ int IDAGetNumNonlinSolvConvFails(void *ida_mem, long int *nncfails)
 
   IDA_mem = (IDAMem) ida_mem;
 
+  *nnfails = IDA_mem->ida_nnf;
+
+  return(IDA_SUCCESS);
+}
+
+/*-----------------------------------------------------------------*/
+
+int IDAGetNonlinSolvStats(void *ida_mem, long int *nniters, long int *nnfails)
+{
+  IDAMem IDA_mem;
+
+  if (ida_mem==NULL) {
+    IDAProcessError(NULL, IDA_MEM_NULL, "IDA",
+                    "IDAGetNonlinSolvStats", MSG_NO_MEM);
+    return(IDA_MEM_NULL);
+  }
+
+  IDA_mem = (IDAMem) ida_mem;
+
+  *nniters = IDA_mem->ida_nni;
+  *nnfails = IDA_mem->ida_nnf;
+
+  return(IDA_SUCCESS);
+}
+
+/*-----------------------------------------------------------------*/
+
+int IDAGetNumStepSolveFails(void *ida_mem, long int *nncfails)
+{
+  IDAMem IDA_mem;
+
+  if (ida_mem==NULL) {
+    IDAProcessError(NULL, IDA_MEM_NULL, "IDA", "IDAGetNumStepSolveFails",
+                    MSG_NO_MEM);
+    return(IDA_MEM_NULL);
+  }
+
+  IDA_mem = (IDAMem) ida_mem;
+
   *nncfails = IDA_mem->ida_ncfn;
 
   return(IDA_SUCCESS);
@@ -1093,20 +1322,165 @@ int IDAGetNumNonlinSolvConvFails(void *ida_mem, long int *nncfails)
 
 /*-----------------------------------------------------------------*/
 
-int IDAGetNonlinSolvStats(void *ida_mem, long int *nniters, long int *nncfails)
+int IDAGetUserData(void *ida_mem, void** user_data)
 {
   IDAMem IDA_mem;
 
   if (ida_mem==NULL) {
-    IDAProcessError(NULL, IDA_MEM_NULL, "IDA",
-                    "IDAGetNonlinSolvStats", MSG_NO_MEM);
+    IDAProcessError(NULL, IDA_MEM_NULL, "IDA", "IDAGetUserData", MSG_NO_MEM);
     return(IDA_MEM_NULL);
   }
 
   IDA_mem = (IDAMem) ida_mem;
 
-  *nniters  = IDA_mem->ida_nni;
-  *nncfails = IDA_mem->ida_ncfn;
+  *user_data = IDA_mem->ida_user_data;
+
+  return(IDA_SUCCESS);
+}
+
+/*-----------------------------------------------------------------*/
+
+int IDAPrintAllStats(void *ida_mem, FILE *outfile, SUNOutputFormat fmt)
+{
+  IDAMem IDA_mem;
+  IDALsMem idals_mem;
+
+  if (ida_mem == NULL) {
+    IDAProcessError(NULL, IDA_MEM_NULL, "IDA", "IDAPrintAllStats",
+                    MSG_NO_MEM);
+    return(IDA_MEM_NULL);
+  }
+
+  IDA_mem = (IDAMem) ida_mem;
+
+  switch(fmt)
+  {
+  case SUN_OUTPUTFORMAT_TABLE:
+    /* step and method stats */
+    fprintf(outfile, "Current time                 = %"RSYM"\n", IDA_mem->ida_tn);
+    fprintf(outfile, "Steps                        = %ld\n", IDA_mem->ida_nst);
+    fprintf(outfile, "Error test fails             = %ld\n", IDA_mem->ida_netf);
+    fprintf(outfile, "NLS step fails               = %ld\n", IDA_mem->ida_ncfn);
+    fprintf(outfile, "Initial step size            = %"RSYM"\n", IDA_mem->ida_h0u);
+    fprintf(outfile, "Last step size               = %"RSYM"\n", IDA_mem->ida_hused);
+    fprintf(outfile, "Current step size            = %"RSYM"\n", IDA_mem->ida_hh);
+    fprintf(outfile, "Last method order            = %d\n", IDA_mem->ida_kused);
+    fprintf(outfile, "Current method order         = %d\n", IDA_mem->ida_kk);
+
+    /* function evaluations */
+    fprintf(outfile, "Residual fn evals            = %ld\n", IDA_mem->ida_nre);
+
+    /* IC calculation stats */
+    fprintf(outfile, "IC linesearch backtrack ops  = %d\n", IDA_mem->ida_nbacktr);
+
+    /* nonlinear solver stats */
+    fprintf(outfile, "NLS iters                    = %ld\n", IDA_mem->ida_nni);
+    fprintf(outfile, "NLS fails                    = %ld\n", IDA_mem->ida_nnf);
+    if (IDA_mem->ida_nst > 0)
+    {
+      fprintf(outfile, "NLS iters per step           = %"RSYM"\n",
+              (realtype) IDA_mem->ida_nre / (realtype) IDA_mem->ida_nst);
+    }
+
+    /* linear solver stats */
+    fprintf(outfile, "LS setups                    = %ld\n", IDA_mem->ida_nsetups);
+    if (IDA_mem->ida_lmem)
+    {
+      idals_mem = (IDALsMem) (IDA_mem->ida_lmem);
+      fprintf(outfile, "Jac fn evals                 = %ld\n", idals_mem->nje);
+      fprintf(outfile, "LS residual fn evals         = %ld\n", idals_mem->nreDQ);
+      fprintf(outfile, "Prec setup evals             = %ld\n", idals_mem->npe);
+      fprintf(outfile, "Prec solves                  = %ld\n", idals_mem->nps);
+      fprintf(outfile, "LS iters                     = %ld\n", idals_mem->nli);
+      fprintf(outfile, "LS fails                     = %ld\n", idals_mem->ncfl);
+      fprintf(outfile, "Jac-times setups             = %ld\n", idals_mem->njtsetup);
+      fprintf(outfile, "Jac-times evals              = %ld\n", idals_mem->njtimes);
+      if (IDA_mem->ida_nni > 0)
+      {
+        fprintf(outfile, "LS iters per NLS iter        = %"RSYM"\n",
+                (realtype) idals_mem->nli / (realtype) IDA_mem->ida_nni);
+        fprintf(outfile, "Jac evals per NLS iter       = %"RSYM"\n",
+                (realtype) idals_mem->nje / (realtype) IDA_mem->ida_nni);
+        fprintf(outfile, "Prec evals per NLS iter      = %"RSYM"\n",
+                (realtype) idals_mem->npe / (realtype) IDA_mem->ida_nni);
+      }
+    }
+
+    /* rootfinding stats */
+    fprintf(outfile, "Root fn evals                = %ld\n", IDA_mem->ida_nge);
+    break;
+
+  case SUN_OUTPUTFORMAT_CSV:
+    /* step and method stats */
+    fprintf(outfile, "Time,%"RSYM, IDA_mem->ida_tn);
+    fprintf(outfile, ",Steps,%ld", IDA_mem->ida_nst);
+    fprintf(outfile, ",Error test fails,%ld", IDA_mem->ida_netf);
+    fprintf(outfile, ",NLS step fails,%ld", IDA_mem->ida_ncfn);
+    fprintf(outfile, ",Initial step size,%"RSYM, IDA_mem->ida_h0u);
+    fprintf(outfile, ",Last step size,%"RSYM, IDA_mem->ida_hused);
+    fprintf(outfile, ",Current step size,%"RSYM, IDA_mem->ida_hh);
+    fprintf(outfile, ",Last method order,%d", IDA_mem->ida_kused);
+    fprintf(outfile, ",Current method order,%d", IDA_mem->ida_kk);
+
+    /* function evaluations */
+    fprintf(outfile, ",Residual fn evals,%ld", IDA_mem->ida_nre);
+
+    /* IC calculation stats */
+    fprintf(outfile, ",IC linesearch backtrack ops,%d", IDA_mem->ida_nbacktr);
+
+    /* nonlinear solver stats */
+    fprintf(outfile, ",NLS iters,%ld", IDA_mem->ida_nni);
+    fprintf(outfile, ",NLS fails,%ld", IDA_mem->ida_nnf);
+    if (IDA_mem->ida_nst > 0)
+    {
+      fprintf(outfile, ",NLS iters per step,%"RSYM,
+              (realtype) IDA_mem->ida_nre / (realtype) IDA_mem->ida_nst);
+    }
+    else
+    {
+      fprintf(outfile, ",NLS iters per step,0");
+    }
+
+    /* linear solver stats */
+    fprintf(outfile, ",LS setups,%ld", IDA_mem->ida_nsetups);
+    if (IDA_mem->ida_lmem)
+    {
+      idals_mem = (IDALsMem) (IDA_mem->ida_lmem);
+      fprintf(outfile, ",Jac fn evals,%ld", idals_mem->nje);
+      fprintf(outfile, ",LS residual evals,%ld", idals_mem->nreDQ);
+      fprintf(outfile, ",Prec setup evals,%ld", idals_mem->npe);
+      fprintf(outfile, ",Prec solves,%ld", idals_mem->nps);
+      fprintf(outfile, ",LS iters,%ld", idals_mem->nli);
+      fprintf(outfile, ",LS fails,%ld", idals_mem->ncfl);
+      fprintf(outfile, ",Jac-times setups,%ld", idals_mem->njtsetup);
+      fprintf(outfile, ",Jac-times evals,%ld", idals_mem->njtimes);
+      if (IDA_mem->ida_nni > 0)
+      {
+        fprintf(outfile, ",LS iters per NLS iter,%"RSYM,
+                (realtype) idals_mem->nli / (realtype) IDA_mem->ida_nni);
+        fprintf(outfile, ",Jac evals per NLS iter,%"RSYM,
+                (realtype) idals_mem->nje / (realtype) IDA_mem->ida_nni);
+        fprintf(outfile, ",Prec evals per NLS iter,%"RSYM,
+                (realtype) idals_mem->npe / (realtype) IDA_mem->ida_nni);
+      }
+      else
+      {
+        fprintf(outfile, ",LS iters per NLS iter,0");
+        fprintf(outfile, ",Jac evals per NLS iter,0");
+        fprintf(outfile, ",Prec evals per NLS iter,0");
+      }
+    }
+
+    /* rootfinding stats */
+    fprintf(outfile, ",Root fn evals,%ld", IDA_mem->ida_nge);
+    fprintf(outfile, "\n");
+    break;
+
+  default:
+    IDAProcessError(IDA_mem, IDA_ILL_INPUT, "IDA", "IDAPrintAllStats",
+                    "Invalid formatting option.");
+    return(IDA_ILL_INPUT);
+  }
 
   return(IDA_SUCCESS);
 }
