@@ -3,7 +3,7 @@
  *                Alan C. Hindmarsh and Radu Serban @ LLNL
  *-----------------------------------------------------------------
  * SUNDIALS Copyright Start
- * Copyright (c) 2002-2021, Lawrence Livermore National Security
+ * Copyright (c) 2002-2022, Lawrence Livermore National Security
  * and Southern Methodist University.
  * All rights reserved.
  *
@@ -412,8 +412,8 @@ int IDASetPreconditioner(void *ida_mem,
 {
   IDAMem   IDA_mem;
   IDALsMem idals_mem;
-  PSetupFn idals_psetup;
-  PSolveFn idals_psolve;
+  SUNPSetupFn idals_psetup;
+  SUNPSolveFn idals_psolve;
   int      retval;
 
   /* access IDALsMem structure */
@@ -566,7 +566,7 @@ int IDAGetNumJacEvals(void *ida_mem, long int *njevals)
 {
   IDAMem   IDA_mem;
   IDALsMem idals_mem;
-  int          retval;
+  int      retval;
 
   /* access IDALsMem structure; store output and return */
   retval = idaLs_AccessLMem(ida_mem, "IDAGetNumJacEvals",
