@@ -2,7 +2,7 @@
  * Programmer(s): Radu Serban @ LLNL
  * -----------------------------------------------------------------
  * SUNDIALS Copyright Start
- * Copyright (c) 2002-2021, Lawrence Livermore National Security
+ * Copyright (c) 2002-2022, Lawrence Livermore National Security
  * and Southern Methodist University.
  * All rights reserved.
  *
@@ -21,11 +21,28 @@
 #include <stdarg.h>
 
 #include "idas/idas.h"
+#include "sundials_context_impl.h"
+#include "sundials_logger_impl.h"
 
 #ifdef __cplusplus  /* wrapper to enable C++ usage */
 extern "C" {
 #endif
 
+#if defined(SUNDIALS_EXTENDED_PRECISION)
+#define RSYM  ".32Lg"
+#define RSYMW "41.32Lg"
+#else
+#define RSYM  ".16g"
+#define RSYMW "23.16g"
+#endif
+
+/*=================================================================*/
+/* Shortcuts                                                       */
+/*=================================================================*/
+
+#define IDA_PROFILER IDA_mem->ida_sunctx->profiler
+#define IDA_LOGGER IDA_mem->ida_sunctx->logger
+
 /*
  * =================================================================
  *   M A I N    I N T E G R A T O R    M E M O R Y    B L O C K
@@ -36,10 +53,21 @@ extern "C" {
 /* Basic IDA constants */
 
 #define HMAX_INV_DEFAULT RCONST(0.0) /* hmax_inv default value          */
+#define HMIN_DEFAULT     RCONST(0.0) /* hmin default value              */
 #define MAXORD_DEFAULT   5           /* maxord default value            */
 #define MXORDP1          6           /* max. number of N_Vectors in phi */
 #define MXSTEP_DEFAULT   500         /* mxstep default value            */
 
+#define ETA_MAX_FX_DEFAULT RCONST(2.0)  /* threshold to increase step size   */
+#define ETA_MIN_FX_DEFAULT RCONST(1.0)  /* threshold to decrease step size   */
+#define ETA_MAX_DEFAULT    RCONST(2.0)  /* max step size increase factor     */
+#define ETA_MIN_DEFAULT    RCONST(0.5)  /* min step size decrease factor     */
+#define ETA_LOW_DEFAULT    RCONST(0.9)  /* upper bound on decrease factor    */
+#define ETA_MIN_EF_DEFAULT RCONST(0.25) /* err test fail min decrease factor */
+#define ETA_CF_DEFAULT     RCONST(0.25) /* NLS failure decrease factor       */
+
+#define DCJ_DEFAULT RCONST(0.25)  /* constant for updating Jacobian/preconditioner */
+
 /* Return values for lower level routines used by IDASolve and functions
    provided to the nonlinear solver */
 
@@ -71,6 +99,8 @@ extern "C" {
 
 typedef struct IDAMemRec {
 
+  SUNContext ida_sunctx;
+
   realtype ida_uround;    /* machine unit roundoff */
 
   /*--------------------------
@@ -280,7 +310,7 @@ typedef struct IDAMemRec {
   realtype ida_h0u;      /* actual initial stepsize                           */
   realtype ida_hh;       /* current step size h                               */
   realtype ida_hused;    /* step size used on last successful step            */
-  realtype ida_rr;       /* rr = hnext / hused                                */
+  realtype ida_eta;      /* eta = hnext / hused                               */
   realtype ida_tn;       /* current internal value of t                       */
   realtype ida_tretlast; /* value of tret previously returned by IDASolve     */
   realtype ida_cj;       /* current value of scalar (-alphas/hh) in Jacobian  */
@@ -306,6 +336,15 @@ typedef struct IDAMemRec {
   int ida_maxord_alloc;  /* value of maxord used when allocating memory       */
   long int ida_mxstep;   /* max number of internal steps for one user call    */
   realtype ida_hmax_inv; /* inverse of max. step size hmax (default = 0.0)    */
+  realtype ida_hmin;     /* min step size hmin (default = 0.0)                */
+
+  realtype ida_eta_max_fx; /* threshold to increase step size */
+  realtype ida_eta_min_fx; /* threshold to decrease step size */
+  realtype ida_eta_max;    /* max step size increase factor   */
+  realtype ida_eta_min;    /* min step size decrease factor   */
+  realtype ida_eta_low;    /* upper bound on decrease factor  */
+  realtype ida_eta_min_ef; /* eta >= eta_min_ef after an error test failure */
+  realtype ida_eta_cf;     /* eta on a nonlinear solver convergence failure */
 
   /*--------
     Counters
@@ -333,6 +372,9 @@ typedef struct IDAMemRec {
   long int ida_nni;      /* number of Newton iterations performed             */
   long int ida_nniS;
 
+  long int ida_nnf;      /* number of Newton convergence failures             */
+  long int ida_nnfS;
+
   long int ida_nsetups;  /* number of lsetup calls                            */
   long int ida_nsetupsS;
 
@@ -439,7 +481,9 @@ typedef struct IDAMemRec {
 
   /* Linear Solver specific memory */
 
-  void *ida_lmem;
+  void *ida_lmem;   /* linear solver interface structure */
+  realtype ida_dcj; /* parameter that determines cj ratio thresholds for calling
+                     * the linear solver setup function */
 
   /* Flag to request a call to the setup routine */
 
@@ -584,7 +628,7 @@ struct CkpntMemRec {
 
   realtype     ck_hh;
   realtype     ck_hused;
-  realtype     ck_rr;
+  realtype     ck_eta;
   realtype     ck_cj;
   realtype     ck_cjlast;
   realtype     ck_cjold;
@@ -983,6 +1027,7 @@ int IDASensResDQ(int Ns, realtype t,
 /* General errors */
 
 #define MSG_MEM_FAIL       "A memory request failed."
+#define MSG_NULL_SUNCTX    "sunctx = NULL illegal."
 #define MSG_NO_MEM         "ida_mem = NULL illegal."
 #define MSG_NO_MALLOC      "Attempt to call before IDAMalloc."
 #define MSG_BAD_NVECTOR    "A required vector operation is not implemented."
@@ -1072,13 +1117,12 @@ int IDASensResDQ(int Ns, realtype t,
 #define MSG_EWT_NOW_BAD    "At " MSG_TIME "some ewt component has become <= 0.0."
 #define MSG_TOO_MUCH_ACC   "At " MSG_TIME "too much accuracy requested."
 
-#define MSG_BAD_T          "Illegal value for t. " MSG_TIME_INT
+#define MSG_BAD_K          "Illegal value for k."
+#define MSG_NULL_DKY       "dky = NULL illegal."
+#define MSG_NULL_DKYP      "dkyp = NULL illegal."
+#define MSG_BAD_T          "Illegal value for t." MSG_TIME_INT
 #define MSG_BAD_TOUT       "Trouble interpolating at " MSG_TIME_TOUT ". tout too far back in direction of integration."
 
-#define MSG_BAD_K        "Illegal value for k."
-#define MSG_NULL_DKY     "dky = NULL illegal."
-#define MSG_NULL_DKYP    "dkyp = NULL illegal."
-
 #define MSG_ERR_FAILS        "At " MSG_TIME_H "the error test failed repeatedly or with |h| = hmin."
 #define MSG_CONV_FAILS       "At " MSG_TIME_H "the corrector convergence failed repeatedly or with |h| = hmin."
 #define MSG_SETUP_FAILED     "At " MSG_TIME "the linear solver setup failed unrecoverably."
@@ -1115,9 +1159,11 @@ int IDASensResDQ(int Ns, realtype t,
 #define MSG_QSRHSFUNC_FIRST "The quadrature right-hand side routine failed at the first call."
 
 /* IDASet* / IDAGet* error messages */
-#define MSG_NEG_MAXORD     "maxord<=0 illegal."
+
+#define MSG_NEG_MAXORD     "maxord <= 0 illegal."
 #define MSG_BAD_MAXORD     "Illegal attempt to increase maximum order."
 #define MSG_NEG_HMAX       "hmax < 0 illegal."
+#define MSG_NEG_HMIN       "hmin < 0 illegal."
 #define MSG_NEG_EPCON      "epcon <= 0.0 illegal."
 #define MSG_BAD_CONSTR     "Illegal values in constraints vector."
 #define MSG_BAD_EPICCON    "epiccon <= 0.0 illegal."
