@@ -2,7 +2,7 @@
  * Programmer(s): David J. Gardner @ LLNL
  * -----------------------------------------------------------------
  * SUNDIALS Copyright Start
- * Copyright (c) 2002-2021, Lawrence Livermore National Security
+ * Copyright (c) 2002-2022, Lawrence Livermore National Security
  * and Southern Methodist University.
  * All rights reserved.
  *
@@ -36,6 +36,7 @@
 extern "C" {
 
 using namespace sundials;
+using namespace sundials::sycl;
 
 
 /* --------------------------------------------------------------------------
@@ -118,10 +119,10 @@ static int GetKernelParameters(N_Vector v, booleantype reduction,
  * -------------------------------------------------------------------------- */
 
 
-N_Vector N_VNewEmpty_Sycl()
+N_Vector N_VNewEmpty_Sycl(SUNContext sunctx)
 {
   /* Create an empty vector object */
-  N_Vector v = N_VNewEmpty();
+  N_Vector v = N_VNewEmpty(sunctx);
   if (v == NULL)
   {
     SUNDIALS_DEBUG_PRINT("ERROR in N_VNewEmpty_Sycl: N_VNewEmpty returned NULL\n");
@@ -205,7 +206,6 @@ N_Vector N_VNewEmpty_Sycl()
 
   /* Initialize content */
   NVEC_SYCL_CONTENT(v)->length             = 0;
-  NVEC_SYCL_CONTENT(v)->own_exec           = SUNFALSE;
   NVEC_SYCL_CONTENT(v)->own_helper         = SUNFALSE;
   NVEC_SYCL_CONTENT(v)->host_data          = NULL;
   NVEC_SYCL_CONTENT(v)->device_data        = NULL;
@@ -228,7 +228,7 @@ N_Vector N_VNewEmpty_Sycl()
 }
 
 
-N_Vector N_VNew_Sycl(sunindextype length, sycl::queue *Q)
+N_Vector N_VNew_Sycl(sunindextype length, ::sycl::queue *Q, SUNContext sunctx)
 {
   /* Check inputs */
   if (Q == NULL)
@@ -244,7 +244,7 @@ N_Vector N_VNew_Sycl(sunindextype length, sycl::queue *Q)
   }
 
   /* Create vector with empty content */
-  N_Vector v = N_VNewEmpty_Sycl();
+  N_Vector v = N_VNewEmpty_Sycl(sunctx);
   if (v == NULL)
   {
     SUNDIALS_DEBUG_PRINT("ERROR in N_VNew_Sycl: N_VNewEmpty_Sycl returned NULL\n");
@@ -253,11 +253,10 @@ N_Vector N_VNew_Sycl(sunindextype length, sycl::queue *Q)
 
   /* Fill content */
   NVEC_SYCL_CONTENT(v)->length             = length;
-  NVEC_SYCL_CONTENT(v)->own_exec           = SUNTRUE;
   NVEC_SYCL_CONTENT(v)->own_helper         = SUNTRUE;
-  NVEC_SYCL_CONTENT(v)->stream_exec_policy = new SyclThreadDirectExecPolicy(SYCL_BLOCKDIM(Q));
-  NVEC_SYCL_CONTENT(v)->reduce_exec_policy = new SyclBlockReduceExecPolicy(SYCL_BLOCKDIM(Q));
-  NVEC_SYCL_CONTENT(v)->mem_helper         = SUNMemoryHelper_Sycl(Q);
+  NVEC_SYCL_CONTENT(v)->stream_exec_policy = new ThreadDirectExecPolicy(SYCL_BLOCKDIM(Q));
+  NVEC_SYCL_CONTENT(v)->reduce_exec_policy = new BlockReduceExecPolicy(SYCL_BLOCKDIM(Q));
+  NVEC_SYCL_CONTENT(v)->mem_helper         = SUNMemoryHelper_Sycl(sunctx);
   NVEC_SYCL_CONTENT(v)->queue              = Q;
   NVEC_SYCL_PRIVATE(v)->use_managed_mem    = SUNFALSE;
 
@@ -282,7 +281,8 @@ N_Vector N_VNew_Sycl(sunindextype length, sycl::queue *Q)
 N_Vector N_VNewWithMemHelp_Sycl(sunindextype length,
                                 booleantype use_managed_mem,
                                 SUNMemoryHelper helper,
-                                sycl::queue *Q)
+                                ::sycl::queue *Q,
+                                SUNContext sunctx)
 {
   /* Check inputs */
   if (Q == NULL)
@@ -310,7 +310,7 @@ N_Vector N_VNewWithMemHelp_Sycl(sunindextype length,
   }
 
   /* Create vector with empty content */
-  N_Vector v = N_VNewEmpty_Sycl();
+  N_Vector v = N_VNewEmpty_Sycl(sunctx);
   if (v == NULL)
   {
     SUNDIALS_DEBUG_PRINT("ERROR in N_VNewWithMemHelp_Sycl: N_VNewEmpty_Sycl returned NULL\n");
@@ -319,10 +319,9 @@ N_Vector N_VNewWithMemHelp_Sycl(sunindextype length,
 
   /* Fill content */
   NVEC_SYCL_CONTENT(v)->length             = length;
-  NVEC_SYCL_CONTENT(v)->own_exec           = SUNTRUE;
   NVEC_SYCL_CONTENT(v)->own_helper         = SUNFALSE;
-  NVEC_SYCL_CONTENT(v)->stream_exec_policy = new SyclThreadDirectExecPolicy(SYCL_BLOCKDIM(Q));
-  NVEC_SYCL_CONTENT(v)->reduce_exec_policy = new SyclBlockReduceExecPolicy(SYCL_BLOCKDIM(Q));
+  NVEC_SYCL_CONTENT(v)->stream_exec_policy = new ThreadDirectExecPolicy(SYCL_BLOCKDIM(Q));
+  NVEC_SYCL_CONTENT(v)->reduce_exec_policy = new BlockReduceExecPolicy(SYCL_BLOCKDIM(Q));
   NVEC_SYCL_CONTENT(v)->mem_helper         = helper;
   NVEC_SYCL_CONTENT(v)->queue              = Q;
   NVEC_SYCL_PRIVATE(v)->use_managed_mem    = use_managed_mem;
@@ -338,7 +337,8 @@ N_Vector N_VNewWithMemHelp_Sycl(sunindextype length,
 }
 
 
-N_Vector N_VNewManaged_Sycl(sunindextype length, sycl::queue *Q)
+N_Vector N_VNewManaged_Sycl(sunindextype length, ::sycl::queue *Q,
+                            SUNContext sunctx)
 {
   /* Check inputs */
   if (Q == NULL)
@@ -354,7 +354,7 @@ N_Vector N_VNewManaged_Sycl(sunindextype length, sycl::queue *Q)
   }
 
   /* Create vector with empty content */
-  N_Vector v = N_VNewEmpty_Sycl();
+  N_Vector v = N_VNewEmpty_Sycl(sunctx);
   if (v == NULL)
   {
     SUNDIALS_DEBUG_PRINT("ERROR in N_VNewManaged_Sycl: N_VNewEmpty_Sycl returned NULL\n");
@@ -363,11 +363,10 @@ N_Vector N_VNewManaged_Sycl(sunindextype length, sycl::queue *Q)
 
   /* Fill content */
   NVEC_SYCL_CONTENT(v)->length             = length;
-  NVEC_SYCL_CONTENT(v)->own_exec           = SUNTRUE;
   NVEC_SYCL_CONTENT(v)->own_helper         = SUNTRUE;
-  NVEC_SYCL_CONTENT(v)->stream_exec_policy = new SyclThreadDirectExecPolicy(SYCL_BLOCKDIM(Q));
-  NVEC_SYCL_CONTENT(v)->reduce_exec_policy = new SyclBlockReduceExecPolicy(SYCL_BLOCKDIM(Q));
-  NVEC_SYCL_CONTENT(v)->mem_helper         = SUNMemoryHelper_Sycl(Q);
+  NVEC_SYCL_CONTENT(v)->stream_exec_policy = new ThreadDirectExecPolicy(SYCL_BLOCKDIM(Q));
+  NVEC_SYCL_CONTENT(v)->reduce_exec_policy = new BlockReduceExecPolicy(SYCL_BLOCKDIM(Q));
+  NVEC_SYCL_CONTENT(v)->mem_helper         = SUNMemoryHelper_Sycl(sunctx);
   NVEC_SYCL_CONTENT(v)->queue              = Q;
   NVEC_SYCL_PRIVATE(v)->use_managed_mem    = SUNTRUE;
 
@@ -390,7 +389,7 @@ N_Vector N_VNewManaged_Sycl(sunindextype length, sycl::queue *Q)
 
 
 N_Vector N_VMake_Sycl(sunindextype length, realtype *h_vdata, realtype *d_vdata,
-                      sycl::queue *Q)
+                      ::sycl::queue *Q, SUNContext sunctx)
 {
   /* Check inputs */
   if (Q == NULL)
@@ -412,22 +411,21 @@ N_Vector N_VMake_Sycl(sunindextype length, realtype *h_vdata, realtype *d_vdata,
   }
 
   /* Create vector with empty content */
-  N_Vector v = N_VNewEmpty_Sycl();
+  N_Vector v = N_VNewEmpty_Sycl(sunctx);
   if (v == NULL)
   {
-    SUNDIALS_DEBUG_PRINT("ERROR in N_VMake_Sycl: N_VNewEmpty_Sycl returned NULL\n");
+    SUNDIALS_DEBUG_PRINT("ERROR in N_VMake_Sycl: N_VMake_Sycl returned NULL\n");
     return NULL;
   }
 
   /* Fill content */
   NVEC_SYCL_CONTENT(v)->length             = length;
-  NVEC_SYCL_CONTENT(v)->own_exec           = SUNTRUE;
   NVEC_SYCL_CONTENT(v)->own_helper         = SUNTRUE;
   NVEC_SYCL_CONTENT(v)->host_data          = SUNMemoryHelper_Wrap(h_vdata, SUNMEMTYPE_HOST);
   NVEC_SYCL_CONTENT(v)->device_data        = SUNMemoryHelper_Wrap(d_vdata, SUNMEMTYPE_DEVICE);
-  NVEC_SYCL_CONTENT(v)->stream_exec_policy = new SyclThreadDirectExecPolicy(SYCL_BLOCKDIM(Q));
-  NVEC_SYCL_CONTENT(v)->reduce_exec_policy = new SyclBlockReduceExecPolicy(SYCL_BLOCKDIM(Q));
-  NVEC_SYCL_CONTENT(v)->mem_helper         = SUNMemoryHelper_Sycl(Q);
+  NVEC_SYCL_CONTENT(v)->stream_exec_policy = new ThreadDirectExecPolicy(SYCL_BLOCKDIM(Q));
+  NVEC_SYCL_CONTENT(v)->reduce_exec_policy = new BlockReduceExecPolicy(SYCL_BLOCKDIM(Q));
+  NVEC_SYCL_CONTENT(v)->mem_helper         = SUNMemoryHelper_Sycl(sunctx);
   NVEC_SYCL_CONTENT(v)->queue              = Q;
   NVEC_SYCL_PRIVATE(v)->use_managed_mem    = SUNFALSE;
 
@@ -451,7 +449,7 @@ N_Vector N_VMake_Sycl(sunindextype length, realtype *h_vdata, realtype *d_vdata,
 
 
 N_Vector N_VMakeManaged_Sycl(sunindextype length, realtype *vdata,
-                             sycl::queue *Q)
+                             ::sycl::queue *Q, SUNContext sunctx)
 {
   /* Check inputs */
   if (Q == NULL)
@@ -473,7 +471,7 @@ N_Vector N_VMakeManaged_Sycl(sunindextype length, realtype *vdata,
   }
 
   /* Create vector with empty content */
-  N_Vector v = N_VNewEmpty_Sycl();
+  N_Vector v = N_VNewEmpty_Sycl(sunctx);
   if (v == NULL)
   {
     SUNDIALS_DEBUG_PRINT("ERROR in N_VMakeManaged_Sycl: N_VNewEmpty_Sycl returned NULL\n");
@@ -482,13 +480,12 @@ N_Vector N_VMakeManaged_Sycl(sunindextype length, realtype *vdata,
 
   /* Fill content */
   NVEC_SYCL_CONTENT(v)->length             = length;
-  NVEC_SYCL_CONTENT(v)->own_exec           = SUNTRUE;
   NVEC_SYCL_CONTENT(v)->own_helper         = SUNTRUE;
   NVEC_SYCL_CONTENT(v)->host_data          = SUNMemoryHelper_Wrap(vdata, SUNMEMTYPE_UVM);
   NVEC_SYCL_CONTENT(v)->device_data        = SUNMemoryHelper_Alias(NVEC_SYCL_CONTENT(v)->host_data);
-  NVEC_SYCL_CONTENT(v)->stream_exec_policy = new SyclThreadDirectExecPolicy(SYCL_BLOCKDIM(Q));
-  NVEC_SYCL_CONTENT(v)->reduce_exec_policy = new SyclBlockReduceExecPolicy(SYCL_BLOCKDIM(Q));
-  NVEC_SYCL_CONTENT(v)->mem_helper         = SUNMemoryHelper_Sycl(Q);
+  NVEC_SYCL_CONTENT(v)->stream_exec_policy = new ThreadDirectExecPolicy(SYCL_BLOCKDIM(Q));
+  NVEC_SYCL_CONTENT(v)->reduce_exec_policy = new BlockReduceExecPolicy(SYCL_BLOCKDIM(Q));
+  NVEC_SYCL_CONTENT(v)->mem_helper         = SUNMemoryHelper_Sycl(sunctx);
   NVEC_SYCL_CONTENT(v)->queue              = Q;
   NVEC_SYCL_PRIVATE(v)->use_managed_mem    = SUNTRUE;
 
@@ -602,21 +599,26 @@ int N_VSetKernelExecPolicy_Sycl(N_Vector x,
                                 SUNSyclExecPolicy* stream_exec_policy,
                                 SUNSyclExecPolicy* reduce_exec_policy)
 {
-  if (x == NULL || stream_exec_policy == NULL || reduce_exec_policy == NULL)
+  if (!x)
   {
-    SUNDIALS_DEBUG_PRINT("ERROR in N_VSetKernelExecPolicy_Sycl: An input is NULL\n");
+    SUNDIALS_DEBUG_PRINT("ERROR in N_VSetKernelExecPolicy_Sycl: The input vector is NULL\n");
     return -1;
   }
 
-  if (NVEC_SYCL_CONTENT(x)->own_exec)
-  {
-    delete NVEC_SYCL_CONTENT(x)->stream_exec_policy;
-    delete NVEC_SYCL_CONTENT(x)->reduce_exec_policy;
-  }
+  /* Delete the old policies */
+  delete NVEC_SYCL_CONTENT(x)->stream_exec_policy;
+  delete NVEC_SYCL_CONTENT(x)->reduce_exec_policy;
 
-  NVEC_SYCL_CONTENT(x)->stream_exec_policy = stream_exec_policy;
-  NVEC_SYCL_CONTENT(x)->reduce_exec_policy = reduce_exec_policy;
-  NVEC_SYCL_CONTENT(x)->own_exec = SUNFALSE;
+  /* Clone input policy or use default */
+  if (stream_exec_policy)
+    NVEC_SYCL_CONTENT(x)->stream_exec_policy = stream_exec_policy->clone();
+  else
+    NVEC_SYCL_CONTENT(x)->stream_exec_policy = new ThreadDirectExecPolicy(SYCL_BLOCKDIM(NVEC_SYCL_QUEUE(x)));
+
+  if (reduce_exec_policy)
+    NVEC_SYCL_CONTENT(x)->reduce_exec_policy = reduce_exec_policy->clone();
+  else
+    NVEC_SYCL_CONTENT(x)->reduce_exec_policy = new BlockReduceExecPolicy(SYCL_BLOCKDIM(NVEC_SYCL_QUEUE(x)));
 
   return 0;
 }
@@ -630,7 +632,8 @@ void N_VCopyToDevice_Sycl(N_Vector x)
   copy_fail = SUNMemoryHelper_Copy(NVEC_SYCL_MEMHELP(x),
                                    NVEC_SYCL_CONTENT(x)->device_data,
                                    NVEC_SYCL_CONTENT(x)->host_data,
-                                   NVEC_SYCL_MEMSIZE(x));
+                                   NVEC_SYCL_MEMSIZE(x),
+                                   NVEC_SYCL_QUEUE(x));
 
   if (copy_fail)
   {
@@ -650,7 +653,8 @@ void N_VCopyFromDevice_Sycl(N_Vector x)
   copy_fail = SUNMemoryHelper_Copy(NVEC_SYCL_MEMHELP(x),
                                    NVEC_SYCL_CONTENT(x)->host_data,
                                    NVEC_SYCL_CONTENT(x)->device_data,
-                                   NVEC_SYCL_MEMSIZE(x));
+                                   NVEC_SYCL_MEMSIZE(x),
+                                   NVEC_SYCL_QUEUE(x));
 
   if (copy_fail)
   {
@@ -704,7 +708,7 @@ N_Vector N_VCloneEmpty_Sycl(N_Vector w)
   }
 
   /* Create vector */
-  N_Vector v = N_VNewEmpty_Sycl();
+  N_Vector v = N_VNewEmpty_Sycl(w->sunctx);
   if (v == NULL)
   {
     SUNDIALS_DEBUG_PRINT("ERROR in N_VCloneEmpty_Sycl: N_VNewEmpty returned NULL\n");
@@ -746,7 +750,6 @@ N_Vector N_VClone_Sycl(N_Vector w)
   }
 
   /* Allocate content */
-  NVEC_SYCL_CONTENT(v)->own_exec           = SUNTRUE;
   NVEC_SYCL_CONTENT(v)->own_helper         = SUNTRUE;
   NVEC_SYCL_CONTENT(v)->stream_exec_policy = NVEC_SYCL_CONTENT(w)->stream_exec_policy->clone();
   NVEC_SYCL_CONTENT(v)->reduce_exec_policy = NVEC_SYCL_CONTENT(w)->reduce_exec_policy->clone();
@@ -807,9 +810,11 @@ void N_VDestroy_Sycl(N_Vector v)
   /* free items in content */
   if (NVEC_SYCL_MEMHELP(v))
   {
-    SUNMemoryHelper_Dealloc(NVEC_SYCL_MEMHELP(v), vc->host_data);
+    SUNMemoryHelper_Dealloc(NVEC_SYCL_MEMHELP(v), vc->host_data,
+                            NVEC_SYCL_QUEUE(v));
     vc->host_data = NULL;
-    SUNMemoryHelper_Dealloc(NVEC_SYCL_MEMHELP(v), vc->device_data);
+    SUNMemoryHelper_Dealloc(NVEC_SYCL_MEMHELP(v), vc->device_data,
+                            NVEC_SYCL_QUEUE(v));
     vc->device_data = NULL;
     if (vc->own_helper) SUNMemoryHelper_Destroy(vc->mem_helper);
     vc->mem_helper = NULL;
@@ -819,6 +824,12 @@ void N_VDestroy_Sycl(N_Vector v)
     SUNDIALS_DEBUG_PRINT("WARNING in N_VDestroy_Sycl: mem_helper was NULL when trying to dealloc data, this could result in a memory leak\n");
   }
 
+  delete vc->stream_exec_policy;
+  delete vc->reduce_exec_policy;
+
+  vc->stream_exec_policy = nullptr;
+  vc->reduce_exec_policy = nullptr;
+
   /* free content struct and vector */
   free(vc);
   free(v);
@@ -838,7 +849,7 @@ void N_VConst_Sycl(realtype c, N_Vector z)
 {
   const sunindextype N      = NVEC_SYCL_LENGTH(z);
   realtype           *zdata = NVEC_SYCL_DDATAp(z);
-  sycl::queue        *Q     = NVEC_SYCL_QUEUE(z);
+  ::sycl::queue      *Q     = NVEC_SYCL_QUEUE(z);
   size_t             nthreads_total, nthreads_per_block;
 
   if (GetKernelParameters(z, SUNFALSE, nthreads_total, nthreads_per_block))
@@ -860,7 +871,7 @@ void N_VLinearSum_Sycl(realtype a, N_Vector x, realtype b, N_Vector y, N_Vector
   const realtype     *xdata = NVEC_SYCL_DDATAp(x);
   const realtype     *ydata = NVEC_SYCL_DDATAp(y);
   realtype           *zdata = NVEC_SYCL_DDATAp(z);
-  sycl::queue        *Q     = NVEC_SYCL_QUEUE(z);
+  ::sycl::queue      *Q     = NVEC_SYCL_QUEUE(z);
   size_t             nthreads_total, nthreads_per_block;
 
   if (GetKernelParameters(z, SUNFALSE, nthreads_total, nthreads_per_block))
@@ -882,7 +893,7 @@ void N_VProd_Sycl(N_Vector x, N_Vector y, N_Vector z)
   const realtype     *xdata = NVEC_SYCL_DDATAp(x);
   const realtype     *ydata = NVEC_SYCL_DDATAp(y);
   realtype           *zdata = NVEC_SYCL_DDATAp(z);
-  sycl::queue        *Q     = NVEC_SYCL_QUEUE(z);
+  ::sycl::queue      *Q     = NVEC_SYCL_QUEUE(z);
   size_t             nthreads_total, nthreads_per_block;
 
   if (GetKernelParameters(z, SUNFALSE, nthreads_total, nthreads_per_block))
@@ -904,7 +915,7 @@ void N_VDiv_Sycl(N_Vector x, N_Vector y, N_Vector z)
   const realtype     *xdata = NVEC_SYCL_DDATAp(x);
   const realtype     *ydata = NVEC_SYCL_DDATAp(y);
   realtype           *zdata = NVEC_SYCL_DDATAp(z);
-  sycl::queue        *Q     = NVEC_SYCL_QUEUE(z);
+  ::sycl::queue      *Q     = NVEC_SYCL_QUEUE(z);
   size_t             nthreads_total, nthreads_per_block;
 
   if (GetKernelParameters(z, SUNFALSE, nthreads_total, nthreads_per_block))
@@ -925,7 +936,7 @@ void N_VScale_Sycl(realtype c, N_Vector x, N_Vector z)
   const sunindextype N      = NVEC_SYCL_LENGTH(z);
   const realtype     *xdata = NVEC_SYCL_DDATAp(x);
   realtype           *zdata = NVEC_SYCL_DDATAp(z);
-  sycl::queue        *Q     = NVEC_SYCL_QUEUE(z);
+  ::sycl::queue      *Q     = NVEC_SYCL_QUEUE(z);
   size_t             nthreads_total, nthreads_per_block;
 
   if (GetKernelParameters(z, SUNFALSE, nthreads_total, nthreads_per_block))
@@ -946,7 +957,7 @@ void N_VAbs_Sycl(N_Vector x, N_Vector z)
   const sunindextype N      = NVEC_SYCL_LENGTH(z);
   const realtype     *xdata = NVEC_SYCL_DDATAp(x);
   realtype           *zdata = NVEC_SYCL_DDATAp(z);
-  sycl::queue        *Q     = NVEC_SYCL_QUEUE(z);
+  ::sycl::queue      *Q     = NVEC_SYCL_QUEUE(z);
   size_t             nthreads_total, nthreads_per_block;
 
   if (GetKernelParameters(z, SUNFALSE, nthreads_total, nthreads_per_block))
@@ -967,7 +978,7 @@ void N_VInv_Sycl(N_Vector x, N_Vector z)
   const sunindextype N      = NVEC_SYCL_LENGTH(z);
   const realtype     *xdata = NVEC_SYCL_DDATAp(x);
   realtype           *zdata = NVEC_SYCL_DDATAp(z);
-  sycl::queue        *Q     = NVEC_SYCL_QUEUE(z);
+  ::sycl::queue      *Q     = NVEC_SYCL_QUEUE(z);
   size_t             nthreads_total, nthreads_per_block;
 
   if (GetKernelParameters(z, SUNFALSE, nthreads_total, nthreads_per_block))
@@ -988,7 +999,7 @@ void N_VAddConst_Sycl(N_Vector x, realtype b, N_Vector z)
   const sunindextype N      = NVEC_SYCL_LENGTH(z);
   const realtype     *xdata = NVEC_SYCL_DDATAp(x);
   realtype           *zdata = NVEC_SYCL_DDATAp(z);
-  sycl::queue        *Q     = NVEC_SYCL_QUEUE(z);
+  ::sycl::queue      *Q     = NVEC_SYCL_QUEUE(z);
   size_t             nthreads_total, nthreads_per_block;
 
   if (GetKernelParameters(z, SUNFALSE, nthreads_total, nthreads_per_block))
@@ -1009,7 +1020,7 @@ realtype N_VDotProd_Sycl(N_Vector x, N_Vector y)
   const sunindextype N      = NVEC_SYCL_LENGTH(x);
   const realtype     *xdata = NVEC_SYCL_DDATAp(x);
   const realtype     *ydata = NVEC_SYCL_DDATAp(y);
-  sycl::queue        *Q     = NVEC_SYCL_QUEUE(x);
+  ::sycl::queue      *Q     = NVEC_SYCL_QUEUE(x);
   size_t             nthreads_total, nthreads_per_block;
 
   if (InitializeReductionBuffer(x, ZERO))
@@ -1026,7 +1037,7 @@ realtype N_VDotProd_Sycl(N_Vector x, N_Vector y)
   realtype *sum = NVEC_SYCL_DBUFFERp(x);
 
   SYCL_FOR_REDUCE(Q, nthreads_total, nthreads_per_block, item,
-                  sum, sycl::ONEAPI::plus<realtype>(),
+                  sum, ::sycl::plus<realtype>(),
                   GRID_STRIDE_XLOOP(item, i, N)
                   {
                     sum += xdata[i] * ydata[i];
@@ -1045,7 +1056,7 @@ realtype N_VMaxNorm_Sycl(N_Vector x)
 {
   const sunindextype N      = NVEC_SYCL_LENGTH(x);
   const realtype     *xdata = NVEC_SYCL_DDATAp(x);
-  sycl::queue        *Q     = NVEC_SYCL_QUEUE(x);
+  ::sycl::queue      *Q     = NVEC_SYCL_QUEUE(x);
   size_t             nthreads_total, nthreads_per_block;
 
   if (InitializeReductionBuffer(x, ZERO))
@@ -1062,7 +1073,7 @@ realtype N_VMaxNorm_Sycl(N_Vector x)
   realtype *max = NVEC_SYCL_DBUFFERp(x);
 
   SYCL_FOR_REDUCE(Q, nthreads_total, nthreads_per_block, item,
-                  max, sycl::ONEAPI::maximum<realtype>(),
+                  max, ::sycl::maximum<realtype>(),
                   GRID_STRIDE_XLOOP(item, i, N)
                   {
                     max.combine(abs(xdata[i]));
@@ -1082,7 +1093,7 @@ realtype N_VWSqrSumLocal_Sycl(N_Vector x, N_Vector w)
   const sunindextype N      = NVEC_SYCL_LENGTH(x);
   const realtype     *xdata = NVEC_SYCL_DDATAp(x);
   const realtype     *wdata = NVEC_SYCL_DDATAp(w);
-  sycl::queue        *Q     = NVEC_SYCL_QUEUE(x);
+  ::sycl::queue      *Q     = NVEC_SYCL_QUEUE(x);
   size_t             nthreads_total, nthreads_per_block;
 
   if (InitializeReductionBuffer(x, ZERO))
@@ -1099,7 +1110,7 @@ realtype N_VWSqrSumLocal_Sycl(N_Vector x, N_Vector w)
   realtype *sum = NVEC_SYCL_DBUFFERp(x);
 
   SYCL_FOR_REDUCE(Q, nthreads_total, nthreads_per_block, item,
-                  sum, sycl::ONEAPI::plus<realtype>(),
+                  sum, ::sycl::plus<realtype>(),
                   GRID_STRIDE_XLOOP(item, i, N)
                   {
                     sum += xdata[i] * wdata[i] * xdata[i] * wdata[i];
@@ -1128,7 +1139,7 @@ realtype N_VWSqrSumMaskLocal_Sycl(N_Vector x, N_Vector w, N_Vector id)
   const realtype     *xdata  = NVEC_SYCL_DDATAp(x);
   const realtype     *wdata  = NVEC_SYCL_DDATAp(w);
   const realtype     *iddata = NVEC_SYCL_DDATAp(id);
-  sycl::queue        *Q      = NVEC_SYCL_QUEUE(x);
+  ::sycl::queue      *Q      = NVEC_SYCL_QUEUE(x);
   size_t             nthreads_total, nthreads_per_block;
 
   if (InitializeReductionBuffer(x, ZERO))
@@ -1145,7 +1156,7 @@ realtype N_VWSqrSumMaskLocal_Sycl(N_Vector x, N_Vector w, N_Vector id)
   realtype *sum = NVEC_SYCL_DBUFFERp(x);
 
   SYCL_FOR_REDUCE(Q, nthreads_total, nthreads_per_block, item,
-                  sum, sycl::ONEAPI::plus<realtype>(),
+                  sum, ::sycl::plus<realtype>(),
                   GRID_STRIDE_XLOOP(item, i, N)
                   {
                     if (iddata[i] > ZERO)
@@ -1173,7 +1184,7 @@ realtype N_VMin_Sycl(N_Vector x)
 {
   const sunindextype N      = NVEC_SYCL_LENGTH(x);
   const realtype     *xdata = NVEC_SYCL_DDATAp(x);
-  sycl::queue        *Q     = NVEC_SYCL_QUEUE(x);
+  ::sycl::queue      *Q     = NVEC_SYCL_QUEUE(x);
   size_t             nthreads_total, nthreads_per_block;
 
   if (InitializeReductionBuffer(x, std::numeric_limits<realtype>::max()))
@@ -1190,7 +1201,7 @@ realtype N_VMin_Sycl(N_Vector x)
   realtype *min = NVEC_SYCL_DBUFFERp(x);
 
   SYCL_FOR_REDUCE(Q, nthreads_total, nthreads_per_block, item,
-                  min, sycl::ONEAPI::minimum<realtype>(),
+                  min, ::sycl::minimum<realtype>(),
                   GRID_STRIDE_XLOOP(item, i, N)
                   {
                     min.combine(xdata[i]);
@@ -1215,7 +1226,7 @@ realtype N_VL1Norm_Sycl(N_Vector x)
 {
   const sunindextype N      = NVEC_SYCL_LENGTH(x);
   const realtype     *xdata = NVEC_SYCL_DDATAp(x);
-  sycl::queue        *Q     = NVEC_SYCL_QUEUE(x);
+  ::sycl::queue      *Q     = NVEC_SYCL_QUEUE(x);
   size_t             nthreads_total, nthreads_per_block;
 
   if (InitializeReductionBuffer(x, ZERO))
@@ -1232,7 +1243,7 @@ realtype N_VL1Norm_Sycl(N_Vector x)
   realtype *sum = NVEC_SYCL_DBUFFERp(x);
 
   SYCL_FOR_REDUCE(Q, nthreads_total, nthreads_per_block, item,
-                  sum, sycl::ONEAPI::plus<realtype>(),
+                  sum, ::sycl::plus<realtype>(),
                   GRID_STRIDE_XLOOP(item, i, N)
                   {
                     sum += abs(xdata[i]);
@@ -1252,7 +1263,7 @@ void N_VCompare_Sycl(realtype c, N_Vector x, N_Vector z)
   const sunindextype N      = NVEC_SYCL_LENGTH(z);
   const realtype     *xdata = NVEC_SYCL_DDATAp(x);
   realtype           *zdata = NVEC_SYCL_DDATAp(z);
-  sycl::queue        *Q     = NVEC_SYCL_QUEUE(z);
+  ::sycl::queue      *Q     = NVEC_SYCL_QUEUE(z);
   size_t             nthreads_total, nthreads_per_block;
 
   if (GetKernelParameters(z, SUNFALSE, nthreads_total, nthreads_per_block))
@@ -1273,7 +1284,7 @@ booleantype N_VInvTest_Sycl(N_Vector x, N_Vector z)
   const sunindextype N      = NVEC_SYCL_LENGTH(z);
   const realtype     *xdata = NVEC_SYCL_DDATAp(x);
   realtype           *zdata = NVEC_SYCL_DDATAp(z);
-  sycl::queue        *Q     = NVEC_SYCL_QUEUE(z);
+  ::sycl::queue      *Q     = NVEC_SYCL_QUEUE(z);
   size_t             nthreads_total, nthreads_per_block;
 
   if (InitializeReductionBuffer(x, ZERO))
@@ -1290,7 +1301,7 @@ booleantype N_VInvTest_Sycl(N_Vector x, N_Vector z)
   realtype *sum = NVEC_SYCL_DBUFFERp(x);
 
   SYCL_FOR_REDUCE(Q, nthreads_total, nthreads_per_block, item,
-                  sum, sycl::ONEAPI::plus<realtype>(),
+                  sum, ::sycl::plus<realtype>(),
                   GRID_STRIDE_XLOOP(item, i, N)
                   {
                     if (xdata[i] == ZERO)
@@ -1318,7 +1329,7 @@ booleantype N_VConstrMask_Sycl(N_Vector c, N_Vector x, N_Vector m)
   const realtype     *cdata = NVEC_SYCL_DDATAp(c);
   const realtype     *xdata = NVEC_SYCL_DDATAp(x);
   realtype           *mdata = NVEC_SYCL_DDATAp(m);
-  sycl::queue        *Q     = NVEC_SYCL_QUEUE(x);
+  ::sycl::queue      *Q     = NVEC_SYCL_QUEUE(x);
   size_t             nthreads_total, nthreads_per_block;
 
   if (InitializeReductionBuffer(x, ZERO))
@@ -1335,7 +1346,7 @@ booleantype N_VConstrMask_Sycl(N_Vector c, N_Vector x, N_Vector m)
   realtype *sum = NVEC_SYCL_DBUFFERp(x);
 
   SYCL_FOR_REDUCE(Q, nthreads_total, nthreads_per_block, item,
-                  sum, sycl::ONEAPI::plus<realtype>(),
+                  sum, ::sycl::plus<realtype>(),
                   GRID_STRIDE_XLOOP(item, i, N)
                   {
                     bool test =
@@ -1359,7 +1370,7 @@ realtype N_VMinQuotient_Sycl(N_Vector num, N_Vector denom)
   const sunindextype N      = NVEC_SYCL_LENGTH(num);
   const realtype     *ndata = NVEC_SYCL_DDATAp(num);
   const realtype     *ddata = NVEC_SYCL_DDATAp(denom);
-  sycl::queue        *Q     = NVEC_SYCL_QUEUE(num);
+  ::sycl::queue      *Q     = NVEC_SYCL_QUEUE(num);
   size_t             nthreads_total, nthreads_per_block;
 
   if (InitializeReductionBuffer(num, std::numeric_limits<realtype>::max()))
@@ -1376,7 +1387,7 @@ realtype N_VMinQuotient_Sycl(N_Vector num, N_Vector denom)
   realtype *min = NVEC_SYCL_DBUFFERp(num);
 
   SYCL_FOR_REDUCE(Q, nthreads_total, nthreads_per_block, item,
-                  min, sycl::ONEAPI::minimum<realtype>(),
+                  min, ::sycl::minimum<realtype>(),
                   GRID_STRIDE_XLOOP(item, i, N)
                   {
                     if (ddata[i] != ZERO)
@@ -1401,7 +1412,7 @@ int N_VLinearCombination_Sycl(int nvec, realtype* c, N_Vector* X, N_Vector z)
 {
   const sunindextype N      = NVEC_SYCL_LENGTH(z);
   realtype           *zdata = NVEC_SYCL_DDATAp(z);
-  sycl::queue        *Q     = NVEC_SYCL_QUEUE(z);
+  ::sycl::queue      *Q     = NVEC_SYCL_QUEUE(z);
   size_t             nthreads_total, nthreads_per_block;
 
   /* Fused op workspace shortcuts */
@@ -1458,7 +1469,7 @@ int N_VScaleAddMulti_Sycl(int nvec, realtype* c, N_Vector x, N_Vector* Y,
 {
   const sunindextype N      = NVEC_SYCL_LENGTH(x);
   const realtype     *xdata = NVEC_SYCL_DDATAp(x);
-  sycl::queue        *Q     = NVEC_SYCL_QUEUE(x);
+  ::sycl::queue      *Q     = NVEC_SYCL_QUEUE(x);
   size_t             nthreads_total, nthreads_per_block;
 
   /* Shortcuts to the fused op workspace */
@@ -1527,7 +1538,7 @@ int N_VLinearSumVectorArray_Sycl(int nvec,
                                  N_Vector* Z)
 {
   const sunindextype N  = NVEC_SYCL_LENGTH(Z[0]);
-  sycl::queue        *Q = NVEC_SYCL_QUEUE(Z[0]);
+  ::sycl::queue      *Q = NVEC_SYCL_QUEUE(Z[0]);
   size_t             nthreads_total, nthreads_per_block;
 
   /* Shortcuts to the fused op workspace */
@@ -1588,7 +1599,7 @@ int N_VLinearSumVectorArray_Sycl(int nvec,
 int N_VScaleVectorArray_Sycl(int nvec, realtype* c, N_Vector* X, N_Vector* Z)
 {
   const sunindextype N  = NVEC_SYCL_LENGTH(Z[0]);
-  sycl::queue        *Q = NVEC_SYCL_QUEUE(Z[0]);
+  ::sycl::queue      *Q = NVEC_SYCL_QUEUE(Z[0]);
   size_t             nthreads_total, nthreads_per_block;
 
   /* Shortcuts to the fused op workspace arrays */
@@ -1649,7 +1660,7 @@ int N_VScaleVectorArray_Sycl(int nvec, realtype* c, N_Vector* X, N_Vector* Z)
 int N_VConstVectorArray_Sycl(int nvec, realtype c, N_Vector* Z)
 {
   const sunindextype N  = NVEC_SYCL_LENGTH(Z[0]);
-  sycl::queue        *Q = NVEC_SYCL_QUEUE(Z[0]);
+  ::sycl::queue      *Q = NVEC_SYCL_QUEUE(Z[0]);
   size_t             nthreads_total, nthreads_per_block;
 
   /* Shortcuts to the fused op workspace arrays */
@@ -1697,7 +1708,7 @@ int N_VScaleAddMultiVectorArray_Sycl(int nvec, int nsum, realtype* c,
                                      N_Vector* X, N_Vector** Y, N_Vector** Z)
 {
   const sunindextype N  = NVEC_SYCL_LENGTH(X[0]);
-  sycl::queue        *Q = NVEC_SYCL_QUEUE(X[0]);
+  ::sycl::queue      *Q = NVEC_SYCL_QUEUE(X[0]);
   size_t             nthreads_total, nthreads_per_block;
 
   /* Shortcuts to the fused op workspace */
@@ -1770,7 +1781,7 @@ int N_VLinearCombinationVectorArray_Sycl(int nvec, int nsum, realtype* c,
                                          N_Vector** X, N_Vector* Z)
 {
   const sunindextype N  = NVEC_SYCL_LENGTH(Z[0]);
-  sycl::queue        *Q = NVEC_SYCL_QUEUE(Z[0]);
+  ::sycl::queue      *Q = NVEC_SYCL_QUEUE(Z[0]);
   size_t             nthreads_total, nthreads_per_block;
 
   /* Shortcuts to the fused op workspace arrays */
@@ -1857,12 +1868,13 @@ int N_VBufPack_Sycl(N_Vector x, void *buf)
   copy_fail = SUNMemoryHelper_Copy(NVEC_SYCL_MEMHELP(x),
                                    buf_mem,
                                    NVEC_SYCL_CONTENT(x)->device_data,
-                                   NVEC_SYCL_MEMSIZE(x));
+                                   NVEC_SYCL_MEMSIZE(x),
+                                   NVEC_SYCL_QUEUE(x));
 
   /* synchronize with the host */
   NVEC_SYCL_QUEUE(x)->wait_and_throw();
 
-  SUNMemoryHelper_Dealloc(NVEC_SYCL_MEMHELP(x), buf_mem);
+  SUNMemoryHelper_Dealloc(NVEC_SYCL_MEMHELP(x), buf_mem, NVEC_SYCL_QUEUE(x));
 
   return (copy_fail ? -1 : 0);
 }
@@ -1880,12 +1892,13 @@ int N_VBufUnpack_Sycl(N_Vector x, void *buf)
   copy_fail = SUNMemoryHelper_Copy(NVEC_SYCL_MEMHELP(x),
                                    NVEC_SYCL_CONTENT(x)->device_data,
                                    buf_mem,
-                                   NVEC_SYCL_MEMSIZE(x));
+                                   NVEC_SYCL_MEMSIZE(x),
+                                   NVEC_SYCL_QUEUE(x));
 
   /* synchronize with the host */
   NVEC_SYCL_QUEUE(x)->wait_and_throw();
 
-  SUNMemoryHelper_Dealloc(NVEC_SYCL_MEMHELP(x), buf_mem);
+  SUNMemoryHelper_Dealloc(NVEC_SYCL_MEMHELP(x), buf_mem, NVEC_SYCL_QUEUE(x));
 
   return (copy_fail ? -1 : 0);
 }
@@ -2018,7 +2031,8 @@ static int AllocateData(N_Vector v)
   if (vcp->use_managed_mem)
   {
     alloc_fail = SUNMemoryHelper_Alloc(NVEC_SYCL_MEMHELP(v), &(vc->device_data),
-                                       NVEC_SYCL_MEMSIZE(v), SUNMEMTYPE_UVM);
+                                       NVEC_SYCL_MEMSIZE(v), SUNMEMTYPE_UVM,
+                                       NVEC_SYCL_QUEUE(v));
     if (alloc_fail)
     {
       SUNDIALS_DEBUG_PRINT("ERROR in AllocateData: SUNMemoryHelper_Alloc failed for SUNMEMTYPE_UVM\n");
@@ -2028,14 +2042,16 @@ static int AllocateData(N_Vector v)
   else
   {
     alloc_fail = SUNMemoryHelper_Alloc(NVEC_SYCL_MEMHELP(v), &(vc->host_data),
-                                       NVEC_SYCL_MEMSIZE(v), SUNMEMTYPE_HOST);
+                                       NVEC_SYCL_MEMSIZE(v), SUNMEMTYPE_HOST,
+                                       NVEC_SYCL_QUEUE(v));
     if (alloc_fail)
     {
       SUNDIALS_DEBUG_PRINT("ERROR in AllocateData: SUNMemoryHelper_Alloc failed to alloc SUNMEMTYPE_HOST\n");
     }
 
     alloc_fail = SUNMemoryHelper_Alloc(NVEC_SYCL_MEMHELP(v), &(vc->device_data),
-                                       NVEC_SYCL_MEMSIZE(v), SUNMEMTYPE_DEVICE);
+                                       NVEC_SYCL_MEMSIZE(v), SUNMEMTYPE_DEVICE,
+                                       NVEC_SYCL_QUEUE(v));
     if (alloc_fail)
     {
       SUNDIALS_DEBUG_PRINT("ERROR in AllocateData: SUNMemoryHelper_Alloc failed to alloc SUNMEMTYPE_DEVICE\n");
@@ -2072,7 +2088,7 @@ static int InitializeReductionBuffer(N_Vector v, const realtype value, size_t n)
     /* allocate pinned memory on the host */
     alloc_fail = SUNMemoryHelper_Alloc(NVEC_SYCL_MEMHELP(v),
                                        &(vcp->reduce_buffer_host), bytes,
-                                       SUNMEMTYPE_PINNED);
+                                       SUNMEMTYPE_PINNED, NVEC_SYCL_QUEUE(v));
     if (alloc_fail)
     {
       SUNDIALS_DEBUG_PRINT("WARNING in InitializeReductionBuffer: SUNMemoryHelper_Alloc failed to alloc SUNMEMTYPE_PINNED, using SUNMEMTYPE_HOST instead\n");
@@ -2080,7 +2096,7 @@ static int InitializeReductionBuffer(N_Vector v, const realtype value, size_t n)
       /* if pinned alloc failed, allocate plain host memory */
       alloc_fail = SUNMemoryHelper_Alloc(NVEC_SYCL_MEMHELP(v),
                                          &(vcp->reduce_buffer_host), bytes,
-                                         SUNMEMTYPE_HOST);
+                                         SUNMEMTYPE_HOST, NVEC_SYCL_QUEUE(v));
       if (alloc_fail)
       {
         SUNDIALS_DEBUG_PRINT("ERROR in InitializeReductionBuffer: SUNMemoryHelper_Alloc failed to alloc SUNMEMTYPE_HOST\n");
@@ -2090,7 +2106,7 @@ static int InitializeReductionBuffer(N_Vector v, const realtype value, size_t n)
     /* allocate device memory */
     alloc_fail = SUNMemoryHelper_Alloc(NVEC_SYCL_MEMHELP(v),
                                        &(vcp->reduce_buffer_dev), bytes,
-                                       SUNMEMTYPE_DEVICE);
+                                       SUNMEMTYPE_DEVICE, NVEC_SYCL_QUEUE(v));
     if (alloc_fail)
     {
       SUNDIALS_DEBUG_PRINT("ERROR in InitializeReductionBuffer: SUNMemoryHelper_Alloc failed to alloc SUNMEMTYPE_DEVICE\n");
@@ -2117,7 +2133,7 @@ static int InitializeReductionBuffer(N_Vector v, const realtype value, size_t n)
   }
 
   /* deallocate the wrapper */
-  SUNMemoryHelper_Dealloc(NVEC_SYCL_MEMHELP(v), value_mem);
+  SUNMemoryHelper_Dealloc(NVEC_SYCL_MEMHELP(v), value_mem, NVEC_SYCL_QUEUE(v));
 
   return ((alloc_fail || copy_fail) ? -1 : 0);
 }
@@ -2132,12 +2148,14 @@ static void FreeReductionBuffer(N_Vector v)
 
   /* free device mem */
   if (vcp->reduce_buffer_dev != NULL)
-    SUNMemoryHelper_Dealloc(NVEC_SYCL_MEMHELP(v), vcp->reduce_buffer_dev);
+    SUNMemoryHelper_Dealloc(NVEC_SYCL_MEMHELP(v), vcp->reduce_buffer_dev,
+                            NVEC_SYCL_QUEUE(v));
   vcp->reduce_buffer_dev = NULL;
 
   /* free host mem */
   if (vcp->reduce_buffer_host != NULL)
-    SUNMemoryHelper_Dealloc(NVEC_SYCL_MEMHELP(v), vcp->reduce_buffer_host);
+    SUNMemoryHelper_Dealloc(NVEC_SYCL_MEMHELP(v), vcp->reduce_buffer_host,
+                            NVEC_SYCL_QUEUE(v));
   vcp->reduce_buffer_host = NULL;
 
   /* reset allocated memory size */
@@ -2189,7 +2207,7 @@ static int FusedBuffer_Init(N_Vector v, int nreal, int nptr)
     /* allocate pinned memory on the host */
     alloc_fail = SUNMemoryHelper_Alloc(NVEC_SYCL_MEMHELP(v),
                                        &(vcp->fused_buffer_host), bytes,
-                                       SUNMEMTYPE_PINNED);
+                                       SUNMEMTYPE_PINNED, NVEC_SYCL_QUEUE(v));
     if (alloc_fail)
     {
       SUNDIALS_DEBUG_PRINT("WARNING in FusedBuffer_Init: SUNMemoryHelper_Alloc failed to alloc SUNMEMTYPE_PINNED, using SUNMEMTYPE_HOST instead\n");
@@ -2197,7 +2215,7 @@ static int FusedBuffer_Init(N_Vector v, int nreal, int nptr)
       /* if pinned alloc failed, allocate plain host memory */
       alloc_fail = SUNMemoryHelper_Alloc(NVEC_SYCL_MEMHELP(v),
                                          &(vcp->fused_buffer_host), bytes,
-                                         SUNMEMTYPE_HOST);
+                                         SUNMEMTYPE_HOST, NVEC_SYCL_QUEUE(v));
       if (alloc_fail)
       {
         SUNDIALS_DEBUG_PRINT("ERROR in FusedBuffer_Init: SUNMemoryHelper_Alloc failed to alloc SUNMEMTYPE_HOST\n");
@@ -2208,7 +2226,7 @@ static int FusedBuffer_Init(N_Vector v, int nreal, int nptr)
     /* allocate device memory */
     alloc_fail = SUNMemoryHelper_Alloc(NVEC_SYCL_MEMHELP(v),
                                        &(vcp->fused_buffer_dev), bytes,
-                                       SUNMEMTYPE_DEVICE);
+                                       SUNMEMTYPE_DEVICE, NVEC_SYCL_QUEUE(v));
     if (alloc_fail)
     {
       SUNDIALS_DEBUG_PRINT("ERROR in FusedBuffer_Init: SUNMemoryHelper_Alloc failed to alloc SUNMEMTYPE_DEVICE\n");
@@ -2353,14 +2371,14 @@ static int FusedBuffer_Free(N_Vector v)
   if (vcp->fused_buffer_host)
   {
     SUNMemoryHelper_Dealloc(NVEC_SYCL_MEMHELP(v),
-                            vcp->fused_buffer_host);
+                            vcp->fused_buffer_host, NVEC_SYCL_QUEUE(v));
     vcp->fused_buffer_host = NULL;
   }
 
   if (vcp->fused_buffer_dev)
   {
     SUNMemoryHelper_Dealloc(NVEC_SYCL_MEMHELP(v),
-                            vcp->fused_buffer_dev);
+                            vcp->fused_buffer_dev, NVEC_SYCL_QUEUE(v));
     vcp->fused_buffer_dev = NULL;
   }
 
