@@ -3,7 +3,7 @@
  *                Aaron Collier @ LLNL
  * -----------------------------------------------------------------
  * SUNDIALS Copyright Start
- * Copyright (c) 2002-2021, Lawrence Livermore National Security
+ * Copyright (c) 2002-2022, Lawrence Livermore National Security
  * and Southern Methodist University.
  * All rights reserved.
  *
@@ -20,20 +20,23 @@
 #include <stdlib.h>
 #include <math.h>
 
+#include <sundials/sundials_types.h>
 #include <sundials/sundials_math.h>
 
-static booleantype sunIsInf(realtype a)
+static long double sunNextafterl(long double from, long double to);
+
+static booleantype sunIsInf(sunrealtype a)
 {
-#if (__STDC_VERSION__ >= 199901L)
+#if defined(__cplusplus) || defined(SUNDIALS_C_COMPILER_HAS_ISINF_ISNAN)
   return(isinf(a));
 #else
   return(a < -BIG_REAL || a > BIG_REAL);
 #endif
 }
 
-static booleantype sunIsNaN(realtype a)
+static booleantype sunIsNaN(sunrealtype a)
 {
-#if ( __STDC_VERSION__ >= 199901L)
+#if defined(__cplusplus) || defined(SUNDIALS_C_COMPILER_HAS_ISINF_ISNAN)
   return(isnan(a));
 #else
   /* Most compilers/platforms follow NaN != a,
@@ -44,10 +47,10 @@ static booleantype sunIsNaN(realtype a)
 #endif
 }
 
-realtype SUNRpowerI(realtype base, int exponent)
+sunrealtype SUNRpowerI(sunrealtype base, int exponent)
 {
   int i, expt;
-  realtype prod;
+  sunrealtype prod;
 
   prod = RCONST(1.0);
   expt = abs(exponent);
@@ -56,30 +59,32 @@ realtype SUNRpowerI(realtype base, int exponent)
   return(prod);
 }
 
-realtype SUNRpowerR(realtype base, realtype exponent)
+sunrealtype SUNRpowerR(sunrealtype base, sunrealtype exponent)
 {
   if (base <= RCONST(0.0)) return(RCONST(0.0));
 
-#if defined(SUNDIALS_USE_GENERIC_MATH)
-  return((realtype) pow((double) base, (double) exponent));
-#elif defined(SUNDIALS_DOUBLE_PRECISION)
+#if defined(__cplusplus) || defined(SUNDIALS_C_COMPILER_HAS_MATH_PRECISIONS)
+#if defined(SUNDIALS_DOUBLE_PRECISION)
   return(pow(base, exponent));
 #elif defined(SUNDIALS_SINGLE_PRECISION)
   return(powf(base, exponent));
 #elif defined(SUNDIALS_EXTENDED_PRECISION)
   return(powl(base, exponent));
 #endif
+#else
+  return((sunrealtype) pow((double) base, (double) exponent));
+#endif
 }
 
-booleantype SUNRCompare(realtype a, realtype b)
+booleantype SUNRCompare(sunrealtype a, sunrealtype b)
 {
   return(SUNRCompareTol(a, b, 10*UNIT_ROUNDOFF));
 }
 
-booleantype SUNRCompareTol(realtype a, realtype b, realtype tol)
+booleantype SUNRCompareTol(sunrealtype a, sunrealtype b, sunrealtype tol)
 {
-  realtype diff;
-  realtype norm;
+  sunrealtype diff;
+  sunrealtype norm;
 
   /* If a and b are exactly equal.
    * This also covers the case where a and b are both inf under IEEE 754.
@@ -106,3 +111,67 @@ booleantype SUNRCompareTol(realtype a, realtype b, realtype tol)
    */
   return(diff >= SUNMAX(10*UNIT_ROUNDOFF, tol*norm));
 }
+
+long double sunNextafterl(long double from, long double to)
+{
+#if defined(__cplusplus) || defined(SUNDIALS_C_COMPILER_HAS_MATH_PRECISIONS)
+  return nextafterl(from, to);
+#else
+  union {
+    long double f;
+    int i;
+  } u;
+
+  u.i = 0;
+  u.f = from;
+
+  /* if either are NaN, then return NaN via the sum */
+  if (sunIsNaN((sunrealtype) from) || sunIsNaN((sunrealtype) to)) { return from + to; }
+
+  if (from == to) {
+    return to;
+  }
+
+  /* ordering is -0.0, +0.0 so nextafter(-0.0, 0.0) should give +0.0
+     and nextafter(0.0, -0.0) should give -0.0 */
+  if (from == 0) {
+    u.i = 1;
+    return  to > 0 ? u.f : -u.f;
+  }
+
+  if ((from > 0) == (to > from)) {
+    u.i++;
+  } else {
+    u.i--;
+  }
+
+  return u.f;
+#endif
+}
+
+sunrealtype SUNStrToReal(const char* str)
+{
+  char* end;
+#if defined(__cplusplus) || defined(SUNDIALS_C_COMPILER_HAS_MATH_PRECISIONS)
+#if defined(SUNDIALS_EXTENDED_PRECISION)
+  return strtold(str, &end);
+#elif defined(SUNDIALS_DOUBLE_PRECISION)
+  return strtod(str, &end);
+#elif defined(SUNDIALS_SINGLE_PRECISION)
+  return strtof(str, &end);
+#else
+#error "Should not be here, no SUNDIALS precision defined, report to github.com/LLNL/sundials/issues"
+#endif
+#else
+#if defined(SUNDIALS_EXTENDED_PRECISION)
+  /* Use strtod, but then round down to the closest double value
+     since strtod will effectively round up to the closest long double. */
+  double val = strtod(str, &end);
+  return (sunrealtype) sunNextafterl(val, -0.0);
+#elif defined(SUNDIALS_DOUBLE_PRECISION)
+  return strtod(str, &end);
+#elif defined(SUNDIALS_SINGLE_PRECISION)
+  return strtod(str, &end);
+#endif
+#endif
+}
