@@ -3,7 +3,7 @@
  * Based on sundials_sptfqmr.c code, written by Aaron Collier @ LLNL
  * -----------------------------------------------------------------
  * SUNDIALS Copyright Start
- * Copyright (c) 2002-2021, Lawrence Livermore National Security
+ * Copyright (c) 2002-2022, Lawrence Livermore National Security
  * and Southern Methodist University.
  * All rights reserved.
  *
@@ -22,7 +22,8 @@
 #include <sunlinsol/sunlinsol_sptfqmr.h>
 #include <sundials/sundials_math.h>
 
-#include "sundials_debug.h"
+#include "sundials_context_impl.h"
+#include "sundials_logger_impl.h"
 
 #define ZERO RCONST(0.0)
 #define ONE  RCONST(1.0)
@@ -36,21 +37,6 @@
 #define SPTFQMR_CONTENT(S)  ( (SUNLinearSolverContent_SPTFQMR)(S->content) )
 #define LASTFLAG(S)         ( SPTFQMR_CONTENT(S)->last_flag )
 
-/*
- * -----------------------------------------------------------------
- * deprecated wrapper functions
- * -----------------------------------------------------------------
- */
-
-SUNLinearSolver SUNSPTFQMR(N_Vector y, int pretype, int maxl)
-{ return(SUNLinSol_SPTFQMR(y, pretype, maxl)); }
-
-int SUNSPTFQMRSetPrecType(SUNLinearSolver S, int pretype)
-{ return(SUNLinSol_SPTFQMRSetPrecType(S, pretype)); }
-
-int SUNSPTFQMRSetMaxl(SUNLinearSolver S, int maxl)
-{ return(SUNLinSol_SPTFQMRSetMaxl(S, maxl)); }
-
 /*
  * -----------------------------------------------------------------
  * exported functions
@@ -61,15 +47,15 @@ int SUNSPTFQMRSetMaxl(SUNLinearSolver S, int maxl)
  * Function to create a new SPTFQMR linear solver
  */
 
-SUNLinearSolver SUNLinSol_SPTFQMR(N_Vector y, int pretype, int maxl)
+SUNLinearSolver SUNLinSol_SPTFQMR(N_Vector y, int pretype, int maxl, SUNContext sunctx)
 {
   SUNLinearSolver S;
   SUNLinearSolverContent_SPTFQMR content;
 
   /* check for legal pretype and maxl values; if illegal use defaults */
-  if ((pretype != PREC_NONE)  && (pretype != PREC_LEFT) &&
-      (pretype != PREC_RIGHT) && (pretype != PREC_BOTH))
-    pretype = PREC_NONE;
+  if ((pretype != SUN_PREC_NONE)  && (pretype != SUN_PREC_LEFT) &&
+      (pretype != SUN_PREC_RIGHT) && (pretype != SUN_PREC_BOTH))
+    pretype = SUN_PREC_NONE;
   if (maxl <= 0)
     maxl = SUNSPTFQMR_MAXL_DEFAULT;
 
@@ -82,7 +68,7 @@ SUNLinearSolver SUNLinSol_SPTFQMR(N_Vector y, int pretype, int maxl)
 
   /* Create linear solver */
   S = NULL;
-  S = SUNLinSolNewEmpty();
+  S = SUNLinSolNewEmpty(sunctx);
   if (S == NULL) return(NULL);
 
   /* Attach operations */
@@ -136,6 +122,9 @@ SUNLinearSolver SUNLinSol_SPTFQMR(N_Vector y, int pretype, int maxl)
   content->PData       = NULL;
   content->print_level = 0;
   content->info_file   = stdout;
+#if SUNDIALS_LOGGING_LEVEL >= SUNDIALS_LOGGING_INFO
+  content->info_file   = (sunctx->logger->info_fp) ? sunctx->logger->info_fp : stdout;
+#endif
 
   /* Allocate content */
   content->r_star = N_VClone(y);
@@ -179,8 +168,8 @@ SUNLinearSolver SUNLinSol_SPTFQMR(N_Vector y, int pretype, int maxl)
 int SUNLinSol_SPTFQMRSetPrecType(SUNLinearSolver S, int pretype)
 {
   /* Check for legal pretype */
-  if ((pretype != PREC_NONE)  && (pretype != PREC_LEFT) &&
-      (pretype != PREC_RIGHT) && (pretype != PREC_BOTH)) {
+  if ((pretype != SUN_PREC_NONE)  && (pretype != SUN_PREC_LEFT) &&
+      (pretype != SUN_PREC_RIGHT) && (pretype != SUN_PREC_BOTH)) {
     return(SUNLS_ILL_INPUT);
   }
 
@@ -247,12 +236,12 @@ int SUNLinSolInitialize_SPTFQMR(SUNLinearSolver S)
     return(LASTFLAG(S));
   }
 
-  if ( (content->pretype != PREC_LEFT) &&
-       (content->pretype != PREC_RIGHT) &&
-       (content->pretype != PREC_BOTH) )
-    content->pretype = PREC_NONE;
+  if ( (content->pretype != SUN_PREC_LEFT) &&
+       (content->pretype != SUN_PREC_RIGHT) &&
+       (content->pretype != SUN_PREC_BOTH) )
+    content->pretype = SUN_PREC_NONE;
 
-  if ((content->pretype != PREC_NONE) && (content->Psolve == NULL)) {
+  if ((content->pretype != SUN_PREC_NONE) && (content->Psolve == NULL)) {
     LASTFLAG(S) = SUNLS_PSOLVE_NULL;
     return(LASTFLAG(S));
   }
@@ -266,7 +255,7 @@ int SUNLinSolInitialize_SPTFQMR(SUNLinearSolver S)
 
 
 int SUNLinSolSetATimes_SPTFQMR(SUNLinearSolver S, void* ATData,
-                               ATimesFn ATimes)
+                               SUNATimesFn ATimes)
 {
   /* set function pointers to integrator-supplied ATimes routine
      and data, and return with success */
@@ -279,7 +268,7 @@ int SUNLinSolSetATimes_SPTFQMR(SUNLinearSolver S, void* ATData,
 
 
 int SUNLinSolSetPreconditioner_SPTFQMR(SUNLinearSolver S, void* PData,
-                                       PSetupFn Psetup, PSolveFn Psolve)
+                                       SUNPSetupFn Psetup, SUNPSolveFn Psolve)
 {
   /* set function pointers to integrator-supplied Psetup and PSolve
      routines and data, and return with success */
@@ -319,7 +308,7 @@ int SUNLinSolSetZeroGuess_SPTFQMR(SUNLinearSolver S, booleantype onoff)
 int SUNLinSolSetup_SPTFQMR(SUNLinearSolver S, SUNMatrix A)
 {
   int ier;
-  PSetupFn Psetup;
+  SUNPSetupFn Psetup;
   void* PData;
 
   /* Set shortcuts to SPTFQMR memory structures */
@@ -355,8 +344,8 @@ int SUNLinSolSolve_SPTFQMR(SUNLinearSolver S, SUNMatrix A, N_Vector x,
   booleantype *zeroguess;
   int n, m, ier, l_max;
   void *A_data, *P_data;
-  ATimesFn atimes;
-  PSolveFn psolve;
+  SUNATimesFn atimes;
+  SUNPSolveFn psolve;
   realtype *res_norm;
   int *nli;
   N_Vector sx, sb, r_star, q, d, v, p, *r, u, vtemp1, vtemp2, vtemp3;
@@ -395,10 +384,10 @@ int SUNLinSolSolve_SPTFQMR(SUNLinearSolver S, SUNMatrix A, N_Vector x,
   b_ok = SUNFALSE;
 
   /* set booleantype flags for internal solver options */
-  preOnLeft  = ( (SPTFQMR_CONTENT(S)->pretype == PREC_LEFT) ||
-                 (SPTFQMR_CONTENT(S)->pretype == PREC_BOTH) );
-  preOnRight = ( (SPTFQMR_CONTENT(S)->pretype == PREC_RIGHT) ||
-                 (SPTFQMR_CONTENT(S)->pretype == PREC_BOTH) );
+  preOnLeft  = ( (SPTFQMR_CONTENT(S)->pretype == SUN_PREC_LEFT) ||
+                 (SPTFQMR_CONTENT(S)->pretype == SUN_PREC_BOTH) );
+  preOnRight = ( (SPTFQMR_CONTENT(S)->pretype == SUN_PREC_RIGHT) ||
+                 (SPTFQMR_CONTENT(S)->pretype == SUN_PREC_BOTH) );
   scale_x = (sx != NULL);
   scale_b = (sb != NULL);
 
@@ -409,8 +398,9 @@ int SUNLinSolSolve_SPTFQMR(SUNLinearSolver S, SUNMatrix A, N_Vector x,
     return(SUNLS_ILL_INPUT);
   }
 
-#ifdef SUNDIALS_BUILD_WITH_MONITORING
-  if (SPTFQMR_CONTENT(S)->print_level && SPTFQMR_CONTENT(S)->info_file)
+#if SUNDIALS_LOGGING_LEVEL >= SUNDIALS_LOGGING_INFO
+  if (SPTFQMR_CONTENT(S)->print_level && SPTFQMR_CONTENT(S)->info_file
+      && (SPTFQMR_CONTENT(S)->info_file != S->sunctx->logger->info_fp))
     fprintf(SPTFQMR_CONTENT(S)->info_file, "SUNLINSOL_SPTFQMR:\n");
 #endif
 
@@ -445,7 +435,7 @@ int SUNLinSolSolve_SPTFQMR(SUNLinearSolver S, SUNMatrix A, N_Vector x,
 
   /* Apply left preconditioner and b-scaling to r_star (or really just r_0) */
   if (preOnLeft) {
-    ier = psolve(P_data, r_star, vtemp1, delta, PREC_LEFT);
+    ier = psolve(P_data, r_star, vtemp1, delta, SUN_PREC_LEFT);
     if (ier != 0) {
       *zeroguess  = SUNFALSE;
       LASTFLAG(S) = (ier < 0) ?
@@ -467,14 +457,18 @@ int SUNLinSolSolve_SPTFQMR(SUNLinearSolver S, SUNMatrix A, N_Vector x,
      to do anything */
   *res_norm = r_init_norm = SUNRsqrt(rho[0]);
 
-#ifdef SUNDIALS_BUILD_WITH_MONITORING
+#if SUNDIALS_LOGGING_LEVEL >= SUNDIALS_LOGGING_INFO
   /* print initial residual */
-  if (SPTFQMR_CONTENT(S)->print_level && SPTFQMR_CONTENT(S)->info_file)
+  if (SPTFQMR_CONTENT(S)->print_level && SPTFQMR_CONTENT(S)->info_file
+      && (SPTFQMR_CONTENT(S)->info_file != S->sunctx->logger->info_fp))
   {
     fprintf(SPTFQMR_CONTENT(S)->info_file,
             SUNLS_MSG_RESIDUAL,
             (long int) 0, *res_norm);
   }
+  SUNLogger_QueueMsg(S->sunctx->logger, SUN_LOGLEVEL_INFO,
+    "SUNLinSolSolve_SPFTQMR", "initial-residual",
+    "nli = %li, resnorm = %.16g", (long int) 0, *res_norm);
 #endif
 
   if (r_init_norm <= delta) {
@@ -488,7 +482,7 @@ int SUNLinSolSolve_SPTFQMR(SUNLinearSolver S, SUNMatrix A, N_Vector x,
   else N_VScale(ONE, r_star, vtemp1);
   if (preOnRight) {
     N_VScale(ONE, vtemp1, v);
-    ier = psolve(P_data, v, vtemp1, delta, PREC_RIGHT);
+    ier = psolve(P_data, v, vtemp1, delta, SUN_PREC_RIGHT);
     if (ier != 0) {
       *zeroguess  = SUNFALSE;
       LASTFLAG(S) = (ier < 0) ?
@@ -504,7 +498,7 @@ int SUNLinSolSolve_SPTFQMR(SUNLinearSolver S, SUNMatrix A, N_Vector x,
     return(LASTFLAG(S));
   }
   if (preOnLeft) {
-    ier = psolve(P_data, v, vtemp1, delta, PREC_LEFT);
+    ier = psolve(P_data, v, vtemp1, delta, SUN_PREC_LEFT);
     if (ier != 0) {
       *zeroguess  = SUNFALSE;
       LASTFLAG(S) = (ier < 0) ?
@@ -548,7 +542,7 @@ int SUNLinSolSolve_SPTFQMR(SUNLinearSolver S, SUNMatrix A, N_Vector x,
     if (scale_x) N_VDiv(r[1], sx, r[1]);
     if (preOnRight) {
       N_VScale(ONE, r[1], vtemp1);
-      ier = psolve(P_data, vtemp1, r[1], delta, PREC_RIGHT);
+      ier = psolve(P_data, vtemp1, r[1], delta, SUN_PREC_RIGHT);
       if (ier != 0) {
         *zeroguess  = SUNFALSE;
         LASTFLAG(S) = (ier < 0) ?
@@ -564,7 +558,7 @@ int SUNLinSolSolve_SPTFQMR(SUNLinearSolver S, SUNMatrix A, N_Vector x,
       return(LASTFLAG(S));
     }
     if (preOnLeft) {
-      ier = psolve(P_data, vtemp1, r[1], delta, PREC_LEFT);
+      ier = psolve(P_data, vtemp1, r[1], delta, SUN_PREC_LEFT);
       if (ier != 0) {
         *zeroguess  = SUNFALSE;
         LASTFLAG(S) = (ier < 0) ?
@@ -618,14 +612,18 @@ int SUNLinSolSolve_SPTFQMR(SUNLinearSolver S, SUNMatrix A, N_Vector x,
       /* NOTE: just use approximation to norm of residual, if possible */
       *res_norm = r_curr_norm = tau*SUNRsqrt(m+1);
 
-#ifdef SUNDIALS_BUILD_WITH_MONITORING
+#if SUNDIALS_LOGGING_LEVEL >= SUNDIALS_LOGGING_INFO
       /* print current iteration number and the residual */
-      if (SPTFQMR_CONTENT(S)->print_level && SPTFQMR_CONTENT(S)->info_file)
+      if (SPTFQMR_CONTENT(S)->print_level && SPTFQMR_CONTENT(S)->info_file
+          && (SPTFQMR_CONTENT(S)->info_file != S->sunctx->logger->info_fp))
       {
         fprintf(SPTFQMR_CONTENT(S)->info_file,
                 SUNLS_MSG_RESIDUAL,
                 (long int) *nli, *res_norm);
       }
+      SUNLogger_QueueMsg(S->sunctx->logger, SUN_LOGLEVEL_INFO,
+        "SUNLinSolSolve_SPTFQMR", "iterate-residual",
+        "nli = %li, resnorm = %.16g", (long int) 0, *res_norm);
 #endif
 
       /* Exit inner loop if iteration has converged based upon approximation
@@ -654,7 +652,7 @@ int SUNLinSolSolve_SPTFQMR(SUNLinearSolver S, SUNMatrix A, N_Vector x,
         if (scale_x) N_VDiv(x, sx, vtemp1);
         else N_VScale(ONE, x, vtemp1);
         if (preOnRight) {
-          ier = psolve(P_data, vtemp1, vtemp2, delta, PREC_RIGHT);
+          ier = psolve(P_data, vtemp1, vtemp2, delta, SUN_PREC_RIGHT);
           if (ier != 0) {
             *zeroguess  = SUNFALSE;
             LASTFLAG(S) = (ier < 0) ?
@@ -671,7 +669,7 @@ int SUNLinSolSolve_SPTFQMR(SUNLinearSolver S, SUNMatrix A, N_Vector x,
           return(LASTFLAG(S));
         }
         if (preOnLeft) {
-          ier = psolve(P_data, vtemp2, vtemp1, delta, PREC_LEFT);
+          ier = psolve(P_data, vtemp2, vtemp1, delta, SUN_PREC_LEFT);
           if (ier != 0) {
             *zeroguess  = SUNFALSE;
             LASTFLAG(S) = (ier < 0) ?
@@ -686,7 +684,7 @@ int SUNLinSolSolve_SPTFQMR(SUNLinearSolver S, SUNMatrix A, N_Vector x,
         if (!b_ok) {
           b_ok = SUNTRUE;
           if (preOnLeft) {
-            ier = psolve(P_data, b, vtemp3, delta, PREC_LEFT);
+            ier = psolve(P_data, b, vtemp3, delta, SUN_PREC_LEFT);
             if (ier != 0) {
               *zeroguess  = SUNFALSE;
               LASTFLAG(S) = (ier < 0) ?
@@ -745,7 +743,7 @@ int SUNLinSolSolve_SPTFQMR(SUNLinearSolver S, SUNMatrix A, N_Vector x,
     else N_VScale(ONE, p, vtemp1);
     if (preOnRight) {
       N_VScale(ONE, vtemp1, v);
-      ier = psolve(P_data, v, vtemp1, delta, PREC_RIGHT);
+      ier = psolve(P_data, v, vtemp1, delta, SUN_PREC_RIGHT);
       if (ier != 0) {
         *zeroguess  = SUNFALSE;
         LASTFLAG(S) = (ier < 0) ?
@@ -761,7 +759,7 @@ int SUNLinSolSolve_SPTFQMR(SUNLinearSolver S, SUNMatrix A, N_Vector x,
       return(LASTFLAG(S));
     }
     if (preOnLeft) {
-      ier = psolve(P_data, v, vtemp1, delta, PREC_LEFT);
+      ier = psolve(P_data, v, vtemp1, delta, SUN_PREC_LEFT);
       if (ier != 0) {
         *zeroguess  = SUNFALSE;
         LASTFLAG(S) = (ier < 0) ?
@@ -785,7 +783,7 @@ int SUNLinSolSolve_SPTFQMR(SUNLinearSolver S, SUNMatrix A, N_Vector x,
   if ((converged == SUNTRUE) || (r_curr_norm < r_init_norm)) {
     if (scale_x) N_VDiv(x, sx, x);
     if (preOnRight) {
-      ier = psolve(P_data, x, vtemp1, delta, PREC_RIGHT);
+      ier = psolve(P_data, x, vtemp1, delta, SUN_PREC_RIGHT);
       if (ier != 0) {
         *zeroguess  = SUNFALSE;
         LASTFLAG(S) = (ier < 0) ?
@@ -913,7 +911,6 @@ int SUNLinSolFree_SPTFQMR(SUNLinearSolver S)
 int SUNLinSolSetInfoFile_SPTFQMR(SUNLinearSolver S,
                                  FILE* info_file)
 {
-#ifdef SUNDIALS_BUILD_WITH_MONITORING
   /* check that the linear solver is non-null */
   if (S == NULL)
     return(SUNLS_MEM_NULL);
@@ -921,17 +918,12 @@ int SUNLinSolSetInfoFile_SPTFQMR(SUNLinearSolver S,
   SPTFQMR_CONTENT(S)->info_file = info_file;
 
   return(SUNLS_SUCCESS);
-#else
-  SUNDIALS_DEBUG_PRINT("ERROR in SUNLinSolSetInfoFile_SPTFQMR: SUNDIALS was not built with monitoring\n");
-  return(SUNLS_ILL_INPUT);
-#endif
 }
 
 
 int SUNLinSolSetPrintLevel_SPTFQMR(SUNLinearSolver S,
                                    int print_level)
 {
-#ifdef SUNDIALS_BUILD_WITH_MONITORING
   /* check that the linear solver is non-null */
   if (S == NULL)
     return(SUNLS_MEM_NULL);
@@ -943,8 +935,4 @@ int SUNLinSolSetPrintLevel_SPTFQMR(SUNLinearSolver S,
   SPTFQMR_CONTENT(S)->print_level = print_level;
 
   return(SUNLS_SUCCESS);
-#else
-  SUNDIALS_DEBUG_PRINT("ERROR in SUNLinSolSetPrintLevel_SPTFQMR: SUNDIALS was not built with monitoring\n");
-  return(SUNLS_ILL_INPUT);
-#endif
 }
