@@ -5,7 +5,7 @@
  *     Alan C. Hindmarsh and Radu Serban @ LLNL
  * -----------------------------------------------------------------
  * SUNDIALS Copyright Start
- * Copyright (c) 2002-2021, Lawrence Livermore National Security
+ * Copyright (c) 2002-2022, Lawrence Livermore National Security
  * and Southern Methodist University.
  * All rights reserved.
  *
@@ -20,18 +20,15 @@
 
 #include <stdio.h>
 #include <stdlib.h>
-
-#include <sunmatrix/sunmatrix_dense.h>
 #include <sundials/sundials_math.h>
+#include <sunmatrix/sunmatrix_dense.h>
 
 #define ZERO RCONST(0.0)
 #define ONE  RCONST(1.0)
 
-
 /* Private function prototypes */
-static booleantype SMCompatible_Dense(SUNMatrix A, SUNMatrix B);
-static booleantype SMCompatible2_Dense(SUNMatrix A, N_Vector x, N_Vector y);
-
+static booleantype compatibleMatrices(SUNMatrix A, SUNMatrix B);
+static booleantype compatibleMatrixAndVectors(SUNMatrix A, N_Vector x, N_Vector y);
 
 /*
  * -----------------------------------------------------------------
@@ -43,19 +40,21 @@ static booleantype SMCompatible2_Dense(SUNMatrix A, N_Vector x, N_Vector y);
  * Function to create a new dense matrix
  */
 
-SUNMatrix SUNDenseMatrix(sunindextype M, sunindextype N)
+SUNMatrix SUNDenseMatrix(sunindextype M, sunindextype N, SUNContext sunctx)
 {
   SUNMatrix A;
   SUNMatrixContent_Dense content;
   sunindextype j;
 
   /* return with NULL matrix on illegal dimension input */
-  if ( (M <= 0) || (N <= 0) ) return(NULL);
+  if ((M <= 0) || (N <= 0))
+    return (NULL);
 
   /* Create an empty matrix object */
   A = NULL;
-  A = SUNMatNewEmpty();
-  if (A == NULL) return(NULL);
+  A = SUNMatNewEmpty(sunctx);
+  if (A == NULL)
+    return (NULL);
 
   /* Attach operations */
   A->ops->getid     = SUNMatGetID_Dense;
@@ -70,8 +69,11 @@ SUNMatrix SUNDenseMatrix(sunindextype M, sunindextype N)
 
   /* Create content */
   content = NULL;
-  content = (SUNMatrixContent_Dense) malloc(sizeof *content);
-  if (content == NULL) { SUNMatDestroy(A); return(NULL); }
+  content = (SUNMatrixContent_Dense)malloc(sizeof *content);
+  if (content == NULL) {
+    SUNMatDestroy(A);
+    return (NULL);
+  }
 
   /* Attach content */
   A->content = content;
@@ -79,54 +81,59 @@ SUNMatrix SUNDenseMatrix(sunindextype M, sunindextype N)
   /* Fill content */
   content->M     = M;
   content->N     = N;
-  content->ldata = M*N;
+  content->ldata = M * N;
   content->data  = NULL;
   content->cols  = NULL;
 
   /* Allocate content */
-  content->data = (realtype *) calloc(M * N, sizeof(realtype));
-  if (content->data == NULL) { SUNMatDestroy(A); return(NULL); }
+  content->data = (realtype*)calloc(M * N, sizeof(realtype));
+  if (content->data == NULL) {
+    SUNMatDestroy(A);
+    return (NULL);
+  }
 
-  content->cols = (realtype **) malloc(N * sizeof(realtype *));
-  if (content->cols == NULL) { SUNMatDestroy(A); return(NULL); }
-  for (j=0; j<N; j++) content->cols[j] = content->data + j * M;
+  content->cols = (realtype**)malloc(N * sizeof(realtype*));
+  if (content->cols == NULL) {
+    SUNMatDestroy(A);
+    return (NULL);
+  }
+  for (j = 0; j < N; j++)
+    content->cols[j] = content->data + j * M;
 
-  return(A);
+  return (A);
 }
 
-
 /* ----------------------------------------------------------------------------
- * Function to print the dense matrix 
+ * Function to print the dense matrix
  */
- 
+
 void SUNDenseMatrix_Print(SUNMatrix A, FILE* outfile)
 {
   sunindextype i, j;
-  
-  /* should not be called unless A is a dense matrix; 
+
+  /* should not be called unless A is a dense matrix;
      otherwise return immediately */
   if (SUNMatGetID(A) != SUNMATRIX_DENSE)
     return;
 
   /* perform operation */
-  fprintf(outfile,"\n");
-  for (i=0; i<SM_ROWS_D(A); i++) {
-    for (j=0; j<SM_COLUMNS_D(A); j++) {
+  fprintf(outfile, "\n");
+  for (i = 0; i < SM_ROWS_D(A); i++) {
+    for (j = 0; j < SM_COLUMNS_D(A); j++) {
 #if defined(SUNDIALS_EXTENDED_PRECISION)
-      fprintf(outfile,"%12Lg  ", SM_ELEMENT_D(A,i,j));
+      fprintf(outfile, "%12Lg  ", SM_ELEMENT_D(A, i, j));
 #elif defined(SUNDIALS_DOUBLE_PRECISION)
-      fprintf(outfile,"%12g  ", SM_ELEMENT_D(A,i,j));
+      fprintf(outfile, "%12g  ", SM_ELEMENT_D(A, i, j));
 #else
-      fprintf(outfile,"%12g  ", SM_ELEMENT_D(A,i,j));
+      fprintf(outfile, "%12g  ", SM_ELEMENT_D(A, i, j));
 #endif
     }
-    fprintf(outfile,"\n");
+    fprintf(outfile, "\n");
   }
-  fprintf(outfile,"\n");
+  fprintf(outfile, "\n");
   return;
 }
 
-
 /* ----------------------------------------------------------------------------
  * Functions to access the contents of the dense matrix structure
  */
@@ -174,32 +181,29 @@ realtype** SUNDenseMatrix_Cols(SUNMatrix A)
 realtype* SUNDenseMatrix_Column(SUNMatrix A, sunindextype j)
 {
   if (SUNMatGetID(A) == SUNMATRIX_DENSE)
-    return SM_COLUMN_D(A,j);
+    return SM_COLUMN_D(A, j);
   else
     return NULL;
 }
 
-
 /*
  * -----------------------------------------------------------------
  * implementation of matrix operations
  * -----------------------------------------------------------------
  */
 
-SUNMatrix_ID SUNMatGetID_Dense(SUNMatrix A)
-{
-  return SUNMATRIX_DENSE;
-}
+SUNMatrix_ID SUNMatGetID_Dense(SUNMatrix A) { return SUNMATRIX_DENSE; }
 
 SUNMatrix SUNMatClone_Dense(SUNMatrix A)
 {
-  SUNMatrix B = SUNDenseMatrix(SM_ROWS_D(A), SM_COLUMNS_D(A));
-  return(B);
+  SUNMatrix B = SUNDenseMatrix(SM_ROWS_D(A), SM_COLUMNS_D(A), A->sunctx);
+  return (B);
 }
 
 void SUNMatDestroy_Dense(SUNMatrix A)
 {
-  if (A == NULL) return;
+  if (A == NULL)
+    return;
 
   /* free content */
   if (A->content != NULL) {
@@ -219,8 +223,12 @@ void SUNMatDestroy_Dense(SUNMatrix A)
   }
 
   /* free ops and matrix */
-  if (A->ops) { free(A->ops); A->ops = NULL; }
-  free(A); A = NULL;
+  if (A->ops) {
+    free(A->ops);
+    A->ops = NULL;
+  }
+  free(A);
+  A = NULL;
 
   return;
 }
@@ -228,12 +236,13 @@ void SUNMatDestroy_Dense(SUNMatrix A)
 int SUNMatZero_Dense(SUNMatrix A)
 {
   sunindextype i;
-  realtype *Adata;
+  realtype* Adata;
 
-  /* Perform operation */
+  /* Perform operation A_ij = 0 */
   Adata = SM_DATA_D(A);
-  for (i=0; i<SM_LDATA_D(A); i++)
+  for (i = 0; i < SM_LDATA_D(A); i++)
     Adata[i] = ZERO;
+
   return SUNMAT_SUCCESS;
 }
 
@@ -241,14 +250,14 @@ int SUNMatCopy_Dense(SUNMatrix A, SUNMatrix B)
 {
   sunindextype i, j;
 
-  /* Verify that A and B are compatible */
-  if (!SMCompatible_Dense(A, B))
+  if (!compatibleMatrices(A, B))
     return SUNMAT_ILL_INPUT;
 
-  /* Perform operation */
-  for (j=0; j<SM_COLUMNS_D(A); j++)
-    for (i=0; i<SM_ROWS_D(A); i++)
-      SM_ELEMENT_D(B,i,j) = SM_ELEMENT_D(A,i,j);
+  /* Perform operation B_ij = A_ij */
+  for (j = 0; j < SM_COLUMNS_D(A); j++)
+    for (i = 0; i < SM_ROWS_D(A); i++)
+      SM_ELEMENT_D(B, i, j) = SM_ELEMENT_D(A, i, j);
+
   return SUNMAT_SUCCESS;
 }
 
@@ -256,13 +265,15 @@ int SUNMatScaleAddI_Dense(realtype c, SUNMatrix A)
 {
   sunindextype i, j;
 
-  /* Perform operation */
-  for (j=0; j<SM_COLUMNS_D(A); j++)
-    for (i=0; i<SM_ROWS_D(A); i++) {
-      SM_ELEMENT_D(A,i,j) *= c;
-      if (i == j) 
-        SM_ELEMENT_D(A,i,j) += ONE;
+  /* Perform operation A = c*A + I */
+  for (j = 0; j < SM_COLUMNS_D(A); j++) {
+    for (i = 0; i < SM_ROWS_D(A); i++) {
+      SM_ELEMENT_D(A, i, j) *= c;
+      if (i == j)
+        SM_ELEMENT_D(A, i, j) += ONE;
     }
+  }
+
   return SUNMAT_SUCCESS;
 }
 
@@ -270,14 +281,14 @@ int SUNMatScaleAdd_Dense(realtype c, SUNMatrix A, SUNMatrix B)
 {
   sunindextype i, j;
 
-  /* Verify that A and B are compatible */
-  if (!SMCompatible_Dense(A, B))
+  if (!compatibleMatrices(A, B))
     return SUNMAT_ILL_INPUT;
 
-  /* Perform operation */
-  for (j=0; j<SM_COLUMNS_D(A); j++)
-    for (i=0; i<SM_ROWS_D(A); i++)
-      SM_ELEMENT_D(A,i,j) = c*SM_ELEMENT_D(A,i,j) + SM_ELEMENT_D(B,i,j);
+  /* Perform operation A = c*A + B */
+  for (j = 0; j < SM_COLUMNS_D(A); j++)
+    for (i = 0; i < SM_ROWS_D(A); i++)
+      SM_ELEMENT_D(A, i, j) = c * SM_ELEMENT_D(A, i, j) + SM_ELEMENT_D(B, i, j);
+
   return SUNMAT_SUCCESS;
 }
 
@@ -285,71 +296,67 @@ int SUNMatMatvec_Dense(SUNMatrix A, N_Vector x, N_Vector y)
 {
   sunindextype i, j;
   realtype *col_j, *xd, *yd;
-  
-  /* Verify that A, x and y are compatible */
-  if (!SMCompatible2_Dense(A, x, y))
+
+  if (!compatibleMatrixAndVectors(A, x, y))
     return SUNMAT_ILL_INPUT;
 
-  /* access vector data (return if failure) */
+  /* access vector data (return if NULL data pointers) */
   xd = N_VGetArrayPointer(x);
   yd = N_VGetArrayPointer(y);
   if ((xd == NULL) || (yd == NULL) || (xd == yd))
     return SUNMAT_MEM_FAIL;
 
-  /* Perform operation */
-  for (i=0; i<SM_ROWS_D(A); i++)
+  /* Perform operation y = Ax */
+  for (i = 0; i < SM_ROWS_D(A); i++)
     yd[i] = ZERO;
-  for(j=0; j<SM_COLUMNS_D(A); j++) {
-    col_j = SM_COLUMN_D(A,j);
-    for (i=0; i<SM_ROWS_D(A); i++)
-      yd[i] += col_j[i]*xd[j];
+  for (j = 0; j < SM_COLUMNS_D(A); j++) {
+    col_j = SM_COLUMN_D(A, j);
+    for (i = 0; i < SM_ROWS_D(A); i++)
+      yd[i] += col_j[i] * xd[j];
   }
+
   return SUNMAT_SUCCESS;
 }
 
-int SUNMatSpace_Dense(SUNMatrix A, long int *lenrw, long int *leniw)
+int SUNMatSpace_Dense(SUNMatrix A, long int* lenrw, long int* leniw)
 {
   *lenrw = SM_LDATA_D(A);
   *leniw = 3 + SM_COLUMNS_D(A);
   return SUNMAT_SUCCESS;
 }
 
-
 /*
  * -----------------------------------------------------------------
  * private functions
  * -----------------------------------------------------------------
  */
 
-static booleantype SMCompatible_Dense(SUNMatrix A, SUNMatrix B)
+static booleantype compatibleMatrices(SUNMatrix A, SUNMatrix B)
 {
   /* both matrices must be SUNMATRIX_DENSE */
-  if (SUNMatGetID(A) != SUNMATRIX_DENSE)
-    return SUNFALSE;
-  if (SUNMatGetID(B) != SUNMATRIX_DENSE)
+  if ((SUNMatGetID(A) != SUNMATRIX_DENSE) || (SUNMatGetID(B) != SUNMATRIX_DENSE)) {
     return SUNFALSE;
+  }
 
   /* both matrices must have the same shape */
-  if (SM_ROWS_D(A) != SM_ROWS_D(B))
-    return SUNFALSE;
-  if (SM_COLUMNS_D(A) != SM_COLUMNS_D(B))
+  if ((SM_ROWS_D(A) != SM_ROWS_D(B)) || (SM_COLUMNS_D(A) != SM_COLUMNS_D(B))) {
     return SUNFALSE;
+  }
 
   return SUNTRUE;
 }
 
-
-static booleantype SMCompatible2_Dense(SUNMatrix A, N_Vector x, N_Vector y)
+static booleantype compatibleMatrixAndVectors(SUNMatrix A, N_Vector x, N_Vector y)
 {
-  /*   vectors must be one of {SERIAL, OPENMP, PTHREADS} */ 
-  if ( (N_VGetVectorID(x) != SUNDIALS_NVEC_SERIAL) &&
-       (N_VGetVectorID(x) != SUNDIALS_NVEC_OPENMP) &&
-       (N_VGetVectorID(x) != SUNDIALS_NVEC_PTHREADS) )
+  /* Vectors must provide nvgetarraypointer and cannot be a parallel vector */
+  if (!x->ops->nvgetarraypointer || !y->ops->nvgetarraypointer) {
     return SUNFALSE;
+  }
 
-  /* Optimally we would verify that the dimensions of A, x and y agree, 
-   but since there is no generic 'length' routine for N_Vectors we cannot */
+  /* Check that the dimensions agree */
+  if ((N_VGetLength(x) != SM_COLUMNS_D(A)) || (N_VGetLength(y) != SM_ROWS_D(A))) {
+    return SUNFALSE;
+  }
 
   return SUNTRUE;
 }
-
