@@ -2,7 +2,7 @@
  * Programmer(s): Daniel R. Reynolds @ SMU
  * -----------------------------------------------------------------------------
  * SUNDIALS Copyright Start
- * Copyright (c) 2002-2021, Lawrence Livermore National Security
+ * Copyright (c) 2002-2022, Lawrence Livermore National Security
  * and Southern Methodist University.
  * All rights reserved.
  *
@@ -23,7 +23,8 @@
 #include <sundials/sundials_math.h>
 #include <sundials/sundials_nvector_senswrapper.h>
 
-#include "sundials_debug.h"
+#include "sundials_context_impl.h"
+#include "sundials_logger_impl.h"
 
 /* Internal utility routines */
 static int AndersonAccelerate(SUNNonlinearSolver NLS, N_Vector gval, N_Vector x,
@@ -43,7 +44,7 @@ static void FreeContent(SUNNonlinearSolver NLS);
   Constructor to create a new fixed point solver
   ============================================================================*/
 
-SUNNonlinearSolver SUNNonlinSol_FixedPoint(N_Vector y, int m)
+SUNNonlinearSolver SUNNonlinSol_FixedPoint(N_Vector y, int m, SUNContext sunctx)
 {
   SUNNonlinearSolver NLS;
   SUNNonlinearSolverContent_FixedPoint content;
@@ -61,8 +62,7 @@ SUNNonlinearSolver SUNNonlinSol_FixedPoint(N_Vector y, int m)
     return(NULL);
 
   /* Create nonlinear linear solver */
-  NLS = NULL;
-  NLS = SUNNonlinSolNewEmpty();
+  NLS = SUNNonlinSolNewEmpty(sunctx);
   if (NLS == NULL) return(NULL);
 
   /* Attach operations */
@@ -100,7 +100,10 @@ SUNNonlinearSolver SUNNonlinSol_FixedPoint(N_Vector y, int m)
   content->nconvfails  = 0;
   content->ctest_data  = NULL;
   content->print_level = 0;
-  content->info_file   = NULL;
+  content->info_file   = stdout;
+#if SUNDIALS_LOGGING_LEVEL >= SUNDIALS_LOGGING_INFO
+  content->info_file   = (sunctx->logger->info_fp) ? sunctx->logger->info_fp : stdout;
+#endif
 
   /* Fill allocatable content */
   retval = AllocateContent(NLS, y);
@@ -114,7 +117,8 @@ SUNNonlinearSolver SUNNonlinSol_FixedPoint(N_Vector y, int m)
   Constructor wrapper to create a new fixed point solver for sensitivity solvers
   ============================================================================*/
 
-SUNNonlinearSolver SUNNonlinSol_FixedPointSens(int count, N_Vector y, int m)
+SUNNonlinearSolver SUNNonlinSol_FixedPointSens(int count, N_Vector y, int m,
+                                               SUNContext sunctx)
 {
   SUNNonlinearSolver NLS;
   N_Vector w;
@@ -123,7 +127,7 @@ SUNNonlinearSolver SUNNonlinSol_FixedPointSens(int count, N_Vector y, int m)
   w = N_VNew_SensWrapper(count, y);
 
   /* create nonlinear solver using sensitivity vector wrapper */
-  NLS = SUNNonlinSol_FixedPoint(w, m);
+  NLS = SUNNonlinSol_FixedPoint(w, m, sunctx);
 
   /* free sensitivity vector wrapper */
   N_VDestroy(w);
@@ -210,14 +214,18 @@ int SUNNonlinSolSolve_FixedPoint(SUNNonlinearSolver NLS, N_Vector y0,
   FP_CONTENT(NLS)->niters     = 0;
   FP_CONTENT(NLS)->nconvfails = 0;
 
-#ifdef SUNDIALS_BUILD_WITH_MONITORING
-      /* print current iteration number and the nonlinear residual */
-      if (FP_CONTENT(NLS)->print_level && FP_CONTENT(NLS)->info_file)
-      {
-        fprintf(FP_CONTENT(NLS)->info_file,
-                "SUNNONLINSOL_FIXEDPOINT (nni=%ld):\n",
-                (long int) FP_CONTENT(NLS)->niters);
-      }
+#if SUNDIALS_LOGGING_LEVEL >= SUNDIALS_LOGGING_INFO
+  /* print current iteration number and the nonlinear residual */
+  if (FP_CONTENT(NLS)->print_level && FP_CONTENT(NLS)->info_file
+      && (FP_CONTENT(NLS)->info_file != NLS->sunctx->logger->info_fp))
+  {
+    fprintf(FP_CONTENT(NLS)->info_file,
+            "SUNNONLINSOL_FIXEDPOINT (nni=%ld):\n",
+            (long int) FP_CONTENT(NLS)->niters);
+  }
+  SUNLogger_QueueMsg(NLS->sunctx->logger, SUN_LOGLEVEL_INFO,
+    "SUNNonlinSolSolve_FixedPoint", "begin-iteration",
+    "iter = %ld, nni = %ld", (long int) 0, FP_CONTENT(NLS)->niters);
 #endif
 
   /* Looping point for attempts at solution of the nonlinear system:
@@ -252,15 +260,19 @@ int SUNNonlinSolSolve_FixedPoint(SUNNonlinearSolver NLS, N_Vector y0,
     retval = FP_CONTENT(NLS)->CTest(NLS, ycor, delta, tol, w,
                                     FP_CONTENT(NLS)->ctest_data);
 
-#ifdef SUNDIALS_BUILD_WITH_MONITORING
-      /* print current iteration number and the nonlinear residual */
-      if (FP_CONTENT(NLS)->print_level && FP_CONTENT(NLS)->info_file)
-      {
-        fprintf(FP_CONTENT(NLS)->info_file,
-                SUN_NLS_MSG_RESIDUAL,
-                (long int) FP_CONTENT(NLS)->curiter,
-                N_VWrmsNorm(delta, w));
-      }
+#if SUNDIALS_LOGGING_LEVEL >= SUNDIALS_LOGGING_INFO
+    /* print current iteration number and the nonlinear residual */
+    if (FP_CONTENT(NLS)->print_level && FP_CONTENT(NLS)->info_file
+        && (FP_CONTENT(NLS)->info_file != NLS->sunctx->logger->info_fp))
+    {
+      fprintf(FP_CONTENT(NLS)->info_file,
+              "SUNNONLINSOL_FIXEDPOINT (nni=%ld):\n",
+              (long int) FP_CONTENT(NLS)->niters);
+    }
+    SUNLogger_QueueMsg(NLS->sunctx->logger, SUN_LOGLEVEL_INFO,
+      "SUNNonlinSolSolve_FixedPoint", "end-of-iterate",
+      "iter = %ld, nni = %ld, wrmsnorm = %.16g",  (long int) FP_CONTENT(NLS)->curiter,
+      FP_CONTENT(NLS)->niters, N_VWrmsNorm(delta, w));
 #endif
 
     /* return if successful */
@@ -731,7 +743,6 @@ static void FreeContent(SUNNonlinearSolver NLS)
 int SUNNonlinSolSetInfoFile_FixedPoint(SUNNonlinearSolver NLS,
                                        FILE* info_file)
 {
-#ifdef SUNDIALS_BUILD_WITH_MONITORING
   /* check that the nonlinear solver is non-null */
   if (NLS == NULL)
     return(SUN_NLS_MEM_NULL);
@@ -739,16 +750,11 @@ int SUNNonlinSolSetInfoFile_FixedPoint(SUNNonlinearSolver NLS,
   FP_CONTENT(NLS)->info_file = info_file;
 
   return(SUN_NLS_SUCCESS);
-#else
-  SUNDIALS_DEBUG_PRINT("ERROR in SUNNonlinSolSetInfoFile_FixedPoint: SUNDIALS was not built with monitoring\n");
-  return(SUN_NLS_ILL_INPUT);
-#endif
 }
 
 int SUNNonlinSolSetPrintLevel_FixedPoint(SUNNonlinearSolver NLS,
                                          int print_level)
 {
-#ifdef SUNDIALS_BUILD_WITH_MONITORING
   /* check that the nonlinear solver is non-null */
   if (NLS == NULL)
     return(SUN_NLS_MEM_NULL);
@@ -760,8 +766,4 @@ int SUNNonlinSolSetPrintLevel_FixedPoint(SUNNonlinearSolver NLS,
   FP_CONTENT(NLS)->print_level = print_level;
 
   return(SUN_NLS_SUCCESS);
-#else
-  SUNDIALS_DEBUG_PRINT("ERROR in SUNNonlinSolSetPrintLevel_FixedPoint: SUNDIALS was not built with monitoring\n");
-  return(SUN_NLS_ILL_INPUT);
-#endif
 }
