@@ -2,7 +2,7 @@
  * Programmer(s): Daniel R. Reynolds @ SMU
  *---------------------------------------------------------------
  * SUNDIALS Copyright Start
- * Copyright (c) 2002-2021, Lawrence Livermore National Security
+ * Copyright (c) 2002-2022, Lawrence Livermore National Security
  * and Southern Methodist University.
  * All rights reserved.
  *
@@ -61,9 +61,15 @@ static int dense_MM(SUNMatrix A, SUNMatrix B, SUNMatrix C);
 // Private function to check function return values
 static int check_flag(void *flagvalue, const string funcname, int opt);
 
+// SUNContext for the simulation
+static SUNContext sunctx = NULL;
+
 // Main Program
 int main(int argc, char* argv[])
 {
+  // Create the SUNDIALS context object for this simulation.
+  SUNContext_Create(NULL, &sunctx);
+
   // general problem parameters
   realtype T0 = RCONST(0.0);         // initial time
   realtype Tf = RCONST(0.05);        // final time
@@ -85,12 +91,11 @@ int main(int argc, char* argv[])
   void *arkstep_mem = NULL;       // empty ARKStep memory structure
   void *mristep_mem = NULL;       // empty MRIStep memory structure
   void *inner_mem = NULL;         // empty inner ARKStep memory structure
-  ARKodeButcherTable B = NULL;
   int numfails;
   booleantype fixedpoint;
   realtype t, tcur;
   long int ark_nst, ark_nfe, ark_nfi, ark_nsetups, ark_nje, ark_nfeLS, ark_nni, ark_ncfn;
-  long int mri_nst, mri_nfs, mri_nsetups, mri_nje, mri_nfeLS, mri_nni, mri_ncfn;
+  long int mri_nst, mri_nfse, mri_nfsi, mri_nsetups, mri_nje, mri_nfeLS, mri_nni, mri_ncfn;
 
   // if an argument supplied, set fixedpoint (otherwise use SUNFALSE)
   fixedpoint = SUNFALSE;
@@ -109,30 +114,50 @@ int main(int argc, char* argv[])
   }
 
   // Initialize vector data structure and specify initial condition
-  y = N_VNew_Serial(NEQ);
+  y = N_VNew_Serial(NEQ, sunctx);
   if (check_flag((void *)y, "N_VNew_Serial", 0)) return 1;
   N_VConst(ONE, y);
 
   /* Call ARKStepCreate and MRIStepCreate to initialize the timesteppers */
-  arkstep_mem = ARKStepCreate(NULL, f, T0, y);
+  arkstep_mem = ARKStepCreate(NULL, f, T0, y, sunctx);
   if (check_flag((void *) arkstep_mem, "ARKStepCreate", 0)) return 1;
-  inner_mem = ARKStepCreate(f0, NULL, T0, y);
+
+  inner_mem = ARKStepCreate(f0, NULL, T0, y, sunctx);
   if (check_flag((void *) inner_mem, "ARKStepCreate", 0)) return 1;
-  mristep_mem = MRIStepCreate(f, T0, y, MRISTEP_ARKSTEP, inner_mem);
+
+  MRIStepInnerStepper inner_stepper = NULL;
+  flag = ARKStepCreateMRIStepInnerStepper(inner_mem, &inner_stepper);
+  if (check_flag(&flag, "ARKStepCreateMRIStepInnerStepper", 1)) return 1;
+
+  mristep_mem = MRIStepCreate(NULL, f, T0, y, inner_stepper, sunctx);
   if (check_flag((void *) mristep_mem, "MRIStepCreate", 0)) return 1;
 
-  // Create solve-decoupled DIRK2 (trapezoidal) Butcher table
-  B = ARKodeButcherTable_Alloc(3, SUNFALSE);
+  // Create DIRK2 (trapezoidal) Butcher table
+  ARKodeButcherTable B = ARKodeButcherTable_Alloc(2, SUNFALSE);
   if (check_flag((void *)B, "ARKodeButcherTable_Alloc", 0)) return 1;
-  B->A[1][0] = ONE;
-  B->A[2][0] = RCONST(0.5);
-  B->A[2][2] = RCONST(0.5);
+  B->A[1][0] = RCONST(0.5);
+  B->A[1][1] = RCONST(0.5);
   B->b[0] = RCONST(0.5);
-  B->b[2] = RCONST(0.5);
+  B->b[1] = RCONST(0.5);
   B->c[1] = ONE;
-  B->c[2] = ONE;
   B->q=2;
 
+  // Create solve-decoupled DIRK2 (trapezoidal) Butcher table
+  ARKodeButcherTable Bc = ARKodeButcherTable_Alloc(3, SUNFALSE);
+  if (check_flag((void *)Bc, "ARKodeButcherTable_Alloc", 0)) return 1;
+  Bc->A[1][0] = ONE;
+  Bc->A[2][0] = RCONST(0.5);
+  Bc->A[2][2] = RCONST(0.5);
+  Bc->b[0] = RCONST(0.5);
+  Bc->b[2] = RCONST(0.5);
+  Bc->c[1] = ONE;
+  Bc->c[2] = ONE;
+  Bc->q=2;
+
+  // Create the MIS coupling table
+  MRIStepCoupling C = MRIStepCoupling_MIStoMRI(Bc, 2, 0);
+  if (check_flag((void *)C, "MRIStepCoupling_MIStoMRI", 0)) return 1;
+
   // Set routines
   flag = ARKStepSetUserData(arkstep_mem, (void *) &lamda);   // Pass lamda to user functions
   if (check_flag(&flag, "ARKStepSetUserData", 1)) return 1;
@@ -144,6 +169,7 @@ int main(int argc, char* argv[])
   if (check_flag(&flag, "ARKStepSetTables", 1)) return 1;
   flag = ARKStepSetMaxNumSteps(arkstep_mem, 2*Nt);           // Increase num internal steps
   if (check_flag(&flag, "ARKStepSetMaxNumSteps", 1)) return 1;
+
   flag = MRIStepSetUserData(mristep_mem, (void *) &lamda);   // Pass lamda to user functions
   if (check_flag(&flag, "MRIStepSetUserData", 1)) return 1;
   flag = MRIStepSStolerances(mristep_mem, reltol, abstol);   // Specify tolerances
@@ -152,8 +178,8 @@ int main(int argc, char* argv[])
   if (check_flag(&flag, "MRIStepSetFixedStep", 1)) return 1;
   flag = ARKStepSetFixedStep(inner_mem, Tf/Nt/10);
   if (check_flag(&flag, "ARKStepSetFixedStep", 1)) return 1;
-  flag = MRIStepSetTable(mristep_mem, 2, B);                 // Specify Butcher table
-  if (check_flag(&flag, "MRIStepSetTable", 1)) return 1;
+  flag = MRIStepSetCoupling(mristep_mem, C);                 // Specify Butcher table
+  if (check_flag(&flag, "MRIStepSetCoupling", 1)) return 1;
   flag = MRIStepSetMaxNumSteps(mristep_mem, 2*Nt);           // Increase num internal steps
   if (check_flag(&flag, "MRIStepSetMaxNumSteps", 1)) return 1;
 
@@ -161,11 +187,12 @@ int main(int argc, char* argv[])
   if (fixedpoint) {
 
     // Initialize fixed-point solvers and attach to integrators
-    NLSa = SUNNonlinSol_FixedPoint(y, 50);
+    NLSa = SUNNonlinSol_FixedPoint(y, 50, sunctx);
     if (check_flag((void *)NLSa, "SUNNonlinSol_FixedPoint", 0)) return 1;
     flag = ARKStepSetNonlinearSolver(arkstep_mem, NLSa);
     if (check_flag(&flag, "ARKStepSetNonlinearSolver", 1)) return 1;
-    NLSm = SUNNonlinSol_FixedPoint(y, 50);
+
+    NLSm = SUNNonlinSol_FixedPoint(y, 50, sunctx);
     if (check_flag((void *)NLSm, "SUNNonlinSol_FixedPoint", 0)) return 1;
     flag = MRIStepSetNonlinearSolver(mristep_mem, NLSm);
     if (check_flag(&flag, "MRIStepSetNonlinearSolver", 1)) return 1;
@@ -173,13 +200,14 @@ int main(int argc, char* argv[])
   } else {
 
     // Initialize dense matrix data structures and solvers
-    Aa = SUNDenseMatrix(NEQ, NEQ);
+    Aa = SUNDenseMatrix(NEQ, NEQ, sunctx);
     if (check_flag((void *)Aa, "SUNDenseMatrix", 0)) return 1;
-    LSa = SUNLinSol_Dense(y, Aa);
+    LSa = SUNLinSol_Dense(y, Aa, sunctx);
     if (check_flag((void *)LSa, "SUNLinSol_Dense", 0)) return 1;
-    Am = SUNDenseMatrix(NEQ, NEQ);
+
+    Am = SUNDenseMatrix(NEQ, NEQ, sunctx);
     if (check_flag((void *)Am, "SUNDenseMatrix", 0)) return 1;
-    LSm = SUNLinSol_Dense(y, Am);
+    LSm = SUNLinSol_Dense(y, Am, sunctx);
     if (check_flag((void *)LSm, "SUNLinSol_Dense", 0)) return 1;
 
     // Linear solver interface
@@ -187,6 +215,7 @@ int main(int argc, char* argv[])
     if (check_flag(&flag, "ARKStepSetLinearSolver", 1)) return 1;
     flag = ARKStepSetJacFn(arkstep_mem, Jac);
     if (check_flag(&flag, "ARKStepSetJacFn", 1)) return 1;
+
     flag = MRIStepSetLinearSolver(mristep_mem, LSm, Am);
     if (check_flag(&flag, "MRIStepSetLinearSolver", 1)) return 1;
     flag = MRIStepSetJacFn(mristep_mem, Jac);
@@ -195,9 +224,9 @@ int main(int argc, char* argv[])
     // Specify linearly implicit RHS, with non-time-dependent Jacobian
     flag = ARKStepSetLinear(arkstep_mem, 0);
     if (check_flag(&flag, "ARKStepSetLinear", 1)) return 1;
+
     flag = MRIStepSetLinear(mristep_mem, 0);
     if (check_flag(&flag, "MRIStepSetLinear", 1)) return 1;
-
   }
 
 
@@ -247,7 +276,7 @@ int main(int argc, char* argv[])
   if (check_flag(&flag, "MRIStepGetCurrentTime", 1)) return 1;
   flag = MRIStepGetNumSteps(mristep_mem, &mri_nst);
   if (check_flag(&flag, "MRIStepGetNumSteps", 1)) return 1;
-  flag = MRIStepGetNumRhsEvals(mristep_mem, &mri_nfs);
+  flag = MRIStepGetNumRhsEvals(mristep_mem, &mri_nfse, &mri_nfsi);
   if (check_flag(&flag, "MRIStepGetNumRhsEvals", 1)) return 1;
   flag = MRIStepGetNumNonlinSolvIters(mristep_mem, &mri_nni);
   if (check_flag(&flag, "MRIStepGetNumNonlinSolvIters", 1)) return 1;
@@ -265,7 +294,7 @@ int main(int argc, char* argv[])
   cout << "   Return time = " << t << "\n";
   cout << "   Internal final time = " << tcur << "\n";
   cout << "   Internal solver steps = " << mri_nst << "\n";
-  cout << "   Total RHS evals:  Fs = " << mri_nfs << "\n";
+  cout << "   Total RHS evals:  Fs = " << mri_nfsi << "\n";
   cout << "   Total number of nonlinear iterations = " << mri_nni << "\n";
   cout << "   Total number of nonlinear solver convergence failures = " << mri_ncfn << "\n";
   if (!fixedpoint) {
@@ -282,9 +311,9 @@ int main(int argc, char* argv[])
     numfails += 1;
     cout << "  Internal solver steps error: " << ark_nst << " vs " << mri_nst << "\n";
   }
-  if (ark_nfi != (mri_nfs+mri_nst)) {
+  if ((ark_nfi - ark_nst) != mri_nfsi) {
     numfails += 1;
-    cout << "  RHS evals error: " << ark_nfi << " vs " << mri_nfs << "\n";
+    cout << "  RHS evals error: " << ark_nfi << " vs " << mri_nfsi << "\n";
   }
   if (ark_nni != mri_nni) {
     numfails += 1;
@@ -316,9 +345,12 @@ int main(int argc, char* argv[])
 
   // Clean up and return with successful completion
   ARKodeButcherTable_Free(B);  // Free Butcher table
+  ARKodeButcherTable_Free(Bc); // Free Butcher table
+  MRIStepCoupling_Free(C);     // Free MRI coupling table
   ARKStepFree(&arkstep_mem);   // Free integrator memory
   MRIStepFree(&mristep_mem);
   ARKStepFree(&inner_mem);
+  MRIStepInnerStepper_Free(&inner_stepper);
   if (fixedpoint) {
     SUNNonlinSolFree(NLSa);    // Free nonlinear solvers
     SUNNonlinSolFree(NLSm);
@@ -329,6 +361,8 @@ int main(int argc, char* argv[])
     SUNMatDestroy(Am);
   }
   N_VDestroy(y);               // Free y vector
+
+  SUNContext_Free(&sunctx);
   return (numfails);
 }
 
@@ -379,9 +413,9 @@ static int Jac(realtype t, N_Vector y, N_Vector fy,
 {
   realtype *rdata = (realtype *) user_data;   // cast user_data to realtype
   realtype lam = rdata[0];                    // set shortcut for stiffness parameter
-  SUNMatrix V  = SUNDenseMatrix(3,3);          // create temporary SUNMatrix objects
-  SUNMatrix D  = SUNDenseMatrix(3,3);          // create temporary SUNMatrix objects
-  SUNMatrix Vi = SUNDenseMatrix(3,3);          // create temporary SUNMatrix objects
+  SUNMatrix V  = SUNDenseMatrix(3,3,sunctx);  // create temporary SUNMatrix objects
+  SUNMatrix D  = SUNDenseMatrix(3,3,sunctx);  // create temporary SUNMatrix objects
+  SUNMatrix Vi = SUNDenseMatrix(3,3,sunctx);  // create temporary SUNMatrix objects
 
   SUNMatZero(V);        // initialize temporary matrices to zero
   SUNMatZero(D);        // (not technically required)
